MODULE crsini   
   !!======================================================================
   !!                         ***  MODULE crsini  ***
   !!            Manage the grid coarsening module initialization
   !!======================================================================
   !!  History     2012-05   (J. Simeon, G. Madec, C. Ethe, C. Calone) Original code
   !!----------------------------------------------------------------------
   USE dom_oce                  ! For parameters in par_oce (jperio, lk_vvl)
   USE crs                      ! Coarse grid domain
   USE phycst, ONLY: omega, rad ! physical constants
   USE crsdom
   USE crsdomwri
   USE lib_mpp        ! MPP manager
   USE wrk_nemo       ! work arrays
   USE in_out_manager ! I/O manager
   USE iom            ! I/O manager library
   USE timing         ! Timing

   IMPLICIT NONE
   PRIVATE

   PUBLIC  crs_init

   !! * Substitutions
#  include "domzgr_substitute.h90"

CONTAINS
   
   SUBROUTINE crs_init 
      !!-------------------------------------------------------------------
      !!                     *** SUBROUTINE crs_init
      !!  ** Purpose : Initialization of the grid coarsening module  
      !!               1. Read namelist
      !!               X2. MOVE TO crs_dom.F90 Set the domain definitions for coarse grid
      !!                 a. Define the coarse grid starting/ending indices on parent grid
      !!                    Here is where the T-pivot or F-pivot grids are discerned
      !!                 b. TODO.  Include option for north-centric or equator-centric binning.
      !!                 (centered only for odd reduction factors; even reduction bins bias equator to the south)
      !!               3. Mask and mesh creation. => calls to crsfun
      !!                  a. Use crsfun_mask to generate tmask,umask, vmask, fmask.
      !!                  b. Use crsfun_coordinates to get coordinates
      !!                  c. Use crsfun_UV to get horizontal scale factors
      !!                  d. Use crsfun_TW to get initial vertical scale factors   
      !!               4. Volumes and weights jes.... TODO. Updates for vvl? Where to do this? crsstp.F90?
      !!                  a. Calculate initial coarse grid box volumes: pvol_T, pvol_W
      !!                  b. Calculate initial coarse grid surface-averaging weights
      !!                  c. Calculate initial coarse grid volume-averaging weights
      !!                  
      !!               X5. MOVE TO crs_dom_wri.F90 Using iom_rstput output the initial meshmask.
      !!               ?. Another set of "masks" to generate
      !!                  are the u- and v- surface areas for U- and V- area-weighted means.
      !!                  Need to put this somewhere in section 3?
      !! jes. What do to about the vvl?  GM.  could separate the weighting (denominator), so
      !! output C*dA or C*dV as summation not mran, then do mean (division) at moment of output.
      !! As is, crsfun takes into account vvl.   
      !!      Talked about pre-setting the surface array to avoid IF/ENDIFS and division.
      !!      But have then to make that preset array here and elsewhere.
      !!      that is called every timestep...
      !! 
      !!               - Read in pertinent data ?
      !!-------------------------------------------------------------------
      !! Local variables
      INTEGER  :: ji,jj,jk           ! dummy indices
      INTEGER  :: ierr               ! allocation error status
      INTEGER  :: ios                ! Local integer output status for namelist read
      INTEGER  :: iif, iil, ijf, ijl ! indices for tmask_i_crs construction
      REAL(wp) :: zmin,zmax
      REAL(wp), DIMENSION(:,:,:), POINTER :: zfse3t, zfse3u, zfse3v, zfse3w

      NAMELIST/namcrs/ nn_factx, nn_facty, nn_msh_crs, nn_crs_kz, ln_crs_wn, ln_crs_top, ln_crs_wri
      !!-------------------------------------------------------------------------------------------
      !
      IF( nn_timing == 1 )  CALL timing_start('crs_init')
      !
      IF(lwp) THEN
         WRITE(numout,*)
         WRITE(numout,*) 'crs_init : Initializing the grid coarsening module '
      ENDIF

     !---------------------------------------------------------
     ! 1. Read Namelist file
     !---------------------------------------------------------
     !
     REWIND( numnam_ref )              ! Namelist namrun in reference namelist : Parameters of the run
     READ  ( numnam_ref, namcrs, IOSTAT = ios, ERR = 901)
901  IF( ios /= 0 ) CALL ctl_nam ( ios , 'namcrs in reference namelist', lwp )

     REWIND( numnam_cfg )              ! Namelist namrun in configuration namelist : Parameters of the run
     READ  ( numnam_cfg, namcrs, IOSTAT = ios, ERR = 902 )
902  IF( ios /= 0 ) CALL ctl_nam ( ios , 'namcrs in configuration namelist', lwp )
     IF(lwm) WRITE ( numond, namcrs )

     IF( .NOT. ln_crs )ln_crs_top = .FALSE.  
     IF( .NOT. ln_crs )ln_crs_wri = .FALSE.

     IF ( ln_crs_top .AND. ln_crs_wri ) THEN
        IF(lwp) THEN
           WRITE(numout,*) 'You should clarify your choice'
           WRITE(numout,*) 'Do you want use the CRS module to force the BIO (ln_crs_top=.true.)'
           WRITE(numout,*) 'or to write light files (ln_crs_wri=.true.) ???'
           WRITE(numout,*) 'You have to think about it but for the moment we stop here'
           STOP
        ENDIF
     ENDIF

     IF(lwp) THEN
        WRITE(numout,*)
        WRITE(numout,*) 'crs_init: Namelist namcrs '
        WRITE(numout,*) '   coarsening factor in i-direction      nn_factx   = ', nn_factx
        WRITE(numout,*) '   coarsening factor in j-direction      nn_facty   = ', nn_facty
        WRITE(numout,*) '   create (=1) a mesh file or not (=0)   nn_msh_crs = ', nn_msh_crs
        WRITE(numout,*) '   type of Kz coarsening (0,1,2)         nn_crs_kz  = ', nn_crs_kz

        IF ( .NOT. ln_crs_wri ) THEN
           IF( ln_crs_wn )THEN 
              WRITE(numout,*) '   vertical velocities are coarsened '
           ELSE
              WRITE(numout,*) '   computed using hdivn '
           ENDIF
        ENDIF

        IF( ln_crs_top ) WRITE(numout,*) '   coarsening of physics activated for outputs and BGC model'
        IF( ln_crs_wri ) WRITE(numout,*) '   coarsening activated only for outputs'   

        IF ( .NOT. ln_crs_wri ) THEN
           SELECT CASE ( nn_crs_kz )
              CASE ( 0 ) ; WRITE(numout,*) '   coarsene KZ with MEAN(KZ)'
              CASE ( 1 ) ; WRITE(numout,*) '   coarsene KZ with MIN(KZ)'
              CASE ( 2 ) ; WRITE(numout,*) '   coarsene KZ with MAX(KZ)'
              CASE ( 3 ) ; WRITE(numout,*) '   coarsene KZ with MEANLOG(KZ)'
              CASE ( 4 ) ; WRITE(numout,*) '   coarsene KZ with MEDIANE(KZ)'
           END SELECT
        ENDIF
     ENDIF
              
     rfactx_r = 1. / nn_factx
     rfacty_r = 1. / nn_facty

     !---------------------------------------------------------
     ! 2. Define Global Dimensions of the coarsened grid
     !---------------------------------------------------------
     CALL crs_dom_def      

     !---------------------------------------------------------
     ! 3. Mask and Mesh
     !---------------------------------------------------------

     !  3.a. Get the masks   
 
     CALL crs_dom_msk

     !  3.b. Get the coordinates
     !      Odd-numbered reduction factor, center coordinate on T-cell
     !      Even-numbered reduction factor, center coordinate on U-,V- faces or f-corner.
     !      
     CALL crs_dom_coordinates( gphit, glamt, 'T', gphit_crs, glamt_crs ) 
     CALL crs_dom_coordinates( gphiu, glamu, 'U', gphiu_crs, glamu_crs )       
     CALL crs_dom_coordinates( gphiv, glamv, 'V', gphiv_crs, glamv_crs ) 
     CALL crs_dom_coordinates( gphif, glamf, 'F', gphif_crs, glamf_crs ) 

     !  3.c. Get the horizontal mesh

     !      3.c.1 Horizontal scale factors

     CALL crs_dom_hgr( e1t, e2t, 'T', e1t_crs, e2t_crs )
     CALL crs_dom_hgr( e1u, e2u, 'U', e1u_crs, e2u_crs )
     CALL crs_dom_hgr( e1v, e2v, 'V', e1v_crs, e2v_crs )
     CALL crs_dom_hgr( e1f, e2f, 'F', e1f_crs, e2f_crs )

     WHERE(e1t_crs == 0._wp) e1t_crs=r_inf
     WHERE(e1u_crs == 0._wp) e1u_crs=r_inf
     WHERE(e1v_crs == 0._wp) e1v_crs=r_inf
     WHERE(e1f_crs == 0._wp) e1f_crs=r_inf
     WHERE(e2t_crs == 0._wp) e2t_crs=r_inf
     WHERE(e2u_crs == 0._wp) e2u_crs=r_inf
     WHERE(e2v_crs == 0._wp) e2v_crs=r_inf
     WHERE(e2f_crs == 0._wp) e2f_crs=r_inf

     e1e2t_crs(:,:) = e1t_crs(:,:) * e2t_crs(:,:)
     
     
     !      3.c.2 Coriolis factor  

      SELECT CASE( jphgr_msh )   ! type of horizontal mesh
      CASE ( 0, 1, 4 )           ! mesh on the sphere
         ff_crs(:,:) = 2. * omega * SIN( rad * gphif_crs(:,:) )
      CASE DEFAULT 
         IF(lwp) WRITE(numout,*) 'crsini.F90. crs_init. Only jphgr_msh = 0, 1 or 4 supported' 
      END SELECT 

     !      3.d.1 mbathy ( vertical k-levels of bathymetry )     

     CALL crs_dom_bat
     !
     CALL wrk_alloc(jpi, jpj, jpk, zfse3t, zfse3u, zfse3v, zfse3w )
     !
     zfse3t(:,:,:) = e3t_0(:,:,:) !fse3t(:,:,:)
     zfse3u(:,:,:) = e3u_0(:,:,:) !fse3u(:,:,:)
     zfse3v(:,:,:) = e3v_0(:,:,:) !fse3v(:,:,:)
     zfse3w(:,:,:) = e3w_0(:,:,:) !fse3w(:,:,:)

     !    3.d.2   Surfaces 
     e2e3u_crs(:,:,:)=0._wp
     e2e3u_msk(:,:,:)=0._wp
     e1e3v_crs(:,:,:)=0._wp
     e1e3v_msk(:,:,:)=0._wp
     CALL crs_dom_sfc( tmask, 'W', e1e2w_crs, e1e2w_msk, p_e1=e1t, p_e2=e2t    )
     CALL crs_dom_sfc( umask, 'U', e2e3u_crs, e2e3u_msk, p_e2=e2u, p_e3=zfse3u )
     CALL crs_dom_sfc( vmask, 'V', e1e3v_crs, e1e3v_msk, p_e1=e1v, p_e3=zfse3v )

     DO jk=1,jpk
        DO ji=1,jpi_crs
           DO jj=1,jpj_crs
              facsurfu(ji,jj,jk) = umask_crs(ji,jj,jk) * e2e3u_msk(ji,jj,jk)  
              IF( e2e3u_crs(ji,jj,jk) .NE. 0._wp ) facsurfu(ji,jj,jk) = facsurfu(ji,jj,jk) / e2e3u_crs(ji,jj,jk)
              facsurfv(ji,jj,jk) = vmask_crs(ji,jj,jk) * e1e3v_msk(ji,jj,jk)  
              IF( e1e3v_crs(ji,jj,jk) .NE. 0._wp ) facsurfv(ji,jj,jk) = facsurfv(ji,jj,jk) / e1e3v_crs(ji,jj,jk)
           ENDDO
        ENDDO
     ENDDO

     !    3.d.3   Vertical scale factors
     !
     CALL crs_dom_e3( e1t, e2t, zfse3t, p_sfc_3d_crs=e1e2w_crs, cd_type='T', p_mask=tmask, p_e3_crs=e3t_0_crs, p_e3_max_crs=e3t_max_0_crs)
     CALL crs_dom_e3( e1t, e2t, zfse3w, p_sfc_3d_crs=e1e2w_crs, cd_type='W', p_mask=tmask, p_e3_crs=e3w_0_crs, p_e3_max_crs=e3w_max_0_crs)
     CALL crs_dom_e3( e1u, e2u, zfse3u, p_sfc_2d_crs=e2u_crs  , cd_type='U', p_mask=umask, p_e3_crs=e3u_0_crs, p_e3_max_crs=e3u_max_0_crs)
     CALL crs_dom_e3( e1v, e2v, zfse3v, p_sfc_2d_crs=e1v_crs  , cd_type='V', p_mask=vmask, p_e3_crs=e3v_0_crs, p_e3_max_crs=e3v_max_0_crs)

     WHERE(e3t_max_0_crs == 0._wp) e3t_max_0_crs=r_inf
     WHERE(e3u_max_0_crs == 0._wp) e3u_max_0_crs=r_inf
     WHERE(e3v_max_0_crs == 0._wp) e3v_max_0_crs=r_inf
     WHERE(e3w_max_0_crs == 0._wp) e3w_max_0_crs=r_inf

     DO jk = 1, jpk
        DO ji = 1, jpi_crs
           DO jj = 1, jpj_crs
              IF( e3t_max_0_crs(ji,jj,jk) == 0._wp ) e3t_max_0_crs(ji,jj,jk) = e3t_1d(jk)
              IF( e3w_max_0_crs(ji,jj,jk) == 0._wp ) e3w_max_0_crs(ji,jj,jk) = e3w_1d(jk)
              IF( e3u_max_0_crs(ji,jj,jk) == 0._wp ) e3u_max_0_crs(ji,jj,jk) = e3t_1d(jk)
              IF( e3v_max_0_crs(ji,jj,jk) == 0._wp ) e3v_max_0_crs(ji,jj,jk) = e3t_1d(jk)
           ENDDO
        ENDDO
     ENDDO

#if defined key_vvl
     e3t_max_n_crs=e3t_max_0_crs
     e3u_max_n_crs=e3u_max_0_crs
     e3v_max_n_crs=e3v_max_0_crs
     e3w_max_n_crs=e3w_max_0_crs
#endif

     ht_0_crs(:,:)=0._wp
     DO jk = 1, jpk
        ht_0_crs(:,:)=ht_0_crs(:,:)+e3t_0_crs(:,:,jk)*tmask_crs(:,:,jk)
     ENDDO

#if defined key_vvl
     e3t_0_crs(:,:,:) = e3t_0_crs(:,:,:) * tmask_crs(:,:,:)
     e3u_0_crs(:,:,:) = e3u_0_crs(:,:,:) * umask_crs(:,:,:)
     e3v_0_crs(:,:,:) = e3v_0_crs(:,:,:) * vmask_crs(:,:,:)
     e3w_0_crs(:,:,:) = e3w_0_crs(:,:,:) * tmask_crs(:,:,:)
#endif

     ! Reset 0 to e3t_0 or e3w_0
     DO jk = 1, jpk
        DO ji = 1, jpi_crs
           DO jj = 1, jpj_crs
              IF( e3t_0_crs(ji,jj,jk) == 0._wp ) e3t_0_crs(ji,jj,jk) = e3t_1d(jk)
              IF( e3w_0_crs(ji,jj,jk) == 0._wp ) e3w_0_crs(ji,jj,jk) = e3w_1d(jk)
              IF( e3u_0_crs(ji,jj,jk) == 0._wp ) e3u_0_crs(ji,jj,jk) = e3t_1d(jk)
              IF( e3v_0_crs(ji,jj,jk) == 0._wp ) e3v_0_crs(ji,jj,jk) = e3t_1d(jk)
           ENDDO
        ENDDO
     ENDDO

#if defined key_vvl
     e3t_b_crs(:,:,:) = e3t_0_crs(:,:,:)
     e3u_b_crs(:,:,:) = e3u_0_crs(:,:,:)
     e3v_b_crs(:,:,:) = e3v_0_crs(:,:,:)
     e3w_b_crs(:,:,:) = e3w_0_crs(:,:,:)

     e3t_n_crs(:,:,:) = e3t_0_crs(:,:,:)
     e3u_n_crs(:,:,:) = e3u_0_crs(:,:,:)
     e3v_n_crs(:,:,:) = e3v_0_crs(:,:,:)
     e3w_n_crs(:,:,:) = e3w_0_crs(:,:,:)

     e3t_a_crs(:,:,:) = e3t_0_crs(:,:,:)
     e3u_a_crs(:,:,:) = e3u_0_crs(:,:,:)
     e3v_a_crs(:,:,:) = e3v_0_crs(:,:,:)
     e3w_a_crs(:,:,:) = e3w_0_crs(:,:,:)
#endif

     !    3.d.3   Vertical depth (meters)
     !cbr: il semblerait que p_e3=... ne soit pas utile ici !!!!!!!!!
     CALL crs_dom_ope( gdept_0, 'MAX', 'T', tmask, gdept_0_crs, p_e3=zfse3t, psgn=1.0 ) 
     CALL crs_dom_ope( gdepw_0, 'MAX', 'W', tmask, gdepw_0_crs, p_e3=zfse3w, psgn=1.0 )

     DO jk = 1, jpk
        DO ji = 1, jpi_crs
           DO jj = 1, jpj_crs
              IF( gdept_0_crs(ji,jj,jk) .LE. 0._wp ) gdept_0_crs(ji,jj,jk) = gdept_1d(jk)
              IF( gdepw_0_crs(ji,jj,jk) .LE. 0._wp ) gdepw_0_crs(ji,jj,jk) = gdepw_1d(jk)
           ENDDO
        ENDDO
     ENDDO

#if defined key_vvl
     gdept_n_crs(:,:,:) = gdept_0_crs(:,:,:)
     gdepw_n_crs(:,:,:) = gdepw_0_crs(:,:,:)
#endif

     !---------------------------------------------------------
     ! 4.  Coarse grid ocean volume and averaging weights
     !---------------------------------------------------------
     CALL crs_dom_facvol( tmask, 'T', e1t, e2t, zfse3t, ocean_volume_crs_t, facvol_t )
     !
     bt_crs(:,:,:) = ocean_volume_crs_t(:,:,:) * facvol_t(:,:,:)
     !
     r1_bt_crs(:,:,:) = 0._wp 
     WHERE( bt_crs /= 0._wp ) r1_bt_crs(:,:,:) = 1._wp / bt_crs(:,:,:)

     CALL crs_dom_facvol( tmask, 'W', e1t, e2t, zfse3w, ocean_volume_crs_w, facvol_w )

     !---------------------------------------------------------
     ! 5.  Build tmask_i_crs ( for crsdomwri and lib_fortran_crs (glob_sum)
     !---------------------------------------------------------
     tmask_i_crs(:,:) = tmask_crs(:,:,1)
     iif = jpreci
     iil = nlci_crs - jpreci + 1
     ijf = jpreci
     ijl = nlcj_crs - jprecj + 1

     tmask_i_crs( 1:iif ,    :  ) = 0._wp
     tmask_i_crs(iil:jpi_crs,    :  ) = 0._wp
     tmask_i_crs(   :   , 1:ijf ) = 0._wp
     tmask_i_crs(   :   ,ijl:jpj_crs) = 0._wp

     tpol_crs(1:jpiglo_crs,:) = 1._wp
     fpol_crs(1:jpiglo_crs,:) = 1._wp
     IF( jperio == 3 .OR. jperio == 4 ) THEN
        tpol_crs(jpiglo_crs/2+1:jpiglo_crs,:) = 0._wp
        fpol_crs(       1      :jpiglo_crs,:) = 0._wp
        IF( mjg_crs(nlej_crs) == jpiglo_crs ) THEN
           DO ji = iif+1, iil-1
              tmask_i_crs(ji,nlej_crs-1) = tmask_i_crs(ji,nlej_crs-1) &
              & * tpol_crs(mig_crs(ji),1)
           ENDDO
        ENDIF
     ENDIF
     IF( jperio == 5 .OR. jperio == 6 ) THEN
        tpol_crs(      1       :jpiglo_crs,:)=0._wp
        fpol_crs(jpiglo_crs/2+1:jpiglo_crs,:)=0._wp
     ENDIF
     !
     !---------------------------------------------------------
     ! 6.  Write out coarse meshmask  (see OPA_SRC/DOM/domwri.F90 for ideas later)
     !---------------------------------------------------------

     IF( nn_msh_crs > 0 ) THEN 
        CALL dom_grid_crs   ! Save the parent grid information  & Switch to coarse grid domain
        CALL crs_dom_wri     
        CALL dom_grid_glo   ! Return to parent grid domain
     ENDIF
    
     rhop_crs(:,:,:)=0._wp ; rhd_crs(:,:,:)=0._wp ; rb2_crs(:,:,:)=0._wp
 
     !---------------------------------------------------------
     ! 7. Finish and clean-up
     !---------------------------------------------------------
     CALL wrk_dealloc(jpi, jpj, jpk, zfse3t, zfse3u, zfse3v, zfse3w )

     IF( nn_timing == 1 )  CALL timing_stop('crs_init')

   END SUBROUTINE crs_init
   !!======================================================================
END MODULE crsini
