MODULE crswri
   !!======================================================================
   !!                     ***  MODULE  crsdwri  ***
   !!  Ocean coarsening :  coarse ocean fields
   !!=====================================================================
   !!   2012-07  (J. Simeon, C. Calone, G. Madec, C. Ethe)
   !!----------------------------------------------------------------------

   !!----------------------------------------------------------------------
   !!   crs_wri       : create the standard output files for coarse grid and prep
   !!                       other variables needed to be passed to TOP
   !!----------------------------------------------------------------------
   USE oce             ! ocean dynamics and tracers 
   USE dom_oce         ! ocean space and time domain
   USE sbc_oce         ! Surface boundary condition: ocean fields
   USE lbclnk          ! ocean lateral boundary conditions (or mpp link)
   USE in_out_manager  ! I/O manager
   USE timing          ! preformance summary
   USE wrk_nemo        ! working array
   USE crs
   USE crsdom
   USE domvvl
   USE iom
   
   IMPLICIT NONE
   PRIVATE

   INTERFACE crs_wri
      MODULE PROCEDURE crs_wri_2D, crs_wri_3D
   END INTERFACE
   PUBLIC    crs_wri

   !! * Substitutions
#  include "domzgr_substitute.h90"
#  include "vectopt_loop_substitute.h90"
   !!----------------------------------------------------------------------
   !! NEMO/OPA 3.3 , NEMO Consortium (2010)
   !! $Id $
   !! Software governed by the CeCILL licence     (NEMOGCM/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------
CONTAINS

   SUBROUTINE crs_wri_2D( p_fld, cd_op, cd_type, varname, context )
      !!---------------------------------------------------------------------
      !!                  ***  ROUTINE crs_wri  ***
      !!                   
      !! ** Purpose :   Basic output of coarsened dynamics and tracer fields 
      !!      NETCDF format is used by default 
      !!      1. Accumulate in time the dimensionally-weighted fields
      !!      2. At time of output, rescale [1] by dimension and time
      !!         to yield the spatial and temporal average.
      !!
      !! ** Method  :  
      !!----------------------------------------------------------------------
      !!  Arguments
      REAL(wp), DIMENSION(jpi,jpj),             INTENT(in)           :: p_fld ! T, U, V or W on parent grid
      CHARACTER(len=*),                         INTENT(in)           :: cd_op ! Operation SUM, MAX or MIN
      CHARACTER(len=1),                         INTENT(in)           :: cd_type ! grid type U,V 
      CHARACTER(len=*),                         INTENT(in)           :: varname
      CHARACTER(len=*),                         INTENT(in)           :: context
      CHARACTER(len=256) :: ctmp
      !!
      REAL(wp), POINTER, DIMENSION(:,:,:) :: zfse3, zmsk
      !!----------------------------------------------------------------------

      IF( nn_timing == 1 )   CALL timing_start('crs_wri_2D')

      !  Initialize arrays
      CALL wrk_alloc( jpi, jpj, jpk, zfse3, zmsk )

      ctmp = TRIM(context)//'_crs'

      CALL iom_swap( ctmp )    ! swap on the coarse grid

      SELECT CASE ( TRIM(cd_type) )
      CASE ( 'T' )
      !---------------------------------------------------------------------------------------------------
      !variables at now time : 
      !---------------------------------------------------------------------------------------------------
#if defined key_vvl
         zfse3(:,:,:) = e3t_n(:,:,:)
#else
         zfse3(:,:,:) = e3t_0(:,:,:)
#endif
         zmsk = tmask
         CALL crs_dom_ope( p_fld, TRIM(cd_op), 'T', zmsk, sshn_crs , p_e12=e1e2t, p_e3=zfse3, psgn=1.0 )
         CALL iom_put( TRIM(varname), sshn_crs )   ! 2D output 
      CASE ( 'U' )
      !---------------------------------------------------------------------------------------------------
      !variables at now time : 
      !---------------------------------------------------------------------------------------------------
#if defined key_vvl
         zfse3(:,:,:) = e3u_n(:,:,:)
#else
         zfse3(:,:,:) = e3u_0(:,:,:)
#endif
         zmsk = umask
         CALL crs_dom_ope( p_fld , TRIM(cd_op), 'U', zmsk, utau_crs, p_e12=e2u, p_e3=zfse3, p_surf_crs=e2e3u_msk(:,:,1), psgn=-1.0 )
         CALL iom_put( TRIM(varname), utau_crs )   ! 2D output 
      CASE ( 'V' )
      !---------------------------------------------------------------------------------------------------
      !variables at now time : 
      !---------------------------------------------------------------------------------------------------
#if defined key_vvl
         zfse3(:,:,:) = e3v_n(:,:,:)
#else
         zfse3(:,:,:) = e3v_0(:,:,:)
#endif
         zmsk = vmask
         CALL crs_dom_ope( p_fld , TRIM(cd_op), 'V', zmsk, vtau_crs, p_e12=e1v, p_e3=zfse3, p_surf_crs=e1e3v_msk(:,:,1), psgn=-1.0 )
         CALL iom_put( TRIM(varname), vtau_crs )   ! 2D output 
      END SELECT

      !---------------------------------------------------------------------------------------------------
      !  free memory
      CALL wrk_dealloc( jpi, jpj, jpk, zfse3, zmsk )
      !
      ctmp = TRIM(context)
      CALL iom_swap( ctmp )     ! return back on high-resolution grid
      !
      IF( nn_timing == 1 )   CALL timing_stop('crs_wri_2D')
      !
   END SUBROUTINE crs_wri_2D

   SUBROUTINE crs_wri_3D( p_fld, cd_op, cd_type, varname, context )
      !!---------------------------------------------------------------------
      !!                  ***  ROUTINE crs_wri  ***
      !!                   
      !! ** Purpose :   Basic output of coarsened dynamics and tracer fields 
      !!      NETCDF format is used by default 
      !!      1. Accumulate in time the dimensionally-weighted fields
      !!      2. At time of output, rescale [1] by dimension and time
      !!         to yield the spatial and temporal average.
      !!
      !! ** Method  :  
      !!----------------------------------------------------------------------
      !!  Arguments
      REAL(wp), DIMENSION(jpi,jpj,jpk),         INTENT(in)           :: p_fld ! T, U, V or W on parent grid
      CHARACTER(len=*),                         INTENT(in)           :: cd_op ! Operation SUM, MAX or MIN
      CHARACTER(len=1),                         INTENT(in)           :: cd_type ! grid type U,V 
      CHARACTER(len=*),                         INTENT(in)           :: varname
      CHARACTER(len=*),                         INTENT(in)           :: context
      CHARACTER(len=256) :: ctmp
      !!
      REAL(wp), POINTER, DIMENSION(:,:,:) :: zfse3, zmsk
      !!----------------------------------------------------------------------

      IF( nn_timing == 1 )   CALL timing_start('crs_wri_3D')

      !  Initialize arrays
      CALL wrk_alloc( jpi, jpj, jpk, zfse3, zmsk )

      ctmp = TRIM(context)//'_crs'

      CALL iom_swap( ctmp )    ! swap on the coarse grid

      SELECT CASE ( TRIM(cd_type) )
      CASE ( 'T' )
      !---------------------------------------------------------------------------------------------------
      !variables at now time : 
      !---------------------------------------------------------------------------------------------------
#if defined key_vvl
         zfse3(:,:,:) = e3t_n(:,:,:)
#else
         zfse3(:,:,:) = e3t_0(:,:,:)
#endif
         zmsk = tmask
         CALL crs_dom_ope( p_fld, TRIM(cd_op), 'T', zmsk, tsn_crs(:,:,:,1), p_e12=e1e2t, p_e3=zfse3, psgn=1.0 )
         CALL iom_put( TRIM(varname), tsn_crs(:,:,:,1) )   ! 3D output 
      CASE ( 'U' )
      !---------------------------------------------------------------------------------------------------
      !variables at now time : 
      !---------------------------------------------------------------------------------------------------
#if defined key_vvl
         zfse3(:,:,:) = e3u_n(:,:,:)
#else
         zfse3(:,:,:) = e3u_0(:,:,:)
#endif
         zmsk = umask
         CALL crs_dom_ope( p_fld , TRIM(cd_op), 'U', zmsk, un_crs, p_e12=e2u, p_e3=zfse3, p_surf_crs=e2e3u_msk, psgn=-1.0 )
         CALL iom_put( TRIM(varname), un_crs )   ! 3D output 
      CASE ( 'V' )
      !---------------------------------------------------------------------------------------------------
      !variables at now time : 
      !---------------------------------------------------------------------------------------------------
#if defined key_vvl
         zfse3(:,:,:) = e3v_n(:,:,:)
#else
         zfse3(:,:,:) = e3v_0(:,:,:)
#endif
         zmsk = vmask
         CALL crs_dom_ope( p_fld , TRIM(cd_op), 'V', zmsk, vn_crs, p_e12=e1v, p_e3=zfse3, p_surf_crs=e1e3v_msk, psgn=-1.0 )
         CALL iom_put( TRIM(varname), vn_crs )   ! 3D output 
      END SELECT

      !---------------------------------------------------------------------------------------------------
      !  free memory
      CALL wrk_dealloc( jpi, jpj, jpk, zfse3, zmsk )
      !
      ctmp = TRIM(context)
      CALL iom_swap( ctmp )     ! return back on high-resolution grid
      !
      IF( nn_timing == 1 )   CALL timing_stop('crs_wri_3D')
      !
   END SUBROUTINE crs_wri_3D
   !!======================================================================
END MODULE crswri
