MODULE diaharm_cmc
   !!======================================================================
   !!                       ***  MODULE  diaharm_cmc  ***
   !! Ocean physics:  on-line T-tide analyse based on slide-window and least-square function-fit
   !!======================================================================
   !! History : 1.0  !  2017-06  Yimin Liu, Gregory Smith, Wei Yu, Frederic Dupont,Francois Roy
   !!----------------------------------------------------------------------
#if defined key_cmc_diaharm  && defined key_tide
   !!----------------------------------------------------------------------
   !!   'key_cmc_diaharm'  :    CMC on-line T-tide analyse
   !!----------------------------------------------------------------------
   !!   dia_harm_cmc          : the main subroutine
   !!   dia_harm_cmc_rot_sld  : rotation and sliding window.
   !!   dia_harm_cmc_init     : initialization
   !!   dia_harm_cmc_rst      : restart
   !!   inverse_matrix        : inverse matrix based on LaPack
   !!----------------------------------------------------------------------
   USE par_kind,       only : wp
   USE par_oce,        only : jpi,jpj,jpk
   USE dom_oce,        only : rdt, tmask
   USE oce,            only : sshn
   USE lbclnk,         only : lbc_lnk
   USE in_out_manager, only : nit000, nitend,  lwp, lrst_oce, numout, numror, nitrst, numrow, numnam_cfg, numnam_ref
   USE iom_def,        only : jpdom_autoglo, jpdom_unknown
   USE iom,            only : iom_get, iom_rstput, iom_varid, iom_put, iom_use

   IMPLICIT NONE
   PRIVATE

   LOGICAL, PUBLIC                        ::  lk_diaharm_cmc =  &
                                                       .TRUE.  ! default logical
   REAL(wp)                               ::  sldwnd_tdhm      ! slide window length in day.
   INTEGER                                ::  nstp_tdhm = 4    ! tide harmon analyses in model step.
   LOGICAL                                ::  ln_str_tdhm =     &
                                                       .FALSE. ! start harmonic analyses
   REAL(wp)                               ::  relay_crt        ! Rayleigh criterion to select freqency
   REAL(wp)                               ::  max_period_hour  ! max period in hour to select
   LOGICAL                                ::  ln_exclude_25hr = &
                                                       .FALSE. ! excluding periods close to 25 hour

   INTEGER                                ::  nb_frq           ! the number of frequencies used for harmonic analyses
   INTEGER                                ::  nbfqa_tdhm       ! =nb_frq*2+1;
   REAL(wp),ALLOCATABLE, DIMENSION(:    ) ::  omega_harmo      ! omega used  for harmonic analyses

   REAL(wp),ALLOCATABLE, DIMENSION(:,:,:) ::  rot_tdhm         ! SO(3) operator;
   REAL(wp),ALLOCATABLE, DIMENSION(:,:,:) ::  rot_tdhm_0       ! SO(3) operator for gap filling;
   REAL(wp),ALLOCATABLE, DIMENSION(:,:  ) ::  eem_tdhm         ! Coefficient Matrix  3X3D
   REAL(wp),ALLOCATABLE, DIMENSION(:,:,:) ::  eyv_tdhm         ! ssh mapping onto Fouries series.
   REAL(wp)                               ::  sldwgt_tdhm      ! slide window weight
   REAL(wp)                               ::  rb_accum

   REAL(wp),ALLOCATABLE, DIMENSION(:,:  ) ::  wk_eem_tdhm      ! work sapce
   REAL(wp),ALLOCATABLE, DIMENSION(:    ) ::  wk_eyv_tdhm

   LOGICAL                                ::  ln_diag_matrix_tdhm =   &
                                                       .FALSE. ! for printing out the initial value of the matrix
   REAL(wp),PARAMETER                     ::  rp_fill_value =         &
                                                  -999999.0_wp ! for filling empty arrays

   REAL(wp),ALLOCATABLE, DIMENSION(  :,:) ::  tdhn             ! tide harmonic analysis field combining all constituent (ssh only so far)

! additional options, only recommended for expert users
   REAL(wp)                               ::  time_gap_sec =          &
                                                    86400.0_wp ! time-gap between the fisrt step of present run and last step of previous run.
   LOGICAL                                ::  ln_time_gap_permit =    &
                                                       .FALSE. ! time-gap permit;   'true' not recommended.
   LOGICAL                                ::  ln_first_call_rot_sld = &
                                                       .TRUE.  ! apply time_gap_sec in the first time of calling rot_sld;

   PUBLIC :: dia_harm_cmc,  tdhn, ln_str_tdhm

   !! * Substitutions
   !!----------------------------------------------------------------------
   !! NEMO/OPA 3.3 , NEMO Consortium (2010)
   !! $Id: module_example 4147 2013-11-04 11:51:55Z cetlod $ 
   !! Software governed by the CeCILL licence     (NEMOGCM/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------
CONTAINS

   SUBROUTINE dia_harm_cmc( kt )
      !!----------------------------------------------------------------------
      !!                    ***  ROUTINE dia_harm_cmc  ***
      !!
      !! ** Purpose :   on-line T-tide analyse
      !!
      !! ** Method  :   based on slide-window and least-square function-fit
      !!
      !!               
      !!----------------------------------------------------------------------
      !!
      INTEGER , INTENT(in)     ::   kt
      !!
      INTEGER  ::   jn, jm, ierr
      !!--------------------------------------------------------------------

      IF ( kt == nit000 ) THEN
         CALL dia_harm_cmc_init ( kt )
         IF ( ln_diag_matrix_tdhm .AND. lwp ) then
            WRITE(numout,*) ' matrix eem_tdhm Load <<<<<<< kt=', kt
            WRITE(numout,'(20f10.5)') eem_tdhm
            WRITE(numout,*) ' >>>>>>>> END eem_tdhm'
            CALL flush(numout)
         ENDIF
      ENDIF 
    
      !!!!
      IF ( mod ( kt, nstp_tdhm) == 0 ) THEN

         IF ( ln_first_call_rot_sld  .AND.  ln_time_gap_permit ) THEN
            CALL dia_harm_cmc_rot_sld (rot_tdhm_0);  DEALLOCATE ( rot_tdhm_0 );  ln_first_call_rot_sld=.false.;
         ELSE
            CALL dia_harm_cmc_rot_sld (rot_tdhm  )
         ENDIF

         IF ( ln_str_tdhm ) THEN

            !get inverse wk_eem_tdhm
            CALL  inverse_matrix(eem_tdhm,  wk_eem_tdhm, kt );   ! NB. keep eem_tdhm for next step

            !get tdhn
            tdhn(:,:) = 0.0

            DO jm=1,nbfqa_tdhm        ! all components in solution space.
               DO jn=1,nb_frq         ! NB.  excluding the mean.
                  tdhn(:,:)= tdhn(:,:) + wk_eem_tdhm(jn,jm) * eyv_tdhm(:,:,jm)
               ENDDO
            ENDDO

            tdhn(:,:)= tdhn(:,:) * tmask (:,:,1)

         ENDIF 
      ENDIF  ! mod ( kt, nstp_tdhm) == 0

      ! Write outputs
      ! =============
      IF (iom_use('ssh_tide')) CALL iom_put( 'ssh_tide' , tdhn )

      !!!!
      IF( lrst_oce )  CALL dia_harm_cmc_rst( kt, 'WRITE' )
      !
      IF ( kt == nitend) DEALLOCATE ( rot_tdhm, eem_tdhm, wk_eem_tdhm, eyv_tdhm, wk_eyv_tdhm, omega_harmo )
      !
   END SUBROUTINE dia_harm_cmc

   SUBROUTINE dia_harm_cmc_rot_sld ( rot_zz )
      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE dia_harm_cmc_rot_sld  ***
      !!                   
      !! ** Purpose :   rotate vector and matrix, update with slide-weight
      !!
      !!         | r11   r12   0   |
      !! rot =   | r21   r22   0   |
      !!         |   0    0    1   |
      !!
      !!         | EE11  EE12 EE13 |
      !! EEm =   | EE21  EE22 EE23 |
      !!         | Ee31  EE32 EE3  |
      !!
      !! EYv =   | EY1   EY2   EY3 |'
      !! 
      !! new EEm  = (1-alf) * rot * EEm *rot' +  alfa*cos_components
      !! new EYv  = (1-alf) * rot * EYv       +  alfa*cos_components
      !! 
      !!----------------------------------------------------------------------
      !! Arguments
      REAL(wp), DIMENSION(:,:,:), intent(in) ::  rot_zz
      !! locals
      INTEGER  ::  ierr,  jn1, jn2, jn3,   jm1,jm2,jm3,  ji,jj
      !!--------------------------------------------------------------------

       jn3=nbfqa_tdhm;
       jm3=nbfqa_tdhm;

     ! update E * SSH
       do jj=1,jpj
       do ji=1,jpi
         ! rot vector
         do jn1=1,nbfqa_tdhm;
          wk_eyv_tdhm(jn1) = eyv_tdhm(ji,jj,jn1);
         enddo 
         do jn1=1,nb_frq; jn2=jn1+nb_frq ;
           eyv_tdhm(ji,jj,jn1) = rot_zz(1,1,jn1) * wk_eyv_tdhm(jn1) + rot_zz(1,2,jn1) * wk_eyv_tdhm(jn2)
           eyv_tdhm(ji,jj,jn2) = rot_zz(2,1,jn1) * wk_eyv_tdhm(jn1) + rot_zz(2,2,jn1) * wk_eyv_tdhm(jn2)
         enddo

         ! using slide weight
         eyv_tdhm(ji,jj,:)     = eyv_tdhm(ji,jj,:) * (1.0-sldwgt_tdhm)
         do jn1=1,nb_frq;
           eyv_tdhm(ji,jj,jn1) = eyv_tdhm(ji,jj,jn1) + sshn(ji,jj) * sldwgt_tdhm
         enddo
           eyv_tdhm(ji,jj,jn3) = eyv_tdhm(ji,jj,jn3) + sshn(ji,jj) * sldwgt_tdhm
       enddo
       enddo

     !..............................
     ! update E * E'
       ! left rot
       wk_eem_tdhm(:,:) = eem_tdhm(:,:)
       do jn1=1,nb_frq;   jn2=jn1+nb_frq;
          eem_tdhm(jn1,:) = rot_zz(1,1,jn1) * wk_eem_tdhm(jn1,:) + rot_zz(1,2,jn1) * wk_eem_tdhm(jn2,:)
          eem_tdhm(jn2,:) = rot_zz(2,1,jn1) * wk_eem_tdhm(jn1,:) + rot_zz(2,2,jn1) * wk_eem_tdhm(jn2,:)
       enddo

       ! right rot,   NB. Transpose of rot matrix:  rot12 <==> rot21.
       wk_eem_tdhm(:,:) = eem_tdhm(:,:)
       do jm1=1,nb_frq;   jm2=jm1+nb_frq;
          eem_tdhm(:,jm1) = wk_eem_tdhm(:,jm1) * rot_zz(1,1,jm1) + wk_eem_tdhm(:,jm2) * rot_zz(1,2,jm1)
          eem_tdhm(:,jm2) = wk_eem_tdhm(:,jm1) * rot_zz(2,1,jm1) + wk_eem_tdhm(:,jm2) * rot_zz(2,2,jm1)
       enddo

       ! using slide weight
        eem_tdhm(:,:)        = eem_tdhm(:,:) * (1.0-sldwgt_tdhm)

        do jn1=1,nb_frq;   do jm1=1,nb_frq
           eem_tdhm(jn1,jm1) = eem_tdhm(jn1,jm1) + sldwgt_tdhm
        enddo;enddo

        do jn1=1,nb_frq
           eem_tdhm(jn1,jm3) = eem_tdhm(jn1,jm3) + sldwgt_tdhm
        enddo

        do jm1=1,nb_frq
           eem_tdhm(jn3,jm1) = eem_tdhm(jn3,jm1) + sldwgt_tdhm
        enddo

        eem_tdhm(jn3,jm3)    = eem_tdhm(jn3,jm3) + sldwgt_tdhm

        rb_accum  = rb_accum + 1.0_wp
   END SUBROUTINE dia_harm_cmc_rot_sld

   SUBROUTINE dia_harm_cmc_init (kt)
      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE dia_harm_cmc_init  ***
      !!                   
      !! ** Purpose :   initialization of dia_harm_cmc
      !!
      !!----------------------------------------------------------------------

      USE tideini,        only : omega_tide, nb_harmo;

      INTEGER,  INTENT(in) ::  kt

      REAL(wp)             ::  zrdt, zrdt_harmo,  zcos,zsin
      INTEGER              ::  ierr, jn
      Logical              ::  ln_use_input_freq = .FALSE.
      character(len=256)   ::  fname_const_freq
      
      NAMELIST/namharm_cmc/ln_str_tdhm, nstp_tdhm, sldwnd_tdhm, & 
                           ln_use_input_freq, fname_const_freq, relay_crt, max_period_hour, &
                           ln_exclude_25hr, ln_time_gap_permit, time_gap_sec
      !!--------------------------------------------------------------------
       
      !... load namelist.
      REWIND( numnam_ref )
      READ  ( numnam_ref, namharm_cmc, IOSTAT = ierr)
      IF ( ierr /= 0 ) THEN;
        write(0,*) ' diaharm_cmc:: ERROR read namharm_cmc in namelist ref'  ;  call flush(0); stop;
      ENDIF

      REWIND( numnam_cfg )
      READ  ( numnam_cfg, namharm_cmc, IOSTAT = ierr)
      IF ( ierr /= 0 ) THEN;
        write(0,*) ' diaharm_cmc:: ERROR read namharm_cmc in namelist cfg'  ;  call flush(0); stop;
      ENDIF

      IF( lwp )THEN
         WRITE(numout,*)
         WRITE(numout,*) 'diaharm_cmc : tide harmonic analyses'
         WRITE(numout,*)'~~~~~~~~~~~                              '
         WRITE(numout,*)' start analyses in this cycle    = ', ln_str_tdhm
         WRITE(numout,*)' the step number of analyses     = ', nstp_tdhm
         WRITE(numout,*)' slide window length in day      = ', sldwnd_tdhm
         WRITE(numout,*)' use input constituents frequency= ', ln_use_input_freq
         WRITE(numout,*)' input const_freq file name      = ', trim(fname_const_freq)
         WRITE(numout,*)' Rayleigh criterion              = ', relay_crt
         WRITE(numout,*)' max period in hour to select    = ', max_period_hour
         WRITE(numout,*)' Excluding periods too close to 25 hours'
         WRITE(numout,*)'                                 = ', ln_exclude_25hr
         WRITE(numout,*)' gap filling on/off              = ', ln_time_gap_permit
         IF (ln_time_gap_permit) THEN
            WRITE(numout,*)' time gap to fill (s)            = ', time_gap_sec
            WRITE(numout,*)' .......... or in (hours)        = ', time_gap_sec/3600.0_wp
            WRITE(numout,*)' .......... or in (days)         = ', time_gap_sec/86400.0_wp
         ENDIF
         CALL FLUSH(numout)
      ENDIF

      ! load frequncy
      IF ( ln_use_input_freq ) then
        call load_harmo_freq ( fname_const_freq )
      ELSE
        nb_frq = nb_harmo
        allocate ( omega_harmo(nb_frq) , stat=ierr )
        omega_harmo(:) = omega_tide(:) 
      ENDIF

      !... open space
      nbfqa_tdhm=nb_frq*2+1;

      ALLOCATE ( rot_tdhm         (2,2,            nb_frq), &
                 eem_tdhm         (nbfqa_tdhm, nbfqa_tdhm), &
                 wk_eem_tdhm      (nbfqa_tdhm, nbfqa_tdhm), &
                 eyv_tdhm         (jpi,jpj,    nbfqa_tdhm), &
                 wk_eyv_tdhm      (            nbfqa_tdhm), &
                 tdhn             (jpi,jpj               ), &
                                             stat = ierr );

      !Roy, to avoid nans in XIOS output when harmonic analysis is not started 
      rot_tdhm(:,:,:)=0._wp
      eem_tdhm(:,:)=0._wp
      wk_eem_tdhm(:,:)=0._wp
      eyv_tdhm(:,:,:)=0._wp
      wk_eyv_tdhm(:)=0._wp
      tdhn(:,:)=0._wp

      !... spacify some parameters
      zrdt_harmo  = rdt * real( nstp_tdhm, wp)
      IF ( zrdt_harmo - 3600.0_wp > 10.0_wp ) then
         WRITE(0,*) ' W A R N NING :: time-interval for harmonic analyses larger than 1 hour'; call flush(0);
         STOP
      ENDIF

      sldwgt_tdhm = zrdt_harmo / (sldwnd_tdhm*86400.0_wp);

      !... initial E*E' and E*Y
      CALL dia_harm_cmc_rst (kt, 'READ')


      ! check spin up period
      IF ( ln_str_tdhm .AND. rb_accum < 86400.0_wp*7.0_wp / zrdt_harmo ) THEN
        ln_str_tdhm = .FALSE.
        IF( lwp ) THEN
           WRITE(numout,*)' Warning:: diaharm_cmc needs 7 days to spin up, rb_accum=', rb_accum
           WRITE(numout,*)' ln_str_tdhm is reset to false :'
           WRITE(numout,*)' start analyses in this cycle    = ', ln_str_tdhm
           CALL FLUSH(numout)
        ENDIF
      ENDIF

      !... defined SO(3) operator.
      DO jn=1, nb_frq
        zcos= cos( omega_harmo(jn) * zrdt_harmo );
        zsin= sin( omega_harmo(jn) * zrdt_harmo );
        rot_tdhm(1,1,jn) = zcos;  rot_tdhm(1,2,jn) =-zsin;
        rot_tdhm(2,1,jn) = zsin;  rot_tdhm(2,2,jn) = zcos;
      ENDDO

      ! in case there is time-gap
      IF ( ln_time_gap_permit ) THEN
        ALLOCATE ( rot_tdhm_0(2,2, nb_frq), stat = ierr );
        DO jn=1, nb_frq
          zcos= cos( omega_harmo(jn) * (zrdt_harmo+time_gap_sec) );
          zsin= sin( omega_harmo(jn) * (zrdt_harmo+time_gap_sec) );
          rot_tdhm_0(1,1,jn) = zcos;  rot_tdhm_0(1,2,jn) =-zsin;
          rot_tdhm_0(2,1,jn) = zsin;  rot_tdhm_0(2,2,jn) = zcos;
        ENDDO
      ENDIF

   END SUBROUTINE dia_harm_cmc_init

   SUBROUTINE dia_harm_cmc_rst ( kt, cdrw )
      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE dia_harm_cmc_rst  ***
      !!                   
      !! ** Purpose :   input or output restart data, and coincidence check with previous cycle.
      !!
      !!----------------------------------------------------------------------
      USE dom_oce,       only : nldi,nldj,  nlei,nlej

      CHARACTER(len=*), INTENT(in) ::   cdrw   ! "READ"/"WRITE" flag
      INTEGER,          INTENT(in) ::   kt

      REAL(wp)                                :: wkr
      REAL(wp)                                :: zsldwnd_tdhm, znb_frq
      REAL(wp), ALLOCATABLE, DIMENSION(:,:)   :: wkio_2d
      REAL(wp), ALLOCATABLE, DIMENSION(:)     :: wkio_1d, zomega_harmo, wkio_1dz
      INTEGER                                 :: ierr, jn, jm, jk
      INTEGER                                 :: jn1, jn2
      CHARACTER(len=10)                       :: cwk
      INTEGER                                 :: ji, jj
      INTEGER                                 :: jnbfqa_tdhm_sqr, jmax_dim_1d, np_1d, jp

      !!----------------------------------------------------------------------
      
      jnbfqa_tdhm_sqr = nbfqa_tdhm*nbfqa_tdhm
      jmax_dim_1d     = jnbfqa_tdhm_sqr + nb_frq  + 1
      np_1d           = ( jmax_dim_1d - 1 ) / jpk + 1

      ALLOCATE ( wkio_2d(jpk,np_1d), wkio_1d(jmax_dim_1d), wkio_1dz(jpk) )
      wkio_2d(:,:) = rp_fill_value; wkio_1d(:) = rp_fill_value

      !............
      IF ( trim(cdrw) == 'READ' ) THEN

         ! init in any case
         tdhn    (:,:)   = 0.0_wp
         eem_tdhm(:,:)   = 0.0_wp
         eem_tdhm(nbfqa_tdhm, nbfqa_tdhm) = exp(-1.0_wp);  ! increase stability for the first spin-up cycles.
         rb_accum        = 0.0_wp
         eyv_tdhm(:,:,:) = 0.0_wp

         if ( .NOT. ln_str_tdhm ) return   ! ROY; and set ldstop=.TRUE. in following 'wdtdh, 'ndtdh', 'tdhx01'

         ! load parameters
         IF( iom_varid( numror, 'wdtdh', ldstop = .TRUE. ) > 0 ) THEN
            CALL iom_get( numror,'wdtdh', zsldwnd_tdhm )
            IF ( ABS(sldwnd_tdhm - zsldwnd_tdhm) > 0.001_wp )  then
               WRITE(0,*) ' diaharm_cmc:: parameter changed sldwnd_tdhm', zsldwnd_tdhm ; call flush(0); stop;
            ENDIF
         ENDIF
         IF( iom_varid( numror, 'ndtdh', ldstop = .TRUE. ) > 0 ) THEN
            CALL iom_get( numror,'ndtdh', znb_frq )
            IF ( ABS(REAL(nb_frq,wp) - znb_frq ) > 0.001_wp )  then
               WRITE(0,*) ' diaharm_cmc:: parameter changed nb_frq', znb_frq; call flush(0); stop;
            ENDIF
         ENDIF
            ! check with previous cycle

         ! load and extract ey
         IF( iom_varid( numror, 'tdhx01', ldstop = .TRUE. ) > 0 ) THEN
            DO jn1=1,nb_frq;   jn2=jn1+nb_frq
              WRITE(cwk,'(a,I2.2)') 'tdhx', jn1
              CALL iom_get( numror, jpdom_autoglo, TRIM(cwk), eyv_tdhm(:,:,jn1) )
              WRITE(cwk,'(a,I2.2)') 'tdhy', jn1
              CALL iom_get( numror, jpdom_autoglo, TRIM(cwk), eyv_tdhm(:,:,jn2) )
            ENDDO 
            ! read the mean ey as well
            cwk = 'tdhm'
            jn2 = nbfqa_tdhm
            CALL iom_get(   numror, jpdom_autoglo, TRIM(cwk), eyv_tdhm(:,:,jn2) )
         ENDIF

         ! load tdhn
         IF( iom_varid( numror, 'tdhn',ldstop = .TRUE. ) > 0 ) THEN
            CALL iom_get( numror, jpdom_autoglo,'tdhn', tdhn )
         ENDIF
         
         ! load ee
         IF( iom_varid( numror, 'tdhee001',ldstop = .TRUE. ) > 0 ) THEN
            ! loop over jp to reconstruct matrix and auxiliaries
            DO jp= 1, np_1d
               WRITE(cwk,'(a,I3.3)') 'tdhee', jp
               CALL iom_get( numror, jpdom_unknown, TRIM(cwk), wkio_1dz )
               wkio_2d(:,jp) = wkio_1dz(:)
            ENDDO

            ! get the wkio_1d
            jn=0;
            loop_jkr: DO jp= 1, np_1d
                DO jk=1, jpk
                   IF ( abs(wkio_2d(jk,jp) - rp_fill_value) > 1.0_wp ) THEN
                     jn= jn + 1
                     wkio_1d(jn)=wkio_2d(jk,jp)
                     IF ( jn >= jmax_dim_1d )  EXIT loop_jkr
                   ENDIF
                 ENDDO
            ENDDO loop_jkr
            IF ( jn < jmax_dim_1d ) THEN
               WRITE(0,*) 'diaharm_cmc:  load rst 1d wrong, jn < jmax_dim_1d', jn, jmax_dim_1d
               CALL flush(0)
               STOP
            ENDIF

            eem_tdhm(:,:) = RESHAPE( wkio_1d(1:jnbfqa_tdhm_sqr), (/nbfqa_tdhm, nbfqa_tdhm/) ); ! matrix loaded

            ! check omega_harmo with previous cycle
            ALLOCATE ( zomega_harmo(nb_frq) );
            zomega_harmo(:) = wkio_1d( 1+jnbfqa_tdhm_sqr : nb_frq+jnbfqa_tdhm_sqr )
            wkr =  SUM( ABS(zomega_harmo(:)-omega_harmo(:)) ) * 3600.0_wp
            IF ( wkr > 0.05 ) THEN
               WRITE(0,*) ' diaharm_cmc:: omega_harmo or its sequency is changed '; call flush(0)
               WRITE(0,*) ' loaded   zomega_harmo='
               WRITE(0,'(20f10.5)') zomega_harmo
               WRITE(0,*) ' specified omega_harmo='
               WRITE(0,'(20f10.5)')  omega_harmo
               WRITE(0,*) ' total diff = ', wkr
               CALL flush(0)
               STOP 
            ENDIF
            DEALLOCATE (zomega_harmo)
            rb_accum = wkio_1d(jmax_dim_1d)
         ENDIF


      !............
      ELSEIF( TRIM(cdrw) == 'WRITE' ) THEN

         IF(lwp) WRITE(numout,*) '---- dia_harm_cmc_rst ---- at kt=',kt

         ! output  constant
         CALL iom_rstput( kt, nitrst, numrow, 'wdtdh', sldwnd_tdhm     )
         CALL iom_rstput( kt, nitrst, numrow, 'ndtdh', REAL(nb_frq,wp) )

         ! output  ey
         DO jn1=1,nb_frq;   jn2=jn1+nb_frq
            WRITE(cwk,'(a,I2.2)') 'tdhx', jn1
            CALL iom_rstput( kt, nitrst, numrow, trim(cwk), eyv_tdhm(:,:,jn1) )
            WRITE(cwk,'(a,I2.2)') 'tdhy', jn1
            CALL iom_rstput( kt, nitrst, numrow, trim(cwk), eyv_tdhm(:,:,jn2) )
         ENDDO 
         ! write the mean ey as well
         cwk = 'tdhm'
         jn2 = nbfqa_tdhm
         CALL iom_rstput(    kt, nitrst, numrow, trim(cwk), eyv_tdhm(:,:,jn2) )

         ! tdhn ! FD only useful if harmonic analysis frequency not multiple of restart
         CALL iom_rstput(    kt, nitrst, numrow, 'tdhn', tdhn )

         ! assemble matrix and auxiliaries
         wkio_1d(1:jnbfqa_tdhm_sqr                         ) = RESHAPE(eem_tdhm(:,:),  (/ jnbfqa_tdhm_sqr /));
         wkio_1d(1+jnbfqa_tdhm_sqr : nb_frq+jnbfqa_tdhm_sqr) = omega_harmo(:)
         wkio_1d(                               jmax_dim_1d) = rb_accum
         jn=0;
         loop_jkw: DO jp= 1, np_1d
             DO jk=1, jpk
                jn= jn + 1
                wkio_2d(jk,jp) = wkio_1d(jn);
                IF ( jn >= jmax_dim_1d )  EXIT loop_jkw
             ENDDO
         ENDDO loop_jkw

         ! output the matrix
         DO jp= 1, np_1d
            wkio_1dz(:) = wkio_2d(:,jp)
            WRITE(cwk,'(a,I3.3)') 'tdhee', jp
            CALL iom_rstput( kt, nitrst, numrow, TRIM(cwk), wkio_1dz )
         ENDDO
      ENDIF

      DEALLOCATE ( wkio_2d, wkio_1d, wkio_1dz )

   END SUBROUTINE dia_harm_cmc_rst

   SUBROUTINE inverse_matrix(A_in, A, kstp)
      USE lib_mpp,        only : get_unit
      REAL(wp), DIMENSION(:,:), INTENT(IN ) :: A_in
      REAL(wp), DIMENSION(:,:), INTENT(OUT) :: A
      INTEGER , OPTIONAL,       INTENT(IN ) :: kstp

      INTEGER,  PARAMETER                      :: nblock_wk=128  ! block parameter for work array
      REAL(wp), DIMENSION(SIZE(A,1)*nblock_wk) :: work           ! work array for LAPACK
      INTEGER,  DIMENSION(SIZE(A,1)   )        :: ipiv           ! pivot indices
      INTEGER                                  :: n, nwork, info

      ! work space for accurance check of inverse matrix
      INTEGER, parameter :: ip_stp_chk_inv_eem_tdhm=50  ! step to check, 0 not check
      REAL(wp) :: wkr, wk_dr, wk_or, wkl, wk_dl, wk_ol  ! left,right multi diagonal and off-diagonal
      INTEGER  :: ji,jj,jk, iwk_unit
      character(len=256) :: cl_filename_cmc_diag

      ! External procedures defined in LAPACK
      EXTERNAL DGETRF
      EXTERNAL DGETRI
      !!----------------------------------------------------------------------
      A=A_in; n = size(A,1); nwork=n*nblock_wk

      ! DGETRF computes an LU factorization of a general M-by-N matrix A
      !        using partial pivoting with row interchanges.
      CALL DGETRF(n, n, A, n, ipiv, info)
      IF ( info /=0 ) THEN
         WRITE(0,*) 'diaharm_cmc: A Matrix is numerically singular!'; call flush (0); stop
      ENDIF

      ! DGETRI computes the inverse of a matrix using the LU factorization,
      !        which was computed by DGETRF.
      CALL DGETRI(n, A, n, ipiv, work, nwork, info)
      IF ( info /=0 ) THEN
         WRITE(0,*) 'diaharm_cmc: A Matrix inversion failed!'; call flush (0); stop
      ELSE
         IF ( nwork <int(work(1))+1 ) THEN 
            WRITE(0,*) 'diaharm_cmc: warning:: increase nwork for optimum performance', int(work(1))+1
         ENDIF
      ENDIF

      !###############
      !###  accurance check inverse matrix
      IF ( ip_stp_chk_inv_eem_tdhm > 0 ) THEN
        IF ( .NOT. present (kstp) ) THEN
          WRITE(0,*)"diaharm_cmc:  accurance check inverse matrix need input variable kstp"; CALL FLUSH(0); STOP
        ENDIF

        IF ( lwp .and. (mod(kstp-nit000+1, ip_stp_chk_inv_eem_tdhm) == 0 .or. kstp == nitend) ) then
          wk_dr=0.0; wk_or=0.0; wk_dl=0.0; wk_ol=0.0;
          DO jj=1,n; DO ji=1,n  ! for each element
             wkr=0.0;  wkl=0.0
             DO jk=1,n
               wkl= wkl + A   (ji,jk)*A_in(jk,jj)    ! left  inverse
               wkr= wkr + A_in(ji,jk)*A   (jk,jj)    ! right inverse
             ENDDO
             IF ( ji == jj ) THEN;  wk_dr= max(wk_dr, abs(wkr-1.0)); wk_dl= max(wk_dl, abs(wkl-1.0));
             ELSE;                  wk_or= max(wk_or, abs(wkr    )); wk_ol= max(wk_ol, abs(wkl    ));
             ENDIF
          ENDDO; ENDDO  ! for each element

          WRITE(numout,*) ' Tide Harmonic Anal: Accurance of inverse matrix eem_tdhm, kstp=', kstp
          WRITE(numout,*) '     optimum performance minimum nwork=', int(work(1))+1
          WRITE(numout,*) '     left  diagonal and off-diag element accurance=', wk_dl, wk_ol
          WRITE(numout,*) '     right diagonal and off-diag element accurance=', wk_dr, wk_or
          CALL FLUSH(numout)

         ENDIF  ! mod=0 and lwp;
      ENDIF     ! ip_stp_chk_inv_eem_tdhm >0
      !###
      !###############

   END SUBROUTINE inverse_matrix

   SUBROUTINE load_harmo_freq (cd_fename)
      USE lib_mpp,        only : get_unit
      USE phycst,         only : rpi
      CHARACTER(LEN=*), INTENT (IN)       :: cd_fename 

      INTEGER                             :: iwk_unit, ierr,  jn,jn_all,jn_ok
      CHARACTER(LEN=256)                  :: cwk
      REAL(wp)                            :: wk_freq, wk_diff
      REAL(wp), DIMENSION(:), ALLOCATABLE :: wk_okfreq
      LOGICAL                             :: ll_exist
      INTEGER, PARAMETER                  :: jp_tot_freq=146

      !!----------------------------------------------------------------------
      !! Data References: R. Pawlowicz, B. Beardsley, and S. Lentz, 
      !!                  "Classical tidal harmonic analysis including error estimates
      !!                  in MATLAB using T_TIDE", Computers and Geosciences 28 (2002), 929-937
      !!----------------------------------------------------------------------
      iwk_unit = get_unit();
      INQUIRE(FILE=trim(cd_fename), EXIST=ll_exist)
      IF ( .NOT. ll_exist ) THEN
         write(0,*)'diaharm_cmc:: file not exist ' // trim(cd_fename); call flush(0); stop
      ENDIF
      OPEN( iwk_unit, file=trim(cd_fename), form='FORMATTED', action='read', iostat=ierr)
      IF ( ierr /= 0 )THEN 
         WRITE(0,*)'diaharm_cmc:: open file wrong ' // trim(cd_fename); call flush(0); stop
      ENDIF

      ! read header
      jn_ok=0;
      READ(iwk_unit,*, iostat=ierr) cwk, jn_all;    jn_ok=jn_ok+ierr
      READ(iwk_unit,*, iostat=ierr)            ;    jn_ok=jn_ok+ierr
      READ(iwk_unit,*, iostat=ierr)            ;    jn_ok=jn_ok+ierr
      IF ( jn_ok /= 0 )THEN 
         WRITE(0,*)'diaharm_cmc:: read header wrong ' // trim(cd_fename), jn; call flush(0); stop
      ENDIF
      IF ( jn_all /= jp_tot_freq ) THEN
         write(0,*)'diaharm_cmc:: freq number wrong,jn_all,jp_tot_freq=', jn_all,jp_tot_freq; call flush(0); stop
      ENDIF

      ALLOCATE ( wk_okfreq(jn_all), stat=ierr );    wk_okfreq(:)=rp_fill_value

      ! read contents
      IF( lwp )THEN
         WRITE(numout,*) 
         WRITE(numout,*)' diaharm_cmc : the selected period IN HOUR for harmonic analysis'
      ENDIF
      jn_ok=0;
      DO jn=1, jn_all
         READ(iwk_unit,*, iostat=ierr) wk_freq, wk_diff, cwk
         IF ( ierr /= 0 ) THEN
            WRITE(0,*)'diaharm_cmc:: read jnth records wrong ' // trim(cd_fename), jn; call flush(0); stop
         ENDIF
         IF ( wk_diff >= relay_crt .AND. wk_freq >= 1.0_wp/max_period_hour ) THEN
            IF ( ln_exclude_25hr .AND.  abs(1.0_wp/wk_freq - 25.0_wp) <= 0.5_wp ) THEN
               IF( lwp )THEN
                  WRITE(numout,"(a, f12.6, a)") "excluding this freq close to 25 plus-minus 0.5 hour: ", 1.0/wk_freq,  '  '//trim(cwk);
                  CALL FLUSH(numout)
               ENDIF
            ELSE
               jn_ok=jn_ok+1;
               wk_okfreq(jn_ok) = wk_freq
               IF( lwp )THEN
                  WRITE(numout,'(I3.3, f12.6, a)') jn_ok, 1.0/wk_freq,  '  '//trim(cwk);       call flush(numout)
               ENDIF 
            ENDIF
         ENDIF
      ENDDO
      CLOSE (iwk_unit)

      ! specify to nb_frq and omega_harmo
      ALLOCATE ( omega_harmo(jn_ok), stat=ierr);  omega_harmo(:) =rp_fill_value
      nb_frq=jn_ok; 
      omega_harmo(:) = wk_okfreq(1:nb_frq) * (2.0*rpi/3600.0)  ! unit: hertz * rad
      IF( lwp )THEN
          WRITE(numout,*) ' diaharm_cmc : selected period for harmonic-analysis OK'
          CALL FLUSH(numout)
      ENDIF 

      DEALLOCATE (wk_okfreq, stat=ierr)

   END SUBROUTINE load_harmo_freq
#else
   LOGICAL, PUBLIC             :: lk_diaharm_cmc = .FALSE.
!
CONTAINS
   SUBROUTINE dia_harm_cmc( kt )
      INTEGER , INTENT(in)     ::   kt
      WRITE(*,*) 'dia_harm_cmc: You should not have seen this print! error?'
   END SUBROUTINE dia_harm_cmc
#endif
   !!======================================================================
END MODULE diaharm_cmc
