MODULE dynadv_sl

!FRoy OK ???? (Check if my interpretation of gdept is OK below
! (new names gdept_0 ==> gdept_1d , gdept ==> gdept_0)
! (new names gdepw_0 ==> gdepw_1d , gdepw ==> gdepw_0)
! (new names e3t_0 ==> e3t_1d , e3t ==> e3t_0)

   USE par_oce          ! ocean parameters
   USE dom_oce          ! ocean space and time domain
   USE oce              ! ocean dynamics and tracers variables
   USE in_out_manager   ! I/O manager
   USE wrk_nemo         ! Memory allocation
   USE lib_mpp          ! distributed memory computing library
   USE lbclnk           ! ocean lateral boundary conditions (or mpp link)
   USE prtctl 
   USE semilag_adv      ! semi-Lagrangian advection core routines

   IMPLICIT NONE
   PRIVATE

   PUBLIC dyn_adv_sl_t   ! Momentum advection with t-point averaging
   PUBLIC dyn_adv_sl_uv  ! Momentum advection to u, v points directly

   !! * Substitutions
#  include "domzgr_substitute.h90"
#  include "vectopt_loop_substitute.h90"

CONTAINS

!copyright (C) 2001  MSC-RPN COMM  %%%MC2%%%
!***s/r slag3d -- Upstream interpolation: Semi-Lagrangian Scheme
!*

subroutine dyn_adv_sl_t (kt,do_limit,toler)
   ! Advect horizontal momentum in a Semi-Lagrangian manner and post-process
   ! the results to return a momentum trend.

   ! Unlike dyn_adv_sl_split, this subroutine re-uses the trajectories computed
   ! with tra_adv_sl by:

   ! 1) averaging 'before' momentum values from velocity points to tracer points
   ! 2) Upwinding these averaged momentum values (using t-trajectories)
   ! 3) Computing the averaged u, v trends
   ! 4) Re-averaging these trends back to the respective momentum points

   ! This avoids the need to duplicate (triplicate) the relatively expensive
   ! set of trajectory computations, but it does introduce diffusion into the
   ! process.  The spatial order of accuracy will be lower than that provided
   ! by the interpolation alone, and there may be impacts on overall stability.

   INTEGER,  INTENT(in) :: kt ! Parameters
   LOGICAL,  INTENT(in) :: do_limit
   REAL(wp), INTENT(in) :: toler

   REAL(wp), POINTER, DIMENSION(:,:,:) :: vel_avg_b, vel_avg_i, vel_avg_trend
   REAL(wp) :: dt_leapfrog
   INTEGER  :: ierr

   IF(lwp) THEN
      WRITE(numout,*)
      WRITE(numout,*) 'dyn_adv_sl_t : Semi_Lagragian treatment of momentum via t-point advection'
      WRITE(numout,*) '~~~~~~~~~~~~~'
      WRITE(numout,*)
   ENDIF
   
   IF ( .not. allocated( xd ) ) THEN
     ALLOCATE( xd(jpi,jpj,jpk), yd(jpi,jpj,jpk), zd(jpi,jpj,jpk), STAT=ierr )
     IF( ierr /= 0 )   CALL ctl_warn('dyn_adv_sl_t: failed to allocate arrays')
   ENDIF
   CALL wrk_alloc( jpi, jpj, jpk, vel_avg_b )
   CALL wrk_alloc( jpi, jpj, jpk, vel_avg_i )
   CALL wrk_alloc( jpi, jpj, jpk, vel_avg_trend )

   dt_leapfrog = 2*rdt
   if (kt == nit000 .and. neuler == 0) then
      ! If this is the first timestep and the Euler step has
      ! not yet happened, then set dt_leapfrog to rdt
      dt_leapfrog = rdt
   endif

   if (trajectory_kt /= kt ) then
      ! If the trajectories are not stored at the current timestep, compute them
      CALL traject_sld( kt, dt_leapfrog )
      trajectory_kt = kt
   endif
   ! Average 'before' u-velocities to t-points
   vel_avg_b(2:jpi,:,:) = 0.5*(ub(1:jpi-1,:,:)+ub(2:jpi,:,:))
   ! Exchange processor halo points, since fully valid data is necessary for the semilag interpolation
   call lbc_lnk(vel_avg_b,'T',-1.) ! Include sign reversal at north fold

   ! Interpolate using t-point trajectories.  The physical boundaries lie along u and v points, so here
   ! when interpolating the proper boundary condition is the little-used 'zero_between' along the appropriate
   ! direction
   ! Call interpolation without limiting of overshoots
   call upwnd3_new (vel_avg_i, vel_avg_b, tmask, xd, yd, zd, 0.0, 0.0, 0.0, 'zero_between', 'noflux_between', 'noflux_between', do_limit, toler )

   vel_avg_trend = (vel_avg_i - vel_avg_b)/dt_leapfrog

   ! Average trend back to u-points and add to ua
   ! For points not near a boundary, this is a simple average; velocity points along a boundary
   ! then the proper 'average' is to copy the trend from the t-point back to the velocity point.
   ! This requires a degree of trickery with the masks to ensure no division by zero -- at
   ! masked points the denominator of this average will be 1, but at not-masked points the
   ! 1 - umask term will cancel.
   ua(1:jpi-1,:,:) = ua(1:jpi-1,:,:) + & 
                     (tmask(1:jpi-1,:,:)*vel_avg_trend(1:jpi-1,:,:) + &
                      tmask(2:jpi,:,:)  *vel_avg_trend(2:jpi,:,:)) / &
                     (tmask(1:jpi-1,:,:) + tmask(2:jpi,:,:) + 1 - umask(1:jpi-1,:,:))
   ! Average 'before' v-velocities to t-points
   vel_avg_b(:,2:jpj,:) = 0.5*(vb(:,1:jpj-1,:) + vb(:,2:jpj,:))
   ! Exchange
   call lbc_lnk(vel_avg_b,'T',-1.) ! With sign exchange at north fold
   ! Call interpolation
   call upwnd3_new (vel_avg_i, vel_avg_b, tmask, xd, yd, zd, 0.0, 0.0, 0.0, 'noflux_between', 'zero_between', 'noflux_between', do_limit, toler )
   vel_avg_trend = (vel_avg_i - vel_avg_b)/dt_leapfrog

   ! Average, as with u
   va(:,1:jpj-1,:) = va(:,1:jpj-1,:) + & 
                     (tmask(:,1:jpj-1,:)*vel_avg_trend(:,1:jpj-1,:) + &
                      tmask(:,2:jpj,:)  *vel_avg_trend(:,2:jpj,:)) / &
                     (tmask(:,1:jpj-1,:) + tmask(:,2:jpj,:) + 1 - vmask(:,1:jpj-1,:))

   ! Control volume printing, if activated
   if (ln_ctl) then
      call prt_ctl( tab3d_1 = ua, clinfo1 = ' slm_t adv - Ua: ', mask1 = umask, &
                    tab3d_2 = va, clinfo2 = ' Va: ', mask2 = vmask, clinfo3 = 'dyn')
   endif

   CALL wrk_dealloc( jpi, jpj, jpk, vel_avg_b )
   CALL wrk_dealloc( jpi, jpj, jpk, vel_avg_i )
   CALL wrk_dealloc( jpi, jpj, jpk, vel_avg_trend )

end subroutine dyn_adv_sl_t


subroutine dyn_adv_sl_uv (kt,do_limit,toler)
   ! Advect horizontal momentum in a Semi-Lagrangian manner, and post-process
   ! the results to return a momentum trend.  By presenting the result as a
   ! "trend", this should ensure compatibility with the remainder of the dynamics
   ! code.

   ! It is important to mention explicitly that semi-Lagrangian advection does
   ! not carry any particular guarantees about conservation.  Modifications of
   ! the semi-Lagrangian scheme to address these concerns do exist, but their
   ! implementation would be rather cumbersome.

   ! This routine proceeds in three parts, independently for u and v

   ! First, construct an initial guess for [u,v,w] velocities at our [u,v]
   ! points.  Multiply by dt to construct a one-timestep initial displacement.

   ! Second, iterate this displacement (Tsl_iter times) to converge to revised
   ! trajectories.  This displacement is based on "now" timestep velocities.

   ! Third-and-a-half, subtract the "before" velocities to construct the advection
   ! trend.  This permits dyn_adv_sl to act as a drop-in replacement for other
   ! advection schemes.

   ! Local variables:

   INTEGER,  INTENT(in) :: kt  ! Parameters
   LOGICAL,  INTENT(in) :: do_limit
   REAL(wp), INTENT(in) :: toler

   REAL(wp), POINTER, DIMENSION(:,:,:) :: ui, vi, wi    ! Interpolated velocities
   REAL(wp), POINTER, DIMENSION(:,:,:) :: uo, vo, wo    ! Original velocities
   REAL(wp), POINTER, DIMENSION(:,:,:) :: u0d, v0d, w0d ! Displacements
   REAL(wp)  :: u0p, v0p, w0p, pscale
   REAL(wp)  :: uoh, voh, woh, uih, vih, wih, uimask, vimask, wimask
   REAL(wp)  :: u0new, v0new, w0new
   INTEGER   :: ji, jj, jk, id, jd, iff, jf, iter, kp1, km1
   INTEGER   :: maxiter, maxji, maxjj, maxjk, itertotal, numcells
   real(wp)  :: dt_leapfrog, ee
   integer   :: ierr
     
   IF(lwp) THEN
      WRITE(numout,*)
      WRITE(numout,*) 'dyn_adv_sl_uv : Semi_Lagragian treatment of momentum at velocity points'
      WRITE(numout,*) '~~~~~~~~~~~~~'
   ENDIF

   IF ( .not. allocated( xd ) ) THEN
     ALLOCATE( xd(jpi,jpj,jpk), yd(jpi,jpj,jpk), zd(jpi,jpj,jpk), STAT=ierr )
     IF( ierr /= 0 )   CALL ctl_warn('dyn_adv_sl_uv: failed to allocate arrays')
   ENDIF
   CALL wrk_alloc( jpi, jpj, jpk, ui )
   CALL wrk_alloc( jpi, jpj, jpk, vi )
   CALL wrk_alloc( jpi, jpj, jpk, wi )
   CALL wrk_alloc( jpi, jpj, jpk, uo )
   CALL wrk_alloc( jpi, jpj, jpk, vo )
   CALL wrk_alloc( jpi, jpj, jpk, wo )
   CALL wrk_alloc( jpi, jpj, jpk, u0d )
   CALL wrk_alloc( jpi, jpj, jpk, v0d )
   CALL wrk_alloc( jpi, jpj, jpk, w0d )

   super_noisy = .false.

   ! xd, yd, zd will be used for displacements-in-cells for interpolation, 
   ! and they are defined in the module
   ! Initialize
   xd = 0; yd = 0; zd = 0;
   ui = 0; vi = 0; wi = 0;
   uo = 0; vo = 0; wo = 0;

   ! By default, we compute a phantom trend based on the difference between
   ! what semi-lagrangian advection would give us after one step and
   ! the *before*-time values for velocity.
   dt_leapfrog = 2*rdt
   if (kt == nit000 .and. neuler == 0) then
      ! If this is the first timestep and the Euler step has
      ! not yet happened, then set dt_leapfrog to rdt
      dt_leapfrog = rdt
   endif

   ! After this call, the trajectories will not be valid at t-points anymore, so reset
   ! the flag to -1
   trajectory_kt = -1

   !! Upwinding of U

   ! As our initial guess, use existing velocities at the lexically-same points.
   ! This will tend to "break" near corners near corners, where a u-point may
   ! exist and a same-numbered v-point may not exist, but that's okay provided
   ! off-grid points have zero velocities.  Then, the first iteration will
   ! correctly interpolate.

   ! Convert velocities into grid-cell displacements, by multiplying through
   ! by the timestep and dividing by the grid scale factor (e[123]u)
   if (print_vdiag /= 0) then
      write(numout,*) 'Semilag advection of velocity'
      write(numout,*) 'Before velocities u'
      write(numout,'(5E11.3)') ub(diag_i-2:diag_i+2,diag_j-2:diag_j+2,diag_k)
      write(numout,*) 'v'
      write(numout,'(5E11.3)') vb(diag_i-2:diag_i+2,diag_j-2:diag_j+2,diag_k)
      write(numout,*) 'Local velocities u'
      write(numout,'(5E11.3)') un(diag_i-2:diag_i+2,diag_j-2:diag_j+2,diag_k)
      write(numout,*) 'v'
      write(numout,'(5E11.3)') vn(diag_i-2:diag_i+2,diag_j-2:diag_j+2,diag_k)
      write(numout,*) 'w'
      write(numout,'(1E11.3)') wn(diag_i,diag_j,diag_k)
   endif

   uo = un
   vo(1:jpi-1,2:jpj,:) = 0.25*(vn(1:jpi-1,1:jpj-1,:) + & ! Below and to the left
                               vn(2:jpi  ,1:jpj-1,:) + & ! Below and to the right
                               vn(1:jpi-1,2:jpj  ,:) + & ! Above and to the left
                               vn(2:jpi  ,2:jpj  ,:))    ! Above and to the right
   ! For z-displacement, e3u is not available.  Instead, we use e3t_0, defined
   ! on the same vertical level.  Extra care should be taken for partial
   ! vertical levels.

   ! z-displacements are negative because the grid counts upwards-by-index
   ! as we go towards the ocean floor, but the velocities themselves are
   ! positive towards the surface.

   ! To account for partial steps, at this stage we take the z-displacement
   ! **relative to the background level** without partial-stepping.  This
   ! creates an offset for any partial step, which will be undone inside
   ! the interpolation routine.
   wo(1:jpi-1,:,1:jpk-1) = 0.25*(wn(2:jpi,:,1:jpk-1) + wn(2:jpi,:,2:jpk) + &
                             wn(1:jpi-1,:,1:jpk-1) + wn(1:jpi-1,:,2:jpk))
   u0d = uo*dt_leapfrog
   v0d = vo*dt_leapfrog
   w0d = wo*dt_leapfrog
   do jk = 1,jpkm1
      xd(:,:,jk) = u0d(:,:,jk)/e1u(:,:)*umask(:,:,jk)
      yd(:,:,jk) = v0d(:,:,jk)/e2u(:,:)*umask(:,:,jk)
   end do
   ! Limit initial trajectories to 0.5 cell-lengths in lateral directions, to prevent
   ! first guesses from "skipping past" boundaries
   do ji=1,jpi
      do jj=1,jpj
         do jk=1,jpk
            if (abs(xd(ji,jj,jk)) > 0.5 .or. abs(yd(ji,jj,jk)) > 0.5) then
               ee = sqrt(xd(ji,jj,jk)**2 + yd(ji,jj,jk)**2)
               u0d(ji,jj,jk) = 0.5*u0d(ji,jj,jk)/ee
               v0d(ji,jj,jk) = 0.5*v0d(ji,jj,jk)/ee
               w0d(ji,jj,jk) = 0.5*w0d(ji,jj,jk)/ee
               xd(ji,jj,jk) = u0d(ji,jj,jk)/e1u(ji,jj)
               yd(ji,jj,jk) = v0d(ji,jj,jk)/e2u(ji,jj)
            end if
         end do
      end do
   end do
   do ji = 1,jpi
     do jj = 1,jpj 
         zd(ji,jj,1:jpk-1) = (-w0d(ji,jj,1:jpk-1) + gdept_1d(1:jpk-1) - gdept_0(ji,jj,1:jpk-1))/ e3t_1d(1:jpk-1) * umask(ji,jj,1:jpk-1)
      end do
   end do

   if (print_vdiag /= 0) then
      write(numout,*) 'Initial U-velocities'
      write(numout,'(3E12.4)') uo(diag_i,diag_j,diag_k), vo(diag_i,diag_j,diag_k), wo(diag_i,diag_j,diag_k)
      write(numout,*) 'Initial U-displacement guesses'
      write(numout,'(3E12.4)') u0d(diag_i,diag_j,diag_k), v0d(diag_i,diag_j,diag_k), w0d(diag_i,diag_j,diag_k)
      write(numout,'(3E12.4)') xd(diag_i,diag_j,diag_k), yd(diag_i,diag_j,diag_k), zd(diag_i,diag_j,diag_k)
   endif
   
   maxiter = 0
   itertotal = 0
   numcells = 0

   do jk = 1,jpk-1
      do jj = 2,jpj-1
!CS 2018/07/04         do ji = 1,jpi-1
         do ji = 2,jpi-1
            ! Skip masked cells
            if (umask(ji,jj,jk) == 0.0) cycle

            numcells = numcells + 1

            ! Otherwise, interpolate to find the velocity at the 'foot' of the current candidate trajectory, repeating
            ! until convergence

            ! Save local origin velocity
            uoh = uo(ji,jj,jk)
            voh = vo(ji,jj,jk)
            woh = wo(ji,jj,jk)

            do iter = 1,Tsl_iter

               u0p = u0d(ji,jj,jk)
               v0p = v0d(ji,jj,jk)
               w0p = w0d(ji,jj,jk)

               !if (ji == diag_i .and. jj == diag_j .and. jk == diag_k) then
               !   super_noisy = .true.
               !else
               !   super_noisy = .false.
               !endif


               ! Perform linear interpolation; here the u-grid is the native grid whereas
               ! v and w require appropriate offsets

               call upwnd1 (uih, uimask, un, umask, ji, xd(ji,jj,jk), &
                            jj, yd(ji,jj,jk), jk, zd(ji,jj,jk), 0.0, 0.0, 0.0, &
                            'zero_mask', 'noflux_between', 'noflux_between')

               ! Interpolate v.  Given the way grid numbering works, we wish to subtract
               ! half of a grid cell in 'y' to match physical locations.  That is, the
               ! point (i,j) on the u-grid is the same physical location as (i+0.5,j-0.5)
               ! on the v-grid, which turns into a (negative of the) offset given to upwnd3_new.  
               ! See figure 3.2 of the NEMO manual for details of grid numbering.

               call upwnd1 (vih, vimask, vn, vmask, ji, xd(ji,jj,jk), &
                            jj, yd(ji,jj,jk), jk, zd(ji,jj,jk), -0.5, 0.5, 0.0, &
                            'noflux_between', 'zero_mask', 'noflux_between')

               ! Interpolate w.  This introduces a half-cell offset in both the horizontal (i)
               ! and vertical (k), since w is given on a grid offset in depth between t-points.
               ! To ask for the point (i,j,k) on the u-grid, we need to ask for (i+0.5,j,k+0.5).
               
               call upwnd1 (wih, wimask, wn, tmask, ji, xd(ji,jj,jk), &
                            jj, yd(ji,jj,jk), jk, zd(ji,jj,jk), -0.5, 0.0, -0.5, &
                            'noflux_between', 'noflux_between', 'zero_mask')

               ! Sanity checking: if any of the interpolated masks is nonzero, we're inside
               ! the boundary

               if (uimask == 0 .or. vimask == 0 .or. wimask == 0) then
                  uih = 0
                  vih = 0
                  wih = 0
               endif

               if (iter > Tsl_cut .and. uimask == 1.0 .and. vimask == 1.0 .and. wimask == 1.0) then
                  if (print_vdiag /= 0 .and. ji == diag_i .and. jj == diag_j .and. jk == diag_k) then 
                     write(numout,*) 'Ending with confirmed in-domain point @ iter', iter
                  endif
                  exit
               endif

               if (print_vdiag /= 0 .and. ji == diag_i .and. jj == diag_j .and. jk == diag_k) then 
                  write(numout,*) 'Interpolated velocities @ iter', iter
                  write(numout,'(3E12.4)') uih, vih, wih
               endif

               call expstep(dt_leapfrog, uoh, voh, woh, uih, vih, wih, u0p, v0p, w0p, u0new, v0new, w0new)

!               if (iter > (4*Tsl_iter/5)) then
!                  u0d(ji,jj,jk) = 0.9375*u0p + 0.0625*u0new
!                  v0d(ji,jj,jk) = 0.9375*v0p + 0.0625*v0new
!                  w0d(ji,jj,jk) = 0.9375*w0p + 0.0625*w0new
               if (iter > (3*Tsl_cut/4)) then
                  u0d(ji,jj,jk) = 0.875*u0p + 0.125*u0new
                  v0d(ji,jj,jk) = 0.875*v0p + 0.125*v0new
                  w0d(ji,jj,jk) = 0.875*w0p + 0.125*w0new
               elseif (iter > (2*Tsl_cut/4)) then
                  u0d(ji,jj,jk) = 0.75*u0p + 0.25*u0new
                  v0d(ji,jj,jk) = 0.75*v0p + 0.25*v0new
                  w0d(ji,jj,jk) = 0.75*w0p + 0.25*w0new
               elseif (iter > (Tsl_cut/4)) then
                  u0d(ji,jj,jk) = 0.5*u0p + 0.5*u0new
                  v0d(ji,jj,jk) = 0.5*v0p + 0.5*v0new
                  w0d(ji,jj,jk) = 0.5*w0p + 0.5*w0new
               else
                  u0d(ji,jj,jk) = u0new
                  v0d(ji,jj,jk) = v0new
                  w0d(ji,jj,jk) = w0new
               endif

               ! Recalculate cell dispalcements
               xd(ji,jj,jk) = u0d(ji,jj,jk) / e1u(ji,jj)
               yd(ji,jj,jk) = v0d(ji,jj,jk) / e2u(ji,jj)
               zd(ji,jj,jk) = (-w0d(ji,jj,jk) + gdept_1d(jk) - gdept_0(ji,jj,jk))/e3t_1d(jk)


               if (print_vdiag /= 0 .and. ji == diag_i .and. jj == diag_j .and. jk == diag_k) then
                  write(numout,*) 'Revised displacements (u), iter', iter
                  write(numout,'(3E12.4)') u0d(diag_i,diag_j,diag_k), v0d(diag_i,diag_j,diag_k), w0d(diag_i,diag_j,diag_k)
                  write(numout,'(3E12.4)') xd(diag_i,diag_j,diag_k), yd(diag_i,diag_j,diag_k), zd(diag_i,diag_j,diag_k)
                  write (numout,*) 'Delta'
                  write (numout,'(1E12.4)') ((u0d(diag_i,diag_j,diag_k)-u0p)**2 + &
                                             (v0d(diag_i,diag_j,diag_k)-v0p)**2 + &
                                             (w0d(diag_i,diag_j,diag_k)-w0p)**2)**0.5
               endif

               ! Break early if the trajectory is converged within tolerance
               if ((u0d(ji,jj,jk)-u0p)**2/e1u(ji,jj)**2 + &
                   (v0d(ji,jj,jk)-v0p)**2/e2u(ji,jj)**2 + &
                   (w0d(ji,jj,jk)-w0p)**2/e3t_1d(jk)**2 < (0.001)**2) then
                  u0d(ji,jj,jk) = u0p
                  v0d(ji,jj,jk) = v0p
                  w0d(ji,jj,jk) = w0p
                  xd(ji,jj,jk) = u0d(ji,jj,jk) / e1u(ji,jj)
                  yd(ji,jj,jk) = v0d(ji,jj,jk) / e2u(ji,jj)
                  zd(ji,jj,jk) = (-w0d(ji,jj,jk) + gdept_1d(jk) - gdept_0(ji,jj,jk))/e3t_1d(jk)
                  exit
               endif
            end do
            if (iter > maxiter) then
               maxiter = iter
               maxji = ji
               maxjj = jj
               maxjk = jk
            endif

            itertotal = itertotal + iter
         end do
      end do
   end do

   if (.true. .or. print_vdiag /= 0) then
      write (numout, *) 'Maximum of', maxiter, 'u-iterations at cell'
      write (numout, *) maxji, maxjj, maxjk
      write (numout, * ) itertotal, 'u-trajectory iterations in', numcells
   endif

   !if (print_vdiag /= 0) print_diag = 1

   ! Now, we've hopefully converged on a trajectory.  Interpolate ub along
   ! this trajectory to get our updated value.
   ! Call interpolation
   call upwnd3_new (ui, ub, umask, xd, yd, zd, 0.0, 0.0, 0.0, & 
                    'zero_mask', 'noflux_between', 'noflux_between', do_limit, toler )
   print_diag = 0;

   ! Our "trend" from the semi-Lagrangian advection will simply be the first difference
   ! between the interpolated velocity and the 'now' velocity.  This conceptually matches
   ! what's done with the existing advection schemes such as cen2.  This trend is added
   ! to the 'after' velocity (which, until dyn_nxt is called in step.F90, is really the
   ! trend accumulator.)

   ua = ua + (ui - ub)/dt_leapfrog
   if (print_vdiag /= 0) then
      write(numout,*) 'Interpolated u'
      write(numout,'(3E12.4)') ui(diag_i-1:diag_i+1,diag_j-1:diag_j+1,diag_k)
      write(numout,*) 'trend'
      write(numout,'(1E12.4)') (ui(diag_i,diag_j,diag_k)-ub(diag_i,diag_j,diag_k))/rdt
   endif

   ! Repeat this process for 'v', beginning with re-initialized displacements.
   uo = 0; wo = 0;
   vo = vn
   uo(2:jpi,1:jpj-1,:) = 0.25*(un(1:jpi-1,2:jpj  ,:) + & ! Above and to the left
                               un(2:jpi  ,2:jpj  ,:) + & ! Above and to the right
                               un(1:jpi-1,1:jpj-1,:) + & ! Below and to the left
                               un(2:jpi  ,1:jpj-1,:))    ! Below and to the right
   wo(:,1:jpj-1,1:jpk-1) = 0.25*(wn(:,1:jpj-1,1:jpk-1) + wn(:,2:jpj,1:jpk-1) + &
                                 wn(:,1:jpj-1,2:jpk)   + wn(:,2:jpj,2:jpk))
   u0d = uo*dt_leapfrog
   v0d = vo*dt_leapfrog
   w0d = wo*dt_leapfrog
   do jk = 1,jpk
      xd(:,:,jk) = uo(:,:,jk) * dt_leapfrog * vmask(:,:,jk) / e1v ! use v-point grid scaling factors
      yd(:,:,jk) = vo(:,:,jk) * dt_leapfrog * vmask(:,:,jk) / e2v
   enddo
   ! Limit initial trajectories to 0.5 cell-lengths in lateral directions, to prevent
   ! first guesses from "skipping past" boundaries
   do ji=1,jpi
      do jj=1,jpj
         do jk=1,jpk
            if (abs(xd(ji,jj,jk)) > 0.5 .or. abs(yd(ji,jj,jk)) > 0.5) then
               ee = sqrt(xd(ji,jj,jk)**2 + yd(ji,jj,jk)**2)
               u0d(ji,jj,jk) = 0.5*u0d(ji,jj,jk)/ee
               v0d(ji,jj,jk) = 0.5*v0d(ji,jj,jk)/ee
               w0d(ji,jj,jk) = 0.5*w0d(ji,jj,jk)/ee
               xd(ji,jj,jk) = u0d(ji,jj,jk)/e1v(ji,jj)
               yd(ji,jj,jk) = v0d(ji,jj,jk)/e2v(ji,jj)
            end if
         end do
      end do
   end do
   do ji = 1,jpi
      do jj = 1,jpj
         zd(ji,jj,:) = (-wo(ji,jj,:) * dt_leapfrog +gdept_1d(:) - gdept_0(ji,jj,:))* vmask(ji,jj,:) / e3t_1d(:)
      end do
   end do
   if (print_vdiag /= 0) then
      write(numout,*) 'Initial V-velocities'
      write(numout,'(3E12.4)') uo(diag_i,diag_j,diag_k), vo(diag_i,diag_j,diag_k), wo(diag_i,diag_j,diag_k)
      write(numout,*) 'Initial V-displacement guesses'
      write(numout,'(3E12.4)') u0d(diag_i,diag_j,diag_k), v0d(diag_i,diag_j,diag_k), w0d(diag_i,diag_j,diag_k)
      write(numout,'(3E12.4)') xd(diag_i,diag_j,diag_k), yd(diag_i,diag_j,diag_k), zd(diag_i,diag_j,diag_k)
   endif

   maxiter = 0
   itertotal = 0
   numcells = 0

   do jk = 1,jpk-1
!CS 2018/07/04      do jj = 1,jpj-1
      do jj = 2,jpj-1
         do ji = 2,jpi-1
            ! Skip masked cells
            if (vmask(ji,jj,jk) == 0.0) cycle

            numcells = numcells + 1

            ! Otherwise, interpolate to find the velocity at the 'foot' of the current candidate trajectory, repeating
            ! until convergence

            ! Save local origin velocity
            uoh = uo(ji,jj,jk)
            voh = vo(ji,jj,jk)
            woh = wo(ji,jj,jk)

            do iter = 1,Tsl_iter

               u0p = u0d(ji,jj,jk)
               v0p = v0d(ji,jj,jk)
               w0p = w0d(ji,jj,jk)


               ! Perform linear interpolation; here the u-grid is the native grid whereas
               ! v and w require appropriate offsets

               call upwnd1 (uih, uimask, un, umask, ji, xd(ji,jj,jk), &
                            jj, yd(ji,jj,jk), jk, zd(ji,jj,jk), 0.5, -0.5, 0.0, &
                            'zero_mask', 'noflux_between', 'noflux_between')

               ! Interpolate v.  Given the way grid numbering works, we wish to subtract
               ! half of a grid cell in 'y' to match physical locations.  That is, the
               ! point (i,j) on the u-grid is the same physical location as (i+0.5,j-0.5)
               ! on the v-grid, which turns into a (negative of the) offset given to upwnd3_new.  
               ! See figure 3.2 of the NEMO manual for details of grid numbering.

               call upwnd1 (vih, vimask, vn, vmask, ji, xd(ji,jj,jk), &
                            jj, yd(ji,jj,jk), jk, zd(ji,jj,jk), 0.0, 0.0, 0.0, &
                            'noflux_between', 'zero_mask', 'noflux_between')

               ! Interpolate w.  This introduces a half-cell offset in both the horizontal (i)
               ! and vertical (k), since w is given on a grid offset in depth between t-points.
               ! To ask for the point (i,j,k) on the u-grid, we need to ask for (i+0.5,j,k+0.5).
               
               call upwnd1 (wih, wimask, wn, tmask, ji, xd(ji,jj,jk), &
                            jj, yd(ji,jj,jk), jk, zd(ji,jj,jk), 0.0, -0.5, -0.5, &
                            'noflux_between', 'noflux_between', 'zero_mask')

               ! Sanity checking: if any of the interpolated masks is nonzero, we're inside
               ! the boundary

               if (uimask == 0 .or. vimask == 0 .or. wimask == 0) then
                  uih = 0
                  vih = 0
                  wih = 0
               endif
               
               if (iter > Tsl_cut .and. uimask == 1.0 .and. vimask == 1.0 .and. wimask == 1.0) then
                  if (print_vdiag /= 0 .and. ji == diag_i .and. jj == diag_j .and. jk == diag_k) then 
                     write(numout,*) 'Ending with confirmed in-domain point @ iter', iter
                  endif
                  exit
               endif

               if (print_vdiag /= 0 .and. ji == diag_i .and. jj == diag_j .and. jk == diag_k) then 
                  write(numout,*) 'Interpolated velocities @ iter', iter
                  write(numout,'(3E12.4)') uih, vih, wih
               endif

               call expstep(dt_leapfrog, uoh, voh, woh, uih, vih, wih, u0p, v0p, w0p, u0new, v0new, w0new)

!               if (iter > (4*Tsl_iter/5)) then
!                  u0d(ji,jj,jk) = 0.9375*u0p + 0.0625*u0new
!                  v0d(ji,jj,jk) = 0.9375*v0p + 0.0625*v0new
!                  w0d(ji,jj,jk) = 0.9375*w0p + 0.0625*w0new
               if (iter > (3*Tsl_cut/4)) then
                  u0d(ji,jj,jk) = 0.875*u0p + 0.125*u0new
                  v0d(ji,jj,jk) = 0.875*v0p + 0.125*v0new
                  w0d(ji,jj,jk) = 0.875*w0p + 0.125*w0new
               elseif (iter > (2*Tsl_cut/4)) then
                  u0d(ji,jj,jk) = 0.75*u0p + 0.25*u0new
                  v0d(ji,jj,jk) = 0.75*v0p + 0.25*v0new
                  w0d(ji,jj,jk) = 0.75*w0p + 0.25*w0new
               elseif (iter > (Tsl_cut/4)) then
                  u0d(ji,jj,jk) = 0.5*u0p + 0.5*u0new
                  v0d(ji,jj,jk) = 0.5*v0p + 0.5*v0new
                  w0d(ji,jj,jk) = 0.5*w0p + 0.5*w0new
               else
                  u0d(ji,jj,jk) = u0new
                  v0d(ji,jj,jk) = v0new
                  w0d(ji,jj,jk) = w0new
               endif

               ! Recalculate cell dispalcements
               xd(ji,jj,jk) = u0d(ji,jj,jk) / e1v(ji,jj)
               yd(ji,jj,jk) = v0d(ji,jj,jk) / e2v(ji,jj)
               zd(ji,jj,jk) = (-w0d(ji,jj,jk) + gdept_1d(jk) - gdept_0(ji,jj,jk))/e3t_1d(jk)


               if (print_vdiag /= 0 .and. ji == diag_i .and. jj == diag_j .and. jk == diag_k) then
                  write(numout,*) 'Revised displacements (v), iter', iter
                  write(numout,'(3E12.4)') u0d(diag_i,diag_j,diag_k), v0d(diag_i,diag_j,diag_k), w0d(diag_i,diag_j,diag_k)
                  write(numout,'(3E12.4)') xd(diag_i,diag_j,diag_k), yd(diag_i,diag_j,diag_k), zd(diag_i,diag_j,diag_k)
                  write (numout,*) 'Delta'
                  write (numout,'(1E12.4)') ((u0d(diag_i,diag_j,diag_k)-u0p)**2 + &
                                             (v0d(diag_i,diag_j,diag_k)-v0p)**2 + &
                                             (w0d(diag_i,diag_j,diag_k)-w0p)**2)**0.5
               endif

               ! Break early if the trajectory is converged within tolerance
               if ((u0d(ji,jj,jk)-u0p)**2/e1v(ji,jj)**2 + &
                   (v0d(ji,jj,jk)-v0p)**2/e2v(ji,jj)**2 + &
                   (w0d(ji,jj,jk)-w0p)**2/e3t_1d(jk)**2 < (0.001)**2) then
                  u0d(ji,jj,jk) = u0p
                  v0d(ji,jj,jk) = v0p
                  w0d(ji,jj,jk) = w0p
                  xd(ji,jj,jk) = u0d(ji,jj,jk) / e1v(ji,jj)
                  yd(ji,jj,jk) = v0d(ji,jj,jk) / e2v(ji,jj)
                  zd(ji,jj,jk) = (-w0d(ji,jj,jk) + gdept_1d(jk) - gdept_0(ji,jj,jk))/e3t_1d(jk)
                  exit
               endif
            end do
            if (iter > maxiter) then
               maxiter = iter
               maxji = ji
               maxjj = jj
               maxjk = jk
            endif

            itertotal = itertotal + iter
         end do
      end do
   end do

   if (.true. .or. print_vdiag /= 0) then
      write (numout, *) 'Maximum of', maxiter, 'v-iterations at cell'
      write (numout, *) maxji, maxjj, maxjk
      write (numout, * ) itertotal, 'v-trajectory iterations in', numcells
   endif
   ! One final interpolation of v, without limiting of overshoots
   if (print_vdiag /= 0) print_diag=1
   ! Call interpolation
   call upwnd3_new(vi, vb, vmask, xd, yd, zd, 0.0, 0.0, 0.0, &
                   'noflux_between','zero_mask','noflux_between', do_limit, toler )

   ! And build the corresponding trend
   va = va + (vi-vb)/dt_leapfrog
   if (print_vdiag /= 0) then
      write(numout,*) 'Interpolated v'
      write(numout,'(3E12.4)') vi(diag_i-1:diag_i+1,diag_j-1:diag_j+1,diag_k)
      write(numout,*) 'trend'
      write(numout,'(2E12.4)') (vi(diag_i,diag_j,diag_k)-vb(diag_i,diag_j,diag_k))/rdt
   endif
   print_diag = 0

   if (ln_ctl) then
      call prt_ctl( tab3d_1 = ua, clinfo1 = ' slm_uv adv - Ua: ', mask1 = umask, &
                    tab3d_2 = va, clinfo2 = ' Va: ', mask2 = vmask, clinfo3 = 'dyn')
   endif

   CALL wrk_dealloc( jpi, jpj, jpk, ui )
   CALL wrk_dealloc( jpi, jpj, jpk, vi )
   CALL wrk_dealloc( jpi, jpj, jpk, wi )
   CALL wrk_dealloc( jpi, jpj, jpk, uo )
   CALL wrk_dealloc( jpi, jpj, jpk, vo )
   CALL wrk_dealloc( jpi, jpj, jpk, wo )
   CALL wrk_dealloc( jpi, jpj, jpk, u0d )
   CALL wrk_dealloc( jpi, jpj, jpk, v0d )
   CALL wrk_dealloc( jpi, jpj, jpk, w0d )

end subroutine dyn_adv_sl_uv
!*---------------------------------------------------------------------
!*---------------------------------------------------------------------

end module dynadv_sl
