MODULE fldread_rpn
   !!======================================================================
   !!                       ***  MODULE  fldread_rpn  ***
   !! Ocean forcing:  read input field for surface boundary condition 
   !!                 (rpn files)
   !!=====================================================================
   !! History :  9.0  !  08-07  (F. Roy with original subroutines 
   !!                                           provided by  M. Desgagne) 
   !!----------------------------------------------------------------------
#if defined key_rpne
   !!----------------------------------------------------------------------
   !!   fld_read_rpn : 
   !!                   read input fields used for the computation of the
   !!                   surface boundary condition (written in rpn files)
   !!----------------------------------------------------------------------
   USE oce             ! ocean dynamics and tracers
   USE dom_oce         ! ocean space and time domain
   USE phycst          ! ???
   USE daymod          ! calendar
   USE in_out_manager  ! I/O manager
   USE fldread
   USE sbc_rpn
   USE sbc_rpntls
   USE sbcblk_rpnc, only: cplatm_step
   USE geo2ocean
   USE lib_mpp
   USE sbc_oce, only: ln_apr_dyn

   IMPLICIT NONE
   PRIVATE   

   !! * Routine accessibility

   INTEGER :: iun_std(1000), &  ! Unit number vector
  &           nfile_std, &      ! Number of std files
  &           kt_sbc            ! Time step used for atm. data interpolation, or treatment of coupling 
   LOGICAL :: atm_file_L

   REAL(wp) ::   dt_gem_atm = 365.*24.  !: spacing between forcing fields (GEM) (hours at this stage)
   REAL(wp) ::   dt_gem_prc = 365.*24.  !: precip dt used for cumulation in GEM (hours at this stage)
                    !: equal to dt_gem_atm if no average in pre-process
   LOGICAL  ::   ln_gem_intrp = .FALSE.   !: switch to activate linear time-interpolation
   INTEGER  ::   nh_gem_offs = 0          !: offset to match dates when searching forcing fields
                    !: example, if daily averages correspond to hour 3, nh_gem_offs = 3
                    !: must be positive
   LOGICAL  ::   ln_gem_avgrad = .FALSE.   !: switch to activate 24h averaging for radiation
   INTEGER, PUBLIC, DIMENSION(2) :: lev_gem_forc=(/ 28257976, 11950 /) 
                    !: Vertical level descriptor (equivalent to ip1), the first
                    !: one is the newstyle version, and the second one the
                    !: oldstyle. It allows to mix oldstyle and newstyle code
                    !: in atmospheric forcing std files.
                    !: The oldstyle code (second value) will only be used
                    !: if fields coded in newstyle are not found.
                    !: The default here is eta=0.995
   REAL(wp), PUBLIC, SAVE ::   &
                 zlev_gem   = -9.       !: optional fixed forcing level from namelist
                                        !: allow to skip reading of PX,P0
                                        !: do nothing if left to -9.
   NAMELIST/namsbc_readrpn/ dt_gem_atm, dt_gem_prc, ln_gem_intrp, nh_gem_offs, &
  &                         lev_gem_forc,ln_gem_avgrad,zlev_gem

   PUBLIC   fld_read_rpn   ! called by sbc... modules

   !!----------------------------------------------------------------------
   !!   OPA 9.0 , LOCEAN-IPSL (2006) 
   !! $Id: fldread.F90 1156 2008-06-26 16:06:45Z rblod $
   !! Software governed by the CeCILL licence (modipsl/doc/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------

CONTAINS

   SUBROUTINE fld_read_rpn( kt, kn_fsbc, sd, sdc )
      !!---------------------------------------------------------------------
      !!                    ***  ROUTINE fld_read_rpn  ***
      !!                   
      !! ** Purpose :   provide at each time step atmospheric variables
      !!                needed by sbc routine (core)
      !!                (UU,VV,TT, etc.) 
      !!
      !! ** Method  :   READ each input fields in STD files using RPN libs
      !!      and intepolate it to the model time-step.
      !!      Forcing files must be listed in "liste_inputfiles",located in
      !!      execution directory
      !!----------------------------------------------------------------------
      INTEGER  , INTENT(in   )               ::   kt        ! ocean time step
      INTEGER  , INTENT(in   )               ::   kn_fsbc   ! sbc computation period (in time step) 
      TYPE(FLD), INTENT(inout), DIMENSION(:) ::   sd        ! input field related variables
      TYPE(FLD), INTENT(inout), DIMENSION(:), &
                                    OPTIONAL ::   sdc       ! input field related variables, coupling
                                                            ! if mixed with atm fields
      !!
      INTEGER  ::   jf         ! dummy indices
      INTEGER  ::   ios        ! namelist error control

      !!---------------------------------------------------------------------

      IF( kt == nit000 ) THEN

        IF ( (.not. ln_rpncpl) .or. ln_rpncmx ) THEN

          REWIND( numnam_ref )              ! Namelist namsbc_readrpn in reference namelist : rpn bulk parameters
          READ  ( numnam_ref, namsbc_readrpn, IOSTAT = ios, ERR = 91)
91        IF( ios /= 0 ) CALL ctl_nam ( ios , 'namsbc_readrpn in reference namelist', lwp )
   !
          REWIND( numnam_cfg )              ! Namelist namsbc_readrpn in configuration namelist : rpn bulk parameters
          READ  ( numnam_cfg, namsbc_readrpn, IOSTAT = ios, ERR = 92 )
92        IF( ios /= 0 ) CALL ctl_nam ( ios , 'namsbc_readrpn in configuration namelist', lwp )

          IF(lwp) THEN
            WRITE(numout,*)
            WRITE(numout,*) 'fld_read_rpn: '
            WRITE(numout,*) '~~~~~~~~'
            WRITE(numout,*) '          Namelist namsbc_readrpn'
            WRITE(numout,*) '          GEM forcing spacing     dt_gem_atm (h) = ', dt_gem_atm
            WRITE(numout,*) '          GEM precip spacing      dt_gem_prc (h) = ', dt_gem_prc
            WRITE(numout,*) '          Interpolation in time   ln_gem_intrp   = ', ln_gem_intrp
            WRITE(numout,*) '          Starting date offset    nh_gem_offs    = ', nh_gem_offs
            WRITE(numout,*) '          Forcing levels          lev_gem_forc   = ', lev_gem_forc
            WRITE(numout,*) '          Forced atm lev if pos.  zlev_gem       = ', zlev_gem
            WRITE(numout,*) '          24h averaged radiation  ln_gem_avgrad  = ', ln_gem_avgrad
            call flush(numout)
          ENDIF

          dt_gem_atm=dt_gem_atm*3600.    !hours to sec.
          dt_gem_prc=dt_gem_prc*3600.    !hours to sec.
        ENDIF

        DO jf = 1, SIZE( sd )                     !    LOOP OVER FIELD    !
          sd(jf)%fnow(:,:,:)   = 0.0
          sd(jf)%fdta(:,:,:,:) = 0.0 ! tricky but working
        ENDDO

        IF ( ln_rpncmx ) THEN
          IF ( .not. PRESENT(sdc) ) call ctl_stop('fld_read_rpn: ln_rpncmx=T and sdc not present, ABORT')
          DO jf = 1, SIZE( sd )                     !    LOOP OVER FIELD    !
            sdc(jf)%fnow(:,:,:)   = 0.0
            sdc(jf)%fdta(:,:,:,:) = 0.0 ! tricky but working
          ENDDO
        ENDIF

#if key_cmc_sam2
        kt_sbc=kt-nit000;    !NB. ndate0 is reset to match nit000 in SAM2
#else
        kt_sbc=kt-1
#endif
        if ( (.not. ln_rpncpl) .or. ln_rpncmx ) call init_atm (sd)

      ENDIF

      ! update field at each kn_fsbc time-step
#if key_cmc_sam2
      IF( MOD( kt-nit000, kn_fsbc ) == 0 ) THEN
        kt_sbc=kt-nit000+1
            if (nrstdt==1) call ctl_stop('Unknown time step method for cmc forcing, nrstdt')
        IF ( ln_rpncmx ) THEN
          call update_atm (sd,sdc)
        ELSE
          call update_atm (sd)
        ENDIF
      ENDIF
#else
      IF( MOD( kt-1, kn_fsbc ) == 0 ) THEN
        kt_sbc=kt
        if (nrstdt==0) then
          if (nit000 /= 1) &
     &call ctl_stop('Unknown time step method for cmc forcing, nit000')
        endif
        if (nrstdt==1) &
     &call ctl_stop('Unknown time step method for cmc forcing, nrstdt')
        IF ( ln_rpncmx ) THEN
          call update_atm (sd,sdc)
        ELSE
          call update_atm (sd)
        ENDIF
      ENDIF
#endif

   END SUBROUTINE fld_read_rpn

   SUBROUTINE init_atm (sd)
      implicit none
      TYPE(FLD), INTENT(inout), DIMENSION(:) ::   sd
      integer yy,mo,dd,hh,mm,ss,dum
      character*16 datev
      real*8  dayfrac,one,sid,rsid
      parameter(one=1.0d0, sid=86400.0d0, rsid=one/sid)

      if ( .not. ln_rpncpl ) then
        ! Was done in cplatm_init, nh_gem_offs not active
        if (lwp) then
          write(numout,*) 'INIT_ATM: ndate0=', ndate0
          write(numout,*) 'INIT_ATM: ndate0_hour=', ndate0_hour
        endif
        write(Mod_runstrt_S(1:8),'(i8.8)') ndate0
        Mod_runstrt_S(9:9)='.'
        write(Mod_runstrt_S(10:11),'(i2.2)') nh_gem_offs + ndate0_hour
        Mod_runstrt_S(12:16)='0000 '  !Already assumed in opa
      endif

      if(lwp) call atm_openf
      dayfrac = dble(kt_sbc)*rdt*rsid
      if (lwp) then
        write(numout,*) 'Mod_runstrt_S=',trim(Mod_runstrt_S)
        write(numout,*) 'dayfrac=',dayfrac
        write(numout,*) 'kt_sbc=',kt_sbc
        write(numout,*) 'rdt=',rdt
        write(numout,*) 'rsid=',rsid
      endif
      call incdatsd  (datev,Mod_runstrt_S,dayfrac)
      call prsdate   (yy,mo,dd,hh,mm,ss,dum,datev,1)
      call pdfjdate2 (tforc_2,yy,mo,dd,hh,mm,ss)
      call atm_getdata (datev,.false.,.true.,sd)

      return
   END SUBROUTINE init_atm

   SUBROUTINE update_atm (sd, sdc)
      implicit none
      TYPE(FLD), INTENT(inout), DIMENSION(:) ::   sd
      TYPE(FLD), INTENT(inout), DIMENSION(:), &
                                OPTIONAL     ::   sdc

      if ( ln_rpncpl ) then
        if (present(sdc)) then
          ! Assumes ln_rpncmx=T
          call cplatm_step (sdc, kt_sbc) 
          call linear_tint (sd)
        else
          call cplatm_step (sd, kt_sbc) 
        endif
      else
!       linear time interpolation of atm data fields
        call linear_tint (sd)
      endif

      return
   END SUBROUTINE update_atm

    !--------------------------------------------
    ! Specific routine for STD file management  
    !-------------------------- -----------------
    
      SUBROUTINE atm_openf
!??      USE in_out_manager
      implicit none
      integer maxnfile
      parameter ( maxnfile=1000 )
      character*512 filename(maxnfile),fn
      integer  fnom,fstouv,fstlnk
      external fnom,fstouv,fstlnk
      integer err,err1,err2,i,cnt,unf,wkoffit
      nfile_std = 0
      cnt       = 0
      unf       = 0
      if (fnom(unf,'liste_inputfiles','SEQ+OLD',0).lt.0) then
        call ctl_stop('atm_openf: liste_inputfiles, error fnom')
      endif
 77   cnt=cnt+1
      if (cnt.gt.maxnfile) then 
        call ctl_stop('atm_openf: maxnfile not large enough ---ABORT---')
      endif
      read (unf, '(a)', end = 9120) filename(cnt)
      goto 77
 9120 nfile_std = cnt - 1
      close(unf)
      do cnt = 1, nfile_std
         fn  = filename(cnt)
         err = wkoffit(fn)
         if ((err.ne.1).and.(err.ne.33)) then
            filename(cnt) = '@#$%^&'
         endif
      end do
      i=0
      do cnt = 1, nfile_std
         if (filename(cnt).ne.'@#$%^&') then
            i = i+1
            filename(i) = filename(cnt)
         endif
      end do
      nfile_std = i
      if (nfile_std.lt.1) then 
         call ctl_stop('atm_openf: NO STD FILE available ---ABORT---')
      endif
      do cnt = 1, nfile_std
         iun_std(cnt) = 0
         err1 = FNOM  (iun_std(cnt),trim(filename(cnt)),'RND+OLD',0)
         if (lwp) write (numout,1001) trim(filename(cnt)),iun_std(cnt)
         err2 = FSTOUV(iun_std(cnt),'RND')
         if ((err1.lt.0).or.(err2.lt.0)) &
           call ctl_stop('atm_openf: err1... ---ABORT---')
      end do
      err = fstlnk (iun_std,nfile_std)
      atm_file_L = .true.
 1001 format ('Opening STD file: ',a,2x,'UNIT= ',i4)
 1002 format ('Opening BINARY file: ',a,2x,'UNIT= ',i4)
      return

      END SUBROUTINE atm_openf


      SUBROUTINE linear_tint (sd)
      implicit none
      TYPE(FLD), INTENT(inout), DIMENSION(:) ::   sd     
      INTEGER  ::   jf         ! dummy indices

      character*16 datev,daten
      integer yy,mo,dd,hh,mm,ss,dum,datm
      real*8  tx,dayfrac,one,sid,rsid
      real    b
      parameter(one=1.0d0, sid=86400.0d0, rsid=one/sid)
      dayfrac = dble(kt_sbc)*rdt*rsid 
      call incdatsd  (datev,Mod_runstrt_S,dayfrac)

      if (datev.gt.current_atmf) then
         dayfrac = dble(dt_gem_atm)*rsid 
         call incdatsd (daten,current_atmf,dayfrac)
         call atm_getdata (daten,.true.,.true.,sd)
      endif
      call prsdate   (yy,mo,dd,hh,mm,ss,dum,datev,1)
      call pdfjdate2 (tx,yy,mo,dd,hh,mm,ss)
      if (ln_gem_intrp) then
        b=(tx-tforc_1)/(tforc_2-tforc_1)
! Using averaged fields with date stamp corresponding to the end of
! the average serie
       else
        b=1.
      endif

      if (lwp) then
        write(numout,*) 'NEW GEM ATM FIELD UPDATE:'
        write(numout,*) 'kt_sbc=',kt_sbc
        write(numout,*) 'dayfrac=',dayfrac
        write(numout,*) 'datev=',datev
        write(numout,*) 'b=',b
        write(numout,'(a,f18.10,1x,i4.4,5(1x,i2.2))') ' tx,yy,mo,dd,hh,mm,ss=',tx,yy,mo,dd,hh,mm,ss
        if (kt_sbc.eq.nit000) call flush(numout)
      endif

      DO jf = 1, SIZE( sd )
        call inter_field3 (b,sd(jf)%fdta(:,:,1,1), &
                             sd(jf)%fdta(:,:,1,2), &
                             sd(jf)%fnow(:,:,1),jpi,jpj)
      ENDDO

 1001 format (/' ----- ABORT ----- No ATM data valid at: ',a)
      return

      END SUBROUTINE linear_tint

      SUBROUTINE atm_getdata (datev,put,get,sd)
      implicit none
      character*(*) datev
      logical put,get
      TYPE(FLD), INTENT(inout), DIMENSION(:) ::   sd
      integer yy,mo,dd,hh,mm,ss,datm,dum,jf
      tforc_1        = tforc_2

      DO jf = 1, SIZE( sd )     
        sd(jf)%fdta(:,:,1,1) = sd(jf)%fdta(:,:,1,2)
      ENDDO

      call atm_read_rpn (datev,sd)

      current_atmf = datev
      call prsdate   (yy,mo,dd,hh,mm,ss,dum,datev,1)
      call pdfjdate2 (tforc_2,yy,mo,dd,hh,mm,ss)

      return
      END SUBROUTINE atm_getdata

      SUBROUTINE atm_read_rpn (datev,sd)
      implicit none
      character*(*) datev
      TYPE(FLD), INTENT(inout), DIMENSION(:) ::   sd
      integer datm
      real KNAMS,kprec
      parameter (KNAMS=0.514791)
      integer i,j,iavg,navg,nivr,nivt,nivm
      integer, dimension(2) :: lev_nul=(/-1,-1/), lev_wrk

      REAL(wp) , ALLOCATABLE, DIMENSION(:,:,:) :: wrkavg
      real*8  dayfrac,one,sid,rsid
      character*16 datew
      parameter(one=1.0d0, sid=86400.0d0, rsid=one/sid)
      real(wp) theta,pi,Tv

      if (.not.atm_file_L.and.lwp) call atm_openf
      kprec=1000./dt_gem_prc

! CAREFUL Must be structured based on sbcblk_rpn.F90 indexing
! Assuming that winds and HU,TT are at same height, will need adaptation
! when using GEM staggered coordinates
      call readfstatm (  sd(1)%fdta (:,:,1,2),jpi,jpj,'UUOR',datev,lev_gem_forc,KNAMS,0.,nivr )
      call readfstatm (  sd(2)%fdta (:,:,1,2),jpi,jpj,'VVOR',datev,lev_gem_forc,KNAMS,0.,nivr )
      nivm=nivr
      call readfstatm (  sd(3)%fdta (:,:,1,2),jpi,jpj,'HU'  ,datev,lev_gem_forc,1.0  ,0.,nivr )
      nivt=nivr
      if (lwp) then
        write(numout,*) 'atm_read_rpn: thermodynamic atm level: ', nivt
        write(numout,*) 'atm_read_rpn: momentum      atm level: ', nivm
      endif
      if (.not.ln_gem_avgrad) then 
        call readfstatm (sd(4)%fdta (:,:,1,2),jpi,jpj,'FB'  ,datev,lev_nul, 1.0  ,0.,nivr )
        call readfstatm (sd(5)%fdta (:,:,1,2),jpi,jpj,'FI'  ,datev,lev_nul, 1.0  ,0.,nivr )
      else
        if (lwp) write(numout,*) 'atm_read_rpn: RADIATION AVERAGING: (SKIP READING FB,FI)',kt_sbc
      endif
      call readfstatm (  sd(6)%fdta (:,:,1,2),jpi,jpj,'TT'  ,datev,lev_gem_forc,1.0  ,273.16,nivr )
      if (nivr /= nivt) call ctl_stop('atm_read_rpn: TT and HU should have same ip1 -- WILL ABORT')
      call readfstatm (  sd(7)%fdta (:,:,1,2),jpi,jpj,'PR'  ,datev,lev_nul, kprec,0.,nivr )
      if (zlev_gem.lt.0.) then
        lev_wrk(:) = nivt
        call readfstatm (sd(11)%fdta(:,:,1,2),jpi,jpj,'PX'  ,datev,lev_wrk, 100.0,0.,nivr )
        if ( nivm /= nivt ) then ! staggered case
        lev_wrk(:) = nivm
        call readfstatm (sd(12)%fdta(:,:,1,2),jpi,jpj,'PX'  ,datev,lev_wrk, 100.0,0.,nivr )
        else
        sd(12)%fdta(:,:,1,2)=sd(11)%fdta(:,:,1,2)
        endif
        call readfstatm (sd(13)%fdta(:,:,1,2),jpi,jpj,'P0'  ,datev,lev_nul, 100.0,0.,nivr )
        if(ln_apr_dyn) &
        call readfstatm (sd(15)%fdta(:,:,1,2),jpi,jpj,'PN'  ,datev,lev_nul, 100.0,0.,nivr )
        do j=1,jpj
        do i=1,jpi
           ! make sure possible garbage values from pre-processing on land do not cause problems
           ! new cases where source data covers water, but not the whole NEMO grid
           if ( tmask(i,j,1) < 0.1 ) then
              sd(1)%fdta (i,j,1,2) = 0.
              sd(2)%fdta (i,j,1,2) = 0.
              sd(3)%fdta (i,j,1,2) = 0.2
              sd(4)%fdta (i,j,1,2) = 0.
              sd(5)%fdta (i,j,1,2) = 0.
              sd(6)%fdta (i,j,1,2) = 275.
              sd(7)%fdta (i,j,1,2) = 0.
              sd(11)%fdta (i,j,1,2) = 10000.
              sd(12)%fdta (i,j,1,2) = 10000.
              sd(13)%fdta (i,j,1,2) = 10000.
              sd(15)%fdta (i,j,1,2) = 10000.
           endif
        enddo
        enddo
!   ... Thickness of first atm. model layer
!   ... (hydrostatic relation approximating Tv_bar=Tv(layer=1))
        do j=1,jpj
        do i=1,jpi
          Tv= sd(6)%fdta(i,j,1,2) * (1.0D0 + DELTA*sd(3)%fdta(i,j,1,2))
          sd(9 )%fdta(i,j,1,2) = -RGASD*Tv/grav*log(sd(11)%fdta(i,j,1,2)/sd(13)%fdta(i,j,1,2))
          sd(10)%fdta(i,j,1,2) = -RGASD*Tv/grav*log(sd(12)%fdta(i,j,1,2)/sd(13)%fdta(i,j,1,2))
          if ( tmask(i,j,1) < 0.1 ) then
             sd(9 )%fdta(i,j,1,2) = 20.
             sd(10)%fdta(i,j,1,2) = 20.
          endif
        enddo
        enddo
       else
        if (lwp) write(numout,*) 'atm_read_rpn: ATM FORCING LEVEL FIXED TO: ', zlev_gem
        sd(9 )%fdta(:,:,1,2) = zlev_gem
        sd(10)%fdta(:,:,1,2) = zlev_gem
        sd(11)%fdta(:,:,1,2) = 100000.
        sd(12)%fdta(:,:,1,2) = 100000.
        sd(13)%fdta(:,:,1,2) = 100000.
        sd(15)%fdta(:,:,1,2) = 100000.
        if (lwp) write(numout,*) 'atm_read_rpn: ATM PRESSURE COLUMN FIXED TO:  100000'
      endif

      if (ln_gem_avgrad) then 
        navg=(24.*3600.)/dt_gem_atm
        if (navg.lt.2) call ctl_stop('ln_gem_avgrad=T==>wrong navg')
        if (datev(10:15).eq.'000000'.or.kt_sbc.eq.0) then
          !ASSUMING that the first nemo time step is at hour 0 (kt_sbc.eq.0 in case not so)
          !ASSUMING that time step divides 24h evenly (datev(10:15).eq.'000000' once a day)
          if (lwp) write(numout,*) 'atm_read_rpn: RADIATION AVERAGING:',kt_sbc,navg
          allocate(wrkavg(jpi,jpj,navg))
          if (lwp) write(numout,*) 'atm_read_rpn: ========FB,DATE:',datev
          call readfstatm   (wrkavg(:,:,1   ),jpi,jpj,'FB',datev,lev_nul,1.0,0.,nivr )
          do iavg=2,navg
            dayfrac = dble(iavg-1)*dt_gem_atm*rsid
            call incdatsd  (datew,datev,dayfrac)
            if (lwp) write(numout,*) 'atm_read_rpn: ========FB,DATE:',datew
            call readfstatm (wrkavg(:,:,iavg),jpi,jpj,'FB',datew,lev_nul,1.0,0.,nivr )
            wrkavg(:,:,1)=wrkavg(:,:,1)+wrkavg(:,:,iavg)
          enddo
          wrkavg(:,:,1)=wrkavg(:,:,1)/navg
          sd(4)%fdta (:,:,1,2) = wrkavg(:,:,1)
          sd(4)%fdta (:,:,1,1) = wrkavg(:,:,1)  
                          !If linear interp, will force it to constant
          if (lwp) write(numout,*) 'atm_read_rpn: ========FI,DATE:',datev
          call readfstatm   (wrkavg(:,:,1   ),jpi,jpj,'FI',datev,lev_nul,1.0,0.,nivr )
          do iavg=2,navg
            dayfrac = dble(iavg-1)*dt_gem_atm*rsid
            call incdatsd  (datew,datev,dayfrac)
            if (lwp) write(numout,*) 'atm_read_rpn: ========FI,DATE:',datew
            call readfstatm (wrkavg(:,:,iavg),jpi,jpj,'FI',datew,lev_nul,1.0,0.,nivr )
            wrkavg(:,:,1)=wrkavg(:,:,1)+wrkavg(:,:,iavg)
          enddo
          wrkavg(:,:,1)=wrkavg(:,:,1)/navg
          sd(5)%fdta (:,:,1,2) = wrkavg(:,:,1)
          sd(5)%fdta (:,:,1,1) = wrkavg(:,:,1)  
                          !If linear interp, will force it to constant
          deallocate(wrkavg)
        endif
      endif

!     ... Unit conversion for precipitations (m --> kg/m**2/s)
!     ... PR/dt_gem_prc=>m/s
!     ... density * m/s = kg/m**2/s, rho=1000 kg/m**3

! Converts precip to snow if T<273.16, 
! sd(7)%fdta (:,:,1,2) not set to zero because used as total precip

      do j=1,jpj 
      do i=1,jpi 
        sd(8)%fdta(i,j,1,2)=0.
        if (sd(6)%fdta(i,j,1,2).le.273.16) sd(8)%fdta(i,j,1,2)=sd(7)%fdta(i,j,1,2)
                ! Now using upper level, 8 corresponds to snow
      enddo
      enddo

! Converts sd(6)%fdta (:,:,1,2) to potential temperature
      sd(6)%fdta(:,:,1,2) = sd(6)%fdta(:,:,1,2)*    &
     &                (sd(13)%fdta(:,:,1,2)/sd(11)%fdta(:,:,1,2))**CAPPA

      return
      
      END SUBROUTINE atm_read_rpn

      SUBROUTINE readfstatm (f,jpi,jpj,nomvar,dat,niv,factm,facta,nivr)

      USE mpi
#if defined key_rpne_sam2
      USE lib_mpp, ONLY: mpi_comm_opa
#endif

      implicit none

      character*(*) nomvar,dat
      integer jpi,jpj
      INTEGER niv(2)
      real f(jpi,jpj),factm,facta
      INTEGER, INTENT(out) :: nivr
      INTEGER fstinl,fstluk,ip1_all,errout

      integer ni1,nj1,nk1,err,nlis,lislon,i,j,datm,mi,mj
      parameter (nlis = 1024)
      integer liste (nlis)
      real*4 wrk(jpidta,jpjdta)

      nivr=0
      errout=0
      if (lwp) then
        call datp2f (datm,dat)
        if (niv(1).eq.-1) then
          err = fstinl (iun_std,ni1,nj1,nk1,datm,' ',niv(1),-1,-1,' ',nomvar,  &
     &                                                   liste,lislon,nlis)
          nivr=niv(1)
         else
          err = fstinl (iun_std,ni1,nj1,nk1,datm,' ',niv(1),-1,-1,' ',nomvar,  &
     &                                                   liste,lislon,nlis)
          nivr=niv(1)
          if (lislon.ne.1) then
          err = fstinl (iun_std,ni1,nj1,nk1,datm,' ',niv(2),-1,-1,' ',nomvar,  &
     &                                                   liste,lislon,nlis)
          nivr=niv(2)
          endif
        endif
        if (lislon.ne.1) then
          if (kt_sbc.le.nitend) then
            print*, 'Variable ',nomvar,' valid: ',dat,  &
     &              ' NOT found ---WILL ABORT---'
            errout=-1
           else
            write(numout,*) 'Variable ',nomvar,' valid: ',dat, 'ip1=', nivr,   &
     &              ' NOT found ---' 
            write(numout,*) 'Will produce a non-perfect TKE2 RESTART'
            write(numout,*) 'FIELD 2 IS NOT UPDATED'
          endif
         else
          err = fstluk (wrk,liste(1),ni1,nj1,nk1)
        endif
      endif
      call MPI_bcast(errout,1,         MPI_INTEGER,0,MPI_COMM_OPA,err ) 
      call MPI_bcast(lislon,1,         MPI_INTEGER,0,MPI_COMM_OPA,err ) 
      call MPI_bcast(nivr  ,1,         MPI_INTEGER,0,MPI_COMM_OPA,err ) 
      if (errout < 0) call ctl_stop('readfstatm: Variable '//nomvar//' NOT found')
      if (lislon.eq.1) then
        call MPI_bcast(wrk,jpidta*jpjdta,MPI_REAL,   0,MPI_COMM_OPA,err ) 
        do i = 1,jpi
        do j = 1,jpj
           mi=mig(i)
           mj=mjg(j)
           if (mi.gt.jpidta) mi=jpidta
           if (mj.gt.jpjdta) mj=jpjdta
           if (mi.lt.1) mi=1
           if (mj.lt.1) mj=1
           f(i,j) = wrk(mi,mj)
        enddo
        enddo
        f   = f*factm + facta
      endif
      return
      END SUBROUTINE readfstatm

#else
   !!----------------------------------------------------------------------
   !!   Dummy module                   NO fldread_rpn module
   !!----------------------------------------------------------------------
#endif

   !!==============================================================================

END MODULE fldread_rpn
