MODULE mpp_rpn

#if defined key_rpne

   !!=====================================================================
   !!                    ***  MODULE  mpp_rpn ***
   !! Library to gather local arrays to global for coupling
   !! or distribute global arrays
   !!====================================================================
   !! History :  1.0  ! 2015  (F. Roy, J-M Belanger) Original code 
   !!--------------------------------------------------------------------

   USE dom_oce
   USE wrk_nemo
#if defined key_mpp_mpi
   USE lib_mpp, only : mppsync, mppgather, mppscatter
#endif

   IMPLICIT NONE
   PRIVATE

   PUBLIC glbcoll4, glbdist4, glbcoll8, glbdist8, glbdisti

   !! Gather distribute variables
   INTEGER, PARAMETER        :: localRoot = 0  ! local MPI process number
!   REAL(wp), DIMENSION(jpij) :: rbuffer        ! Temporary buffer for exchange 
!   REAL(wp), DIMENSION(jpij) :: sbuffer        ! Temporary buffer for exchange
!   REAL(wp), DIMENSION(jpi,jpj,jpnij) :: png
!   REAL(wp), DIMENSION(jpi,jpj) :: wrk
   REAL(wp), DIMENSION(:),     POINTER :: rbuffer
   REAL(wp), DIMENSION(:),     POINTER :: sbuffer
   REAL(wp), DIMENSION(:,:,:), POINTER      :: png
   REAL(wp), DIMENSION(:,:),   POINTER      :: wrk
      
CONTAINS

   SUBROUTINE glbcoll4( r_global_data, r_local_data )

      IMPLICIT NONE

      !!---------------------------------------------------------------------
      !!              ***  ROUTINE glbcoll4  ***
      !!
      !! ** Purpose : - Gather real data from all mpi processes form global array
      !!                on the root process. 
      !!                Use for model coupling field communication through coupler.  
      !!                Output array precision may differ from input array   
      !!----------------------------------------------------------------------
      !! * Arguments
      !!
      REAL(wp), INTENT( IN )  :: r_local_data(jpi,jpj)   

      REAL(sp)               :: r_global_data(jpiglo,jpjglo)
      !!
      !! * Local declarations
      !!
      !
      INTEGER                :: ji, jj, jn ! local loop indices
      !!
      !!--------------------------------------------------------------------
      !!

      CALL wrk_alloc( jpi, jpj, jpnij, png )
      IF ( jpnij > 1 ) THEN
         CALL mppsync
         CALL mppgather (r_local_data,0,png)
         CALL mppsync
       ELSE
         png(:,:,1)=r_local_data(:,:)
      ENDIF

      IF ( nproc==0 .and. jpnij > 1 ) THEN
        DO jn=1,jpnij
           DO jj=1,nlcjt(jn)
              DO ji=1,nlcit(jn)
                 r_global_data(ji+nimppt(jn)-1,jj+njmppt(jn)-1)=png(ji,jj,jn)
              ENDDO
           ENDDO
        ENDDO
       ELSEIF ( jpnij == 1 ) THEN
#if !defined key_mpp_mpi
        r_global_data(:,:) = png(:,:,1)
#endif
      ENDIF
      CALL wrk_dealloc( jpi, jpj, jpnij, png )

      RETURN

   END SUBROUTINE glbcoll4

   SUBROUTINE glbcoll8( r_global_data, r_local_data )

      IMPLICIT NONE

      !!---------------------------------------------------------------------
      !!              ***  ROUTINE glbcoll8  ***
      !!
      !! ** Purpose : - Gather real data from all mpi processes form global array
      !!                on the root process. 
      !!                Use for model coupling field communication through coupler.  
      !!                Output array precision may differ from input array   
      !!----------------------------------------------------------------------
      !! * Arguments
      !!
      REAL(wp), INTENT( IN )  :: r_local_data(jpi,jpj)   

      REAL(wp)               :: r_global_data(jpiglo,jpjglo)
      !!
      !! * Local declarations
      !!
      !
      INTEGER                :: ji, jj, jn ! local loop indices
      !!
      !!--------------------------------------------------------------------
      !!

      CALL wrk_alloc( jpi, jpj, jpnij, png )
      IF ( jpnij > 1 ) THEN
         CALL mppsync
         CALL mppgather (r_local_data,0,png)
         CALL mppsync
       ELSE
         png(:,:,1)=r_local_data(:,:)
      ENDIF

      IF ( nproc==0 .and. jpnij > 1 ) THEN
        DO jn=1,jpnij
           DO jj=1,nlcjt(jn)
              DO ji=1,nlcit(jn)
                 r_global_data(ji+nimppt(jn)-1,jj+njmppt(jn)-1)=png(ji,jj,jn)
              ENDDO
           ENDDO
        ENDDO
       ELSEIF ( jpnij == 1 ) THEN
#if !defined key_mpp_mpi
        r_global_data(:,:) = png(:,:,1)
#endif
      ENDIF
      CALL wrk_dealloc( jpi, jpj, jpnij, png )

      RETURN

   END SUBROUTINE glbcoll8

   SUBROUTINE glbdist4( r_local_data, r_global_data )

      IMPLICIT NONE

      !!---------------------------------------------------------------------
      !!              ***  ROUTINE glbdist4  ***
      !!
      !! ** Purpose : - Redistribute real data from root process to all mpi processes 
      !!                
      !!----------------------------------------------------------------------
      !! * Arguments
      !!
      REAL(wp), INTENT( OUT ) :: r_local_data(jpi,jpj)   

      REAL(sp), INTENT(IN)     :: r_global_data(jpiglo,jpjglo)

      !!
      !! * Local declarations
      !!
      !
      INTEGER                :: ji, jj, jn ! local loop indices
      !!
      !!--------------------------------------------------------------------
      !!

      CALL wrk_alloc( jpi, jpj, jpnij, png )
      IF ( nproc==0 .and. jpnij > 1 ) THEN
         png(:,:,:)=0.0
         DO jn=1,jpnij
            DO jj=1,nlcjt(jn)
               DO ji=1,nlcit(jn)
                  png(ji,jj,jn)=r_global_data(ji+nimppt(jn)-1,jj+njmppt(jn)-1)
               ENDDO
            ENDDO
         ENDDO
       ELSEIF ( jpnij == 1 ) THEN
#if !defined key_mpp_mpi
         png(:,:,1) = r_global_data(:,:)
#endif
      ENDIF

      IF ( jpnij > 1) THEN
         CALL mppsync
         CALL mppscatter (png,0,r_local_data)
         CALL mppsync
      ELSE
         r_local_data(:,:)=png(:,:,1)
      ENDIF
      CALL wrk_dealloc( jpi, jpj, jpnij, png )

      RETURN

   END SUBROUTINE glbdist4

   SUBROUTINE glbdist8( r_local_data, r_global_data )

      IMPLICIT NONE

      !!---------------------------------------------------------------------
      !!              ***  ROUTINE glbdist8  ***
      !!
      !! ** Purpose : - Redistribute real data from root process to all mpi processes 
      !!                
      !!----------------------------------------------------------------------
      !! * Arguments
      !!
      REAL(wp), INTENT( OUT ) :: r_local_data(jpi,jpj)   

      REAL(wp), INTENT(IN)     :: r_global_data(jpiglo,jpjglo)

      !!
      !! * Local declarations
      !!
      !
      INTEGER                :: ji, jj, jn ! local loop indices
      !!
      !!--------------------------------------------------------------------
      !!

      CALL wrk_alloc( jpi, jpj, jpnij, png )
      IF ( nproc==0 .and. jpnij > 1 ) THEN
         png(:,:,:)=0.0
         DO jn=1,jpnij
            DO jj=1,nlcjt(jn)
               DO ji=1,nlcit(jn)
                  png(ji,jj,jn)=r_global_data(ji+nimppt(jn)-1,jj+njmppt(jn)-1)
               ENDDO
            ENDDO
         ENDDO
       ELSEIF ( jpnij == 1 ) THEN 
#if !defined key_mpp_mpi
         png(:,:,1) = r_global_data(:,:)
#endif
      ENDIF

      IF ( jpnij > 1) THEN
         CALL mppsync
         CALL mppscatter (png,0,r_local_data)
         CALL mppsync
      ELSE
         r_local_data(:,:)=png(:,:,1)
      ENDIF
      CALL wrk_dealloc( jpi, jpj, jpnij, png )

      RETURN

   END SUBROUTINE glbdist8

   SUBROUTINE glbdisti( i_local_data, i_global_data )

      IMPLICIT NONE

      !!---------------------------------------------------------------------
      !!              ***  ROUTINE glbdist4  ***
      !!
      !! ** Purpose : - Redistribute real data from root process to all mpi processes 
      !!                
      !!----------------------------------------------------------------------
      !! * Arguments
      !!
      INTEGER, INTENT( OUT )  :: i_local_data(jpi,jpj)   

      INTEGER, INTENT(IN)     :: i_global_data(jpiglo,jpjglo)

      !!
      !! * Local declarations
      !!
      !
      INTEGER                :: ji, jj, jn ! local loop indices
      !!
      !!--------------------------------------------------------------------
      !!

      CALL wrk_alloc( jpi, jpj, jpnij, png )
      CALL wrk_alloc( jpi, jpj, wrk )
      IF ( nproc==0 .and. jpnij > 1 ) THEN
         png(:,:,:)=0.0
         DO jn=1,jpnij
            DO jj=1,nlcjt(jn)
               DO ji=1,nlcit(jn)
                  png(ji,jj,jn)=i_global_data(ji+nimppt(jn)-1,jj+njmppt(jn)-1)
               ENDDO
            ENDDO
         ENDDO
       ELSEIF ( jpnij == 1 ) THEN
#if !defined key_mpp_mpi
         png(:,:,1) = i_global_data(:,:)
#endif
      ENDIF

      IF ( jpnij > 1) THEN
         CALL mppsync
         CALL mppscatter (png,0,wrk)
         i_local_data(:,:)=nint(wrk(:,:))
         CALL mppsync
      ELSE
         i_local_data(:,:)=nint(png(:,:,1))
      ENDIF
      CALL wrk_dealloc( jpi, jpj, jpnij, png )
      CALL wrk_dealloc( jpi, jpj, wrk )

      RETURN

   END SUBROUTINE glbdisti

   !!======================================================================
#else

   !!======================================================================
   !! EMPTY module: mpp_rpn
   !!======================================================================

#endif

END MODULE mpp_rpn
