MODULE sbc_rpn
   !!======================================================================
   !!                       ***  MODULE  sbc_rpn  ***
   !! Surface module :   variables defined in core memory for RPN forcing
   !!                    and coupling
   !!======================================================================
   !!            1.0  ! 2015  (F. Roy) initial module
   !!----------------------------------------------------------------------

#if defined key_rpne
   !!----------------------------------------------------------------------
   !!   sbc_rpn_alloc : allocation of sbc arrays
   !!----------------------------------------------------------------------
   USE par_oce        ! ocean parameters
   USE in_out_manager ! I/O manager
   USE lib_mpp        ! MPP library
#if defined key_lim3
   USE par_ice,   only: jpl
#elif defined key_lim2
   USE par_ice_2, only: jpl
#endif

   IMPLICIT NONE
   PRIVATE

   PUBLIC   sbc_rpn_alloc   ! routine called in sbcmod.F90
   
   !!----------------------------------------------------------------------
   !!           Bulk parameters and constants (RPN)
   !!----------------------------------------------------------------------

   REAL(wp), PUBLIC, PARAMETER ::   CPD    =.100546e+4     ! J K-1 kg-1; specific heat of dry air
   REAL(wp), PUBLIC, PARAMETER ::   CHLC   =.2501e+7       ! J kg-1; latent heat of condensation
   REAL(wp), PUBLIC, PARAMETER ::   CHLF   =.334e+6        ! J kg-1; latent heat of fusion
   REAL(wp), PUBLIC, PARAMETER ::   CHLS   =CHLC+CHLF      ! J kg-1; latent heat of sublimation
   REAL(wp), PUBLIC, PARAMETER ::   STEF   =.566948e-7     ! J m-2 s-1 K-4; Stefan-Boltzmann constant
   REAL(wp), PUBLIC, PARAMETER ::   KARMAN =.40            ! ; Von Karman constant
   REAL(wp), PUBLIC, PARAMETER ::   RGASD  =.28705e+3      ! J K-1 kg-1; gas constant 
                                                           ! for dry air
   REAL(wp), PUBLIC, PARAMETER ::   RGASV  =.46151e+3      ! J K-1 kg-1; gas constant 
                                                           ! for water vapour
   REAL(wp), PUBLIC, PARAMETER ::   EPS1   =.6219480022101 ! ; RGASD/RGASV
   REAL(wp), PUBLIC, PARAMETER ::   DELTA  =.6077686814144 ! ; 1/EPS1 - 1
   REAL(wp), PUBLIC, PARAMETER ::   BETA   = 0.018         ! ; Hard coded param from GEM 
                                                           ! (see water.ftn)
                                                           !   used in Charnock Formula
   REAL(wp), PUBLIC, PARAMETER ::   BETASR = -1            ! Parameters for Smith et al. (1992) and Rascle 2008 combination
                                                           ! estimating wave age and sea roughness
   REAL(wp), PUBLIC, PARAMETER ::   ALPHASR = 0.45         ! ...
                                                           ! ...
   REAL(wp), PUBLIC, PARAMETER ::   USTRMIN=0.01           ! ; Hard coded param from GEM 
                                                           ! (see water.ftn, ZFRVMIN)
                                                           !   used with Z0TRDPS300
   REAL(wp), PUBLIC, PARAMETER ::   CAPPA  =.28549121795   ! RGASD/CPD
   REAL(wp), PUBLIC, PARAMETER ::   Z0MAX  = 5.E-3         ! m; Maximum roughness length
   REAL(wp), PUBLIC, PARAMETER ::   Z0MIN  = 1.5E-5        ! m; Minimum roughness length
   REAL(wp), PUBLIC, PARAMETER ::   Z0HCON = 4.E-5         ! m; Special roughness length for tropics
   REAL(wp), PUBLIC, PARAMETER ::   k_visc =  1.5e-5       ! Kinematic viscosity estimate at ~20oc
   REAL(wp), PUBLIC, PARAMETER ::   am = 0.11              ! Roughness adjustment coefficient (Beljaars qjrms 1994)

   REAL(sp), PARAMETER, PUBLIC :: VAMIN = 1.0e-4           !
   LOGICAL,  PARAMETER, PUBLIC :: WATER_TDIAGLIM = .false. !
   REAL(sp), PARAMETER, PUBLIC :: ht=1.5, hu=10.

   !!----------------------------------------------------------------------
   !!           Namelist for the Ocean Surface Boundary Condition (RPN)
   !!----------------------------------------------------------------------

   REAL(wp), PUBLIC, SAVE      ::   Z0ICE  = 5.e-4         ! Ice roughness length
   REAL(wp), PUBLIC, SAVE      ::   ZALEV_t = 40           ! Approximative atm. forcing height (m)
   REAL(wp), PUBLIC, SAVE      ::   ZALEV_m = 40           ! Approximative atm. forcing height (m)
                                        !This code with ZALEV_t /= ZALEV_m has never been tested
                                        !Only tested with ZALEV_t=ZALEV_m=40
                                        !CAREFUL!!!!
   REAL(wp), PUBLIC, SAVE      ::   qsalfac = 0.98         ! Factor for surface specific humidity
                                                           ! related to the effect of salinity

   REAL(sp), PUBLIC, SAVE      ::   beta_prandtl  = 1.0  ! Prandtl number for neutral profile
   REAL(sp), PUBLIC, SAVE      ::   sl_rineutral  = 0.0  ! Define bulk Ri values for near-neutral regime in the surface layer

   REAL(wp), PUBLIC, SAVE      ::   emis_sea = 0.984         ! Sea surface emissivity (Konda et al.; J. of Oceanography 1994)

   LOGICAL, PUBLIC, SAVE       ::   diusst_coolskin  = .FALSE.
                                        !: logical flag to activate cool skin parameterization
   LOGICAL, PUBLIC, SAVE       ::   diusst_warmlayer = .FALSE.
                                        !: logical flag to activate warm layer parameterization (not used with
                                        !  coupled model)

   LOGICAL, PUBLIC, SAVE       ::  sl_z0ref = .false.           !  Use a reference roughness for surface layer calculations

   REAL(sp), PUBLIC, SAVE,  &
             DIMENSION(2)      ::   Z0TLAT = (/25.,30./) 
                                        ! degrees; Used for roughness length treatment

   LOGICAL, PUBLIC, SAVE       ::   ln_rpnblk_theo = .FALSE. 
                                        !: multiplication factor for precipitation
   LOGICAL, PUBLIC, SAVE       ::   ln_rpnblk_core = .TRUE.     
                                        !: logical flag to use TURB_CORE
   LOGICAL, PUBLIC, SAVE       ::   ln_rpnblk_rphy = .FALSE.      
                                        !: logical flag to use flxsurf4 (RPN Physics)
   LOGICAL, PUBLIC, SAVE       ::   ln_rpncpl = .false. , ln_rpncmx = .false.
                                        ! Activate coupling with RPN atmospheric model
   character(LEN=16), &
            PUBLIC, SAVE       ::   z0mtype='beljaars', z0ttype='DEACU12'
   CHARACTER(LEN=16), PUBLIC, SAVE       ::   z0mtypeU, z0ttypeU

   LOGICAL, PUBLIC, SAVE       ::   ln_cdinc_mask = .false. ! Increase Cd box activation
   REAL(wp), PUBLIC,SAVE       ::   rn_cdinc = 1._wp       ! Increase Cd factor

!# Taken from GEMDM sfc_options.F90
!# Z0MTYPE:  Momentum roughness length formulation over water
!# * 'CHARNOCK' : #TODO: define
!# * 'BELJAARS' : #TODO: define
!# Z0TTYPE: Thermal roughness length formulation over water
!# * 'MOMENTUM' : Uses z0h = z0m (replaces key z0trdps300=.false.)
!# * 'DEACU12'  : #TODO: define  (replaces key z0trdps300=.true.)
!# * 'ECMWF'    : #TODO: define  (New formulation used by ECMWF)

!# M-O stability functions formulation available in GEM (sfclayer_mod)
!#
!  stable case             |    unstable case 
!#-------------------------------------------------------------------
!# DELAGE97     (default)  |    DELAGE92     (default)
!# LOCK07                  |    DYER74
!# BELJAARS91              |

 CHARACTER(LEN=16), PUBLIC, SAVE   ::   sl_func_unstab='DELAGE92', sl_func_stab='DELAGE97' 

   !!----------------------------------------------------------------------
   !!              Ocean Surface Boundary Condition fields
   !!----------------------------------------------------------------------
   !!                                                     !! ocean   !! ice 
   REAL(wp), PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:)   :: alb_o_rpn            !: albedo
   REAL(wp), PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:)   :: t4a_o_rpn            !: radiative temperature [K**4]
   REAL(wp), PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:)   :: qsb_o_rpn            !: sensible heat flux [W/m2]
   REAL(wp), PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:)   :: qla_o_rpn            !: latent heat flux [W/m2]
   REAL(wp), PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:)   :: qlw_o_rpn            !: upward long wave [W/m2]
   REAL(wp), PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:)   :: uta_o_rpn            !: u wind stress [N/m2]
   REAL(wp), PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:)   :: vta_o_rpn            !: v wind stress [N/m2]
   REAL(wp), PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:)   :: to2m_rpn,  ti2m_rpn  !: diagnostic potential temperature at 1,5 m [K] 
   REAL(wp), PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:)   :: qo2m_rpn,  qi2m_rpn  !: diagnostic specific humidity at 1,5 m [K] 
   REAL(wp), PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:)   :: uo10_rpn,  ui10_rpn  !: diagnostic u-wind at 10 m [m/s] 
   REAL(wp), PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:)   :: vo10_rpn,  vi10_rpn  !: diagnostic v-wind at 10 m [m/s] 
   REAL(wp), PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:)   :: rhor_rpn             !: air density (km/m3)
   REAL(wp), PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:)   :: patm_rpn             !: atm pressure (N/m2)
   REAL(wp), PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:)   :: ustr_rpn             !: friction velocity U* (m/s)
   REAL(wp), PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:)   :: qsu_o_rpn, qsu_i_rpn !: surf. specific humidity (kg/kg)
   REAL(wp), PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:)   :: ilm_o_rpn, ilm_i_rpn !: 1/L (L=Monin-Obukhov Length)
   REAL(wp), PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:)   :: z0m_o_rpn, z0m_i_rpn !: Roughness length over ice (momentum) [m]
   REAL(wp), PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:)   :: z0h_o_rpn, z0h_i_rpn !: Roughness length over ice (heat and moisture) [m]
   REAL(wp), PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:)   ::            fr0_i_rpn !: ice fraction if coupled with CICE (begining of time step)
   REAL(wp), PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:)   ::            cat_i_rpn !: potential temperature (K)
   REAL(wp), PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:)   ::            zlm_i_rpn !: atmos momentum level height (m)
   REAL(wp), PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:)   ::            zlt_i_rpn !: atmos thermo   level height (m)
   REAL(wp), PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:)   ::            t4a_i_rpn !: radiative temperature [K**4]
   REAL(wp), PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:)   ::            qsb_i_rpn !: sensible heat flux [W/m2]
   REAL(wp), PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:)   ::            qlw_i_rpn !: upward long wave [W/m2]

   REAL(wp), PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:)   :: formlsr_rpn          !: corresponding GEM BETA  when using
                                                                                 !: combined Smith et al 1992 and Rascle 2008
                                                                                 !: z0m option
   !!----------------------------------------------------------------------
   !! Read the namsbc_rpn here for both sbcblk_rpn.F90 and sbcblk_rpnc.F90
   !!----------------------------------------------------------------------

   NAMELIST/namsbc_rpn/ ln_rpnblk_theo, ln_rpnblk_core, ln_rpnblk_rphy,               &
                     Z0ICE, ZALEV_t, ZALEV_m, z0mtype, z0ttype, Z0TLAT, qsalfac,      &
                     diusst_coolskin,diusst_warmlayer,                                &
                     sl_z0ref,beta_prandtl,sl_rineutral,sl_func_stab,sl_func_unstab,  &
                     ln_cdinc_mask, rn_cdinc

   PUBLIC :: namsbc_rpn !: All items in namelist

   !! * Substitutions
#  include "vectopt_loop_substitute.h90"
   !!----------------------------------------------------------------------
   !! NEMO/OPA 4.0 , NEMO Consortium (2011)
   !! $Id: sbc_rpn.F90 4990 2014-12-15 16:42:49Z timgraham $
   !! Software governed by the CeCILL licence     (NEMOGCM/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------
CONTAINS

   INTEGER FUNCTION sbc_rpn_alloc()
      !!---------------------------------------------------------------------
      !!                  ***  FUNCTION sbc_rpn_alloc  ***
      !!---------------------------------------------------------------------
      INTEGER :: ierr(5)
      !!---------------------------------------------------------------------
      ierr(:) = 0
      !
      ALLOCATE( alb_o_rpn(jpi,jpj) ,                                          &
         &      t4a_o_rpn(jpi,jpj) , STAT=ierr(1) ) 
         !
      ALLOCATE( qsb_o_rpn(jpi,jpj) , qlw_o_rpn(jpi,jpj), &
         &      qla_o_rpn(jpi,jpj) , qsu_o_rpn(jpi,jpj), &
                ilm_o_rpn(jpi,jpj) , z0m_o_rpn(jpi,jpj), &
                z0h_o_rpn(jpi,jpj) , formlsr_rpn(jpi,jpj), STAT=ierr(2) ) 
         !
      ALLOCATE( uta_o_rpn(jpi,jpj) , to2m_rpn(jpi,jpj) , qo2m_rpn(jpi,jpj) ,  &
         &      vta_o_rpn(jpi,jpj) , ti2m_rpn(jpi,jpj) , qi2m_rpn(jpi,jpj) , STAT=ierr(3) ) 
         !
      ALLOCATE( uo10_rpn(jpi,jpj) ,  ui10_rpn(jpi,jpj) , ustr_rpn(jpi,jpj) ,  &
         &      vo10_rpn(jpi,jpj) ,  vi10_rpn(jpi,jpj) , rhor_rpn(jpi,jpj) ,  &
         &      STAT=ierr(4) ) 
         !
#if defined key_cice
      ALLOCATE( fr0_i_rpn(jpi,jpj),   cat_i_rpn(jpi,jpj) ,     &
                zlm_i_rpn(jpi,jpj),   zlt_i_rpn(jpi,jpj) ,     &
                t4a_i_rpn(jpi,jpj),   qsb_i_rpn(jpi,jpj) ,     &
                qlw_i_rpn(jpi,jpj),   qsu_i_rpn(jpi,jpj) ,     &
                ilm_i_rpn(jpi,jpj) ,  z0m_i_rpn(jpi,jpj) ,     &
                z0h_i_rpn(jpi,jpj) ,  STAT=ierr(5) )
#endif
      sbc_rpn_alloc = MAXVAL( ierr )
      IF( lk_mpp            )   CALL mpp_sum ( sbc_rpn_alloc )
      IF( sbc_rpn_alloc > 0 )   CALL ctl_warn('sbc_rpn_alloc: allocation of arrays failed')
      !
   END FUNCTION sbc_rpn_alloc

#else
   !!----------------------------------------------------------------------
   !!   Dummy module                   NO sbc_rpn module
   !!----------------------------------------------------------------------
#endif

   !!======================================================================
END MODULE sbc_rpn
