MODULE sbc_rpn_skin 
USE par_kind
USE sbc_rpn
USE sfclayer_mod
USE phycst, only: stefan,grav 

   IMPLICIT NONE
   PRIVATE

!  
!   real, parameter :: am = 0.11        !Roughness adjustment coefficient (Beljaars qjrms 1994)
!   real, parameter :: k_visc = 1.5e-5  !Kinematic viscosity estimate at ~20oc
   real, parameter :: rho_w = 1025.    !Density of seawater (Rudnick and Martin; Dyn. of Atm. and Oceans, 2002)
   real, parameter :: diff_w = 1.45e-7 !Thermal moelcular diffusivity of seawater (National Physical Laboratory tables)
   real, parameter :: cond_w = 0.58    !Thermal conducitivity of seawater (National Physical Laboratory tables)
   real, parameter :: cv_w = 3950.     !Volumetric heat capacity of seawater (National Physical Laboratory tables)
   real, parameter :: nu = 0.3         !Warm layer profile shape parameter (1 = linear)
   real, parameter :: base_depth = 3.  !Depth (m) of the "base" temperature without diurnal variability
   real, parameter :: skin_depth_min = 0.0002 !Minimum depth of the cold skin (Tu and Tsuang; GRL 2005)
   real, parameter :: ref_depth = 10.  !Reference depth (m) for the Artale et al. (JGR 2002) forumulation of Saunders' constant
   real, parameter :: emis = 0.984     !Sea surface emissivity (Konda et al.; J. of Oceanography 1994)
   character(len = *), parameter :: saunders = 'artale' !Calculation strategy for Saunders' constant ('fairall','artale')
   real, parameter :: dsst_epsilon = 1.e-5 !Convergence threshold for warm layer increment
   real, parameter :: delh_tresh = 1.e-10  !Treshold to compute ctu in ocean coupled mode
   real, parameter :: zt_rho = 1.5     !Height used to compute air density in ocean coupled mode
   integer, parameter :: max_iter = 10 !Maximum number of iterations for convergence of warm layer increment
   real, parameter :: tcdk = 0.2731500000000e+03   ! passage k a c         C

   PUBLIC  cool_skin_warm_layer

!----------------------------------------------------------------------------------------------------

!  External arrays (atmospheric forcings) to be passed as arguments to subroutine 
!
!  New ocean historical (permanent) arrays to declare in sbc_oce
!   sst,zdsst,zskin_inc,zskin_depth

!# Initialize zskin_depth to 0 (not done in GEM ???) 

!   alb_oce(:,:) --> 
!   fcor(:,:) --> wfcor(:)
!   qlw(:,:) --> wqlw(:)
!   qsr(:,:) --> wqsr(:) 
!   qla(:,:) --> fv_wat
!   qsb(:,:) --> fc_wat
!   zst(:,:) --> ts      from  sst_m, ssu_m, ssv_m
!   sf(jp_prsg)%fnow(:,:) --> ps
!   sf(jp_hght)%fnow(:,:) --> zztsl
!   sf(jp_hghm)%fnow(:,:) --> zzusl
!   gphit(:,:) --> zdlat
!   z0m(:,:) --> 
!   z0h(:,:) -->
!   sf(jp_tair)%fnow(:,:)
!   sf(jp_humi)%fnow(:,:)
!   wndm(:,:) (relative wind module) -->  
!   zwnd_i(:,:) (relative u wind component)  --> 
!   zwnd_j(:,:) (relative v wind component)  --> 
!   zqsatw(:,:)
 

!   ts(:,:) --> sst(:)                 
!   skin_depth(:,:) --> skin_depth(:)
!   skin_inc(:,:) --> skin_inc(:) 
!   warm_inc(:,:) --> zdsst(:)  
 
! Mandatory inputs to the cool_skin/warm_layer scheme:

! uu,vv,tt,hu, zzuzl,zztsl  (1st dyn and thermo level)
! qs,sst,ps, z0m, z0h,zdlat,zfcor
! frv_a (ustar) and 10m windspeed (from sl_sfclayer call) 
! fv_wat, fc_wat, zfdsi,alvis_wat,zflusolis

!   sst,zdsst,zskin_inc,zskin_depth

! Physical constants needed: phy_cst or sbc_const
!
! tcdk (273.15 conversion from C to K) given by tdpack used by sfclayer_mod)
! grav (tdpack)
! stefan
! karman (from sbcblk_const 0.35 different from GEMDM 0.4)

! delt  (from dom_oce)         

! 1- Declare new NEMO historical fields in sbc_oce
! 2- Call to cool_skin_warm_layer() in itf_flxsurf2 (itf_sl_sfclayer ?)
!    with additional args. Re-use some transform 2D-->1D fields 
!    BUT modifies interface ???
! 3- Add namelist options in sbcblk_itfc and namsbc_itfc (cpl_RPN and
! 4- Feedback on NEMO SST field
! 5- Output new historical fields (skin_inc,skin_depth,dsst) for diagnostics

contains

   subroutine cool_skin_warm_layer(th,hu,spd,uu,vv,ps,zzusl,zztsl,ts,qs,fri,z0m,z0h,zdlat,zfcor,fc_wat,fv_wat, &
                                   alvis_wat,zflusolis,zfdsi,sst,zdsst,zskin_inc,zskin_depth,delt, &
                                   ln_init_zsst,jpi,jpj)
! Arguments
    REAL(sp), INTENT(IN), DIMENSION(:) ::  &
         th,           &    ! Potential temperature at first model level        
         hu,           &    ! Specific humidity at first model level        
         spd,         &    ! (Relative) air speed at first model level 
         uu,           &    ! (Relative) X-wiynd component at first model level 
         vv,           &    ! (Relative) Y-wind component at first model level 
         ps,           &    ! Sea-level pressure
         zzusl,        &    ! Height of first model momentum level
         zztsl,        &    ! Height of first model thermo level
         ts,           &    ! Sea surface temperature (bulk 0.5m layer)
         qs,           &    ! Saturation specific humidity at the surface
         fri,          &    ! Ice fraction
         z0m,          &    ! Momentum roughness length
         z0h,          &    ! Thermo roughness length (T,q)
         zdlat,        &    ! Latitude in radians
         zfcor,        &    ! Coriolis factor
         fc_wat,       &    ! Snesible heat flux at surface
         fv_wat,       &    ! Latent heat flux at surface
         alvis_wat,    &    ! Visible albedo of ocean
         zflusolis,    &    ! Incoming visible radiation at surface
         zfdsi              ! Incoming infrared radiation at surface

    REAL(sp), INTENT(INOUT), DIMENSION(:) ::  &
         sst,          &    ! Ocean Skin Temperature
         zdsst,        &    ! Diurnal warm layer temperature increment at surface 
         zskin_inc,    &    ! Cool skin temperature increment
         zskin_depth        ! Cool skin depth

    REAL(wp), INTENT(IN) :: delt
    LOGICAL, INTENT(IN):: ln_init_zsst
    INTEGER, INTENT(IN):: jpi,jpj

! Local work arrays

!   REAL(sp), DIMENSION(jpi*jpj) :: z0m_adjust,tva,rhoa,vmod,vdir,vmodd
!   REAL(sp), DIMENSION(jpi*jpj) :: alpha_w,rho_a,frv_w,frv_a,visc_w,skin_solar,q_bal,skin_q
!   REAL(sp), DIMENSION(jpi*jpj) :: warm_increment,stab_func,lambda,ud,vdi,gamma
!   REAL(sp), DIMENSION(jpi*jpj) :: this_inc,prev_inc,denom
 
   REAL(sp), DIMENSION(jpi*jpj) :: tva,rhoa,vmod,vdir,vmodd
   REAL(sp), DIMENSION(jpi*jpj) :: alpha_w,rho_a,frv_w,frv_a,visc_w,skin_solar,q_bal,skin_q
   REAL(sp), DIMENSION(jpi*jpj) :: warm_increment,stab_func,lambda,ud,vdi,gamma
   REAL, DIMENSION(jpi*jpj) :: this_inc,prev_inc,denom


   integer I
!---------------------------------------------------------------------------------------------------------------

   if ( ln_init_zsst ) then
     zdsst=0.0
     zskin_inc=0.0
     zskin_depth=0.0
   endif

   !** ------------------------------------------------------------------

 ! 0.     Precompute local SST from "base" SST (ts) and diurnal components
   ! -----------------------------------------------------------------------
   sst = ts + zdsst + zskin_inc   ! add diurnal warm-layer and cool skin increments


   ! 5.     Prognostic evolution of SST (based on Zeng and Beljaars; GRL 2005)
   ! -------------------------------------------------------------------------

!WARNING: Need tt to compute air density.We only have th (potential temperature). Use th for now 
      ! Preliminary calculations
!JMB      i = sl_prelim(tt,hu,uu,vv,ps,zzusl,rho_air=rho_a,spd_air=vmod,dir_air=vdir,min_wind_speed=VAMIN)
      i = sl_prelim(th,hu,uu,vv,ps,zzusl,rho_air=rho_a,spd_air=vmod,dir_air=vdir,min_wind_speed=VAMIN)
      i = sl_sfclayer(th,hu,vmod,vdir,zzusl,zztsl,sst,qs,z0m,z0h,zdlat,zfcor,hghtm_diag=10.0_sp, &
           ue=frv_a,u_diag=ud,v_diag=vdi,tdiaglim=WATER_TDIAGLIM)

      vmodd = sqrt(ud**2 + vdi**2)
!      alpha_w = 8.75e-6 * (sst-tcdk+9.) !expansion coefficient estimate from Hayes et al. (JGR, 1991)
      alpha_w = max(8.75e-6*(sst-tcdk+9.),0.)
      visc_w = 1.8e-6 - 4.e-8*(sst-tcdk) !kinematic viscosity (from IITC 2011 recommendations at 10oC)
      frv_w = frv_a * sqrt(rho_a/rho_w) !friction velocity of water
      q_bal = -(fv_wat + fc_wat + (emis*stefan*sst**4 - zfdsi)) !surface energy balance (without shortwave)

      ! Compute cool skin properties on request
      COOL_SKIN: if ( diusst_coolskin ) then

         ! Estimate the depth and cooling effect of the cold skin layer (typically 1mm-1cm and 0.3K)
         ! Solar radiation absorbed in the skin layer
         where (zskin_depth < skin_depth_min)
            skin_solar = 0.1 !estimate proposed by Fairall et al. (JGR 1996) section 2.4
         elsewhere
            skin_solar = 0.067 + 11.*zskin_depth - (6.6e-5/zskin_depth) * (1.-exp(-zskin_depth/8.e-4))
         endwhere
         ! Updated skin layer depth (cold skin only exists when net heat is being extracted from the surface)
         skin_q = q_bal + (1.-alvis_wat)*zflusolis*skin_solar
         ! Compute Saunders' constant of proportionality (lambda)
         if (saunders == 'fairall') then !follow Fariall et al. (GRL; 1996)
            where (skin_q <= 0.)
               lambda = 6.*(1. + ((-16.*grav*alpha_w*visc_w**3)/(frv_w**4*diff_w**2*rho_w*cv_w) * (skin_q))**0.75)**(-1./3.)
            elsewhere
               lambda = 6.
            endwhere
         elseif (saunders == 'artale') then !follow Artale et al. (JGR; 2002)
            where (vmodd > 10.)
               gamma = 6.
            elsewhere (vmodd > 7.5)
               gamma = 1.6*vmodd - 10.
            elsewhere
               gamma = 0.2*vmodd + 0.5
            endwhere
            lambda = 86400.*frv_w*cond_w/(gamma*rho_w*cv_w*ref_depth*visc_w)
         else
            print*, 'No Saunders proportionality constants defined for '//trim(saunders)//' in subroutine water()'
            stop
         endif
         zskin_depth = lambda * visc_w / (sqrt(rho_a/rho_w)*frv_a)
         zskin_depth = max(min(zskin_depth,0.01),skin_depth_min) !limit skin depth to physical values
         ! Compute cool skin temperature increment (difference SST - T(zskin_depth))
         zskin_inc = zskin_depth / cond_w * (skin_q)
         if (saunders == 'fairall') zskin_inc = min(zskin_inc,0.)
         where ( fri > 0. ) zskin_inc = 0.

      endif COOL_SKIN

      ! Compute warm layer properties on request
      WARM_LAYER: if ( diusst_warmlayer ) then

         ! Estimate the evolving temperature departure in the warm layer implicitly (secant iterations)
         this_inc = zdsst ; prev_inc = zdsst+10.*dsst_epsilon ; i = 0
         do while (maxval(abs(this_inc-prev_inc)) > dsst_epsilon .and. i < max_iter)
            i = i+1
            where (abs(this_inc-prev_inc) > dsst_epsilon)
               denom = warm_func(this_inc)-warm_func(prev_inc)
               where (abs(denom) > 1.e-8)
                  ! Use local slope to estimate new increment value
                  warm_increment = this_inc - warm_func(this_inc)*((this_inc-prev_inc)/denom)
                  prev_inc = this_inc
                  this_inc = warm_increment
               elsewhere
                  ! Attempt a reset of the increment difference if the function is locally flat
                  warm_increment = this_inc
                  prev_inc = this_inc+10.*dsst_epsilon
               endwhere
            endwhere
         enddo
         zdsst = warm_increment

      endif WARM_LAYER

   return
   
   contains 

   ! Warm layer function for secant iterations
   function warm_func(inc) result(fval)
      ! Argument declarations
      real, dimension(:), intent(in) :: inc             !warm layer increment
      real, dimension(size(inc)) :: fval                !warm layer function result
      ! Local variables
      real, dimension(size(inc)) :: base_bflux,mo_len,stab_func
      ! Compute buoyancy flux at base depth
      where (inc > 0.)
         base_bflux = sqrt(nu*grav*alpha_w/(5.*base_depth)) * rho_w*cv_w*frv_w**2*sqrt(inc)
      elsewhere
         base_bflux = grav*alpha_w*(q_bal+0.64*zflusolis)
      endwhere
      base_bflux = sign(max(abs(base_bflux),1.e-4),base_bflux)
      mo_len = rho_w*cv_w*frv_w**3/(karman*base_bflux) !Monin-Obukhov length in the warm layer
      mo_len = sign(max(abs(mo_len),1.e-4),mo_len)
      ! Compute stability function at base depth
      where (mo_len > 0.)
         stab_func = 1. + 5.*(base_depth/mo_len)
      elsewhere
         stab_func = (1. - 16.*(base_depth/mo_len))**(-0.5)
      endwhere
      ! Compute warm layer function value
      fval = inc - delt*((q_bal + 0.64*zflusolis) / (base_depth*rho_w*cv_w*nu/(nu+1)) - &
           (nu+1)*karman*frv_w/(base_depth*stab_func) * inc) - zdsst
   end function warm_func


   end subroutine cool_skin_warm_layer
 


END MODULE sbc_rpn_skin
