MODULE sbc_rpnflx
#if defined key_rpne
   !!======================================================================
   !!                       ***  MODULE  sbc_flxsurf  ***
   !! Flux formulation:  Interface to RPN-Physics routine for
   !!                    calculating fluxes and diagnostic variables
   !!=====================================================================
   !! History :  1.0   !  05-12  - F. Roy
   !!----------------------------------------------------------------------

   USE par_kind
   USE sfclayer_mod
   USE sfc_options, only: sfc_options_init
   USE dom_oce, only: rdt
   USE sbc_rpn_skin
   USE sbc_rpn
   
   USE lib_mpp, only: ctl_stop

   IMPLICIT NONE
   PRIVATE

   PUBLIC   ITF_FLXSURF2, ITF_SFCLAYER_GEMDM

CONTAINS

   SUBROUTINE ITF_SFCLAYER_GEMDM( numout, jpi,jpj    , fcor,       &
         &                  zt,   zu,    tg   , ta,                &
         &                  qg,   qa   , ps, dU,   uw,  vw,  rhor, lat,      &
         &                  fri,                                   &
         &                  z0m,  z0h  ,                           &
         &                  alvis_wat, zflusolis, zfdsi,           &
         &                  sst,zdsst,zskin_inc,zskin_depth,       &
         &                  Cd    , Ch              , Ce,          &
         &                  ustr  ,                                &
         &                  t2m  ,                                 &
         &                  q2m  ,                                 &
         &                  u10  , v10, ilm, ln_init_zsst )

      !!----------------------------------------------------------------------
      !!                      ***  ROUTINE ITF_SFCLAYER_GEMDM ***
      !!
      !! ** Purpose :   Computes turbulent transfert coefficients of surface
      !!                fluxes according to RPN-PHYSICS
      !!
      !! ** Method  :   call sfclayer_mod.F90 routines
      !!
      !! References :
      !!      RPN
      !! History :
      !!      !  01-16  F. Roy - Initial version
      !!----------------------------------------------------------------------
      !! * Arguments
      INTEGER,  INTENT(in) :: numout,jpi,jpj
      LOGICAL,  INTENT(in) :: ln_init_zsst
      REAL(wp), INTENT(in),  DIMENSION(jpi,jpj) :: &
         fcor,  &         ! Coriolis term
         zt,    &         ! Height of T and q (thermo level)
         zu,    &         ! Height of dU      (momentum level)
         tg,    &         ! Ground temperature (e.g. sst)
         ta,    &         ! Potential temperature at zt
         qg,    &         ! Ground specific humidity
         qa,    &         ! Specific humidity at zt
         ps,    &         ! Sea-surface pressure
         dU,    &         ! Relative total wind intensity at zu (Masked)
                          ! dU to be taken out if First option (see below)
         uw,    &         ! Relative wind intensity at zu (i-direction)
         vw,    &         ! Relative wind intensity at zu (j-direction)
         rhor,  &         ! Air density at surface
         lat,   &         ! Latitude in degrees
         fri,   &         ! Ice fraction
         z0m,   &         ! Roughness length for momentum
         z0h,   &         ! Roughness length for T and q
         alvis_wat, &     ! ocean visible albedo
         zflusolis, &     ! surface incoming solar radiation
         zfdsi            ! surface incoming longwave radiation 

     REAL(wp), INTENT(inout), DIMENSION(jpi,jpj) :: &
         sst,       &     ! Ocean skin T 
         zdsst,     &     ! Diurnal warm layer increment
         zskin_inc, &     ! Cool skin temperature increment
         zskin_depth      ! Cool skin depth

      REAL(wp), INTENT(out), DIMENSION(jpi,jpj) :: &
         Cd,    &         ! Momentum exchange coefficient
         Ch,    &         ! Sensible heat exchange coefficient
         Ce,    &         ! Latent   heat exchange coefficient
         ustr,  &         ! U*
         t2m,   &         ! Potential temperature at diagnostic level
         q2m,   &         ! Specific humidity     at diagnostic level
         u10,   &         ! Wind (i-direction)    at diagnostic level
         v10,   &         ! Wind (j-direction)    at diagnostic level
         ilm              ! 1/L  (ilmo)

      !! * Work variables

      ! ... outputs from n_flxsurf4 and n_diasurf2

      REAL(sp), DIMENSION(jpi*jpj) ::  &
         wcmu,      &     ! See sfclayer_mod.F90 from RPN-Physics
         wctu,      &     
         wilmo,     &     
         wue,       &     
         wuz,       &
         wvz,       &
         wtz,       &
         wqz

      ! ... inputs for sfclayer_mod.F90

      REAL(sp), DIMENSION(jpi*jpj) ::  &
         wfcor,     &     ! See sfclayer_mod.F90 from RPN-Physics
         wzu,       &
         wzt,       &
         wta,       &
         wqa,       &
         wva,       &
         wvd,       &
         wtg,       &
         wqg,       &
         wfr,       &
         wz0,       &
         wz0h,      &
         wla
     REAL(sp),   DIMENSION(jpi*jpj) :: &
         sst1d,        &  ! Ocean skin T 
         zdsst1d,      &  ! Diurnal warm layer increment
         zskin_inc1d,  &  ! Cool skin temperature increment
         zskin_depth1d,&  ! Cool skin depth
         alvis_wat1d,  &  ! Ocean visible albedo
         zflusolis1d,  &  ! Surface incoming solar radiation
         zfdsi1d,      &  ! Surface incoming longwave radiation 
         ps1d,         &  ! Sea-level pressure
         fc_wat1d,     &  ! Sensible heat-flux
         fv_wat1d,     &  ! Latent heat-flux
         rhor1d,       &  ! Latent heat-flux
         Cd1d,         &  ! Turbulent transfer coefficient for momentum at surface
         Ch1d,         &  ! Turbulent transfer coefficient for latent/sensible heat at surface
         zqsatw1d         ! Surface specific humidity at saturation

      ! ... others
      REAL(wp), PARAMETER :: rad = 3.141592653589793_wp / 180._wp
      REAL(wp) ::   zcoef_qsatw
      INTEGER :: i, istat
      LOGICAL, SAVE :: first=.true.
!      REAL(sp), PARAMETER :: VAMIN = 1.0e-4           !Copied from GEM 4.8.rc3
!      LOGICAL,  PARAMETER :: WATER_TDIAGLIM = .false. !Copied from GEM 4.8.rc3
!      REAL(sp), PARAMETER :: ht=1.5, hu=10.

      if (first) then
         i = sfc_options_init()
!JMB  Initialization done in sfc_nml_post_init() (GEMDM:sfc_nml.F90)
!     Add these variable to the namelist namsbc_itfc 
      istat = sl_put('beta', beta_prandtl)
      istat = sl_put('rineutral',sl_rineutral)
      istat = sl_put('tdiaglim',WATER_TDIAGLIM)
      istat = sl_put('sl_stabfunc_stab',sl_func_stab)
      istat = sl_put('sl_stabfunc_unstab',sl_func_unstab)
      istat = sl_put('z0ref',sl_z0ref)

!          call validate(i) .... ?????
         first=.false.
      endif
      ! ... prepare inputs

      call two2one(wfcor(:) , fcor(:,:) )
      call two2one(wzu  (:) , zu  (:,:) )
      call two2one(wzt  (:) , zt  (:,:) )
      call two2one(wta  (:) , ta  (:,:) )
      call two2one(wqa  (:) , qa  (:,:) )
      call two2one(wva  (:) , dU  (:,:) )
      wva  (:) = max(wva(:),VAMIN)
      call two2one(wtg  (:) , tg  (:,:) )
      call two2one(wqg  (:) , qg  (:,:) )
      call two2one(wfr  (:) , fri (:,:) )
      call two2one(wz0  (:) , z0m (:,:) )
      call two2one(wz0h (:) , z0h (:,:) )
      call two2one(wuz  (:) , uw  (:,:) )
      call two2one(wvz  (:) , vw  (:,:) )
      call two2one(wla  (:) , lat (:,:) )
      wla  (:) = wla (:) * rad
      call two2one(alvis_wat1d(:) ,     alvis_wat(:,:) ) 
      call two2one(sst1d  (:) ,         sst(:,:) ) 
      call two2one(zdsst1d  (:) ,       zdsst(:,:) ) 
      call two2one(zskin_inc1d  (:) ,   zskin_inc(:,:) ) 
      call two2one(zskin_depth1d  (:) , zskin_depth(:,:) ) 
      call two2one(zflusolis1d  (:) ,   zflusolis(:,:) ) 
      call two2one(zfdsi1d  (:) ,       zfdsi(:,:) ) 
      call two2one(ps1d  (:) ,       ps(:,:) ) 
      call two2one(rhor1d  (:) ,       rhor(:,:) ) 

!     In GEM:
!     i = sl_prelim(tt,hu,uu,vv,ps,zzusl,rho_air=rho_a,spd_air=vmod,dir_air=vdir,min_wind_speed=VAMIN)

!     First option:
!     i = sl_prelim(wta,wqa,wuz,wvz,wta,wzu,spd_air=wva,dir_air=wvd,min_wind_speed=VAMIN)
!     wva was delt with before in sbcblk_itfc.F90
!     WARNING:Need to apply mask to wva as before for dU if done locally

!     Second option: only dir_air (for computing UU,VV,HU,TT at diagnostic levels)
!     wta and wqa is not used (tt, hu and ps) if not asking for rho_air
!     only wuz and wvz are used for computing spd_air and/or dir_air
      i = sl_prelim(wta,wqa,wuz,wvz,wta,wzu,dir_air=wvd)
      if (i /= SL_OK) then
         call ctl_stop('Aborting in ITF_FLXSURF2 because of error returned by sl_prelim')
      endif
      i = sl_sfclayer(wta,wqa,wva,wvd,wzu,wzt,sst1d,wqg,wz0,wz0h,wla,wfcor, &
           hghtm_diag=hu,hghtt_diag=ht,coefm=wcmu,coeft=wctu, &
           ilmo=wilmo,ue=wue,t_diag=wtz,    &
           q_diag=wqz,u_diag=wuz,v_diag=wvz,tdiaglim=WATER_TDIAGLIM)
      if (i /= SL_OK) then
         call ctl_stop('Aborting in ITF_FLXSURF2 because of error returned by sl_sfclayer')
      endif

! MUST convert rhor(:,:) to 1d

      Cd1d(:) = (wcmu(:)/wue(:))**2
      Ch1d(:) = (wctu(:)/wue(:))*(wcmu(:)/wue(:))
!CDIR COLLAPSE
      fc_wat1d(:)  =  CPD*rhor1d(:)*Ch1d(:)*(sst1d(:) - wta(:)) * wva(:)     ! Sensible Heat

      zcoef_qsatw = qsalfac * 640380.
!JMB      zqsatw1d(:) = zcoef_qsatw / rhor1d(:) * EXP( -5107.4 / sst1d(:) )
      zqsatw1d(:) = zcoef_qsatw / MAX(rhor1d(:),0.1) * EXP( -5107.4 / sst1d(:) )

!CDIR COLLAPSE
      fv_wat1d(:) = CHLC * (rhor1d(:)*Ch1d(:)*( zqsatw1d(:) - wqa(:) ) * wva(:) ) ! Latent Heat

!     In coupled mode diusst_warmlayer should be set to .false. since NEMO already implements 
!     a diurnal cycle.
      if (diusst_coolskin .or. diusst_warmlayer) &
          call cool_skin_warm_layer(wta,wqa,wva,wuz,wvz,ps1d,wzu,wzt,wtg,wqg,wfr,wz0,wz0h,wla,wfcor, &
                                    fc_wat1d,fv_wat1d,alvis_wat1d,zflusolis1d,zfdsi1d,               &
                                    sst1d,zdsst1d,zskin_inc1d,zskin_depth1d,rdt,ln_init_zsst,        &
                                    jpi,jpj)


      wcmu(:) = wcmu(:)/wue(:) !Now CM
      wctu(:) = wctu(:)/wue(:) !Now CT
      ! wta as a tmp variable
      wta(:) = wcmu(:)**2
      call one2two(Cd  (:,:) , wta(:) )
      ! wta as a tmp variable
      wta(:) = wctu(:)*wcmu(:)
      call one2two(Ch  (:,:) , wta(:) )
      Ce  (:,:) = Ch  (:,:)
      call one2two(ustr(:,:) , wue (:) )
      call one2two(ilm (:,:) , wilmo(:) )
      call one2two(t2m (:,:) , wtz(:) )
      call one2two(q2m (:,:) , wqz(:) )
      call one2two(u10 (:,:) , wuz(:) )
      call one2two(v10 (:,:) , wvz(:) )
!JMB
      call one2two( sst(:,:), sst1d(:) )
      call one2two( zdsst(:,:), zdsst1d(:) )
      call one2two( zskin_inc(:,:), zskin_inc1d(:) )
      call one2two( zskin_depth(:,:), zskin_depth1d(:) )


      CONTAINS

      subroutine two2one(tabou,tabin)
      real(wp), intent(in),  dimension(jpi,jpj) :: tabin
      real(sp), intent(out), dimension(jpi*jpj) :: tabou
      integer ji,jj
      do jj=1,jpj
      do ji=1,jpi
         tabou(ji+(jj-1)*jpi)=tabin(ji,jj)
      enddo
      enddo
      end subroutine two2one
      
      subroutine one2two(tabou,tabin)
      real(sp), intent(in),  dimension(jpi*jpj) :: tabin
      real(wp), intent(out), dimension(jpi,jpj) :: tabou
      integer ji,jj
      do jj=1,jpj
      do ji=1,jpi
         tabou(ji,jj)=tabin(ji+(jj-1)*jpi)
      enddo
      enddo
      end subroutine one2two

   END SUBROUTINE ITF_SFCLAYER_GEMDM

   SUBROUTINE ITF_FLXSURF2( numout, jpi,jpj    , fcor,             &
         &                  zt,   zu,    tg   , ta,                &
         &                  qg,   qa   , dU,   uw,  vw,  lat,      &
         &                  z0m,  z0h  ,                           &
         &                  Cd    , Ch              , Ce,          &
         &                  ustr  ,                                &
         &                  t2m  ,                                 &
         &                  q2m  ,                                 &
         &                  u10  , v10, ilm    )

      !!----------------------------------------------------------------------
      !!                      ***  ROUTINE  ITF_FLXSURF2  ***
      !!
      !! ** Purpose :   Computes turbulent transfert coefficients of surface
      !!                fluxes according to RPN-PHYSICS
      !!
      !! ** Method  :   call sfclayer_mod.F90 routines
      !!
      !! References :
      !!      RPN
      !! History :
      !!      !  01-16  F. Roy - Initial version
      !!----------------------------------------------------------------------
      !! * Arguments
      INTEGER,  INTENT(in) :: numout,jpi,jpj
      REAL(wp), INTENT(in),  DIMENSION(jpi,jpj) :: &
         fcor,  &         ! Coriolis term
         zt,    &         ! Height of T and q (thermo level)
         zu,    &         ! Height of dU      (momentum level)
         tg,    &         ! Ground temperature (e.g. sst)
         ta,    &         ! Potential temperature at zt
         qg,    &         ! Ground specific humidity
         qa,    &         ! Specific humidity at zt
         dU,    &         ! Relative total wind intensity at zu (Masked)
                          ! dU to be taken out if First option (see below)
         uw,    &         ! Relative wind intensity at zu (i-direction)
         vw,    &         ! Relative wind intensity at zu (j-direction)
         lat,   &         ! Latitude in degrees
         z0m,   &         ! Roughness length for momentum
         z0h              ! Roughness length for T and q

      REAL(wp), INTENT(out), DIMENSION(jpi,jpj) :: &
         Cd,    &         ! Momentum exchange coefficient
         Ch,    &         ! Sensible heat exchange coefficient
         Ce,    &         ! Latent   heat exchange coefficient
         ustr,  &         ! U*
         t2m,   &         ! Potential temperature at diagnostic level
         q2m,   &         ! Specific humidity     at diagnostic level
         u10,   &         ! Wind (i-direction)    at diagnostic level
         v10,   &         ! Wind (j-direction)    at diagnostic level
         ilm              ! 1/L  (ilmo)

      !! * Work variables

      ! ... outputs from n_flxsurf4 and n_diasurf2

      REAL(sp), DIMENSION(jpi*jpj) ::  &
         wcmu,      &     ! See sfclayer_mod.F90 from RPN-Physics
         wctu,      &     
         wilmo,     &     
         wue,       &     
         wuz,       &
         wvz,       &
         wtz,       &
         wqz

      ! ... inputs for sfclayer_mod.F90

      REAL(sp), DIMENSION(jpi*jpj) ::  &
         wfcor,     &     ! See sfclayer_mod.F90 from RPN-Physics
         wzu,       &
         wzt,       &
         wta,       &
         wqa,       &
         wva,       &
         wvd,       &
         wtg,       &
         wqg,       &
         wz0,       &
         wz0h,      &
         wla

      ! ... others
      REAL(wp), PARAMETER :: rad = 3.141592653589793_wp / 180._wp
      INTEGER i
      LOGICAL, SAVE :: first=.true.
      REAL(sp), PARAMETER :: VAMIN = 1.0e-4           !Copied from GEM 4.8.rc3
      LOGICAL,  PARAMETER :: WATER_TDIAGLIM = .false. !Copied from GEM 4.8.rc3
      REAL(sp), PARAMETER :: ht=1.5, hu=10.

      if (first) then
         i = sfc_options_init()
!          call validate(i) .... ?????
         first=.false.
      endif
      ! ... prepare inputs

      call two2one(wfcor(:) , fcor(:,:) )
      call two2one(wzu  (:) , zu  (:,:) )
      call two2one(wzt  (:) , zt  (:,:) )
      call two2one(wta  (:) , ta  (:,:) )
      call two2one(wqa  (:) , qa  (:,:) )
      call two2one(wva  (:) , dU  (:,:) )
      wva  (:) = max(wva(:),VAMIN)
      call two2one(wtg  (:) , tg  (:,:) )
      call two2one(wqg  (:) , qg  (:,:) )
      call two2one(wz0  (:) , z0m (:,:) )
      call two2one(wz0h (:) , z0h (:,:) )
      call two2one(wuz  (:) , uw  (:,:) )
      call two2one(wvz  (:) , vw  (:,:) )
      call two2one(wla  (:) , lat (:,:) )
      wla  (:) = wla (:) * rad
 
!     In GEM:
!     i = sl_prelim(tt,hu,uu,vv,ps,zzusl,rho_air=rho_a,spd_air=vmod,dir_air=vdir,min_wind_speed=VAMIN)

!     First option:
!     i = sl_prelim(wta,wqa,wuz,wvz,wta,wzu,spd_air=wva,dir_air=wvd,min_wind_speed=VAMIN)
!     wva was delt with before in sbcblk_itfc.F90
!     WARNING:Need to apply mask to wva as before for dU if done locally

!     Second option: only dir_air (for computing UU,VV,HU,TT at diagnostic levels)
!     wta and wqa is not used (tt, hu and ps) if not asking for rho_air
!     only wuz and wvz are used for computing spd_air and/or dir_air
      i = sl_prelim(wta,wqa,wuz,wvz,wta,wzu,dir_air=wvd)
      if (i /= SL_OK) then
         call ctl_stop('Aborting in ITF_FLXSURF2 because of error returned by sl_prelim')
      endif
      i = sl_sfclayer(wta,wqa,wva,wvd,wzu,wzt,wtg,wqg,wz0,wz0h,wla,wfcor, &
           hghtm_diag=hu,hghtt_diag=ht,coefm=wcmu,coeft=wctu, &
           ilmo=wilmo,ue=wue,t_diag=wtz,    &
           q_diag=wqz,u_diag=wuz,v_diag=wvz,tdiaglim=WATER_TDIAGLIM)
      if (i /= SL_OK) then
         call ctl_stop('Aborting in ITF_FLXSURF2 because of error returned by sl_sfclayer')
      endif

      wcmu(:) = wcmu(:)/wue(:) !Now CM
      wctu(:) = wctu(:)/wue(:) !Now CT
      ! wta as a tmp variable
      wta(:) = wcmu(:)**2
      call one2two(Cd  (:,:) , wta(:) )
      ! wta as a tmp variable
      wta(:) = wctu(:)*wcmu(:)
      call one2two(Ch  (:,:) , wta(:) )
      Ce  (:,:) = Ch  (:,:)
      call one2two(ustr(:,:) , wue (:) )
      call one2two(ilm (:,:) , wilmo(:) )
      call one2two(t2m (:,:) , wtz(:) )
      call one2two(q2m (:,:) , wqz(:) )
      call one2two(u10 (:,:) , wuz(:) )
      call one2two(v10 (:,:) , wvz(:) )

      CONTAINS

      subroutine two2one(tabou,tabin)
      real(wp), intent(in),  dimension(jpi,jpj) :: tabin
      real(sp), intent(out), dimension(jpi*jpj) :: tabou
      integer ji,jj
      do jj=1,jpj
      do ji=1,jpi
         tabou(ji+(jj-1)*jpi)=tabin(ji,jj)
      enddo
      enddo
      end subroutine two2one
      
      subroutine one2two(tabou,tabin)
      real(sp), intent(in),  dimension(jpi*jpj) :: tabin
      real(wp), intent(out), dimension(jpi,jpj) :: tabou
      integer ji,jj
      do jj=1,jpj
      do ji=1,jpi
         tabou(ji,jj)=tabin(ji+(jj-1)*jpi)
      enddo
      enddo
      end subroutine one2two

   END SUBROUTINE ITF_FLXSURF2

   !!======================================================================
#else

   !! Empty module

#endif

END MODULE sbc_rpnflx
