MODULE sbcblk_rpn

#if defined key_rpne

   !!======================================================================
   !!                       ***  MODULE  sbcblk_rpn  ***
   !! Ocean forcing:  momentum, heat and freshwater flux formulation
   !!=====================================================================
   !! History :  1.0   !  05-12  F. Roy original code based on sbcblk_core
   !!----------------------------------------------------------------------

   !!----------------------------------------------------------------------
   !!   sbc_blk_rpn     : bulk formulation as ocean surface boundary condition 
   !!                     (forced mode, CORE bulk formulea or RPN physics (rphy=.true.))
   !!   blk_oce_rpn     : computes momentum, heat and freshwater fluxes over ocean
   !!   blk_ice_rpn     : computes momentum, heat and freshwater fluxes over ice
   !!   turb_core_2z_rpn: Computes turbulent transfert coefficients (CORE RPN version)
   !!   itf_flxsurf2    : Computes turbulent transfert coefficients (RPN PHYSICS version)
   !!   cd_neutral_10m  : Estimate of the neutral drag coefficient at 10m   (idem as sbcblk_core.F90)
   !!   psi_m           : universal profile stability function for momentum (idem as sbcblk_core.F90)
   !!   psi_h           : universal profile stability function for temperature and humidity 
   !!                     (idem as sbcblk_core.F90)
   !!----------------------------------------------------------------------
   USE oce             ! ocean dynamics and tracers
   USE dom_oce         ! ocean space and time domain
   USE phycst          ! physical constants
   USE fldread         ! read input fields
   USE fldread_rpn
   USE sbc_oce         ! Surface boundary condition: ocean fields
   USE sbc_rpn         ! Surface boundary condition: ocean fields (RPN)
   USE cyclone         ! Cyclone 10m wind form trac of cyclone centres
   USE sbcdcy          ! surface boundary condition: diurnal cycle
   USE iom             ! I/O manager library
   USE in_out_manager  ! I/O manager
   USE lib_mpp         ! distribued memory computing library
   USE wrk_nemo        ! work arrays
   USE timing          ! Timing
   USE lbclnk          ! ocean lateral boundary conditions (or mpp link)
   USE prtctl          ! Print control
   USE sbcwave, ONLY   :  cdn_wave ! wave module
   USE sbc_ice         ! Surface boundary condition: ice fields
   USE lib_fortran     ! to use key_nosignedzero
   USE sbcblk_rpnc, only : cplatm_ptype, cplatm_rap_dt, cplatm_N_cpldta
   USE sbc_rpnflx
   USE sbcapr
   USE sbc_rpnstringutil


   IMPLICIT NONE
   PRIVATE

   PUBLIC   sbc_blk_rpn         ! routine called in sbcmod module
   PUBLIC   blk_ice_rpn         ! routine called in sbc_ice_lim module
   PUBLIC   turb_core_2z_rpn     ! routine calles in sbcblk_mfs module

   INTEGER , PARAMETER ::   jpfld   = 15          ! maximum number of files to read
   INTEGER , PARAMETER ::   jp_wndi = 1           ! index of 10m (or zzu) wind velocity (i-component) (m/s)    at T-point
   INTEGER , PARAMETER ::   jp_wndj = 2           ! index of 10m (or zzu) wind velocity (j-component) (m/s)    at T-point
   INTEGER , PARAMETER ::   jp_humi = 3           ! index of specific humidity               ( Kg/Kg )
   INTEGER , PARAMETER ::   jp_qsr  = 4           ! index of solar heat                      (W/m2)
   INTEGER , PARAMETER ::   jp_qlw  = 5           ! index of Long wave                       (W/m2)
   INTEGER , PARAMETER ::   jp_tair = 6           ! index of 10m (or zzu) air temperature             (Kelvin)
   INTEGER , PARAMETER ::   jp_prec = 7           ! index of total precipitation (rain+snow) (Kg/m2/s)
   INTEGER , PARAMETER ::   jp_snow = 8           ! index of snow (solid prcipitation)       (kg/m2/s)
   INTEGER , PARAMETER ::   jp_hght = 9           ! index of thermo   level (m)
   INTEGER , PARAMETER ::   jp_hghm = 10          ! index of momentum level (m)
   INTEGER , PARAMETER ::   jp_prst = 11          ! index of thermo   level pressure (Pa)
   INTEGER , PARAMETER ::   jp_prsm = 12          ! index of momentum level pressure (Pa)
   INTEGER , PARAMETER ::   jp_prsg = 13          ! index of sea (ground) level pressure (Pa)
   INTEGER , PARAMETER ::   jp_mcmx = 14          ! index of mixt coupling/forcing mask (forcing==>sf(jp_mcmx)=1)
   INTEGER , PARAMETER ::   jp_prsn = 15          ! index of PN (virtual ground pressure) (Pa)

   TYPE(FLD), ALLOCATABLE, DIMENSION(:) ::   sf   ! structure of input fields (file informations, fields read)
   TYPE(FLD), ALLOCATABLE, DIMENSION(:) ::   sfc  ! structure of input fields (file informations, fields read, coupling interface)

   !                                             !!! RPNE bulk parameters
   REAL(wp), PARAMETER  ::   albo =    0.066      ! ocean albedo assumed to be constant
   REAL(wp), PARAMETER  ::   zeps0 = 1.e-13       ! small number
   REAL(wp), PARAMETER  ::   rhor_ref = 1.22      ! reference air density
   REAL(wp), PARAMETER  ::   zu_diag = 10.        ! hard coded diagnostic level for winds
   REAL(wp), PARAMETER  ::   zt_diag = 1.5        ! hard coded diagnostic level for T and q
   REAL(wp), SAVE       ::   Cice_t               ! transfer coefficient over ice (thermo level)
   REAL(wp), SAVE       ::   Cice_m               ! transfer coefficient over ice (momentum level)

   INTEGER ::  inum                        ! temporary logical unit
#if defined key_cmc_sam2
   public :: sf,jp_wndi,jp_wndj,jp_tair,jp_humi
#endif

   !! * Substitutions
#  include "domzgr_substitute.h90"
#  include "vectopt_loop_substitute.h90"
   !!----------------------------------------------------------------------
   !! NEMO/OPA 3.7 , NEMO-consortium (2014)
   !! $Id: sbcblk_rpn.F90 4990 2014-12-15 16:42:49Z timgraham $
   !! Software governed by the CeCILL licence     (NEMOGCM/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------
CONTAINS

   SUBROUTINE sbc_blk_rpn( kt )
      !!---------------------------------------------------------------------
      !!                    ***  ROUTINE sbc_blk_rpn  ***
      !!
      !! ** Purpose :   provide at each time step the surface ocean fluxes
      !!      (momentum, heat, freshwater and runoff) 
      !!      C A U T I O N : never mask the surface stress fields
      !!                      the stress is assumed to be in the (i,j) mesh referential
      !!
      !! ** Method  :   Apply different choices of bulk formulas who will
      !!                return fluxes according to GEM forcing type
      !! ** Action  :   defined at each time-step at the air-sea interface
      !!              - utau, vtau  i- and j-component of the wind stress
      !!              - taum        wind stress module at T-point
      !!              - wndm        wind speed  module at T-point over free ocean or leads in presence of sea-ice
      !!              - qns, qsr    non-solar and solar heat fluxes
      !!              - emp         upward mass flux (evapo. - precip.)
      !!              - sfx         salt flux due to freezing/melting (non-zero only if ice is present)
      !!                            (set in limsbc(_2).F90)
      !!
      !! ** References :   Large & Yeager, 2004 / Large & Yeager, 2008
      !!                   Brodeau et al. Ocean Modelling 2010,
      !!                   Delage, 1997 / Delage and Girard, 1992 / 
      !!                   Benoit et al, 1989
      !!----------------------------------------------------------------------
      INTEGER, INTENT(in) ::   kt   ! ocean time step
      !
      INTEGER  ::   ierror      ! return error code
      INTEGER  ::   ifpr,ji,jj  ! dummy loop indice
      INTEGER  ::   id1, id2
      INTEGER  ::   jfld        ! dummy loop arguments
      INTEGER  ::   ios         ! Local integer output status for namelist read
      REAL(wp) ::   xmask

      !!---------------------------------------------------------------------
      !
      !                                         ! ====================== !
      IF( kt == nit000 ) THEN                   !  First call kt=nit000  !
         !                                      ! ====================== !
         !
         REWIND( numnam_ref )              ! Namelist namsbc_rpn in reference namelist : RPNE bulk parameters
         READ  ( numnam_ref, namsbc_rpn, IOSTAT = ios, ERR = 901)
901      IF( ios /= 0 ) CALL ctl_nam ( ios , 'namsbc_rpn in reference namelist', lwp )
         !
         REWIND( numnam_cfg )              ! Namelist namsbc_rpn in configuration namelist : RPNE bulk parameters
         READ  ( numnam_cfg, namsbc_rpn, IOSTAT = ios, ERR = 902 )
902      IF( ios /= 0 ) CALL ctl_nam ( ios , 'namsbc_rpn in configuration namelist', lwp )

         IF(lwm) WRITE( numond, namsbc_rpn )

         z0mtypeU=StrUpCase(z0mtype)
         z0ttypeU=StrUpCase(z0ttype)

         Cice_t=(vkarmn/log(ZALEV_t/Z0ICE))**2  !Cice_t at ZALEV_t
         Cice_m=(vkarmn/log(ZALEV_m/Z0ICE))**2  !Cice_m at ZALEV_m

         IF (ln_rpnblk_core.AND.ln_rpnblk_rphy) &
          call ctl_stop('sbcblk_rpn: Choose only one option, ln_rpnblk_core or ln_rpnblk_rphy')

         IF (.not.ln_rpnblk_core.AND..not.ln_rpnblk_rphy) &
          call ctl_stop('sbcblk_rpn: Choose one option, ln_rpnblk_core or ln_rpnblk_rphy')

         IF(lwp) THEN
           WRITE(numout,*)
           WRITE(numout,*) 'sbc_blk_rpn : RPN interface for bulk formulations'
           WRITE(numout,*) '~~~~~~~~'
           WRITE(numout,*) 'Namelist namsbc_rpn:'
           WRITE(numout,*) '  Using hard coded forcing    ln_rpnblk_theo  = ', ln_rpnblk_theo
           WRITE(numout,*) '  Using TURB_CORE blk         ln_rpnblk_core  = ', ln_rpnblk_core
           WRITE(numout,*) '  Using RPN-PHYSICS blk       ln_rpnblk_rphy  = ', ln_rpnblk_rphy
           WRITE(numout,*) '  Surface ice roughness length          Z0ICE = ', Z0ICE
           WRITE(numout,*) '  Approx. atm. height for Cice_t calc ZALEV_t = ', ZALEV_t
           WRITE(numout,*) '  Approx. atm. height for Cice_m calc ZALEV_m = ', ZALEV_m
           WRITE(numout,*) '  Atm - ice derived drag coefficient  Cice_t  = ', Cice_t
           WRITE(numout,*) '  Atm - ice derived drag coefficient  Cice_m  = ', Cice_m
           WRITE(numout,*) '  Roughness trick for RPHY             Z0TLAT = ', Z0TLAT
           WRITE(numout,*) '  Roughness length option RPHY (m)    z0mtype = ', z0mtype
           WRITE(numout,*) '  Roughness length option RPHY (t)    z0ttype = ', z0ttype
           WRITE(numout,*) '  SSS effect on surf spec humidity    qsalfac = ', qsalfac
           WRITE(numout,*) '  Active Cool skin param                         diusst_coolskin = ', diusst_coolskin
           WRITE(numout,*) '  Active warm layer param                       diusst_warmlayer = ',diusst_warmlayer
           WRITE(numout,*) '  Reference roughness                                   sl_z0ref = ',sl_z0ref
           WRITE(numout,*) '  Prandtl number for neutral profile                beta_prandtl = ',beta_prandtl
           WRITE(numout,*) '  Bulk Ri for near-neutral regime in the scf layer  sl_rineutral = ',sl_rineutral
           WRITE(numout,*) '  Non-dimensional M-O stable regime formulation     sl_func_stab = ',sl_func_stab
           WRITE(numout,*) '  Non-dimensional M-O unstable regime formulation  sl_fun_unstab = ',sl_func_unstab
           WRITE(numout,*) '  Increase Cd mask active                         ln_cdinc_mask  = ',ln_cdinc_mask
           WRITE(numout,*) '  Increase Cd factor                               rn_cdinc      = ',rn_cdinc
           call flush(numout)
         ENDIF

         IF( diusst_warmlayer )   &
            &   CALL ctl_stop( 'sbc_blk_rpn: diusst_warmlayer = .true. NO PERMITED IN NEMO' )

         IF( ln_dm2dc )   &
            &   CALL ctl_stop( 'sbc_blk_rpn: ln_dm2dc = .true. not supported by this routine' )
         !                                         ! check: do we plan to use ln_dm2dc with non-daily forcing?
!!         IF( ln_dm2dc .AND. sn_qsr%nfreqh /= 24 )   &
!!            &   CALL ctl_stop( 'sbc_blk_rpn: ln_dm2dc can be activated only with daily short-wave forcing' )
!!         IF( ln_dm2dc .AND. sn_qsr%ln_tint ) THEN
!!            CALL ctl_warn( 'sbc_blk_rpn: ln_dm2dc is taking care of the temporal interpolation of daily qsr',   &
!!               &         '              ==> We force time interpolation = .false. for qsr' )
!!            sn_qsr%ln_tint = .false.
!!         ENDIF
         !                                         ! store namelist information in an array
         jfld = jpfld
         !
         ALLOCATE( sf(jfld), STAT=ierror )         ! set sf structure
         IF( ierror > 0 )   CALL ctl_stop( 'STOP', 'sbc_blk_rpn: unable to allocate sf structure' )
         IF ( ln_rpncpl ) THEN
            IF (cplatm_ptype.eq.22) THEN
              cplatm_N_cpldta=cplatm_rap_dt
            ELSEIF (cplatm_ptype.eq.11) THEN
              cplatm_N_cpldta=1
            ELSE
              call ctl_stop('sbc_blk_rpn: Unknown config or atm proc type')
            ENDIF
            IF ( ln_rpncmx ) then
              ALLOCATE( sfc(jfld), STAT=ierror )         ! set sf structure
              IF( ierror > 0 )   CALL ctl_stop( 'STOP', 'sbc_blk_rpn: unable to allocate sfc structure' )
              DO ifpr= 1, jfld
                ALLOCATE( sfc(ifpr)%fnow(jpi,jpj,1) )
                ALLOCATE( sfc(ifpr)%fdta(jpi,jpj,1,cplatm_N_cpldta) )
                ALLOCATE( sf(ifpr)%fnow(jpi,jpj,1) )
                ALLOCATE( sf(ifpr)%fdta(jpi,jpj,1,2) )
              END DO
            ELSE
              DO ifpr= 1, jfld
                ALLOCATE( sf(ifpr)%fnow(jpi,jpj,1) )
                ALLOCATE( sf(ifpr)%fdta(jpi,jpj,1,cplatm_N_cpldta) )
              END DO
            ENDIF
         ELSE
            DO ifpr= 1, jfld
              ALLOCATE( sf(ifpr)%fnow(jpi,jpj,1) )
              ALLOCATE( sf(ifpr)%fdta(jpi,jpj,1,2) )
            END DO
         ENDIF
         !
         sfx(:,:) = 0._wp                          ! salt flux; zero unless ice is present (computed in limsbc(_2).F90)
         ! First estimation of air density and ustar
         ! (both updated after blk flux computation)
         rhor_rpn(:,:) = rhor_ref
         ustr_rpn(:,:) = 0.
         IF( ln_rstart ) THEN
           id1 = iom_varid( numror, 'rhor_rpn'  , ldstop = .FALSE. )
           id2 = iom_varid( numror, 'ustr_rpn'  , ldstop = .FALSE. )
           if ( id1 > 0 ) CALL iom_get( numror, jpdom_autoglo, 'rhor_rpn', rhor_rpn )   ! Air density
           IF ( ln_rpnblk_rphy .and. id2 > 0 ) &
             CALL iom_get( numror, jpdom_autoglo, 'ustr_rpn', ustr_rpn ) ! Friction velocity
         ENDIF
         !

         IF (diusst_coolskin) THEN
           skinT(:,:)=sst_m(:,:) + rt0
         ENDIF

      ENDIF

      IF ( .not. ln_rpnblk_theo ) THEN
        IF ( ln_rpncpl .and. ln_rpncmx ) THEN
          CALL fld_read_rpn( kt, nn_fsbc, sf, sdc=sfc )
          DO ifpr= 1,jpfld
            IF ( ifpr /= jp_mcmx .and. ifpr /= jp_prsn ) THEN
              DO jj=1,jpj
              DO ji=1,jpi
                xmask=sfc(jp_mcmx)%fnow(ji,jj,1)
                sf(ifpr)%fnow(ji,jj,1)=(1-xmask)*sfc(ifpr)%fnow(ji,jj,1) &
                                      +   xmask *sf( ifpr)%fnow(ji,jj,1)
              ENDDO
              ENDDO
            ENDIF
          ENDDO
        ELSE
          CALL fld_read_rpn( kt, nn_fsbc, sf )
        ENDIF
      ELSE
        sf(jp_hght)%fnow(:,:,1)=40.
        sf(jp_hghm)%fnow(:,:,1)=40.
        sf(jp_humi)%fnow(:,:,1)=0.2
        sf(jp_tair)%fnow(:,:,1)=275.
        sf(jp_prst)%fnow(:,:,1)=100000.
        sf(jp_prsm)%fnow(:,:,1)=100000.
        sf(jp_prsg)%fnow(:,:,1)=100000.
        sf(jp_prsn)%fnow(:,:,1)=100000.
        sf(jp_wndi)%fnow(:,:,1)=10.3  !20 knots
        sf(jp_wndj)%fnow(:,:,1)=0.  
        sf(jp_prec)%fnow(:,:,1)=0.
        sf(jp_snow)%fnow(:,:,1)=0.
        sf(jp_qsr)%fnow(:,:,1)=0.
        sf(jp_qlw)%fnow(:,:,1)=0.
      ENDIF

      IF ( ln_rpncpl .and. kt == nitend+1 ) return

      !                                            ! compute the surface ocean fluxes using RPNE bulk formulea
      IF( MOD( kt - 1, nn_fsbc ) == 0 ) CALL blk_oce_rpn( kt, sf, skinT,sst_m, ssu_m, ssv_m, ln_rpnblk_rphy, kt.eq.nit000  )

      if(ln_apr_dyn) apr(:,:)=sf(jp_prsn)%fnow(:,:,1)

#if defined key_cice
      tprecip(:,:)     = sf(jp_prec)%fnow(:,:,1)  !Total precipitation
      sprecip(:,:)     = sf(jp_snow)%fnow(:,:,1)  !Snow precipitation
      cat_i_rpn(:,:)   = sf(jp_tair)%fnow(:,:,1)  !Potential temperature
      tatm_ice(:,:)    = sf(jp_tair)%fnow(:,:,1)* &
                        (sf(jp_prsg)%fnow(:,:,1)/ &
                         sf(jp_prst)%fnow(:,:,1))**(-CAPPA)
                                                  !Temperature
      qatm_ice(:,:)    = sf(jp_humi)%fnow(:,:,1)  !Specific humidity
      zlm_i_rpn(:,:)   = MAX(0.001,sf(jp_hghm)%fnow(:,:,1))
                                                  !Atmospheric forcing height (momentum)
                                                  !MAX to avoid problems if has unset halo rows
      zlt_i_rpn(:,:)   = MAX(0.001,sf(jp_hght)%fnow(:,:,1))
                                                  !Atmospheric forcing height (temperature)
                                                  !MAX to avoid problems if has unset halo rows
      wndi_ice(:,:)    = sf(jp_wndi)%fnow(:,:,1)  !i-wind component
      wndj_ice(:,:)    = sf(jp_wndj)%fnow(:,:,1)  !j-wind component
      qlw_ice(:,:,1)   = sf(jp_qlw )%fnow(:,:,1)  !Long wave down
      qsr_ice(:,:,1)   = sf(jp_qsr )%fnow(:,:,1)  !Short wave down
#endif

      !                                                ! ---------------------------------------- !
      IF( lrst_oce ) THEN                              !      Write in the ocean restart file     !
         !                                             ! ---------------------------------------- !
         IF(lwp) WRITE(numout,*)
         IF(lwp) WRITE(numout,*) 'sbc_blk_rpn : atm. fields written in ocean restart file ',   &
            &                    'at it= ', kt,' date= ', ndastp
         IF(lwp) WRITE(numout,*) '~~~~'
         CALL iom_rstput( kt, nitrst, numrow, 'rhor_rpn' , rhor_rpn )
         IF ( ln_rpnblk_rphy ) &
           CALL iom_rstput( kt, nitrst, numrow, 'ustr_rpn' , ustr_rpn )
         IF(ln_apr_dyn) &
           CALL iom_rstput( kt, nitrst, numrow, 'apr' , apr )
      ENDIF
      !
   END SUBROUTINE sbc_blk_rpn
   
   SUBROUTINE blk_oce_rpn( kt, sf, pskinT,pst, pu, pv, rphy,  ln_init_zsst )
      !!---------------------------------------------------------------------
      !!                     ***  ROUTINE blk_oce_rpn  ***
      !!
      !! ** Purpose :   provide the momentum, heat and freshwater fluxes at
      !!      the ocean surface at each time step
      !!
      !! ** Method  :   APPLIES flxsurf4 (RPN-Physics) or TURB_CORE (rphy=.false.)
      !! 
      !! ** Outputs : - utau    : i-component of the stress at U-point  (N/m2)
      !!              - vtau    : j-component of the stress at V-point  (N/m2)
      !!              - taum    : Wind stress module at T-point         (N/m2)
      !!              - wndm    : Wind speed module at T-point          (m/s)
      !!              - qsr     : Solar heat flux over the ocean        (W/m2)
      !!              - qns     : Non Solar heat flux over the ocean    (W/m2)
      !!              - emp     : evaporation minus precipitation       (kg/m2/s)
      !!
      !!  ** Nota  :   sf has to be a dummy argument for AGRIF on NEC
      !!---------------------------------------------------------------------

      INTEGER  , INTENT(in   )                 ::   kt    ! time step index
      TYPE(fld), INTENT(inout), DIMENSION(:)   ::   sf    ! input data
      REAL(wp),  INTENT(inout), DIMENSION(:,:) ::   pskinT    ! Ocean skin temperature [Celsius]
      REAL(wp) , INTENT(in)   , DIMENSION(:,:) ::   pst   ! surface temperature                      [Celcius]
      REAL(wp) , INTENT(in)   , DIMENSION(:,:) ::   pu    ! surface current at U-point (i-component) [m/s]
      REAL(wp) , INTENT(in)   , DIMENSION(:,:) ::   pv    ! surface current at V-point (j-component) [m/s]
      LOGICAL,   INTENT(in)                    ::   rphy
      LOGICAL,   INTENT(in)  :: ln_init_zsst

      INTEGER  ::   ji, jj               ! dummy loop indices
      REAL(wp) ::   zcoef_qsatw, zztmp   ! local variable
      REAL(wp), DIMENSION(:,:), POINTER ::   zwnd_i, zwnd_j    ! wind speed components at T-point
      REAL(wp), DIMENSION(:,:), POINTER ::   zqsatw            ! specific humidity at pst
      REAL(wp), DIMENSION(:,:), POINTER ::   zevap             ! evaporation
      REAL(wp), DIMENSION(:,:), POINTER ::   Cd                ! transfer coefficient for momentum      (tau)
      REAL(wp), DIMENSION(:,:), POINTER ::   cdmask            ! mask for enhance the Cd
      REAL(wp), DIMENSION(:,:), POINTER ::   Ch                ! transfer coefficient for sensible heat (Q_sens)
      REAL(wp), DIMENSION(:,:), POINTER ::   Ce                ! tansfert coefficient for evaporation   (Q_lat)
      REAL(wp), DIMENSION(:,:), POINTER ::   zst               ! surface temperature in Kelvin
      REAL(wp), DIMENSION(:,:), POINTER ::   z0m_adjust        ! roughness length formulation over water
      REAL(wp), DIMENSION(:,:), POINTER ::   zt_zu             ! air temperature at wind speed height
      REAL(wp), DIMENSION(:,:), POINTER ::   zq_zu             ! air spec. hum.  at wind speed height
      REAL(wp), DIMENSION(:,:), POINTER ::   wndm10            ! wndm at diagnostic level
                                                               !  (see turb_core_2z_rpn)
      LOGICAL, SAVE :: first=.true.
      !!---------------------------------------------------------------------
      !

      IF( nn_timing == 1 )  CALL timing_start('blk_oce_rpn')
      !
      CALL wrk_alloc( jpi,jpj, zwnd_i, zwnd_j, zqsatw, zevap )
      CALL wrk_alloc( jpi,jpj, Cd, Ch, Ce, zst, zt_zu, zq_zu )
      CALL wrk_alloc( jpi,jpj, wndm10 )
      IF ( rphy ) THEN
        CALL wrk_alloc( jpi,jpj, z0m_adjust )
      ENDIF
      IF ( ln_cdinc_mask ) THEN
        CALL wrk_alloc( jpi,jpj, cdmask )
      ENDIF

      ! local scalars ( place there for vector optimisation purposes)
      zcoef_qsatw = qsalfac * 640380.
      

      zst(:,:) = pst(:,:) + rt0      ! converte Celcius to Kelvin (and set minimum value far above 0 K)
      
      if (diusst_coolskin) then
        skinT(:,:) = pskinT(:,:)              
      else
        skinT(:,:) = zst(:,:)        ! converte Celcius to Kelvin (and set minimum value far above 0 K)
      endif

      ! ----------------------------------------------------------------------------- !
      !      0   Wind components and module at T-point relative to the moving ocean   !
      ! ----------------------------------------------------------------------------- !

      ! ... components ( U10m - U_oce ) at T-point (unmasked)
      zwnd_i(:,:) = 0.e0  
      zwnd_j(:,:) = 0.e0
#if defined key_cyclone
      CALL wnd_cyc( kt, zwnd_i, zwnd_j )    ! add analytical tropical cyclone (Vincent et al. JGR 2012)
      DO jj = 2, jpjm1
         DO ji = fs_2, fs_jpim1   ! vect. opt.
            sf(jp_wndi)%fnow(ji,jj,1) = sf(jp_wndi)%fnow(ji,jj,1) + zwnd_i(ji,jj)
            sf(jp_wndj)%fnow(ji,jj,1) = sf(jp_wndj)%fnow(ji,jj,1) + zwnd_j(ji,jj)
         END DO
      END DO
#endif
      DO jj = 2, jpjm1
         DO ji = fs_2, fs_jpim1   ! vect. opt.
!froy - be careful if you put out currents here, see after itf_flxsurf2 or turb_core_2z_rpn
            zwnd_i(ji,jj) = (  sf(jp_wndi)%fnow(ji,jj,1) - 0.5 * ( pu(ji-1,jj  ) + pu(ji,jj) )  )
            zwnd_j(ji,jj) = (  sf(jp_wndj)%fnow(ji,jj,1) - 0.5 * ( pv(ji  ,jj-1) + pv(ji,jj) )  )
         END DO
      END DO
      CALL lbc_lnk( zwnd_i(:,:) , 'T', -1. )
      CALL lbc_lnk( zwnd_j(:,:) , 'T', -1. )
      ! ... scalar wind ( = | U?m - U_oce | ) at T-point (masked)
      wndm(:,:) = SQRT(  zwnd_i(:,:) * zwnd_i(:,:)   &
         &             + zwnd_j(:,:) * zwnd_j(:,:)  ) * tmask(:,:,1)

      ! ----------------------------------------------------------------------------- !
      !      I   Radiative FLUXES                                                     !
      ! ----------------------------------------------------------------------------- !

      ! ocean albedo assumed to be constant + modify now Qsr to include the diurnal cycle                    ! Short Wave
      zztmp = 1. - albo
      alb_o_rpn(:,:) = albo
      IF( ln_dm2dc ) THEN   ;   qsr(:,:) = zztmp * sbc_dcy( sf(jp_qsr)%fnow(:,:,1) ) * tmask(:,:,1)
      ELSE                  ;   qsr(:,:) = zztmp *          sf(jp_qsr)%fnow(:,:,1)   * tmask(:,:,1)
      ENDIF
      qlw_o_rpn (:,:) = ( sf(jp_qlw)%fnow(:,:,1) - STEF   * skinT(:,:)*skinT(:,:)*skinT(:,:)*skinT(:,:)  ) * tmask(:,:,1)  ! Long  Wave
      t4a_o_rpn (:,:) = ( sf(jp_qlw)%fnow(:,:,1) - qlw_o_rpn(:,:)  ) / STEF * tmask(:,:,1)
      ! ----------------------------------------------------------------------------- !
      !     II    Turbulent FLUXES                                                    !
      ! ----------------------------------------------------------------------------- !

      ! ... specific humidity at SST and IST
      zqsatw(:,:) = zcoef_qsatw / MAX(rhor_rpn(:,:),0.1) * EXP( -5107.4 / skinT(:,:) ) 
                     !MAX to avoid problems if has unset halo rows 
                     !nemo2cice does that...mmmm

      qsu_o_rpn(:,:) = zqsatw(:,:)

      IF ( rphy ) THEN

        call prep_itf_flxsurf(z0m_o_rpn,z0m_adjust,z0h_o_rpn)

        ! ... RPN-Physics Bulk formulae, computation of Cd, Ch, Ce at T-point :
#if ! defined key_cice
        call ctl_stop('NO COUPLING POSSIBLE WITHOUT CICE') 
#endif
        call itf_sfclayer_gemdm( numout, jpi,jpj    , ff,                                        &
         &                    sf(jp_hght)%fnow(:,:,1), sf(jp_hghm)%fnow(:,:,1),                  &
         &                    zst   , sf(jp_tair)%fnow(:,:,1),                                   &
         &                    zqsatw, sf(jp_humi)%fnow(:,:,1), sf(jp_prsg)%fnow(:,:,1), wndm,    &
         &                    zwnd_i, zwnd_j          , rhor_rpn, gphit,                         &
         &                    fr0_i_rpn ,                                                        &
         &                    z0m_o_rpn   , z0h_o_rpn,                                           &
         &                    alb_o_rpn, qsr, sf(jp_qlw)%fnow(:,:,1),                            &
         &                    skinT,warm_inc,skin_inc,skin_depth,                                &  
         &                    Cd    , Ch              , Ce,                                      &
         &                    ustr_rpn  ,                                                        &
         &                    to2m_rpn  ,                                                        &
         &                    qo2m_rpn  ,                                                        &
         &                    uo10_rpn  , vo10_rpn , ilm_o_rpn, ln_init_zsst  )

!        call itf_flxsurf2( numout, jpi,jpj    , ff,                          &
!                           sf(jp_hght)%fnow(:,:,1), sf(jp_hghm)%fnow(:,:,1), &
!                                zst   ,             sf(jp_tair)%fnow(:,:,1), &
!                                zqsatw,             sf(jp_humi)%fnow(:,:,1), &
!                                wndm, zwnd_i, zwnd_j, gphit,                 &
!                                z0m_o_rpn  , z0h_o_rpn  ,                    &
!                                Cd    , Ch              , Ce,                &
!                                ustr_rpn  ,                                  &
!                                to2m_rpn  ,                                  &
!                                qo2m_rpn  ,                                  &
!                                uo10_rpn  , vo10_rpn , ilm_o_rpn )

! Adjust dU10 with currents to obtain diagnostic winds
        DO jj = 2, jpjm1
          DO ji = fs_2, fs_jpim1
             uo10_rpn(ji,jj)= uo10_rpn(ji,jj) + 0.5 * ( pu(ji-1,jj  ) + pu(ji,jj) )
             vo10_rpn(ji,jj)= vo10_rpn(ji,jj) + 0.5 * ( pv(ji  ,jj-1) + pv(ji,jj) )
          ENDDO
        ENDDO

      ELSE

        ! ... NCAR Bulk formulae, computation of Cd, Ch, Ce at T-point :

        call turb_core_2z_rpn(sf(jp_hght)%fnow(:,:,1), sf(jp_hghm)%fnow(:,:,1), &
                              zst,                     sf(jp_tair)%fnow(:,:,1), &
                              zqsatw,                  sf(jp_humi)%fnow(:,:,1), &
                              wndm, Cd    , Ch              , Ce,               &
                              zt_zu, zq_zu,                                     &
                              T_a_d=to2m_rpn  ,                                 &
                              q_a_d=qo2m_rpn  ,                                 &
                              dU_d =wndm10,                                     &
                              ilmo =ilm_o_rpn,                                  &
                              z0m  =z0m_o_rpn,                                  &
                              z0h  =z0h_o_rpn)

! Compute diagnostic winds to be sent to atmospheric model
        DO jj = 2, jpjm1
          DO ji = fs_2, fs_jpim1
            uo10_rpn(ji,jj) = sf(jp_wndi)%fnow(ji,jj,1) !Should be exceptional
            vo10_rpn(ji,jj) = sf(jp_wndj)%fnow(ji,jj,1) !Should be exceptional
            IF (wndm(ji,jj).gt.zeps0) THEN
              uo10_rpn(ji,jj)= wndm10(ji,jj)/wndm(ji,jj)*zwnd_i(ji,jj)  &
                                 + 0.5 * ( pu(ji-1,jj  ) + pu(ji,jj) )
              vo10_rpn(ji,jj)= wndm10(ji,jj)/wndm(ji,jj)*zwnd_j(ji,jj)  &
                                 + 0.5 * ( pv(ji  ,jj-1) + pv(ji,jj) )
            ENDIF
          ENDDO
        ENDDO

      ENDIF

      CALL lbc_lnk( uo10_rpn(:,:) , 'T', -1. )
      CALL lbc_lnk( vo10_rpn(:,:) , 'T', -1. )

!jlei cdmask is a coefficient in [0,1] giving the mask where to apply the Cd increase
      IF ( ln_cdinc_mask ) THEN
        IF( lwp .and. first ) WRITE(numout,*)'WARNING: Increase of wind drag mask active'
        IF( kt == nit000 ) THEN
                CALL iom_open( 'mask_cdinc.nc', inum )
                CALL iom_get ( inum, jpdom_data, 'mask', cdmask )
                CALL iom_close (inum)
        ENDIF
        WHERE (cdmask > 0.5_wp) Cd(:,:) = Cd(:,:) * rn_cdinc
      ENDIF
!jlei end


! ... tau module, i and j component
      DO jj = 1, jpj
         DO ji = 1, jpi
            zztmp = rhor_rpn(ji,jj) * wndm(ji,jj) * Cd(ji,jj)
            taum  (ji,jj) = zztmp * wndm  (ji,jj)
            zwnd_i(ji,jj) = zztmp * zwnd_i(ji,jj)
            zwnd_j(ji,jj) = zztmp * zwnd_j(ji,jj)
         END DO
      END DO

      CALL iom_put( "taum_oce", taum )   ! output wind stress module

      ! ... utau, vtau at U- and V_points, resp.
      !     Note the use of 0.5*(2-umask) in order to unmask the stress along coastlines
      !     Note the use of MAX(tmask(i,j),tmask(i+1,j) is to mask tau over ice shelves
      DO jj = 1, jpjm1
         DO ji = 1, fs_jpim1
            utau(ji,jj) = 0.5 * ( 2. - umask(ji,jj,1) ) * ( zwnd_i(ji,jj) + zwnd_i(ji+1,jj  ) ) &
               &          * MAX(tmask(ji,jj,1),tmask(ji+1,jj,1))
            vtau(ji,jj) = 0.5 * ( 2. - vmask(ji,jj,1) ) * ( zwnd_j(ji,jj) + zwnd_j(ji  ,jj+1) ) &
               &          * MAX(tmask(ji,jj,1),tmask(ji,jj+1,1))
         END DO
      END DO
      CALL lbc_lnk( utau(:,:), 'U', -1. )
      CALL lbc_lnk( vtau(:,:), 'V', -1. )

      uta_o_rpn(:,:) = utau(:,:) !for coupling utau will eventually include ice stress
      vta_o_rpn(:,:) = vtau(:,:) !for coupling vtau will eventually include ice stress

      !  Turbulent fluxes over ocean
      ! -----------------------------

      IF( rphy ) THEN 
        ! Ch and Ce based on CM and CT from GEM (see sbc_flxsurf)
        ! No height adjustment needed like for turb_core_2z_rpn (where Ch valid at zu)
        ! CM=k/fm(zu), CT=k/fh(zt), thus
        ! Qh=rhoCpT*u*, where T* and u* is an invariant
        ! T*=CT(zt) x (skinT-Tair(zt))
        ! u*=CM(zu) x dU(zu)
        ! Cd=CM**2, Ch=CM x CT
        zevap(:,:) = rhor_rpn(:,:)*Ce(:,:)*( zqsatw(:,:) - sf(jp_humi)%fnow(:,:,1) ) * wndm(:,:)    ! Evaporation - Condensation
        qsb_o_rpn (:,:)  = rhor_rpn(:,:)*CPD*Ch(:,:)*( skinT   (:,:) - sf(jp_tair)%fnow(:,:,1) ) * wndm(:,:)     ! Sensible Heat
      ELSE
         !! q_air and t_air are not given at 10m (wind reference height)
         ! Values of temp. and hum. adjusted to height of wind during bulk algorithm iteration must be used!!!
        zevap(:,:) = rhor_rpn(:,:)*Ce(:,:)*( zqsatw(:,:) - zq_zu(:,:) ) * wndm(:,:)    ! Evaporation - Condensation
        qsb_o_rpn (:,:)  = rhor_rpn(:,:)*CPD*Ch(:,:)*( skinT   (:,:) - zt_zu(:,:) ) * wndm(:,:)     ! Sensible Heat
      ENDIF
      qla_o_rpn (:,:) = CHLC * zevap(:,:)                                                              ! Latent Heat

      IF(ln_ctl) THEN
         CALL prt_ctl( tab2d_1=qla_o_rpn, clinfo1=' blk_oce_rpn: qla_o_rpn    : ', tab2d_2=Ce , clinfo2=' Ce  : ' )
         CALL prt_ctl( tab2d_1=qsb_o_rpn, clinfo1=' blk_oce_rpn: qsb_o_rpn    : ', tab2d_2=Ch , clinfo2=' Ch  : ' )
         CALL prt_ctl( tab2d_1=qlw_o_rpn, clinfo1=' blk_oce_rpn: qlw_o_rpn    : ', tab2d_2=qsr, clinfo2=' qsr : ' )
         CALL prt_ctl( tab2d_1=zqsatw, clinfo1=' blk_oce_rpn: zqsatw : ', tab2d_2=zst, clinfo2=' zst : ' )
         CALL prt_ctl( tab2d_1=utau  , clinfo1=' blk_oce_rpn: utau   : ', mask1=umask,   &
            &          tab2d_2=vtau  , clinfo2=              ' vtau : '  , mask2=vmask )
         CALL prt_ctl( tab2d_1=wndm  , clinfo1=' blk_oce_rpn: wndm   : ')
         CALL prt_ctl( tab2d_1=zst   , clinfo1=' blk_oce_rpn: zst    : ')
      ENDIF
       
      ! ----------------------------------------------------------------------------- !
      !     III    Total FLUXES                                                       !
      ! ----------------------------------------------------------------------------- !
     
      emp (:,:) = (  zevap(:,:)                                          &   ! mass flux (evap. - precip.)
         &         - sf(jp_prec)%fnow(:,:,1) ) * tmask(:,:,1)
      qns(:,:) = qlw_o_rpn(:,:) - qsb_o_rpn(:,:) - qla_o_rpn(:,:)        &   ! Downward Non Solar flux
         &     - sf(jp_snow)%fnow(:,:,1) * lfus                          &   ! remove latent melting heat for solid precip
         &     - zevap(:,:) * pst(:,:) * rcp                             &   ! remove evap heat content at SST
         &     + ( sf(jp_prec)%fnow(:,:,1) - sf(jp_snow)%fnow(:,:,1) )   &   ! add liquid precip heat content at Tair
         &     * ( sf(jp_tair)%fnow(:,:,1) - rt0 ) * rcp                 &
         &     + sf(jp_snow)%fnow(:,:,1)                                 &   ! add solid  precip heat content at min(Tair,Tsnow)
         &     * ( MIN( sf(jp_tair)%fnow(:,:,1), rt0_snow ) - rt0 ) * cpic * tmask(:,:,1)
      ! NOTE: if (.not. lk_vvl), these new terms for melt, evap, precip are comensated by
      !       a positive term with emp in tra_sbc
      !
      CALL iom_put( "qlw_oce",   qlw_o_rpn )      ! output downward longwave heat over the ocean
      CALL iom_put( "qsb_oce", - qsb_o_rpn )      ! output downward sensible heat over the ocean
      CALL iom_put( "qla_oce", - qla_o_rpn )      ! output downward latent   heat over the ocean
      CALL iom_put( "qhc_oce",   &
               qns-qlw_o_rpn+qsb_o_rpn+qla_o_rpn )! output downward heat content of E-P over the ocean
      CALL iom_put( "qns_oce",   qns  )                 ! output downward non solar heat over the ocean
      CALL iom_put( "qsr_oce",   qsr  )                 ! output downward solar heat over the ocean
      !
      IF(ln_ctl) THEN
         CALL prt_ctl(tab2d_1=qsb_o_rpn, clinfo1=' blk_oce_rpn: qsb_o_rpn: ', tab2d_2=qlw_o_rpn  , clinfo2=' qlw_o_rpn   : ')
         CALL prt_ctl(tab2d_1=qla_o_rpn, clinfo1=' blk_oce_rpn: qla_o_rpn: ', tab2d_2=qsr  , clinfo2=' qsr   : ')
         CALL prt_ctl(tab2d_1=pst  , clinfo1=' blk_oce_rpn: pst    : ', tab2d_2=emp  , clinfo2=' emp   : ')
         CALL prt_ctl(tab2d_1=utau , clinfo1=' blk_oce_rpn: utau   : ', mask1=umask,   &
            &         tab2d_2=vtau , clinfo2=              ' vtau  : ' , mask2=vmask )
      ENDIF
      !
      ! Update air density for next time step
      rhor_rpn(:,:) = sf(jp_prsg)%fnow(:,:,1)/(RGASD * to2m_rpn(:,:)*(1.+DELTA*qo2m_rpn(:,:)))
      if ( zlev_gem >= 0. ) rhor_rpn(:,:) = rhor_ref
      first = .false.
      !
      CALL wrk_dealloc( jpi,jpj, zwnd_i, zwnd_j, zqsatw, zevap )
      CALL wrk_dealloc( jpi,jpj, Cd, Ch, Ce, zst, zt_zu, zq_zu )
      CALL wrk_dealloc( jpi,jpj, wndm10 )
      IF ( rphy ) THEN
        CALL wrk_dealloc( jpi,jpj, z0m_adjust )
      ENDIF
      IF ( ln_cdinc_mask ) THEN
        CALL wrk_dealloc( jpi,jpj, cdmask )
      ENDIF

      !
      IF( nn_timing == 1 )  CALL timing_stop('blk_oce_rpn')
      !
   END SUBROUTINE blk_oce_rpn

   SUBROUTINE blk_ice_rpn(   pst   , pui   , pvi   , palb ,   &
      &                      p_taui, p_tauj, p_qns , p_qsr,        &
      &                      p_qla , p_qsb,  p_qlw , p_t4a, p_dqns, p_dqla,       &
      &                      p_tpr , p_spr ,                  &
      &                      p_fr1 , p_fr2 , cd_grid, pdim, rphy  ) 
      !!---------------------------------------------------------------------
      !!                     ***  ROUTINE blk_ice_rpn  ***
      !!
      !! ** Purpose :   provide the surface boundary condition over sea-ice
      !!
      !! ** Method  :   compute momentum, heat and freshwater exchanged
      !!                between atmosphere and sea-ice using rpn bulk
      !!                formulea, ice variables and read atmmospheric fields.
      !!                NB: if rphy=.false. (CORE for ocean), the drag coefficient
      !!                is assumed constant (Cice_m),
      !!                else the ice drag coefficient is computed with RPN-PHYSICS
      !!                Warning: For the moment, assume ijpl=1 and tau is not 
      !!                decomposed into multi-category as should if applied
      !! 
      !! caution : the net upward water flux has with mm/day unit
      !!---------------------------------------------------------------------

      USE sbc_rpn, only : ti2m_rpn,qi2m_rpn,ui10_rpn,vi10_rpn

      REAL(wp), INTENT(in   ), DIMENSION(:,:,:)   ::   pst      ! ice surface temperature (>0, =rt0 over land) [Kelvin]
      REAL(wp), INTENT(in   ), DIMENSION(jpi,jpj) ::   pui      ! ice surface velocity (i- and i- components      [m/s]
      REAL(wp), INTENT(in   ), DIMENSION(jpi,jpj) ::   pvi      !    at I-point (B-grid) or U & V-point (C-grid)
      REAL(wp), INTENT(in   ), DIMENSION(:,:,:)   ::   palb     ! ice albedo (clear sky) (alb_ice_cs)               [%]
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj) ::   p_taui   ! i- & j-components of surface ice stress        [N/m2]
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj) ::   p_tauj   !   at I-point (B-grid) or U & V-point (C-grid)
      REAL(wp), INTENT(  out), DIMENSION(:,:,:)   ::   p_qns    ! non solar heat flux over ice (T-point)         [W/m2]
      REAL(wp), INTENT(  out), DIMENSION(:,:,:)   ::   p_qsr    !     solar heat flux over ice (T-point)         [W/m2]
      REAL(wp), INTENT(  out), DIMENSION(:,:,:)   ::   p_qla    ! latent    heat flux over ice (T-point)         [W/m2]
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj) ::   p_qsb    ! sensible  heat flux over ice (T-point)         [W/m2]
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj) ::   p_qlw    ! long wave heat flux over ice (T-point)         [W/m2]
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj) ::   p_t4a    ! radiation temperature (T-point) (to atm.)  [K**4]
      REAL(wp), INTENT(  out), DIMENSION(:,:,:)   ::   p_dqns   ! non solar heat sensistivity  (T-point)         [W/m2]
      REAL(wp), INTENT(  out), DIMENSION(:,:,:)   ::   p_dqla   ! latent    heat sensistivity  (T-point)         [W/m2]
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj) ::   p_tpr    ! total precipitation          (T-point)      [Kg/m2/s]
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj) ::   p_spr    ! solid precipitation          (T-point)      [Kg/m2/s]
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj) ::   p_fr1    ! 1sr fraction of qsr penetration in ice (T-point)  [%]
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj) ::   p_fr2    ! 2nd fraction of qsr penetration in ice (T-point)  [%]
      CHARACTER(len=1), INTENT(in   )             ::   cd_grid  ! ice grid ( C or B-grid)
      INTEGER, INTENT(in   )                      ::   pdim     ! number of ice categories
      LOGICAL, INTENT(in   )                      ::   rphy
      !!
      INTEGER  ::   ji, jj, jl    ! dummy loop indices
      INTEGER  ::   ijpl          ! number of ice categories (size of 3rd dim of input arrays)
      REAL(wp) ::   zst2, zst3, zcoef_dqlw, zwnorm, C_work, zztmp
      REAL(wp), DIMENSION(:,:)  , POINTER ::   z_wnds_t          ! wind speed ( = | U10m - U_ice | ) at T-point
      REAL(wp), DIMENSION(:,:)  , POINTER ::   z_wnds_f          ! wind speed ( = | U10m - U_ice | ) at F-point
      REAL(wp), DIMENSION(:,:)  , POINTER ::   zwndi_t, zwndj_t  ! relative wind components at T-point
      REAL(wp), DIMENSION(:,:)  , POINTER ::   zwndi_f, zwndj_f  ! relative wind components at F-point
      REAL(wp), DIMENSION(:,:)  , POINTER ::   zrhor_f, ztair_f,  &
                                               zhumi_f, zhght_f,  &
                                               zhghm_f, zpst_f   ! Thermo variables at F-point
      REAL(wp), DIMENSION(:,:)  , POINTER ::   zsq_t             ! Ground level humidity at T-point
      REAL(wp), DIMENSION(:,:)  , POINTER ::   zsq_f             ! Ground level humidity at F-point
      REAL(wp), DIMENSION(:,:)  , POINTER ::   zff_f             ! Coriolis param at F-point
      REAL(wp), DIMENSION(:,:)  , POINTER ::   zz0_wft           ! roughness length working array
      REAL(wp), DIMENSION(:,:)  , POINTER ::   Cd                ! transfer coefficient for momentum      (tau)
      REAL(wp), DIMENSION(:,:)  , POINTER ::   Ch                ! transfer coefficient for sensible heat (Q_sens)
      REAL(wp), DIMENSION(:,:)  , POINTER ::   Ce                ! tansfert coefficient for evaporation   (Q_lat)
      REAL(wp), DIMENSION(:,:)  , POINTER ::   ustri             ! friction velocity for ice

      REAL(wp), DIMENSION(:,:,:), POINTER ::   z_dqlw            ! long wave heat sensitivity over ice
      REAL(wp), DIMENSION(:,:,:), POINTER ::   z_dqsb            ! sensible  heat sensitivity over ice
      !!---------------------------------------------------------------------
      !
      IF( nn_timing == 1 )  CALL timing_start('blk_ice_rpn')
      !
      CALL wrk_alloc( jpi,jpj, z_wnds_t, zwndi_t, zwndj_t, zrhor_f )
      CALL wrk_alloc( jpi,jpj, z_wnds_f, zwndi_f, zwndj_f, ztair_f )
      CALL wrk_alloc( jpi,jpj, zhumi_f, zhght_f, zhghm_f, zpst_f )
      CALL wrk_alloc( jpi,jpj, zsq_t, zsq_f, zff_f, zz0_wft )
      CALL wrk_alloc( jpi,jpj, Cd, Ch, Ce, ustri )
      CALL wrk_alloc( jpi,jpj,pdim, z_dqlw, z_dqsb ) 

#if defined key_lim3
      call ctl_stop('blk_ice_rpn: not compatible with lim3 - ABORT')
#endif

      if (pdim /= 1) call ctl_stop('blk_ice_rpn: not compatible with multi-category - ABORT') 
      
      ijpl  = pdim                            ! number of ice categories

      ! local scalars ( place there for vector optimisation purposes)
      zcoef_dqlw = 4.0 * 0.95 * STEF  

!!gm brutal....
      z_wnds_t(:,:) = 0.e0
      p_taui  (:,:) = 0.e0
      p_tauj  (:,:) = 0.e0
!!gm end

      ! ----------------------------------------------------------------------------- !
      !    Ground humidity and roughness length used in flxsurf4                      !
      ! ----------------------------------------------------------------------------- !
      zsq_t  (:,:) = 11637800. * EXP( -5897.8 / pst(:,:,1) ) / rhor_rpn(:,:)
      IF ( rphy ) zz0_wft(:,:) = Z0ICE

      ! ----------------------------------------------------------------------------- !
      !    Wind components and module relative to the moving ocean ( U10m - U_ice )   !
      ! ----------------------------------------------------------------------------- !
      SELECT CASE( cd_grid )
      CASE( 'I' )                  ! B-grid ice dynamics :   I-point (i.e. F-point with sea-ice indexation)
         !                           and scalar wind at T-point ( = | U?m - U_ice | ) (masked)
         DO jj = 2, jpjm1
            DO ji = 2, jpim1   ! B grid : NO vector opt
               ! ... scalar wind at I-point (fld being at T-point)
               zwndi_f(ji,jj) = 0.25 * (  sf(jp_wndi)%fnow(ji-1,jj,1  )      &
                                 + sf(jp_wndi)%fnow(ji  ,jj  ,1)             &
                  &              + sf(jp_wndi)%fnow(ji-1,jj-1,1)             &
                                 + sf(jp_wndi)%fnow(ji  ,jj-1,1)  ) - pui(ji,jj)
               zwndj_f(ji,jj) = 0.25 * (  sf(jp_wndj)%fnow(ji-1,jj,1  )      &
                                 + sf(jp_wndj)%fnow(ji  ,jj  ,1)             &
                  &              + sf(jp_wndj)%fnow(ji-1,jj-1,1)             &
                                 + sf(jp_wndj)%fnow(ji  ,jj-1,1)  ) - pvi(ji,jj)
               z_wnds_f(ji,jj)= SQRT( zwndi_f(ji,jj) * zwndi_f(ji,jj)        &
                                    + zwndj_f(ji,jj) * zwndj_f(ji,jj) )
               ! ... scalar wind at T-point (fld being at T-point)
               zwndi_t(ji,jj) = sf(jp_wndi)%fnow(ji,jj,1)                    &
                                 - 0.25 * (  pui(ji,jj+1) + pui(ji+1,jj+1)   &
                  &                        + pui(ji,jj  ) + pui(ji+1,jj  )  )
               zwndj_t(ji,jj) = sf(jp_wndj)%fnow(ji,jj,1)                    &
                                 - 0.25 * (  pvi(ji,jj+1) + pvi(ji+1,jj+1)   &
                  &                        + pvi(ji,jj  ) + pvi(ji+1,jj  )  )
               z_wnds_t(ji,jj)= SQRT( zwndi_t(ji,jj) * zwndi_t(ji,jj)        &
                                    + zwndj_t(ji,jj) * zwndj_t(ji,jj) ) * tmask(ji,jj,1)
            END DO
         END DO
         CALL lbc_lnk( zwndi_t , 'T', -1. )
         CALL lbc_lnk( zwndj_t , 'T', -1. )
         CALL lbc_lnk( z_wnds_t, 'T',  1. )
         CALL lbc_lnk( zwndi_f , 'I', -1. )
         CALL lbc_lnk( zwndj_f , 'I', -1. )
         CALL lbc_lnk( z_wnds_f, 'I',  1. )

         IF ( rphy ) THEN

           DO jj = 2, jpjm1
              DO ji = 2, jpim1
                 ! ... thermo variables at I-point (fld being at T-point)
                 ztair_f(ji,jj) = 0.25 * (  sf(jp_tair)%fnow(ji-1,jj,1  )  &
                                   + sf(jp_tair)%fnow(ji  ,jj  ,1)         &
                    &              + sf(jp_tair)%fnow(ji-1,jj-1,1)         &
                                   + sf(jp_tair)%fnow(ji  ,jj-1,1)  )
                 zhumi_f(ji,jj) = 0.25 * (  sf(jp_humi)%fnow(ji-1,jj,1  )  &
                                   + sf(jp_humi)%fnow(ji  ,jj  ,1)         &
                    &              + sf(jp_humi)%fnow(ji-1,jj-1,1)         &
                                   + sf(jp_humi)%fnow(ji  ,jj-1,1)  )
                 zhght_f(ji,jj) = 0.25 * (  sf(jp_hght)%fnow(ji-1,jj,1  )  &
                                   + sf(jp_hght)%fnow(ji  ,jj  ,1)         &
                    &              + sf(jp_hght)%fnow(ji-1,jj-1,1)         &
                                   + sf(jp_hght)%fnow(ji  ,jj-1,1)  )
                 zhghm_f(ji,jj) = 0.25 * (  sf(jp_hghm)%fnow(ji-1,jj,1  )  &
                                   + sf(jp_hghm)%fnow(ji  ,jj  ,1)         &
                    &              + sf(jp_hghm)%fnow(ji-1,jj-1,1)         &
                                   + sf(jp_hghm)%fnow(ji  ,jj-1,1)  )
                 zrhor_f(ji,jj) = 0.25 * (  rhor_rpn(ji-1,jj  )            &
                                   + rhor_rpn(ji  ,jj  )                   &
                    &              + rhor_rpn(ji-1,jj-1)                   &
                                   + rhor_rpn(ji  ,jj-1)  )
                 zpst_f(ji,jj)  = 0.25 * (   pst(ji-1,jj,1  )              &
                                   +  pst(ji  ,jj,1  )                     &
                    &              +  pst(ji-1,jj-1,1)                     &
                                   +  pst(ji  ,jj-1,1)  )
                 zsq_f(ji,jj)   = 0.25 * ( zsq_t(ji-1,jj  )                &
                                   +zsq_t(ji  ,jj  )                       &
                    &              +zsq_t(ji-1,jj-1)                       &
                                   +zsq_t(ji  ,jj-1)  )
                 zff_f(ji,jj)   = 0.25 * (    ff(ji-1,jj  )                &
                                   +   ff(ji  ,jj  )                       &
                    &              +   ff(ji-1,jj-1)                       &
                                   +   ff(ji  ,jj-1)  )
              END DO
           END DO

           CALL lbc_lnk( ztair_f,  'I',  1. )
           CALL lbc_lnk( zhumi_f,  'I',  1. )
           CALL lbc_lnk( zhght_f,  'I',  1. )
           CALL lbc_lnk( zhghm_f,  'I',  1. )
           CALL lbc_lnk( zrhor_f,  'I',  1. )
           CALL lbc_lnk( zpst_f,   'I',  1. )
           CALL lbc_lnk( zsq_f,    'I',  1. )
           CALL lbc_lnk( zff_f,    'I',  1. )

           ! A FIRST CALL TO FLXSURF4 FOR tau at F-points (ice model use)
           ! All exchanges recomputed later on T-Points
           ! as well as diagnostic winds for atm. model 
           CALL itf_flxsurf2( numout, jpi,jpj    , zff_f,              &
           &                    zhght_f,zhghm_f,   zpst_f, ztair_f,    &
           &                    zsq_f,  zhumi_f, z_wnds_f,             &
           &                    zwndi_f,zwndj_f         , gphif,       &
           &                    zz0_wft , zz0_wft       ,              &
           &                    Cd    , Ch              , Ce,          &
           &                    ustri ,                                &
           &                    ti2m_rpn  ,                            &
           &                    qi2m_rpn  ,                            &
           &                    ui10_rpn  , vi10_rpn , ilm_i_rpn )

           DO jj = 1, jpj
              DO ji = 1, jpi
                 zwnorm = Cd(ji,jj) * zrhor_f(ji,jj) * z_wnds_f(ji,jj)
                 ! ... ice stress at I or F -point
                 p_taui(ji,jj) = zwnorm * zwndi_f(ji,jj)
                 p_tauj(ji,jj) = zwnorm * zwndj_f(ji,jj)
              END DO
           END DO

         ELSE

           DO jj = 2, jpjm1
              DO ji = 2, jpim1
                 zwnorm = Cice_m * rhor_rpn(ji,jj) * z_wnds_f(ji,jj)
                 ! ... ice stress at I-point
                 p_taui(ji,jj) = zwnorm * zwndi_f(ji,jj)
                 p_tauj(ji,jj) = zwnorm * zwndj_f(ji,jj)
  
!froy            Since no stability functions are computed ,
!                can only assume a logarithmic profile for
!                diagnostic variables to be sent to coupler 
!                These only depend on the value of Cice and
!                atmospheric variables (see Monin-Obukhov theory)
                 ui10_rpn(ji,jj)=sf(jp_wndi)%fnow(ji,jj,1) +                      &
     &           sqrt(Cice_m)/KARMAN*( sf(jp_wndi)%fnow(ji,jj,1)                  &
     &                              - 0.25 * (pui(ji,jj+1) + pui(ji+1,jj+1)       &
     &                                      + pui(ji,jj  ) + pui(ji+1,jj  )) )*   &
     &           log(zu_diag/sf(jp_hghm)%fnow(ji,jj,1))
                 vi10_rpn(ji,jj)=sf(jp_wndj)%fnow(ji,jj,1) +                      &
     &           sqrt(Cice_m)/KARMAN*( sf(jp_wndj)%fnow(ji,jj,1)                  &
     &                              - 0.25 * (pvi(ji,jj+1) + pvi(ji+1,jj+1)       &
     &                                      + pvi(ji,jj  ) + pvi(ji+1,jj  )) )*   &
     &           log(zu_diag/sf(jp_hghm)%fnow(ji,jj,1))
              END DO
           END DO

           CALL lbc_lnk( ui10_rpn,     'T', -1. )
           CALL lbc_lnk( vi10_rpn,     'T', -1. )

         ENDIF

         CALL lbc_lnk( p_taui  , 'I', -1. )
         CALL lbc_lnk( p_tauj  , 'I', -1. )

         !
      CASE( 'C' )                  ! C-grid ice dynamics :   U & V-points (same as ocean)
         DO jj = 2, jpj
            DO ji = fs_2, jpi   ! vect. opt.
               zwndi_t(ji,jj) = (  sf(jp_wndi)%fnow(ji,jj,1)  &
                                 - 0.5 * ( pui(ji-1,jj  ) + pui(ji,jj) )  )
               zwndj_t(ji,jj) = (  sf(jp_wndj)%fnow(ji,jj,1)  &
                                 - 0.5 * ( pvi(ji  ,jj-1) + pvi(ji,jj) )  )
               z_wnds_t(ji,jj)  = SQRT( zwndi_t(ji,jj) * zwndi_t(ji,jj)  &
                                      + zwndj_t(ji,jj) * zwndj_t(ji,jj) ) * tmask(ji,jj,1)
            END DO
         END DO

         CALL lbc_lnk( zwndi_t,  'T', -1. )
         CALL lbc_lnk( zwndj_t,  'T', -1. )
         CALL lbc_lnk( z_wnds_t, 'T',  1. )
         !

         IF ( .not. rphy ) THEN

           DO jj = 2, jpj
              DO ji = fs_2, jpi   ! vect. opt.
!froy            Since no stability functions are computed ,
!                can only assume a logarithmic profile for
!                diagnostic variables to be sent to coupler 
!                These only depend on the value of Cice and
!                atmospheric variables (see Monin-Obukhov theory)
                 ui10_rpn(ji,jj)=sf(jp_wndi)%fnow(ji,jj,1) +                      &
     &           sqrt(Cice_m)/KARMAN*( sf(jp_wndi)%fnow(ji,jj,1)                  &
     &                              - 0.5 * ( pui(ji-1,jj  ) + pui(ji,jj) ) )*    &
     &           log(zu_diag/sf(jp_hghm)%fnow(ji,jj,1))
                 vi10_rpn(ji,jj)=sf(jp_wndj)%fnow(ji,jj,1) +                      &
     &           sqrt(Cice_m)/KARMAN*( sf(jp_wndj)%fnow(ji,jj,1)                  &
     &                              - 0.5 * ( pvi(ji  ,jj-1) + pvi(ji,jj) ) )*    &
     &           log(zu_diag/sf(jp_hghm)%fnow(ji,jj,1))
              END DO
           END DO
#if defined key_vectopt_loop
#endif
           DO jj = 2, jpjm1
              DO ji = fs_2, fs_jpim1   ! vect. opt.
                 p_taui(ji,jj) = 0.5 * Cice_m * rhor_rpn(ji,jj) * &
                                 ( z_wnds_t(ji+1,jj) + z_wnds_t(ji,jj) ) &
                    &          * ( 0.5 * (sf(jp_wndi)%fnow(ji+1,jj,1) + sf(jp_wndi)%fnow(ji,jj,1) ) &
                                   - pui(ji,jj) )
                 p_tauj(ji,jj) = 0.5 * Cice_m * rhor_rpn(ji,jj) * &
                                 ( z_wnds_t(ji,jj+1) + z_wnds_t(ji,jj) ) &
                    &          * ( 0.5 * (sf(jp_wndj)%fnow(ji,jj+1,1) + sf(jp_wndj)%fnow(ji,jj,1) ) &
                                   - pvi(ji,jj) )
              END DO
           END DO

           CALL lbc_lnk( p_taui  , 'U', -1. )
           CALL lbc_lnk( p_tauj  , 'V', -1. )
           CALL lbc_lnk( ui10_rpn, 'T', -1. )
           CALL lbc_lnk( vi10_rpn, 'T', -1. )

         ENDIF

      END SELECT

      IF ( .not. rphy ) THEN
        ti2m_rpn=sf(jp_tair)%fnow(:,:,1) + &
       &   sqrt(Cice_t)/KARMAN*(sf(jp_tair)%fnow(:,:,1) - pst(:,:,1)) *  &
       &   log(zt_diag/sf(jp_hght)%fnow(:,:,1))
        qi2m_rpn=sf(jp_humi)%fnow(:,:,1) + &
       &   sqrt(Cice_t)/KARMAN*(sf(jp_humi)%fnow(:,:,1) - 11637800. * EXP( -5897.8 / pst(:,:,1) ) /  &
       &   rhor_rpn(:,:)) *  &
       &   log(zt_diag/sf(jp_hght)%fnow(:,:,1))
!froy - convert z_wnds_t from m level to t level
        z_wnds_t(:,:)=z_wnds_t(:,:) + &
     &           sqrt(Cice_m)/KARMAN*( z_wnds_t(:,:) )*   &
     &           log(ZALEV_t/ZALEV_m)
      ENDIF

      zztmp = 1. / ( 1. - albo )
      !                                     ! ========================== !
      DO jl = 1, ijpl                       !  Loop over ice categories  !
         !                                  ! ========================== !
         ! ijpl=1  assumed !!

         if ( rphy ) call CdChCe_wnd10(pst,zsq_t,z_wnds_t,zwndi_t,zwndj_t,zz0_wft, &
                                       Cd,Ch,Ce,ustri,pui,pvi,p_taui,p_tauj,cd_grid)

         DO jj = 1 , jpj
            DO ji = 1, jpi
               ! ----------------------------!
               !      I   Radiative FLUXES   !
               ! ----------------------------!
               zst2 = pst(ji,jj,jl) * pst(ji,jj,jl)
               zst3 = pst(ji,jj,jl) * zst2
               ! Short Wave (sw)
               p_qsr(ji,jj,jl) = zztmp * ( 1. - palb(ji,jj,jl) ) * qsr(ji,jj)
               ! Long  Wave (lw)
               p_qlw (ji,jj) = 0.95 * (  sf(jp_qlw)%fnow(ji,jj,1)       &                         
                  &                - STEF   * pst(ji,jj,jl) * zst3  ) * tmask(ji,jj,1)
               p_t4a (ji,jj) = ( sf(jp_qlw)%fnow(ji,jj,1) - p_qlw (ji,jj) ) / STEF * tmask(ji,jj,1)
               ! lw sensitivity
               z_dqlw(ji,jj,jl) = zcoef_dqlw * zst3                                               

               ! ----------------------------!
               !     II    Turbulent FLUXES  !
               ! ----------------------------!

               ! ... turbulent heat fluxes
               C_work=Cice_t
               ! Sensible Heat
               IF ( rphy ) C_work = Ch(ji,jj)
               p_qsb(ji,jj) = rhor_rpn(ji,jj) * CPD * C_work * z_wnds_t(ji,jj) * ( pst(ji,jj,jl) - sf(jp_tair)%fnow(ji,jj,1) )
               ! Sensible heat sensitivity (Dqsb_ice/Dtn_ice)
               z_dqsb(ji,jj,jl) = rhor_rpn(ji,jj) * CPD * C_work * z_wnds_t(ji,jj)

               ! Latent Heat
               IF ( rphy ) C_work = Ce(ji,jj)
               p_qla(ji,jj,jl) = rhor_rpn(ji,jj) * CHLS  * C_work * z_wnds_t(ji,jj)   & 
                  &                    * (  zsq_t(ji,jj) - sf(jp_humi)%fnow(ji,jj,1) ) 
               ! Latent heat sensitivity for ice (Dqla/Dtn_ice)
               p_dqla(ji,jj,jl) = rhor_rpn(ji,jj) * CHLS * C_work * z_wnds_t(ji,jj)   &
                                * ( 5897.8 /  zst2 )  * zsq_t(ji,jj)
               !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
               !!!! zsq_t(:,:) = 11637800. * EXP( -5897.8 / pst(:,:,jl) ) / rhor_rpn(:,:)
               !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

               ! ----------------------------!
               !     III    Total FLUXES     !
               ! ----------------------------!
               ! Downward Non Solar flux
               p_qns (ji,jj,jl) =     p_qlw (ji,jj) - p_qsb (ji,jj) - p_qla (ji,jj,jl)      
               ! Total non solar heat flux sensitivity for ice
               p_dqns(ji,jj,jl) = - ( z_dqlw(ji,jj,jl) + z_dqsb(ji,jj,jl) + p_dqla(ji,jj,jl) )    
            END DO
            !
         END DO
         !
      END DO
      !
      !--------------------------------------------------------------------
      ! FRACTIONs of net shortwave radiation which is not absorbed in the
      ! thin surface layer and penetrates inside the ice cover
      ! ( Maykut and Untersteiner, 1971 ; Ebert and Curry, 1993 )
      !
      p_fr1(:,:) = ( 0.18 * ( 1.0 - cldf_ice ) + 0.35 * cldf_ice )
      p_fr2(:,:) = ( 0.82 * ( 1.0 - cldf_ice ) + 0.65 * cldf_ice )
      !
      p_tpr(:,:) = sf(jp_prec)%fnow(:,:,1)       ! total precipitation [kg/m2/s]
      p_spr(:,:) = sf(jp_snow)%fnow(:,:,1)       ! solid precipitation [kg/m2/s]
      CALL iom_put( 'snowpre', p_spr * 86400. )                  ! Snow precipitation
      CALL iom_put( 'precip' , p_tpr * 86400. )                  ! Total precipitation
      !
      IF(ln_ctl) THEN
         CALL prt_ctl(tab3d_1=p_qla   , clinfo1=' blk_ice_rpn: p_qla  : ', tab2d_2=p_qsb   , clinfo2=' p_qsb  : ', kdim=ijpl)
         CALL prt_ctl(tab2d_1=p_qlw   , clinfo1=' blk_ice_rpn: p_qlw  : ', tab3d_2=p_dqla  , clinfo2=' p_dqla : ', kdim=ijpl)
         CALL prt_ctl(tab3d_1=z_dqsb  , clinfo1=' blk_ice_rpn: z_dqsb : ', tab3d_2=z_dqlw  , clinfo2=' z_dqlw : ', kdim=ijpl)
         CALL prt_ctl(tab3d_1=p_dqns  , clinfo1=' blk_ice_rpn: p_dqns : ', tab3d_2=p_qsr   , clinfo2=' p_qsr  : ', kdim=ijpl)
         CALL prt_ctl(tab3d_1=pst     , clinfo1=' blk_ice_rpn: pst    : ', tab3d_2=p_qns   , clinfo2=' p_qns  : ', kdim=ijpl)
         CALL prt_ctl(tab2d_1=p_tpr   , clinfo1=' blk_ice_rpn: p_tpr  : ', tab2d_2=p_spr   , clinfo2=' p_spr  : ')
         CALL prt_ctl(tab2d_1=p_taui  , clinfo1=' blk_ice_rpn: p_taui : ', tab2d_2=p_tauj  , clinfo2=' p_tauj : ')
         CALL prt_ctl(tab2d_1=z_wnds_t, clinfo1=' blk_ice_rpn: z_wnds_t : ')
      ENDIF

      CALL wrk_dealloc( jpi,jpj, z_wnds_t, zwndi_t, zwndj_t, zrhor_f )
      CALL wrk_dealloc( jpi,jpj, z_wnds_f, zwndi_f, zwndj_f, ztair_f )
      CALL wrk_dealloc( jpi,jpj, zhumi_f, zhght_f, zhghm_f, zpst_f )
      CALL wrk_dealloc( jpi,jpj, zsq_t, zsq_f, zff_f, zz0_wft )
      CALL wrk_dealloc( jpi,jpj, Cd, Ch, Ce, ustri )
      CALL wrk_dealloc( jpi,jpj,pdim, z_dqlw, z_dqsb ) 

      !
      IF( nn_timing == 1 )  CALL timing_stop('blk_ice_rpn')

   END SUBROUTINE blk_ice_rpn

   SUBROUTINE prep_itf_flxsurf(z0mm,z0mm_adjust,z0tt)

      REAL(wp), DIMENSION(:,:), INTENT(inout) ::   z0mm              ! roughness length for momentum 
      REAL(wp), DIMENSION(:,:), INTENT(inout) ::   z0mm_adjust       ! roughness length for momentum 
      REAL(wp), DIMENSION(:,:), INTENT(inout) ::   z0tt              ! roughness length for heat and moisture

      INTEGER :: ji, jj

      ! ... Roughness length based on GEM operational setup

      select case (z0mtype)
      case ('BELJAARS','CHARNOCK')
         select case (z0mtype)
         case ('CHARNOCK')
              z0mm_adjust(:,:) = 0.
         case ('BELJAARS')
              z0mm_adjust(:,:) = am*k_visc/max(ustr_rpn(:,:),USTRMIN)
         end select
         z0mm(:,:) = MAX( MIN( BETA*ustr_rpn(:,:)**2/grav+ z0mm_adjust(:,:),Z0MAX ) , Z0MIN )
      case ('SMTHRSCL')
         ! formlsr_rpn=ALPHASR*(Cp/ua*)**BETASR (Smith et al 1992, and others)
         ! Cp/ua*=wave_age=30tanh(0.6/ua*) (Rascle 2008)
         formlsr_rpn(:,:)=ALPHASR*( 30.*TANH( 0.6/MAX(ustr_rpn(:,:),USTRMIN) ) )**BETASR
         z0mm(:,:) = MAX( MIN( formlsr_rpn(:,:)*ustr_rpn(:,:)**2/grav,Z0MAX ) , Z0MIN )
      case DEFAULT
         WRITE(numout,*) 'prep_itf_flxsurf: Unsupported z0mtype: '//trim(z0mtype)
         stop
      end select

      ! ... Special treatment in tropics
!     Note:  For |lat| >= Z0TLAT(2)  Charnock's relation is used
!            For |lat| <= Z0TLAT(1)  Z0HCON is used.
!            For Z0TLAT(1) < |lat| < Z0TLAT(2)
!            we do a linear interpolation between Charnock and Z0HCON.

      select case (z0ttype)
      case ('DEACU12')
        DO jj=1,jpj
        DO ji=1,jpi
           ustr_rpn(ji,jj) = MAX(USTRMIN,  ustr_rpn(ji,jj))
              ! no impact since ustr re-computed in flxsurf4 with z0m just
              ! computed earlier (with ustr not modified)
              ! scary but ok
           z0tt(ji,jj) = MIN(2.e-05/ustr_rpn(ji,jj), 1.e-04)
        END DO
        END DO

!in GEM(water)        z0h = min(2.e-5/max(zfrv,ZFRVMIN),1.e-4)

      case ('ECMWF')
!in GEM(water)        z0h = min(6.e-6/max(zfrv,ZFRVMIN),5.e-4)

        DO jj=1,jpj
        DO ji=1,jpi
           ustr_rpn(ji,jj) = MAX(USTRMIN,  ustr_rpn(ji,jj))
              ! no impact since ustr re-computed in flxsurf4 with z0m just
              ! computed earlier (with ustr not modified)
              ! scary but ok
           z0tt(ji,jj) = MIN(6.e-06/ustr_rpn(ji,jj), 5.e-04)
        END DO
        END DO

      case ('MOMENTUM')
        z0tt = z0mm
      case DEFAULT
        WRITE(numout,*) 'Unsupported z0ttype: '//trim(z0ttype)
      stop
      end select

      DO jj=1,jpj
      DO ji=1,jpi
        IF (ABS(gphit(ji,jj)) .GE. Z0TLAT(2)) THEN
          z0tt(ji,jj) = z0tt(ji,jj)
        ELSE IF (ABS(gphit(ji,jj)) .LE. Z0TLAT(1)) THEN
          z0tt(ji,jj) = Z0HCON
        ELSE
          z0tt(ji,jj)=( ((ABS(gphit(ji,jj))-Z0TLAT(1))/(Z0TLAT(2)-Z0TLAT(1))) &
     &                  *(z0tt(ji,jj)-Z0HCON) ) + Z0HCON
        ENDIF
      ENDDO
      ENDDO
   END SUBROUTINE prep_itf_flxsurf

   SUBROUTINE CdChCe_wnd10(pst,zsq_t,z_wnds_t,zwndi_t,zwndj_t,zz0_wft, &
                           Cd,Ch,Ce,ustri,pui,pvi,p_taui,p_tauj,cd_grid)

   REAL(wp), DIMENSION(:,:,:) :: pst
   REAL(wp), DIMENSION(:,:)   :: zsq_t,z_wnds_t,zwndi_t,zwndj_t,zz0_wft, &
                                 Cd,Ch,Ce,ustri,pui,pvi,p_taui,p_tauj
   CHARACTER(len=1), INTENT(in) ::   cd_grid  ! ice grid ( C or B-grid)`
   REAL(wp) :: zwnorm
   INTEGER  :: ji, jj, jl

         ! ------------------------------------------------------!
         !   a)    Turbulent exchange coefficients at T-points   !
         ! ------------------------------------------------------!

         jl = 1
         CALL itf_flxsurf2( numout, jpi,jpj    , ff,                 &
                              sf(jp_hght)%fnow(:,:,1), sf(jp_hghm)%fnow(:,:,1),    &
                              pst(:,:,jl),             sf(jp_tair)%fnow(:,:,1),    &
                              zsq_t,                   sf(jp_humi)%fnow(:,:,1),    &
                              z_wnds_t, zwndi_t, zwndj_t, gphit,     &
                              zz0_wft , zz0_wft       ,              &
                              Cd    , Ch              , Ce,          &
                              ustri ,                                &
                              ti2m_rpn  ,                            &
                              qi2m_rpn  ,                            &
                              ui10_rpn  , vi10_rpn , ilm_o_rpn )

         ! ------------------------------------------------------!
         !   b)    Stress and diagnostic winds at T-points       !
         ! ------------------------------------------------------!

         SELECT CASE( cd_grid )
         CASE( 'B' )                  ! B-grid ice dynamics :   I-point (i.e. F-point with sea-ice indexation)
            ! tau was computed earlier at I-points if B grid
            DO jj = 2, jpjm1
               DO ji = 2, jpim1
                  ! ... diagnostic winds at T-points
                  ui10_rpn(ji,jj)=ui10_rpn(ji,jj) + 0.25 * (  pui(ji,jj+1) + pui(ji+1,jj+1)   &
                        &                           + pui(ji,jj  ) + pui(ji+1,jj  )  )
                  vi10_rpn(ji,jj)=vi10_rpn(ji,jj) + 0.25 * (  pvi(ji,jj+1) + pvi(ji+1,jj+1)   &
                        &                           + pvi(ji,jj  ) + pvi(ji+1,jj  )  )
               END DO
            END DO
         CASE( 'C' )                  ! C-grid ice dynamics :   U & V-points (same as ocean)
            DO jj = 1, jpj
               DO ji = 1, jpi
                  zwnorm = Cd(ji,jj) * rhor_rpn(ji,jj) * z_wnds_t(ji,jj)
                  ! ... ice stress at T-point
                  p_taui(ji,jj) = zwnorm * zwndi_t(ji,jj)
                  p_tauj(ji,jj) = zwnorm * zwndj_t(ji,jj)
               END DO
            END DO
            CALL lbc_lnk( p_taui  , 'U', -1. )
            CALL lbc_lnk( p_tauj  , 'V', -1. )
            DO jj = 2, jpj
               DO ji = fs_2, jpi   ! vect. opt.
                  ! ... diagnostic winds at T-points
                  ui10_rpn(ji,jj)=ui10_rpn(ji,jj) + 0.5 * ( pui(ji-1,jj  ) + pui(ji,jj) )
                  vi10_rpn(ji,jj)=vi10_rpn(ji,jj) + 0.5 * ( pvi(ji  ,jj-1) + pvi(ji,jj) )
               END DO
            END DO
         END SELECT

         CALL lbc_lnk( ui10_rpn,     'T', -1. )
         CALL lbc_lnk( vi10_rpn,     'T', -1. )

   END SUBROUTINE CdChCe_wnd10

   SUBROUTINE turb_core_2z_rpn( zt, zu, sst, T_zt, q_sat, q_zt, dU,    &
      &                      Cd, Ch, Ce , T_zu, q_zu,                  &
      &                      T_a_d, q_a_d, dU_d, ilmo, z0m, z0h )
      !!----------------------------------------------------------------------
      !!                      ***  ROUTINE  turb_core  ***
      !!
      !! ** Purpose :   Computes turbulent transfert coefficients of surface
      !!                fluxes according to Large & Yeager (2004) and Large & Yeager (2008)
      !!                If relevant (zt /= zu), adjust temperature and humidity from height zt to zu
      !!
      !! ** Method : Monin Obukhov Similarity Theory 
      !!             + Large & Yeager (2004,2008) closure: CD_n10 = f(U_n10)
      !!
      !! ** References :   Large & Yeager, 2004 / Large & Yeager, 2008
      !!
      !! ** Last update: Laurent Brodeau, June 2014:
      !!    - handles both cases zt=zu and zt/=zu
      !!    - optimized: less 2D arrays allocated and less operations
      !!    - better first guess of stability by checking air-sea difference of virtual temperature
      !!       rather than temperature difference only...
      !!    - added function "cd_neutral_10m" that uses the improved parametrization of 
      !!      Large & Yeager 2008. Drag-coefficient reduction for Cyclone conditions!
      !!    - using code-wide physical constants defined into "phycst.mod" rather than redifining them
      !!      => 'vkarmn' and 'grav'
      !!----------------------------------------------------------------------
!froy 2D zt, zu
      REAL(wp), INTENT(in   ), DIMENSION(jpi,jpj) ::   zt       ! height for T_zt and q_zt                   [m]
      REAL(wp), INTENT(in   ), DIMENSION(jpi,jpj) ::   zu       ! height for dU                              [m]
      REAL(wp), INTENT(in   ), DIMENSION(jpi,jpj) ::   sst      ! sea surface temperature              [Kelvin]
      REAL(wp), INTENT(in   ), DIMENSION(jpi,jpj) ::   T_zt     ! potential air temperature            [Kelvin]
      REAL(wp), INTENT(in   ), DIMENSION(jpi,jpj) ::   q_sat    ! sea surface specific humidity         [kg/kg]
      REAL(wp), INTENT(in   ), DIMENSION(jpi,jpj) ::   q_zt     ! specific air humidity                 [kg/kg]
      REAL(wp), INTENT(in   ), DIMENSION(jpi,jpj) ::   dU       ! relative wind module at zu            [m/s]
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj) ::   Cd       ! transfer coefficient for momentum         (tau)
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj) ::   Ch       ! transfer coefficient for sensible heat (Q_sens)
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj) ::   Ce       ! transfert coefficient for evaporation   (Q_lat)
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj) ::   T_zu     ! air temp. shifted at zu                     [K]
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj) ::   q_zu     ! spec. hum.  shifted at zu               [kg/kg]
!froy
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj), &
                    OPTIONAL                      ::   T_a_d,  &      ! T_a at hard coded diagnostic level
                                                       q_a_d,  &      ! q_a at hard coded diagnostic level
                                                       dU_d,   &      ! dU  at hard coded diagnostic level
                                                       ilmo,   &      ! 1/L
                                                       z0m,    &      ! Derived roughness length for momentum (m)
                                                       z0h            ! Derived roughness length for heat (m)

      !
      REAL(wp), PARAMETER ::   epsTq   = 1.e-12,&  ! Minimum absolute value for dT and dq
                               zu_diag = 10.,   &  ! hard coded diagnostic level for winds
                               zt_diag = 1.5       ! hard coded diagnostic level for T and q
!
      INTEGER ::   j_itt
      INTEGER , PARAMETER ::   nb_itt = 5       ! number of itterations
      LOGICAL ::   l_zt_equal_zu = .FALSE.      ! if q and t are given at different height than U
      !
      REAL(wp), DIMENSION(:,:), POINTER ::   U_zu          ! relative wind at zu                            [m/s]
      REAL(wp), DIMENSION(:,:), POINTER ::   Ce_n10        ! 10m neutral latent coefficient
      REAL(wp), DIMENSION(:,:), POINTER ::   Ch_n10        ! 10m neutral sensible coefficient
      REAL(wp), DIMENSION(:,:), POINTER ::   sqrt_Cd_n10   ! root square of Cd_n10
      REAL(wp), DIMENSION(:,:), POINTER ::   sqrt_Cd       ! root square of Cd
      REAL(wp), DIMENSION(:,:), POINTER ::   zeta_u        ! stability parameter at height zu
      REAL(wp), DIMENSION(:,:), POINTER ::   zeta_t        ! stability parameter at height zt
      REAL(wp), DIMENSION(:,:), POINTER ::   zpsi_h_u, zpsi_m_u
      REAL(wp), DIMENSION(:,:), POINTER ::   ztmp0, ztmp1, ztmp2
      REAL(wp), DIMENSION(:,:), POINTER ::   stab          ! 1st stability test integer
!froy - 1/L and others are needed to compute diagnostic variables
      REAL(wp), DIMENSION(:,:), POINTER ::   one_L,    &   ! 1/L
                                             zpsi_d,   &   ! psi at diagnostic level
                                             zpsi_h_t, &   ! psi_h at zt
                                             T_star,   &   ! T*
                                             q_star        ! q*
!
      !!----------------------------------------------------------------------

      IF( nn_timing == 1 )  CALL timing_start('turb_core_2z_rpn')
    
      CALL wrk_alloc( jpi,jpj, U_zu, Ce_n10, Ch_n10, sqrt_Cd_n10, sqrt_Cd )
      CALL wrk_alloc( jpi,jpj, zeta_u, stab )
      CALL wrk_alloc( jpi,jpj, zpsi_h_u, zpsi_m_u, ztmp0, ztmp1, ztmp2 )
!froy
      CALL wrk_alloc( jpi,jpj, one_L, zpsi_d, zpsi_h_t, T_star, q_star )

      l_zt_equal_zu = .FALSE.
!froy      IF( ABS(zu - zt) < 0.01 ) l_zt_equal_zu = .TRUE.    ! testing "zu == zt" is risky with double precision

      IF( .NOT. l_zt_equal_zu )   CALL wrk_alloc( jpi,jpj, zeta_t )

      U_zu = MAX( 0.5 , dU )   !  relative wind speed at zu (normally 10m), we don't want to fall under 0.5 m/s

      !! First guess of stability: 
      ztmp0 = T_zt*(1. + 0.608*q_zt) - sst*(1. + 0.608*q_sat) ! air-sea difference of virtual pot. temp. at zt
      stab  = 0.5 + sign(0.5,ztmp0)                           ! stab = 1 if dTv > 0  => STABLE, 0 if unstable

      !! Neutral coefficients at 10m:
      IF( ln_cdgw ) THEN      ! wave drag case
         cdn_wave(:,:) = cdn_wave(:,:) + rsmall * ( 1._wp - tmask(:,:,1) )
         ztmp0   (:,:) = cdn_wave(:,:)
      ELSE
         ztmp0 = cd_neutral_10m( U_zu )
      ENDIF
      sqrt_Cd_n10 = SQRT( ztmp0 )
      Ce_n10  = 1.e-3*( 34.6 * sqrt_Cd_n10 )
      Ch_n10  = 1.e-3*sqrt_Cd_n10*(18.*stab + 32.7*(1. - stab))
    
      !! Initializing transf. coeff. with their first guess neutral equivalents :
      Cd = ztmp0   ;   Ce = Ce_n10   ;   Ch = Ch_n10   ;   sqrt_Cd = sqrt_Cd_n10

      !! Initializing values at z_u with z_t values:
      T_zu = T_zt   ;   q_zu = q_zt

      !!  * Now starting iteration loop
      DO j_itt=1, nb_itt
         !
         ztmp1 = T_zu - sst   ! Updating air/sea differences
         ztmp2 = q_zu - q_sat 
!froy - to avoid overflow in L
         ztmp1 = sign( max(abs(ztmp1),epsTq), ztmp1 )
         ztmp2 = sign( max(abs(ztmp2),epsTq), ztmp2 )
!

         ! Updating turbulent scales :   (L&Y 2004 eq. (7))
!froy
!         ztmp1  = Ch/sqrt_Cd*ztmp1    ! theta*
!         ztmp2  = Ce/sqrt_Cd*ztmp2    ! q*
         T_star  = Ch/sqrt_Cd*ztmp1    ! theta*
         q_star  = Ce/sqrt_Cd*ztmp2    ! q*
!
       
         ztmp0 = T_zu*(1. + 0.608*q_zu) ! virtual potential temperature at zu

!-----------------------------------------------------------------------------------------------------
!froy - issues found in the past (NEMO 3.1)
! turb_core_2z NEMO 3.1 (L. Brodeau)
! ==> Disagreement between Large and Yeager and L. Brodeau's thesis
!!froy         L = (U_star*U_star) &                ! Estimate the Monin-Obukov length at height zu
!!froy              & / (KARMAN*grav/T_vpot_u*(T_star*(1.+0.608*q_zu) + 0.608*T_zu*q_star))
!         L = (U_star*U_star) &                ! Estimate the Monin-Obukov length at height zu
!              & / (KARMAN*grav/T_vpot_u*(T_star + 0.608*T_zu*q_star))
! turb_core_1z NEMO 3.1 (Large and Yeager)
!         !! Estimate the Monin-Obukov length :
!         L  = (U_star**2)/( KARMAN*grav*(T_star/T_vpot + q_star/(q_a + 1./0.608)) )
! leads to (see scientific notes from F. Roy)
!         L  = (U_star**2)/( KARMAN*grav*/T_vpot_u*(T_star + 0.608*T_zu*q_star) )
!-----------------------------------------------------------------------------------------------------

         ! Estimate the inverse of Monin-Obukov length (1/L) at height zu:
!froy - see explanations above (go with Large and Yeager and other theoretical papers)
!         ztmp0 =  (vkarmn*grav/ztmp0*(ztmp1*(1.+0.608*q_zu) + 0.608*T_zu*ztmp2)) / (Cd*U_zu*U_zu) 
         one_L =  (vkarmn*grav/ztmp0*(T_star + 0.608*T_zu*q_star)) / (Cd*U_zu*U_zu) 
         !                                                                     ( Cd*U_zu*U_zu is U*^2 at zu)

         !! Stability parameters :
!froy         zeta_u   = zu*ztmp0   ;  zeta_u = sign( min(abs(zeta_u),10.0), zeta_u )
         zeta_u   = zu*one_L   ;  zeta_u = sign( min(abs(zeta_u),10.0), zeta_u )
         if (present(ilmo)) ilmo = zeta_u/zu
         zpsi_h_u = psi_h( zeta_u )
         zpsi_m_u = psi_m( zeta_u )
       
         !! Shifting temperature and humidity at zu (L&Y 2004 eq. (9b-9c))
         IF ( .NOT. l_zt_equal_zu ) THEN
!froy            zeta_t = zt*ztmp0 ;  zeta_t = sign( min(abs(zeta_t),10.0), zeta_t )
            zeta_t = zt*one_L ;  zeta_t = sign( min(abs(zeta_t),10.0), zeta_t )
!froy
            zpsi_h_t = psi_h( zeta_t )
!
            stab = LOG(zu/zt) - zpsi_h_u + psi_h(zeta_t)  ! stab just used as temp array!!!
!froy
!            T_zu = T_zt + ztmp1/vkarmn*stab    ! ztmp1 is still theta*
!            q_zu = q_zt + ztmp2/vkarmn*stab    ! ztmp2 is still q*
            T_zu = T_zt + T_star/vkarmn*stab
            q_zu = q_zt + q_star/vkarmn*stab
            q_zu = max(0., q_zu)
         END IF
       
         IF( ln_cdgw ) THEN      ! surface wave case
            sqrt_Cd = vkarmn / ( vkarmn / sqrt_Cd_n10 - zpsi_m_u ) 
            Cd      = sqrt_Cd * sqrt_Cd
         ELSE
           ! Update neutral wind speed at 10m and neutral Cd at 10m (L&Y 2004 eq. 9a)...
           !   In very rare low-wind conditions, the old way of estimating the
           !   neutral wind speed at 10m leads to a negative value that causes the code
           !   to crash. To prevent this a threshold of 0.25m/s is imposed.
           ztmp0 = MAX( 0.25 , U_zu/(1. + sqrt_Cd_n10/vkarmn*(LOG(zu/10.) - zpsi_m_u)) ) !  U_n10
           ztmp0 = cd_neutral_10m(ztmp0)                                                 ! Cd_n10
           sqrt_Cd_n10 = sqrt(ztmp0)
       
           Ce_n10  = 1.e-3 * (34.6 * sqrt_Cd_n10)                     ! L&Y 2004 eq. (6b)
           stab    = 0.5 + sign(0.5,zeta_u)                           ! update stability
           Ch_n10  = 1.e-3*sqrt_Cd_n10*(18.*stab + 32.7*(1. - stab))  ! L&Y 2004 eq. (6c-6d)

           !! Update of transfer coefficients:
           ztmp1 = 1. + sqrt_Cd_n10/vkarmn*(LOG(zu/10.) - zpsi_m_u)   ! L&Y 2004 eq. (10a)
           Cd      = ztmp0 / ( ztmp1*ztmp1 )   
           sqrt_Cd = SQRT( Cd )
         ENDIF
         !
         ztmp0 = (LOG(zu/10.) - zpsi_h_u) / vkarmn / sqrt_Cd_n10
         ztmp2 = sqrt_Cd / sqrt_Cd_n10
         ztmp1 = 1. + Ch_n10*ztmp0               
         Ch  = Ch_n10*ztmp2 / ztmp1  ! L&Y 2004 eq. (10b)
         !
         ztmp1 = 1. + Ce_n10*ztmp0               
         Ce  = Ce_n10*ztmp2 / ztmp1  ! L&Y 2004 eq. (10c)
         !
      END DO

      IF (present(z0m)) z0m = zu / ( exp( vkarmn/sqrt(Cd)    + zpsi_m_u ) )
      IF (present(z0h)) z0h = zt / ( exp( vkarmn*sqrt(Cd)/Ch + zpsi_h_t ) )

!froy
      IF (PRESENT(dU_d)) THEN
        IF( ln_cdgw ) THEN
          CALL ctl_stop('turb_core_2z_rpn: ln_cdgw=T, not sure about diagnostic variables in this case ...')
        ENDIF
        !! In this formulation z0 is unknown, must relate with computed
        !! level to eliminate ln(zu_diag/z0)
        !! dU_d=(U_star/vkarmn) * [ ln(zu_diag/z0) - psi_m(zu_diag/L) ]
        !! dU  =(U_star/vkarmn) * [ ln(zu     /z0) - psi_m(zu     /L) ]
        !! ===>
        !! dU_d=dU+(U_star/vkarmn)*[ln(zu_diag/zu)-psi_m(zu_diag/L)+psi_m(zu/L)]
        !! Stability parameters (in Monin-Obukov theory, L is independant of z ...)
        !! Re-use of zeta_u and zeta_t (change of signification!!!!)
        zeta_u = zu_diag*one_L ;  zeta_u   = sign( min(abs(zeta_u),10.0), zeta_u )
        zpsi_d = psi_m( zeta_u )
        dU_d = dU + (sqrt(Cd*U_zu*U_zu)/vkarmn) * ( log(zu_diag/zu) -zpsi_d + zpsi_m_u )
      ENDIF
      IF (PRESENT(T_a_d)) THEN
        if (.not.PRESENT(q_a_d)) &
          CALL ctl_stop('turb_core_2z_rpn: turb_core_2z_rpn:Inconsistent option , add q_a_d...')
        zeta_t = zt_diag*one_L ;  zeta_t   = sign( min(abs(zeta_t),10.0), zeta_t )
        zpsi_d = psi_h( zeta_t )
        T_a_d = T_zt + (T_star/vkarmn) * ( log(zt_diag/zt) -zpsi_d + zpsi_h_t )
        q_a_d = q_zt + (q_star/vkarmn) * ( log(zt_diag/zt) -zpsi_d + zpsi_h_t )
      ENDIF
!

      CALL wrk_dealloc( jpi,jpj, U_zu, Ce_n10, Ch_n10, sqrt_Cd_n10, sqrt_Cd )
      CALL wrk_dealloc( jpi,jpj, zeta_u, stab )
      CALL wrk_dealloc( jpi,jpj, zpsi_h_u, zpsi_m_u, ztmp0, ztmp1, ztmp2 )
!froy
      CALL wrk_dealloc( jpi,jpj, one_L, zpsi_d, zpsi_h_t, T_star, q_star )
!

      IF( .NOT. l_zt_equal_zu ) CALL wrk_dealloc( jpi,jpj, zeta_t )

      IF( nn_timing == 1 )  CALL timing_stop('turb_core_2z_rpn')
      !
   END SUBROUTINE turb_core_2z_rpn


   FUNCTION cd_neutral_10m( zw10 )
      !!----------------------------------------------------------------------
      !! Estimate of the neutral drag coefficient at 10m as a function 
      !! of neutral wind  speed at 10m
      !!
      !! Origin: Large & Yeager 2008 eq.(11a) and eq.(11b)
      !!
      !! Author: L. Brodeau, june 2014
      !!----------------------------------------------------------------------    
      REAL(wp), DIMENSION(jpi,jpj), INTENT(in) ::   zw10           ! scalar wind speed at 10m (m/s)
      REAL(wp), DIMENSION(jpi,jpj)             ::   cd_neutral_10m
      !
      REAL(wp), DIMENSION(:,:), POINTER ::   rgt33
      !!----------------------------------------------------------------------    
      !
      CALL wrk_alloc( jpi,jpj, rgt33 )
      !
      !! When wind speed > 33 m/s => Cyclone conditions => special treatment
      rgt33 = 0.5_wp + SIGN( 0.5_wp, (zw10 - 33._wp) )   ! If zw10 < 33. => 0, else => 1  
!froy - bug introduced in trunk_5044 and corrected in trunk_5744
!      cd_neutral_10m = 1.e-3 * ( &
!         &       (rgt33 + 1._wp)*( 2.7_wp/zw10 + 0.142_wp + zw10/13.09_wp - 3.14807E-10*zw10**6) & ! zw10< 33.
!         &      + rgt33         *      2.34   )                                                    ! zw10 >= 33.
      cd_neutral_10m = 1.e-3 * ( &
         &       (1._wp - rgt33)*( 2.7_wp/zw10 + 0.142_wp + zw10/13.09_wp - 3.14807E-10*zw10**6) & ! zw10< 33.
         &      + rgt33         *      2.34   )                                                    ! zw10 >= 33.
!
      !
      CALL wrk_dealloc( jpi,jpj, rgt33)
      !
   END FUNCTION cd_neutral_10m


   FUNCTION psi_m(pta)   !! Psis, L&Y 2004 eq. (8c), (8d), (8e)
      !-------------------------------------------------------------------------------
      ! universal profile stability function for momentum
      !-------------------------------------------------------------------------------
      REAL(wp), DIMENSION(jpi,jpj), INTENT(in) :: pta
      !
      REAL(wp), DIMENSION(jpi,jpj)             :: psi_m
      REAL(wp), DIMENSION(:,:), POINTER        :: X2, X, stabit
      !-------------------------------------------------------------------------------
      !
      CALL wrk_alloc( jpi,jpj, X2, X, stabit )
      !
      X2 = SQRT( ABS( 1. - 16.*pta ) )  ;  X2 = MAX( X2 , 1. )   ;   X = SQRT( X2 )
      stabit = 0.5 + SIGN( 0.5 , pta )
      psi_m = -5.*pta*stabit  &                                                          ! Stable
         &    + (1. - stabit)*(2.*LOG((1. + X)*0.5) + LOG((1. + X2)*0.5) - 2.*ATAN(X) + rpi*0.5)  ! Unstable
      !
      CALL wrk_dealloc( jpi,jpj, X2, X, stabit )
      !
   END FUNCTION psi_m


   FUNCTION psi_h( pta )    !! Psis, L&Y 2004 eq. (8c), (8d), (8e)
      !-------------------------------------------------------------------------------
      ! universal profile stability function for temperature and humidity
      !-------------------------------------------------------------------------------
      REAL(wp), DIMENSION(jpi,jpj), INTENT(in) ::   pta
      !
      REAL(wp), DIMENSION(jpi,jpj)             ::   psi_h
      REAL(wp), DIMENSION(:,:), POINTER        ::   X2, X, stabit
      !-------------------------------------------------------------------------------
      !
      CALL wrk_alloc( jpi,jpj, X2, X, stabit )
      !
      X2 = SQRT( ABS( 1. - 16.*pta ) )   ;   X2 = MAX( X2 , 1. )   ;   X = SQRT( X2 )
      stabit = 0.5 + SIGN( 0.5 , pta )
      psi_h = -5.*pta*stabit   &                                       ! Stable
         &    + (1. - stabit)*(2.*LOG( (1. + X2)*0.5 ))                ! Unstable
      !
      CALL wrk_dealloc( jpi,jpj, X2, X, stabit )
      !
   END FUNCTION psi_h

   !!======================================================================
#else

   !!----------------------------------------------------------------------
   !!   Default option         Empty module        NO sbcblk_rpn
   !!----------------------------------------------------------------------

#endif
  
END MODULE sbcblk_rpn
