MODULE sbcblk_rpnc

#if defined key_rpne
   !!===========================================================================
   !!                    ***  MODULE sbcblk_rpnc  ***
   !! Coupled O/A : coupled ocean-atmosphere case using GOSSIP 
   !!               NEMO LIM coupled to GEM
   !!===========================================================================
   !! History :   
   !!   NEMO 2.3                !  2008-06  JM Belanger (Original code)
   !!                           !           (inspired by M. Desgagne's work in GEM)
   !!   NEMO 3.0                !  2010-01  JM Belanger + F. Roy (Revision)
   !!   NEMO CONCEPTS 1.0.0     !  2010-09  JM Belanger (cleanup)
   !!   NEMO CONCEPTS 2.0.0     !  2012-05  F. Roy (Flux coupling)
   !!   NEMO CONCEPTS 3.0.0     !  2013-08  F. Roy (coupling with GEM 4.4.5 + clean up)
   !!   NEMO CONCEPTS 3.1.3     !  2016-01  J-M Belanger and F. Roy (YY coupling)
   !!----------------------------------------------------------------------
   !!----------------------------------------------------------------------
   !!          Atmosphere/Ocean coupling via GOSSIP 
   !!          State variable coupling
   !!          Master proc communication 
   !!----------------------------------------------------------------------
   !!----------------------------------------------------------------------

#if defined key_mpp_mpi
   USE lib_mpp, only : mpi_comm_opa     ! OPA local communicator
#endif

   USE daymod                       ! date and time info
   USE dom_oce                      ! ocean space and time domain
   USE in_out_manager               ! I/O manager
   USE phycst

   USE sbc_rpntls
   USE oce
   USE geo2ocean
   USE fldread
   USE lbclnk

   USE sbc_rpn                        ! RPN additions to sbc_oce and sbc_ice
   USE mpp_rpn                        ! gather scatter RPN routines

#if defined key_cice
   USE sbc_ice, only: ht_i,       &   ! ice thickness
                      ht_s,       &   ! snow thickness
                      u_ice,      &   ! ice velocity in x direction
                      v_ice,      &   ! ice velocity in y direction
                      alb_ice,    &   ! ice albedo
                      qla_ice,    &   ! latent heat flux over ice
                      utau_ice,   &   ! wind stress in x direction over ice
                      vtau_ice,   &   ! wind stress in y direction over ice
                      tn_ice          ! ice surface temperature
#endif

#if defined key_lim3 || defined key_lim2
!   ... No support for coupling with LIM
#endif

#if defined key_mpp_mpi
   USE mpi
#endif

   USE sbc_rpnstringutil

   IMPLICIT NONE
   PRIVATE

! Routine accessibility

   PUBLIC cplatm_init,cplatm_config, &
          cplatm_terminate,cplatm_step

! Variables

   integer, PARAMETER    ::                 &
                 cplatm_n_fldou=38,         &! Number of coupling fields to export
                 cplatm_n_fldin=10,         &! Number of coupling fields to import
                 cplatm_nsp_wnd=1,          &! Number of spread points to allow staggering
                                             ! of received winds (surface layer physics)
                 cplatm_oms_max=2,          &! Max number of other model subgrids (oms)
                 cplatm_nwmax=50             ! Max number of remapping weights 

   character(len=3), dimension(cplatm_n_fldou) :: cplatm_cvou_S
   character(len=1), dimension(cplatm_n_fldou) :: cplatm_cvot_S
   character(len=3), dimension(cplatm_n_fldin) :: cplatm_cvin_S
   character(len=1), dimension(cplatm_n_fldin) :: cplatm_cvit_S

! Atmospheric model received
   data cplatm_cvin_S /'FBA','FIA','RTA','TTA',           &
                       'UUA','VVA','QQA','PMA','PTA','P0A'/
   data cplatm_cvit_S /'S'  ,'S'  ,'S'  ,'S'  ,           &
                       'U'  ,'V'  ,'S'  ,'S'  ,'S'  ,'S'  /
! Ocean model sends
   data cplatm_cvou_S /'MCP','ALO','ALI','T4O','T4I',  &
                       'SHO','SHI','LHO','LHI',        &
                       'TXO','TYO','TXI','TYI',        &
                       'ZTO','ZQO','ZUO','ZVO',        &
                       'ZTI','ZQI','ZUI','ZVI',        &
                       'GLI','I8I','SDI','TMO',        &
                       'UUO','VVO','I7I','UUI','VVI',  &
                       'QSO','QSI','ILO','ILI',        &
                       'ZMO','ZMI','ZHO','ZHI'      /
   data cplatm_cvot_S /'S'  ,'S'  ,'S'  ,'S',  'S'  ,  &
                       'S'  ,'S'  ,'S'  ,'S'  ,        &
                       'U'  ,'V'  ,'U'  ,'V'  ,        &
                       'S'  ,'S'  ,'U'  ,'V'  ,        &
                       'S'  ,'S'  ,'U'  ,'V'  ,        &
                       'S'  ,'S'  ,'S'  ,'S'  ,        &
                       'U'  ,'V'  ,'S'  ,'U'  ,'V',    &
                       'S'  ,'S'  ,'S'  ,'S'  ,        &
                       'S'  ,'S'  ,'S'  ,'S'        /

   integer, PUBLIC, SAVE ::   &
              cplatm_atm_dt,         & ! Timestep in seconds (Atmospheric model)
              cplatm_rap_dt,         & ! Ratio of model timesteps (atm_dt / oc_dt MUST be integer)
              cplatm_i_avgb,         & ! Counter for averaging ocean_local_busou
              cplatm_n_avgb,         & ! Counter for averaging ocean_local_busou
              cplatm_W_chan(cplatm_oms_max),         & 
                                       ! GOSSIP channel name for writing info to atmospheric model
              cplatm_R_chan(cplatm_oms_max),         & 
                                       ! GOSSIP channel name for reading info from atmospheric model
              cplatm_ptype,          & ! Atmospheric processing type
              cplatm_N_cpldta,       & ! Number of atmospheric states gathered 
              cplatm_mode,   & ! Coupling mode
                  ! 0 - coupling completely off (atm. model use its own SST-ice analysis)
                  ! 1 - only send ice-ocean initial condition to atm. model
                  ! 2 - full coupled mode
              cplatm_Nsubgrid,   &   ! Number of subgrids in other (atmospheric) model 
              cplatm_nblend,     &   ! Number of points to blend coupling zone for regional grids
              cplatm_Ejmin=999,  &   ! Blending zone min index east boundary
              cplatm_Ejmax=-1 ,  &   ! Blending zone max index east boundary
              cplatm_Wjmin=999,  &   ! Blending zone min index west boundary
              cplatm_Wjmax=-1 ,  &   ! Blending zone max index west boundary
              cplatm_Nimin=999,  &   ! Blending zone min index north boundary
              cplatm_Nimax=-1 ,  &   ! Blending zone max index north boundary
              cplatm_Simin=999,  &   ! Blending zone min index south boundary
              cplatm_Simax=-1,   &   ! Blending zone max index south boundary
              cplatm_ocn_nw_max(cplatm_oms_max), &   
                                     ! Max number of valid wgts for each tile
              cplatm_gni(cplatm_oms_max),        &   
                                     ! Atmospheric mode global dimensions (in i)
              cplatm_gnj(cplatm_oms_max),        &   
                                     ! Atmospheric mode global dimensions (in j)
              cplatm_nxchng(cplatm_oms_max),     &   
                                     ! Number of compressed field points recv
              cplatm_ocn_nxchng(cplatm_oms_max), &   
                                     ! Number of compressed field points sent
              cplatm_ocn_nspread,                &
                                     ! Number of spread points relative to ocean mask
                                     ! (when compressing fields, should be 0 in the ocean in principle)
              cplatm_maxnxi, cplatm_maxnxo,      &
                                     ! Max total number of xchng between models (IO)
              cplatm_dynphy_offset,  &
                                     ! Offset in GEM between global dynamics grid and global physics grid
              cplatm_atmf_nsprd      ! Number of tolerance point to spread atm forcing near the atm
                                     ! model boundary (if ln_rpncmx=.true.), so that near wet points
                                     ! along the atm boundary have information spread from the forcing 
                                     ! database cause no wet points available from the coupled atm domain

   logical, PUBLIC, SAVE ::   &
              cplatm_debug_L         ! Print exchanged fields (debugging)

   character(len=7), PUBLIC, SAVE :: cplocn_R_chan_name(cplatm_oms_max), &
                                     cplocn_W_chan_name(cplatm_oms_max)

   NAMELIST/namsbc_rpncpl/cplatm_mode,       &
                          cplatm_Nsubgrid,   & 
                          cplatm_nblend,     &
                          cplatm_Ejmin,      &
                          cplatm_Ejmax,      &
                          cplatm_Wjmin,      &
                          cplatm_Wjmax,      &
                          cplatm_Nimin,      &
                          cplatm_Nimax,      &
                          cplatm_Simin,      &
                          cplatm_Simax,      &
                          ln_rpncmx,         &
                          cplatm_atmf_nsprd, &
                          cplatm_debug_L

   REAL(wp), DIMENSION (:,:,:,:), ALLOCATABLE, SAVE :: atm_busin    ! Local bus for import
   REAL(wp), DIMENSION (:,:,:),   ALLOCATABLE, SAVE :: atm_busou    ! Local bus for export

! Local versions

   INTEGER,  DIMENSION (:,:,:,:),   ALLOCATABLE, SAVE :: atm_iwgt, atm_jwgt
   REAL(wp), DIMENSION (:,:,:,:),   ALLOCATABLE, SAVE :: atm_wgt
   REAL(wp), DIMENSION (:,:,:),     ALLOCATABLE, SAVE :: atm_mxgt
   INTEGER,  DIMENSION (:,:,:),     ALLOCATABLE, SAVE :: atm_cwgt
   REAL(wp), DIMENSION (:,:,:),     ALLOCATABLE, SAVE :: atm_awgt
   REAL(wp), DIMENSION (:,:,:),     ALLOCATABLE, SAVE :: atm_sint
   REAL(wp), DIMENSION (:,:,:),     ALLOCATABLE, SAVE :: atm_cost
   INTEGER,  DIMENSION (:,:,:),     ALLOCATABLE, SAVE :: atm_ijtoc
   INTEGER,  DIMENSION (:,:),       ALLOCATABLE, SAVE :: ocn_ctoi
   INTEGER,  DIMENSION (:,:),       ALLOCATABLE, SAVE :: ocn_ctoj

! Work parameters related to time and cpus
   real(wp), PARAMETER  :: one=1.0_wp,sid=86400.0_wp
   real(wp), PARAMETER  :: rsid=one/sid
   CHARACTER(LEN=16), DIMENSION(:), ALLOCATABLE :: datev, date_ou
   INTEGER,  PARAMETER  :: RootProc  = 0  ! local MPI process number

! FLUX COUPLING BUS IN

!* Ice-Ocean independant
!* 1- FBA - SW down                          p
!* 2- FIA - LW down                          p
!* 3- RTA - Precipitation                    p
!* Ocean model flux calculation
!* 4- TTA - Air temperature                  d
!* 5- UUA - Wind x component                 d
!* 6- VVA - Wind y component                 d
!* 7- QQA - Specific humidity                d
!* 8- PMA - First momentum level pressure    d
!* 9- PTA - First thermo   level pressure    d
!*10- P0A - Ground level pressure            d
!* Atmospheric model flux calculated (DE-ACTIVATED)
!*11- SHO - Sensible heat flux over water    ?
!*12- SHI - Sensible heat flux over ice      ?
!*13- LHO - Latent heat flux over water      ?
!*14- LHI - Latent heat flux over ice        ?
!*15- TXO - Wind stress x component (water)  ?
!*16- TYO - Wind stress y component (water)  ?
!*17- TXI - Wind stress x component (ice)    ?
!*18- TYI - Wind stress y component (ice)    ?

CONTAINS

!!===========================================================================
   SUBROUTINE cplatm_config

   IMPLICIT NONE
   INTEGER  ::   ios      ! Local integer output status for namelist read


!authors: J-M Belanger - July 2008
!
!version: 
!
!purpose: Reads &namsbc_rpncpl from nemo ocean namelist to initialize
!         coupling configuration
!
!----------------------------------------------------------------------------

! Set defaults for namelist variables

   cplatm_mode         =     2      ! Default full coupling
   cplatm_nblend       =     0      ! Default no blending
   cplatm_Nsubgrid     =     1      ! Default atmospheric model on Yin-Yang grid    
   cplatm_atmf_nsprd   =     2      
   cplatm_debug_L      = .false.

#if defined key_lim3 || defined key_lim2
   call ctl_stop('... No support for coupling with LIM')
#endif

   REWIND( numnam_ref )              ! Namelist namsbc_rpncpl in reference namelist : RPN coupling setup
   READ  ( numnam_ref, namsbc_rpncpl, IOSTAT = ios, ERR = 91)
91 IF( ios /= 0 ) CALL ctl_nam ( ios , 'namsbc_rpncpl in reference namelist', lwp )
   !
   REWIND( numnam_cfg )              ! Namelist namsbc_rpncpl in configuration namelist : RPN coupling setup
   READ  ( numnam_cfg, namsbc_rpncpl, IOSTAT = ios, ERR = 92 )
92 IF( ios /= 0 ) CALL ctl_nam ( ios , 'namsbc_rpncpl in configuration namelist', lwp )

   IF(lwm) WRITE( numond, namsbc_rpncpl )

   if ( .not. ln_rpncmx ) cplatm_atmf_nsprd=0

   IF(lwp) THEN
      WRITE(numout,*)' '
      WRITE(numout,*)' cplatm_config : Coupling configuration'
      WRITE(numout,*)'~~~~~~~~ '
      WRITE(numout,*) '        RPN mixt coupling/forcing mode                  ln_rpncmx   = ', ln_rpncmx
      WRITE(numout,*) '        Tolerance on number of spread atm. '
      WRITE(numout,*) '               forcing points near atm BDY       cplatm_atmf_nsprd  = ', cplatm_atmf_nsprd
      WRITE(numout,*)'         RPN coupled mode                                cplatm_mode = ', cplatm_mode
      WRITE(numout,*)'         RPN number of subgrids in other (atmospheric) model  cplatm_Nsubgrid = ', cplatm_Nsubgrid
      WRITE(numout,*)'         RPN number of points in blending zone         cplatm_nblend = ', cplatm_nblend
      WRITE(numout,*)'         RPN blending SPECS: '
      WRITE(numout,*)'               cplatm_Ejmin = ', cplatm_Ejmin
      WRITE(numout,*)'               cplatm_Ejmax = ', cplatm_Ejmax
      WRITE(numout,*)'               cplatm_Wjmin = ', cplatm_Wjmin
      WRITE(numout,*)'               cplatm_Wjmax = ', cplatm_Wjmax
      WRITE(numout,*)'               cplatm_Nimin = ', cplatm_Nimin
      WRITE(numout,*)'               cplatm_Nimax = ', cplatm_Nimax
      WRITE(numout,*)'               cplatm_Simin = ', cplatm_Simin
      WRITE(numout,*)'               cplatm_Simax = ', cplatm_Simax
   ENDIF

   if ( ln_rpncmx .and. cplatm_Nsubgrid > 1 ) &
      call ctl_stop('cplatm_config: cplatm_Nsubgrid should be one for now if ln_rpncmx=T')

   return  
   END SUBROUTINE cplatm_config
!!===========================================================================

!!===========================================================================
   SUBROUTINE cplatm_init
   IMPLICIT NONE
!
   integer  mgi_init,mgi_open,mgi_write,mgi_read
   external mgi_init,mgi_open,mgi_write,mgi_read
!
   character(LEN=1024) :: fn
   character(LEN=16) :: Mod_runwork_S

   integer :: datstp,i,j,k,ier,err_cpl_init(cplatm_oms_max),err,icpla(cplatm_oms_max),icplo
   integer :: icpl_ou,icpl_in,isnd,cnt,ibidon,oms
   integer :: oc_dt,status,tag
   integer, parameter :: nsend = 51
   character(LEN=512) :: s_send(nsend)
   character(LEN=16)  :: z0mtype_R, z0ttype_R
   logical :: l_send(nsend)
   integer :: i_send(nsend)
   real(kind=sp) :: r_send(nsend), Z0TLAT_R(2)
   integer :: ios, ierror
   INTEGER,  DIMENSION (:,:), ALLOCATABLE :: ocn_ctow


   REWIND( numnam_ref )              ! Namelist namsbc_rpn in reference namelist : rpn bulk parameters
   READ  ( numnam_ref, namsbc_rpn, IOSTAT = ios, ERR = 91)
91 IF( ios /= 0 ) CALL ctl_nam ( ios , 'namsbc_rpn in reference namelist', lwp )
   !
   REWIND( numnam_cfg )              ! Namelist namsbc_rpn in configuration namelist : rpn bulk parameters
   READ  ( numnam_cfg, namsbc_rpn, IOSTAT = ios, ERR = 92 )
92 IF( ios /= 0 ) CALL ctl_nam ( ios , 'namsbc_rpn in configuration namelist', lwp )

   IF( diusst_warmlayer )   &
      &   CALL ctl_stop( 'cplatm_init: diusst_warmlayer = .true. NO PERMITED IN NEMO' )


   err_cpl_init(:)=-1

   z0mtypeU=StrUpCase(z0mtype)
   z0ttypeU=StrUpCase(z0ttype)

   do oms=1,cplatm_Nsubgrid

! Init communication channel for YIN then YAN   
                      
     select case (cplatm_Nsubgrid)
       case(1)
          cplocn_W_chan_name(1)='ocn2atm'
          cplocn_R_chan_name(1)='atm2ocn'
       case(2)                  
          cplocn_W_chan_name(1)='ocn2yin'
          cplocn_R_chan_name(1)='yin2ocn'
          cplocn_W_chan_name(2)='ocn2yan'
          cplocn_R_chan_name(2)='yan2ocn'
       case default
          write(numout,*) 'CPLATM_INIT: cplatm_Nsubgrid case option NOT IMPLEMENTED'  
          goto 999
     end select      

     s_send    = ''
     s_send(1) = 'NEMoce'
     s_send(2) = TRIM(z0mtypeU)
     s_send(3) = TRIM(z0ttypeU)

     do icpl_ou=1, cplatm_n_fldou
       isnd=3+icpl_ou
       if (isnd.gt.nsend) then
          write(6     ,9002) err_cpl_init(oms)
          write(numout,9002) err_cpl_init(oms)
          goto 999
       endif
       s_send(isnd)=cplatm_cvou_S(icpl_ou)//'_'//cplatm_cvot_S(icpl_ou)
     enddo

     do icpl_in=1, cplatm_n_fldin
       isnd=3+cplatm_n_fldou+icpl_in
       if (isnd.gt.nsend) then
          write(6     ,9002) err_cpl_init(oms)
          write(numout,9002) err_cpl_init(oms)
          goto 999
       endif
       s_send(isnd)=cplatm_cvin_S(icpl_in)//'_'//cplatm_cvit_S(icpl_in)
     enddo

     if (lwp) then
       write(numout,*) 'INIT_ATM_CPL: ndate0=', ndate0
       write(numout,*) 'INIT_ATM_CPL: ndate0_hour=', ndate0_hour
     endif
     write(Mod_runstrt_S(1:8),'(i8.8)') ndate0
     Mod_runstrt_S(9:9)='.'
     write(Mod_runstrt_S(10:11),'(i2.2)') ndate0_hour
     Mod_runstrt_S(12:16)='0000 '  !Already assumed in opa

     call datp2f (datstp,Mod_runstrt_S)

     i_send(:) = 0
     i_send(1) = datstp
  
     r_send(:) = 0.0
     r_send(1) = REAL(rdt)

     l_send(:) = .false.
     if (cplatm_mode == 1)  l_send(1) = .true.
     if (cplatm_mode == 0)  l_send(2) = .true.
     if (cplatm_nblend > 0) l_send(3) = .true.

     fn = './cplao_settings.nml'

     call cplao_init  ( 'NEMoce', trim(fn), lwp,                        &
                        s_send,l_send,i_send,r_send,nsend,              & 
                        cplatm_W_chan(oms),cplocn_W_chan_name(oms),     &
                        cplatm_R_chan(oms),cplocn_R_chan_name(oms),     &
                        jpiglo,jpjglo,cplatm_gni(oms),cplatm_gnj(oms),  &
                        cplatm_n_fldin,cplatm_n_fldou,                  &
                        cplatm_ocn_nspread, ibidon,                     &
                        err_cpl_init(oms) )
     cplatm_ocn_nspread = 0 !Force this parameter to 0, do not want
                            !to send any information from outside of tmask

   enddo  !oms

   CALL MPI_BCAST(cplatm_gni, cplatm_oms_max, MPI_INTEGER, RootProc, mpi_comm_opa, ierror)
   CALL MPI_BCAST(cplatm_gnj, cplatm_oms_max, MPI_INTEGER, RootProc, mpi_comm_opa, ierror)

   if (lwp) then

     do oms=1,cplatm_Nsubgrid
        if (err_cpl_init(oms) /= 0 ) then
          write(6     ,9001) err_cpl_init(oms)
          write(numout,9001) err_cpl_init(oms)
          goto 999
        endif

        err_cpl_init(oms) = -1

        datstp=i_send(1)  ! Mod_runstrt_S in CMC date integer format (GEM)
        cplatm_dynphy_offset=i_send(2) 
                          ! Offset in GEM between global dynamic grid and global physics grid

        cplatm_atm_dt = nint(r_send(1)) ! REAL(Cstv_dt_8) (GEM)

        write (numout,*) 'cplatm_init: r_send(1)=',r_send(1)
!
        if (trim(s_send(1)).ne.'GEMatm') then
          write(6     ,9003) err_cpl_init
          write(numout,9003) err_cpl_init
          goto 999
        endif

        do icpl_in=1, cplatm_n_fldin
          isnd=3+icpl_in
          if (trim(s_send(isnd)) /=  &
              cplatm_cvin_S(icpl_in)//'_'//cplatm_cvit_S(icpl_in)) then
              write(6     ,9009) -1
              write(numout,9009) -1
              goto 999
          endif
        enddo

        do icpl_ou=1, cplatm_n_fldou
          isnd=3+cplatm_n_fldin+icpl_ou
          if (trim(s_send(isnd)) /=  &
              cplatm_cvou_S(icpl_ou)//'_'//cplatm_cvot_S(icpl_ou)) then
              write(6     ,9009) -2
              write(numout,9009) -2
              goto 999
          endif
        enddo

! Verify ratio of atm and ocean model timesteps  !

        oc_dt=rdt
        if (cplatm_atm_dt > oc_dt) then                         
  
          if (mod(cplatm_atm_dt,oc_dt) >= 1) then
            call ctl_stop('cplatm_init: cplatm_atm_dt must be a multiple of oc_dt ')
          endif
          cplatm_rap_dt = cplatm_atm_dt / oc_dt
          cplatm_ptype=11 
  
         else                                           
  
          if (mod(oc_dt,cplatm_atm_dt) >= 1) then
            call ctl_stop('cplatm_init: oc_dt must be a multiple of cplatm_atm_dt ')
          endif
          cplatm_rap_dt = oc_dt / cplatm_atm_dt
          cplatm_ptype=22 
  
        endif

!----------------------------------!
! inter-model GOSSIP communication !
!----------------------------------!

        icplo=1
        icpla=0
        ier     = mgi_write(cplatm_W_chan(oms), icplo, 1, 'I')
        err = ier
        ier     = mgi_read (cplatm_R_chan(oms), icpla(oms), 1, 'I')
        err = min(ier,err)

        icplo=icplo+icpla(oms) 
        if ( icplo /= 2 .or. err < 0 ) then
           write (6     ,9900)
           write (numout,9900)
           goto 999
        endif

        Z0TLAT_R(:) = -9.
        ier     = mgi_write(cplatm_W_chan(oms), Z0TLAT,   2, 'R')
        err = ier
        ier     = mgi_read (cplatm_R_chan(oms), Z0TLAT_R, 2, 'R')
        err = min(ier,err)

        if ( ABS(Z0TLAT(1)-Z0TLAT_R(1)) > 0.01 .or. &
             ABS(Z0TLAT(2)-Z0TLAT_R(2)) > 0.01 .or. &
             err < 0 ) then
           write (numout,*) 'err=',err
           write (numout,*) 'Z0TLAT(1)=',Z0TLAT(1)
           write (numout,*) 'Z0TLAT(2)=',Z0TLAT(2)
           write (numout,*) 'Z0TLAT_R(1)=',Z0TLAT_R(1)
           write (numout,*) 'Z0TLAT_R(2)=',Z0TLAT_R(2)
           write (6     ,9901)
           write (numout,9901)
           goto 999
        endif

        z0mtype_R = trim(s_send(2))
        if (trim(z0mtype_R) /= trim(z0mtype)) then
           write (numout,*) 'err=',err
           write (numout,*) 'z0mtype',z0mtype
           write (numout,*) 'z0mtype_R',z0mtype_R
           write (6     ,9902)
           write (numout,9902)
           goto 999
        endif

        z0ttype_R = trim(s_send(3))
        if ( trim(z0ttype) /= trim(z0ttype_R) .or. err < 0 ) then
           write (numout,*) 'err=',err
           write (numout,*) 'z0ttype  =',z0ttype
           write (numout,*) 'z0ttype_R=',z0ttype_R
           write (6     ,9903)
           write (numout,9903)
           goto 999
        endif

     enddo   !oms

   endif     !lwp

   err_cpl_init(:) = 0

999 continue

!----------------------------------------------------------------------!
! BROADCASTING parameters from model handshake to all ocean processes  !
!----------------------------------------------------------------------!

   CALL MPI_BCAST(err_cpl_init,         2, MPI_INTEGER, RootProc, mpi_comm_opa, ierror)
   CALL MPI_BCAST(cplatm_dynphy_offset, 1, MPI_INTEGER, RootProc, mpi_comm_opa, ierror)

   do oms=1,cplatm_Nsubgrid
     if (err_cpl_init(oms) /= 0) then
       call ctl_stop('cplatm_init: initialization problem')
       return
     endif
   enddo

   ALLOCATE(atm_busin(jpi,jpj,cplatm_n_fldin,cplatm_Nsubgrid), stat = ierror)
   IF (ierror > 0) THEN
     write (numout,*) ' cplatm_init: Failure in allocating atm_busin'
     call flush(numout) 
     call ctl_stop(' cplatm_init: Failure in allocating atm_busin')
   ENDIF
   atm_busin(:,:,:,:)=0.

   ALLOCATE(atm_busou(jpi,jpj,cplatm_n_fldou), stat = ierror)
   IF (ierror > 0) THEN
     write (numout,*) ' cplatm_init: Failure in allocating atm_busou'
     call flush(numout) 
     call ctl_stop(' cplatm_init: Failure in allocating atm_busou')
   ENDIF
   atm_busou(:,:,:)=0.

!Allocation for cplatm_distwg()

   if ( cplatm_Nsubgrid > 1 ) then 
     if ( (cplatm_gni(1) /= cplatm_gni(2)) .OR. (cplatm_gnj(1) /= cplatm_gnj(2)) ) then
       write (numout,*) ' cplatm_init: Failure atm subgrid dimensions not equal ',&
                          cplatm_gni(:),cplatm_gnj(:)  
       call flush(numout) 
       call ctl_stop(' cplatm_init: Failure atm subgrid dimensions not equal')
     endif
   endif

   allocate(atm_ijtoc(cplatm_gni(1),cplatm_gnj(1),cplatm_Nsubgrid))
   allocate(ocn_ctoi(jpiglo*jpjglo,cplatm_Nsubgrid),ocn_ctoj(jpiglo*jpjglo,cplatm_Nsubgrid))

   allocate (atm_wgt (jpi,jpj,cplatm_nwmax,cplatm_Nsubgrid) )
   allocate (atm_mxgt(jpi,jpj,cplatm_Nsubgrid   ) )
   allocate (atm_awgt(jpi,jpj,cplatm_Nsubgrid   ) )
   allocate (atm_sint(jpi,jpj,cplatm_Nsubgrid   ) )
   allocate (atm_cost(jpi,jpj,cplatm_Nsubgrid   ) )
   allocate (atm_cwgt(jpi,jpj,cplatm_Nsubgrid   ) )
   allocate (atm_iwgt(jpi,jpj,cplatm_nwmax,cplatm_Nsubgrid) )
   allocate (atm_jwgt(jpi,jpj,cplatm_nwmax,cplatm_Nsubgrid) )

   cplatm_maxnxo=0
   cplatm_maxnxi=0
   do oms=1,cplatm_Nsubgrid   
     call cplatm_distwgt(oms)
     cplatm_maxnxo=max(cplatm_maxnxo,cplatm_ocn_nxchng(oms))
     cplatm_maxnxi=max(cplatm_maxnxi,cplatm_nxchng(oms))
   enddo
   deallocate(atm_awgt)

!...  twist to save memory
   allocate(ocn_ctow(jpiglo*jpjglo,cplatm_Nsubgrid))
   ocn_ctow(:,:)=ocn_ctoi(:,:)
   deallocate(ocn_ctoi)
   allocate(ocn_ctoi(cplatm_maxnxo,cplatm_Nsubgrid))
   ocn_ctoi(1:cplatm_maxnxo,1:cplatm_Nsubgrid)=ocn_ctow(1:cplatm_maxnxo,1:cplatm_Nsubgrid)

   ocn_ctow(:,:)=ocn_ctoj(:,:)
   deallocate(ocn_ctoj)
   allocate(ocn_ctoj(cplatm_maxnxo,cplatm_Nsubgrid))
   ocn_ctoj(1:cplatm_maxnxo,1:cplatm_Nsubgrid)=ocn_ctow(1:cplatm_maxnxo,1:cplatm_Nsubgrid)
   deallocate(ocn_ctow)

!... validation of weight distribution concerning ocean mask
   do j=1,jpj
   do i=1,jpi
      if ( tmask(i,j,1) > 0.1 ) then
        cnt=0
        do oms=1,cplatm_Nsubgrid
          cnt=cnt+atm_cwgt(i,j,oms)
        enddo
        if ( cnt == 0 ) then
          if ( nint(atm_mxgt(i,j,1)) == 0 ) then  ! ln_rpncmx=T ==> cplatm_Nsubgrid = 1
                                            ! else atm_mxgt(i,j,*) = 0
            write(numout,*) 'cplatm_init: tmask(i,j,1) > 0 and cnt = 0 at i,j=',i,j
            call ctl_stop('cplatm_init: wet point not covered by atm grid mapping')
          endif
        endif
      endif
   enddo
   enddo
   
   CALL MPI_BCAST(datstp, 1, MPI_INTEGER, RootProc, mpi_comm_opa, ierror)

   call datf2p (Mod_runwork_S,datstp)
   if ( Mod_runwork_S /= Mod_runstrt_S ) then
     IF(lwp) THEN
        write(numout,*) 'Mod_runwork_S=', Mod_runwork_S
        write(numout,*) 'Mod_runstrt_S=', Mod_runstrt_S
     ENDIF
     call ctl_warn('Mod_runwork_S should ideally be the same as Mod_runstrt_S')
   endif

   CALL MPI_BCAST(cplatm_ptype,  1, MPI_INTEGER, RootProc, mpi_comm_opa, ierror)
   CALL MPI_BCAST(cplatm_atm_dt, 1, MPI_INTEGER, RootProc, mpi_comm_opa, ierror)
   CALL MPI_BCAST(cplatm_rap_dt, 1, MPI_INTEGER, RootProc, mpi_comm_opa, ierror)

   cnt = len(cplatm_cvin_S(1)) * cplatm_n_fldin
   CALL MPI_BCAST(cplatm_cvin_S, cnt, MPI_CHARACTER, RootProc, mpi_comm_opa, ierror)
   cnt = len(cplatm_cvit_S(1)) * cplatm_n_fldin
   CALL MPI_BCAST(cplatm_cvit_S, cnt, MPI_CHARACTER, RootProc, mpi_comm_opa, ierror)
   cnt = len(cplatm_cvou_S(1)) * cplatm_n_fldou
   CALL MPI_BCAST(cplatm_cvou_S, cnt, MPI_CHARACTER, RootProc, mpi_comm_opa, ierror)
   cnt = len(cplatm_cvot_S(1)) * cplatm_n_fldou
   CALL MPI_BCAST(cplatm_cvot_S, cnt, MPI_CHARACTER, RootProc, mpi_comm_opa, ierror)

   IF(lwp) THEN
      write(numout,*) 'cplatm_init: ndastp,adatrj:',ndastp,adatrj
      write(numout,*) 'cplatm_init: cplatm_atm_dt=',cplatm_atm_dt,' cplatm_rap_dt=',cplatm_rap_dt
      write(numout,*) 'cplatm_init: datstp=',datstp,' Mod_runwork_S=',Mod_runwork_S

      write (numout,*) ' cplatm_init: finished normally'
   ENDIF

 9001 format (/'cplatm_init: problem in cplatm_init - ABORT',i3/)
 9002 format (/'cplatm_init: NOT ENOUGH STORAGE FOR s_send - ABORT',i3/)
 9003 format (/'cplatm_init: Model name definition inconsistent - ABORT',i3/)
 9004 format (/'cplatm_init: cplatm_atm_dt must be a multiple of oc_dt - ABORT',i3/)
 9005 format (/'cplatm_init: oc_dt must be a multiple of cplatm_atm_dt - ABORT',i3/)
 9006 format (/'cplatm_init: problems counting weights - ABORT',i3/)
 9007 format (/'cplatm_init: allocation problem - ABORT',i3/)
 9008 format (/'cplatm_init: weight reading problem - ABORT',i3/)
 9009 format (/'cplatm_init: ORDER NOT RESPECTED IN COUPLING SEND RECEIVED FIELDS - ABORT',i3/)

 9900 format (/'cplatm_init: Unable to initialize coupling with atm. model - ABORT'/)
 9901 format (/'cplatm_init: Inconsistent values for Z0TLAT, GEM vs NEMO - ABORT'/)
 9902 format (/'cplatm_init: Inconsistent values for z0mtype, GEM vs NEMO - ABORT'/)
 9903 format (/'cplatm_init: Inconsistent values for z0ttype, GEM vs NEMO - ABORT'/)

!  ---------------------------------------------------------------
   return
   END SUBROUTINE cplatm_init
!!===========================================================================

!!===========================================================================
   SUBROUTINE cplatm_terminate (F_pcomm_L)

   implicit none
!
   logical :: F_pcomm_L

   call cplao_terminate (F_pcomm_L)

   return
   END SUBROUTINE cplatm_terminate
!!===========================================================================

!!===========================================================================
   SUBROUTINE cplatm_distwgt(oms_in)

   IMPLICIT NONE

   integer,intent(IN) :: oms_in
!----------------------------------------------------------------------------

! First call global arrays ( de-allocated after initialization )
   integer,  dimension (:,:,:), allocatable :: atm_iwgt_g ! Atm. model i weight indices
   integer,  dimension (:,:,:), allocatable :: atm_jwgt_g ! Atm. model j weight indices
   integer,  dimension (:,:),   allocatable :: atm_mwgt_g ! Mask coherent with weights
   real(sp), dimension (:,:),   allocatable :: atm_rwgt_g ! TMP VAR
   real(wp), dimension (:,:,:), allocatable :: atm_wgt_g  ! Weights in the world of NEMO
   real(sp), dimension (:,:),   allocatable :: atm_awgt_g ! Atm. grid angle
   integer,  dimension (:,:),   allocatable :: ocn_ijtoc  ! Compressed index sent to atm

   integer i,j,k,k2,nw(cplatm_oms_max),ivar,ic,isp,jsp,msk,imin,imax,jmin,jmax
   integer iproc, status, err_cpl_distwgt(cplatm_oms_max), ier, sum_err_cpl_distwgt
   character*1024 fn
   character(LEN=3) :: csubgrid
   logical          :: ghalo, found, skip_error
   real(wp), parameter :: epsw=1.d-10

   err_cpl_distwgt(oms_in) = -1

   if (oms_in == 1) csubgrid='YIN'  ! use file cplwgt.fst_YIN
   if (oms_in == 2) csubgrid='YAN'  ! use file cplwgt.fst_YAN

   allocate(atm_rwgt_g(jpiglo,jpjglo))

   call glbcoll4(atm_rwgt_g,tmask(:,:,1))

   if (lwp) then

      nw(oms_in)=-1
      if ( cplatm_Nsubgrid == 2 ) then
        fn='./cplwgt.fst'//'_'//csubgrid
       else
        fn='./cplwgt.fst'
      endif

      call cplao_cnte_w(fn,jpiglo,jpjglo,nw(oms_in))

      write (numout,*) 'CPLATM_DISTWGT: oms_in,nw(oms_in)= ',oms_in,nw(oms_in)

      if (nw(oms_in) <= 0 .OR. nw(oms_in) > cplatm_nwmax) then
         write (numout,*) 'cplatm_distwgt: PROBLEM WITH ATM COUPLING WEIGHTS READING'
         goto 999
      endif

      allocate (atm_wgt_g (jpiglo,jpjglo,nw(oms_in)) )
      allocate (atm_iwgt_g(jpiglo,jpjglo,nw(oms_in)) )
      allocate (atm_jwgt_g(jpiglo,jpjglo,nw(oms_in)) )
      allocate (atm_mwgt_g(jpiglo,jpjglo   ) )
      allocate (atm_awgt_g(jpiglo,jpjglo   ) )

      call cplao_read_w(fn,atm_iwgt_g,atm_jwgt_g,atm_mwgt_g, &
                        atm_awgt_g,atm_wgt_g,                &
                        jpiglo,jpjglo,nw(oms_in),            &
                        err_cpl_distwgt(oms_in))

      if (err_cpl_distwgt(oms_in) < 0) then
         write (numout,*) 'cplatm_distwgt: PROBLEM WITH ATM COUPLING WEIGHTS READING 2'
         goto 999
      endif

! Reduce atm_mwgt_g according to tmask for optimization

      do j=1,jpjglo
      do i=1,jpiglo
         !Accept this mask discrepency only if YY or Mixt coupling/forcing (ln_rpncmx=T)
         if ( nint(atm_rwgt_g(i,j)) == 1 .and. &
                   atm_mwgt_g(i,j)  == 0 .and.  cplatm_Nsubgrid == 1 ) then
            if ( .not. ln_rpncmx ) then
              write (numout,*) 'cplatm_distwgt: cplatm_Nsubgrid=1, should not have'
              write (numout,*) 'cplatm_distwgt: any unused ocean point in mapping unless ln_rpncmx=T'
              err_cpl_distwgt(oms_in) = -1
              goto 999
             else
              atm_rwgt_g(i,j) = 0. ! Reduced for compress global send
                                   ! Do not couple under atm. forcing
            endif
         endif
      enddo
      enddo

! Compression of sent coupled global fields
! (based on tmask, atm should not use others)
 
      allocate(ocn_ijtoc(jpiglo,jpjglo))

      ic=0
      ocn_ijtoc(:,:)=-9
      do j=1,jpjglo
      do i=1,jpiglo
         if ( atm_rwgt_g(i,j) > 0.5 ) then
            ic=ic+1
            ocn_ijtoc(i,j)=ic
            ocn_ctoi(ic,oms_in)=i
            ocn_ctoj(ic,oms_in)=j
         endif
      enddo
      enddo
      cplatm_ocn_nxchng(oms_in)=ic

      call cplao_select_chan(cplatm_W_chan(oms_in),cplatm_R_chan(oms_in))      
      call cplao_xchng_cmap(ocn_ijtoc,jpiglo,jpjglo,               &
                            cplatm_ocn_nxchng(oms_in),             &
                            atm_ijtoc(1:cplatm_gni(oms_in),1:cplatm_gnj(oms_in),oms_in), &
                            cplatm_gni(oms_in),cplatm_gnj(oms_in), &
                            cplatm_nxchng(oms_in),err_cpl_distwgt(oms_in))

      if (err_cpl_distwgt(oms_in) < 0) then
         write (numout,*) 'PROBLEM WITH COMPRESSED MAP SENDING, subgrid=',(oms_in)
         goto 999
      endif

      deallocate(ocn_ijtoc)

   endif ! lwp
   deallocate(atm_rwgt_g)

   CALL MPI_BCAST(nw(oms_in),1, MPI_INTEGER, RootProc, mpi_comm_opa,           &
                  err_cpl_distwgt(oms_in))
   CALL MPI_BCAST(cplatm_nxchng(oms_in),1, MPI_INTEGER, RootProc, mpi_comm_opa,&
                  err_cpl_distwgt(oms_in))
   CALL MPI_BCAST(atm_ijtoc(1:cplatm_gni(oms_in),1:cplatm_gnj(oms_in),oms_in), &
                  cplatm_gni(oms_in)*cplatm_gnj(oms_in),                       &
                  MPI_INTEGER, RootProc, mpi_comm_opa,                         &
                  err_cpl_distwgt(oms_in))

   if (.not.lwp) then

      allocate (atm_wgt_g (1,1,nw(oms_in)) )
      allocate (atm_iwgt_g(1,1,nw(oms_in)) )
      allocate (atm_jwgt_g(1,1,nw(oms_in)) )
      allocate (atm_awgt_g(1,1   ) )

   endif

! Distribute weight interpolation variables

   do k=1,nw(oms_in)
     call glbdist8(atm_wgt( :,:,k,oms_in),atm_wgt_g( :,:,k))
     call glbdisti(atm_iwgt(:,:,k,oms_in),atm_iwgt_g(:,:,k))
     call glbdisti(atm_jwgt(:,:,k,oms_in),atm_jwgt_g(:,:,k))
   enddo
   call glbdist4(atm_awgt(:,:,oms_in),atm_awgt_g)

   deallocate (atm_wgt_g,  atm_iwgt_g, atm_jwgt_g, &
               atm_awgt_g )

   if (lwp) deallocate(atm_mwgt_g)

! Adjust rotation angles
! (atm_awgt originally represents dst grid relative to src grid, anticlockwise)

   do j=1,jpj
   do i=1,jpi
     atm_awgt(i,j,oms_in)=-atm_awgt(i,j,oms_in)*rpi/180.
     ! (anticlockwise ==> clockwise, see rotation routine)
     atm_sint(i,j,oms_in)=sin(atm_awgt(i,j,oms_in))
     atm_cost(i,j,oms_in)=cos(atm_awgt(i,j,oms_in))
   enddo
   enddo

! Compute number of valid weights for each tile
! For received fields, must extend outside tmask
! to avoid problems when computing quantities 
! over staggered grid (T, F or U-points)
! with received winds

   imin=cplatm_dynphy_offset+1
   jmin=cplatm_dynphy_offset+1
   imax=cplatm_gni(oms_in)-cplatm_dynphy_offset
   jmax=cplatm_gnj(oms_in)-cplatm_dynphy_offset
   cplatm_ocn_nw_max(oms_in)=0
   do j=1,jpj
   do i=1,jpi
     atm_cwgt(i,j,oms_in) = 0
     msk=0
     do jsp=max(j-cplatm_nsp_wnd,1),min(j+cplatm_nsp_wnd,jpj)
     do isp=max(i-cplatm_nsp_wnd,1),min(i+cplatm_nsp_wnd,jpi)
        if (tmask(isp,jsp,1) > 0.1) msk=1
     enddo
     enddo
     if (msk == 1) then
       found=.false.
       do k=nw(oms_in),1,-1
         if ( .not. found ) then
           if ( abs(atm_wgt(i,j,k,oms_in)) > epsw ) then
             !Small negative weights are possible under limitroff point positions
             found=.true.
             atm_cwgt(i,j,oms_in) = k
           endif
         endif
       enddo
       ghalo = .false.
       do k=1,atm_cwgt(i,j,oms_in)
         if ( atm_iwgt(i,j,k,oms_in) < imin  .or. &
              atm_iwgt(i,j,k,oms_in) > imax  .or. &
              atm_jwgt(i,j,k,oms_in) < jmin  .or. &
              atm_jwgt(i,j,k,oms_in) > jmax )     &
              ghalo = .true.
       enddo
       if ( ghalo ) atm_cwgt(i,j,oms_in) = 0
       cplatm_ocn_nw_max(oms_in) = max(cplatm_ocn_nw_max(oms_in),atm_cwgt(i,j,oms_in))
     endif
   enddo
   enddo

   if ( ln_rpncmx ) then
     atm_mxgt(:,:,oms_in)=1.
    else
     atm_mxgt(:,:,oms_in)=0.
   endif

! Quality control on compressed wet point indices
   if ( cplatm_ocn_nw_max(oms_in) /= 0 ) then
      do j=1,jpj
      do i=1,jpi
        if ( atm_cwgt(i,j,oms_in) > 0 ) atm_mxgt(i,j,oms_in) = 0.
        skip_error=.false.
        do k=1,atm_cwgt(i,j,oms_in)
            ic=atm_ijtoc(atm_iwgt(i,j,k,oms_in),atm_jwgt(i,j,k,oms_in),oms_in)
            if ( ic < 1 .or. ic > cplatm_nxchng(oms_in) ) then
               if ( ic < 1 .and. cplatm_atmf_nsprd > 0 ) then
                 if ( atm_iwgt(i,j,k,oms_in) <= cplatm_atmf_nsprd .or. &
                      atm_iwgt(i,j,k,oms_in) >= (cplatm_gni(oms_in) - cplatm_atmf_nsprd + 1) .or. &
                      atm_jwgt(i,j,k,oms_in) <= cplatm_atmf_nsprd .or. &
                      atm_jwgt(i,j,k,oms_in) >= (cplatm_gnj(oms_in) - cplatm_atmf_nsprd + 1) ) &
                      skip_error=.true.
               endif
               if ( .not. skip_error ) then
                 write (numout,*) 'PROBLEM WITH RECEIVED COMPRESSED INDICES atm_ijtoc'
                 write (numout,*) 'GLOBAL ATM GRID INDEX=',atm_iwgt(i,j,k,oms_in),atm_jwgt(i,j,k,oms_in)
                 write (numout,*) 'ic=',ic
                 write (numout,*) 'i,j,k=',i,j,k
                 write (numout,*) 'atm_iwgt(i,j,k,oms_in)=',atm_iwgt(i,j,k,oms_in)
                 write (numout,*) 'atm_jwgt(i,j,k,oms_in)=',atm_jwgt(i,j,k,oms_in)
                 do k2=1,atm_cwgt(i,j,oms_in)
                   write (numout,*) 'i,j,k2=',i,j,k2
                   write (numout,*) 'atm_iwgt(i,j,k2,oms_in)=',atm_iwgt(i,j,k2,oms_in)
                   write (numout,*) 'atm_jwgt(i,j,k2,oms_in)=',atm_jwgt(i,j,k2,oms_in)
                 enddo
                 write (numout,*) 'TRY TO INCREASE cplao_nspread_atm in cplao_settings.nml'
                 write (numout,*) 'OR IF ln_rpncmx=.true. and GLOBAL ATM GRID INDEX IS NEAR ATM BDY'
                 write (numout,*) 'TRY TO INCREASE cplatm_atmf_nsprd in NEMO namelist'
                 write (numout,*) 'ERROR WILL ABORT!'
                 call flush(numout)
                 err_cpl_distwgt(oms_in)=-1
                 goto 999
               endif
            endif
        enddo
        if ( skip_error ) then
          atm_mxgt(i,j,oms_in) = 1.
          atm_cwgt(i,j,oms_in) = 0.
        endif
      enddo
      enddo
   endif

   !init default bidon land values 
   !for where atm_cwgt(i,j) was set to zero
   !avoid crash in turb_core or flxsurf
   do ivar = 1, cplatm_n_fldin
     SELECT CASE ( cplatm_cvin_S(ivar) )
     CASE ( 'FBA' )
        atm_busin(:,:,ivar,oms_in)=0.
     CASE ( 'FIA' )
        atm_busin(:,:,ivar,oms_in)=0.
     CASE ( 'RTA' )
        atm_busin(:,:,ivar,oms_in)=0.
     CASE ( 'TTA' )
        atm_busin(:,:,ivar,oms_in)=275.
     CASE ( 'UUA' )
        atm_busin(:,:,ivar,oms_in)=0.
     CASE ( 'VVA' )
        atm_busin(:,:,ivar,oms_in)=0.
     CASE ( 'QQA' )
        atm_busin(:,:,ivar,oms_in)=0.2
     CASE ( 'PMA' )
        atm_busin(:,:,ivar,oms_in)=100100.
     CASE ( 'PTA' )
        atm_busin(:,:,ivar,oms_in)=100200.
     CASE ( 'P0A' )
        atm_busin(:,:,ivar,oms_in)=100300.
     CASE DEFAULT
        call ctl_stop('cplatm_distwgt: unknown variable atm_busin')
     END SELECT
   enddo

   err_cpl_distwgt(oms_in) = 0

999 continue

   CALL MPI_REDUCE(err_cpl_distwgt(oms_in), sum_err_cpl_distwgt, &
                                       1, MPI_INTEGER,           &
                                          MPI_SUM,     RootProc, mpi_comm_opa, ier)
   CALL MPI_BCAST(sum_err_cpl_distwgt, 1, MPI_INTEGER, RootProc, mpi_comm_opa, ier)
   if (sum_err_cpl_distwgt /= 0) then
     call ctl_stop('cplatm_distwgt: initialization problem')
     return
   endif

   END SUBROUTINE cplatm_distwgt
!!===========================================================================


!!===========================================================================
   SUBROUTINE cplatm_step (sd,kt_sbc)

#if defined key_mpp_mpi
   USE lib_mpp, only : mpi_comm_opa
#endif

   IMPLICIT NONE
   TYPE(FLD), INTENT(inout), DIMENSION(:) ::   sd
   INTEGER, INTENT( in ) ::   kt_sbc

   integer:: ji,jf
   integer :: send_flag, recv_flag, n_comm, ierror

   integer :: yy,mo,dd,hh,mm,ss,dum
   real*8 :: tx,dayfrac
   real(wp) ::   b

   integer :: k, ii, jj , N_avg, n_atm, n_atm0
        
   IF (kt_sbc == nitend+1) THEN
     call cplatm_busou( kt_sbc )
     DEALLOCATE(atm_busin,stat=ierror)
     DEALLOCATE(atm_busou,stat=ierror)
     return
   ENDIF

   SELECT CASE (cplatm_ptype)   

      CASE ( 11 )

         IF (kt_sbc == 1) THEN
            IF(lwp) WRITE(numout,*)
            IF(lwp) WRITE(numout,*) 'cplatm_step: dt_oce < cplatm_atm_dt, time averaging atm states in atmospheric timestep'
         ENDIF

         IF (.NOT. ALLOCATED(datev)) THEN
           ALLOCATE(datev(0:1),date_ou(0:1))
         ENDIF

         IF (kt_sbc == 1) THEN

           datev  (1)       = Mod_runstrt_S
           date_ou(1)(1:15) = '00000000.000000'
           n_comm = 1
! First receive and steps to provide initial state to atm
!           0             1
!(atm)     ---->--------------------->
!              | 
!              |  
!              |   
!              |         
!              |             
!              v  
!                (i   ii  iii  ... n)    (i   ii  iii  ... n)
!(oce)          ---->---->---->...----> ---->---->---->...---->
!                         1                        2
           IF(lwp) WRITE(numout,*) &
              'cplatm_step: First exchange, receiving 1 atm. field'

         ELSEIF (kt_sbc == cplatm_rap_dt+1) THEN

           dayfrac = dble(kt_sbc-1)            *rdt *rsid
           call incdatsd  (datev(0),Mod_runstrt_S,dayfrac)
           date_ou(0)    = datev(0)
           dayfrac = (dble(kt_sbc-1) + cplatm_rap_dt) *rdt *rsid
           call incdatsd  (datev(1),Mod_runstrt_S,dayfrac)
           date_ou(1)    = datev(1)
           n_comm=2
! Regular exchange plus send for begining sequence of atm run
!               1                    2
!(atm)--------------------->-------------------->
!     ^                                     ^  -
!      -                                 -   -
!        -                          -     - 
!           -                    -    -  
!               -             -  -
!                    -       - v
!                        -    (i   ii  iii  ... n)  
!(oce)                       ---->---->---->...----> 
!                                      2           

         ELSE

           dayfrac = (dble(kt_sbc-1) + cplatm_rap_dt) *rdt *rsid
           call incdatsd  (datev(1),Mod_runstrt_S,dayfrac)
           date_ou(1)    = datev(1)
           n_comm=0
           if ( mod(kt_sbc-1,cplatm_rap_dt) == 0 ) n_comm = 1
! Regular exchange if ocean reached kt_sbc-1
! Exchange done at the begining of sub time step i
!                         3
!(atm)           -------------------->
!                                ^  -
!                             -   -
!                        -     - 
!                     -    -  
!                  -  -
!                 - v
!                   i   ii  iii  ... n)    (i   ii  iii  ... n)
!(oce)            ---->---->---->...----> ---->---->---->...---->
!                           3                        4

         ENDIF
        
         call cplatm_busou( kt_sbc )       ! average ocean buses for export to atmosphere

         if (n_comm /= 0) then
           n_atm = 1
           if (n_comm == 2) &  ! sending initial ocean fields to atm
             call cplatm_xchng (date_ou(0),datev(0),send_flag,recv_flag, &
                                     kt_sbc,0,.false.)
           call cplatm_xchng   (date_ou(1),datev(1),send_flag,recv_flag, &
                                     kt_sbc,1,.true.)
           call cplatm_busin( n_atm, sd )   
           do jf = 1, SIZE( sd )
             sd(jf)%fnow(:,:,1)=sd(jf)%fdta(:,:,1,n_atm)
           enddo
         endif

      CASE ( 22 )

         IF (kt_sbc == 1) THEN
            IF(lwp) WRITE(numout,*)
            IF(lwp) WRITE(numout,*) 'cplatm_step: dt_oce >= cplatm_atm_dt, time averaging every atm states in ocean timestep'
         ENDIF

         IF (.NOT. ALLOCATED(datev)) THEN
              ALLOCATE(datev(0:cplatm_rap_dt),date_ou(0:cplatm_rap_dt))
         ENDIF

         IF (kt_sbc == 1) THEN

           n_atm0=0
           n_comm=0
           datev  (0)       = Mod_runstrt_S
           date_ou(0)(1:15) = '00000000.000000'
! First receive and step to provide initial state to atm
! (atm will advance 1 full sequence before any exchange)
!                       1                         2
!           (0    i   ii  iii  ... n)    (i   ii  iii  ... n)
!(atm)     ---->---->---->---->...----> ---->---->---->...---->
!              | 
!              |  
!              |   
!              |         
!              |             
!              v  
!(oce)         ---------------------->
!                         1
!           if (ln_rstart) call ctl_stop('cplatm_step: should not restart with kt_sbc=1')
            IF(lwp) WRITE(numout,*) &
               'cplatm_step: First exchange, receiving 1 atm. field'

         ELSE

           n_atm0=1
           n_comm=cplatm_rap_dt
           IF (kt_sbc == 2) THEN

             n_atm0=0
! Regular exchange at each ending atm time steps +  1 send for atm start
!                         2                       3
!                (i   ii  iii  ... n)    (i   ii  iii  ... n)
!(atm)          ---->---->---->...----> ---->---->---->...---->
!                   -    -    -      |^
!          (0)          -    -  -    ||
!       <-------- -         -  - -   ||
!                    -          - -  ||
!                      -           - ||
!                        ----------- v|
!(oce)       ----------------------> ---------------------->
!                      1                       2

           ENDIF

! Regular exchange at each ending atm time steps 
!                         4                       5
!                (i   ii  iii  ... n)    (i   ii  iii  ... n)
!(atm)          ---->---->---->...----> ---->---->---->...---->
!                   -    -    -      |^
!                       -    -  -    ||
!                           -  - -   ||
!                               - -  ||
!                                  - ||
!                                    v|
!(oce)       ----------------------> ---------------------->
!                      3                       4
           do n_atm=n_atm0,n_comm
             dayfrac = ( (dble(kt_sbc-1) + dble(n_atm)/cplatm_rap_dt)*rdt )*rsid
             if (n_atm == 0) dayfrac = dble(kt_sbc-1)*rdt           *rsid
             call incdatsd  (datev(n_atm),Mod_runstrt_S,dayfrac)
             date_ou(n_atm) = datev(n_atm)
             if (n_atm.gt.0.and.n_atm.lt.n_comm) then
               date_ou(n_atm)(1:15)='00000000.000000'
             endif
           enddo

         ENDIF
        
         call cplatm_busou( kt_sbc )       ! prepare ocean buses for export to atmosphere
         do n_atm=n_atm0,n_comm

           if (n_atm /= n_comm) then
              call cplatm_xchng (date_ou(n_atm),datev(n_atm),send_flag,recv_flag, &
                                      kt_sbc,n_atm,.false.)
            else
              call cplatm_xchng (date_ou(n_atm),datev(n_atm),send_flag,recv_flag, &
                                      kt_sbc,n_atm,.true.)
           endif
           call cplatm_busin( n_atm, sd )   

         end do

!        Compute time average of atmospheric fields over ocean time step and store in fnow (to be used by bulk formulae)
         DO jf = 1, SIZE( sd )
           sd(jf)%fnow(:,:,1)=0.
           N_avg = cplatm_N_cpldta
           IF (kt_sbc == 1) N_avg = 1
           DO ii=1,N_avg
             sd(jf)%fnow(:,:,1)=sd(jf)%fnow(:,:,1)+sd(jf)%fdta(:,:,1,ii)
           ENDDO
           sd(jf)%fnow(:,:,1)=sd(jf)%fnow(:,:,1)/N_avg
         ENDDO

      CASE DEFAULT

         call ctl_stop('cplatm_step: cplatm_ptype NOT DEFINED')

   END SELECT

   return

   END SUBROUTINE cplatm_step
!!===========================================================================


!!===========================================================================
   SUBROUTINE cplatm_xchng (date_ou,date_in,send_flag,recv_flag, &
                                 kt_sbc,n_atm,reset_busou)
!
   IMPLICIT NONE

   character(LEN=16) :: date_ou    ! Validity date of exported field  
   character(LEN=16) :: date_in    ! Date required for imported field 
   integer :: send_flag   
   integer :: recv_flag 
   integer, intent(in)             :: kt_sbc
   integer, intent(in)             :: n_atm
   logical, intent(in)             :: reset_busou

!
!  Locals:
   real(sp), allocatable, dimension(:,:)  , save :: atm_glb_busou
   real(sp), allocatable, dimension(:,:)  , save :: atm_cmp_busin, atm_cmp_busou
   real(wp), dimension(jpi,jpj)                  :: wk1

   integer :: i,j,k, ivar, ier, ierror, iproc, status, ic,oms
   logical, SAVE      :: first=.true.
   integer            :: ji,jj,ig,jg,nvalid

   real(wp) :: fac, xpond, xval
   logical  :: debug_w

   debug_w=cplatm_debug_L

   ierror = -1

   if ( .not. allocated(atm_glb_busou) ) then
     if (lwp) then
       allocate(atm_glb_busou(jpiglo,jpjglo))
       allocate(atm_cmp_busou(cplatm_maxnxo,cplatm_Nsubgrid))
      else
       allocate(atm_glb_busou(1,1))
     endif
   endif

! Send step

   do ivar = 1, cplatm_n_fldou
      call glbcoll4(atm_glb_busou,atm_busou(:,:,ivar:ivar))

      if (lwp) then

        if ( cplatm_cvou_S(ivar) == 'MCP' ) then

           ! Initial condition for atm. model correspond to n_atm = 0
           ! Otherwise set mask to 0 if cplatm_mode == 1

           if ( cplatm_mode == 1 .and. n_atm /= 0 ) atm_glb_busou(:,:) = 0.

           if ( cplatm_nblend > 0 ) then
             ! linear decay of mask for blending
             DO jj = 1, min( cplatm_nblend, jpjglo )
               fac =  real (jj) / min( cplatm_nblend , jpjglo )
               DO ji = cplatm_Simin, cplatm_Simax
                 atm_glb_busou(ji,         jj) =  &
                 atm_glb_busou(ji,         jj) * fac
               ENDDO
               DO ji = cplatm_Nimin, cplatm_Nimax
                 atm_glb_busou(ji,jpjglo-jj+1) =  &
                 atm_glb_busou(ji,jpjglo-jj+1) * fac
               ENDDO
             ENDDO
             DO ji = 1, min( cplatm_nblend, jpiglo )
               fac =  real (ji) / min( cplatm_nblend , jpiglo )
               DO jj = cplatm_Wjmin, cplatm_Wjmax
                 atm_glb_busou(ji         ,jj) =  &
                 atm_glb_busou(ji         ,jj) * fac
               ENDDO
               DO jj = cplatm_Ejmin, cplatm_Ejmax
                 atm_glb_busou(jpiglo-ji+1,jj) =  &
                 atm_glb_busou(jpiglo-ji+1,jj) * fac
               ENDDO
             ENDDO
           endif

        endif

! Successive sends to YIN and YAN
        do oms=1,cplatm_Nsubgrid

          do i=1,cplatm_ocn_nxchng(oms)
            atm_cmp_busou(i,oms)=atm_glb_busou(ocn_ctoi(i,oms),ocn_ctoj(i,oms))
          enddo

          call cplao_select_chan(cplatm_W_chan(oms),cplatm_R_chan(oms))                
          if ( .not. debug_w ) then
            call cplao_xchng (date_ou,                           &
                              atm_cmp_busou(1:cplatm_ocn_nxchng(oms),oms),    &
                              cplatm_ocn_nxchng(oms), 'put',     &
                              cplatm_cvou_S(ivar),               &
                              send_flag, kt_sbc, ierror)
           else
            call cplao_xchng_debug (date_ou,                     &
                              atm_cmp_busou(1:cplatm_ocn_nxchng(oms),oms),    &
                              cplatm_ocn_nxchng(oms), 'put',     &
                              cplatm_cvou_S(ivar),               &
                              atm_ijtoc(1:cplatm_gni(oms),1:cplatm_gnj(oms),oms), &
                              cplatm_gni(oms), cplatm_gnj(oms),  &
                              send_flag, debug_w, kt_sbc, ierror)
            ! Here atm_ijtoc, cplatm_gni, cplatm_gnj are not used
            ! (put mode)
          endif
          if (ierror < 0) goto 999

        enddo  

      endif

   enddo

   if (.not.allocated(atm_cmp_busin)) &
        allocate(atm_cmp_busin(cplatm_maxnxi,cplatm_Nsubgrid))

! Receive step
   do ivar = 1, cplatm_n_fldin

      do oms=1,cplatm_Nsubgrid

        if (lwp) then
          call cplao_select_chan(cplatm_W_chan(oms),cplatm_R_chan(oms))                
          if ( .not. debug_w ) then
            call cplao_xchng (date_in,                                  &
                              atm_cmp_busin(1:cplatm_nxchng(oms),oms),  &
                              cplatm_nxchng(oms), 'get',                &
                              cplatm_cvin_S(ivar),                      &
                              recv_flag, kt_sbc, ierror)
           else
            call cplao_xchng_debug (date_in,                            &
                              atm_cmp_busin(1:cplatm_nxchng(oms),oms),  &
                              cplatm_nxchng(oms), 'get',                &
                              cplatm_cvin_S(ivar),                      &
                              atm_ijtoc(1:cplatm_gni(oms),1:cplatm_gnj(oms),oms), &
                              cplatm_gni(oms), cplatm_gnj(oms),         &
                              recv_flag, debug_w, kt_sbc, ierror)
            ! Here atm_ijtoc, cplatm_gni, cplatm_gnj are used    
            ! only for debug mode, writing get mode fields
          endif
          if (ierror < 0) goto 999
        endif

        call MPI_bcast(recv_flag, 1, MPI_INTEGER, 0, MPI_COMM_OPA, ier )

        if (recv_flag == 0) then

           ! distributes interpolation values

           call MPI_bcast(atm_cmp_busin(1:cplatm_nxchng(oms),oms), &
                          cplatm_nxchng(oms), MPI_REAL, 0, MPI_COMM_OPA, ier )

           ! distributed interpolation
           if ( cplatm_ocn_nw_max(oms) /= 0 ) then
             do j=1,jpj
             do i=1,jpi
               if ( atm_cwgt(i,j,oms) > 0 ) atm_busin(i,j,ivar,oms)=0.
               do k=1,atm_cwgt(i,j,oms)
                   ic=atm_ijtoc(atm_iwgt(i,j,k,oms),atm_jwgt(i,j,k,oms),oms)
                   atm_busin(i,j,ivar,oms)=atm_busin(i,j,ivar,oms)+ &
     &                   atm_wgt(i,j,k,oms)*atm_cmp_busin(ic,oms)
               enddo
             enddo
             enddo

           endif

        endif

      enddo    !  loop on oms

   enddo

   if (recv_flag == 0) then

     ! vector rotation
     do oms=1,cplatm_Nsubgrid

        if ( cplatm_ocn_nw_max(oms) /= 0 ) then
          do ivar=1, cplatm_n_fldin
             if (cplatm_cvit_S(ivar) == 'U') then
                if (ivar+1.gt.cplatm_n_fldin) then
                   ierror = -1
                   goto 999
                endif
                if (cplatm_cvit_S(ivar+1) /= 'V') then
                   ierror = -1
                   goto 999
                endif
!                call rotate_vector_ninj(atm_busin(:,:,ivar:ivar,oms),     &
!                                        atm_busin(:,:,ivar+1:ivar+1,oms), &
!                                        atm_awgt(:,:,oms),jpi,jpj)
                call rotate_vector_ninj2(atm_busin(:,:,ivar:ivar,oms),     &
                                         atm_busin(:,:,ivar+1:ivar+1,oms), &
                                         atm_sint(:,:,oms),atm_cost(:,:,oms),jpi,jpj)
             endif
          enddo
        endif

     enddo

     ! Put all results in first row of atm_busin
     ! with appropriate ponderation
     if ( cplatm_Nsubgrid > 1 ) then
 
       do j=1,jpj
       do i=1,jpi
          nvalid=0
          do oms=1, cplatm_Nsubgrid
            if ( atm_cwgt(i,j,oms) > 0 ) then
               nvalid=nvalid+1 
            endif
          enddo
          xpond=1.
          if ( nvalid > 0 ) then
            xpond=xpond/nvalid
            do ivar = 1, cplatm_n_fldin
               xval=0.
               do oms=1, cplatm_Nsubgrid
                 if ( atm_cwgt(i,j,oms) > 0 ) &
                    xval=xval+atm_busin(i,j,ivar,oms)*xpond
               enddo
               atm_busin(i,j,ivar,1)=xval
            enddo
          endif
       enddo
       enddo

     endif

   endif

   if (recv_flag /= 0) then
     ! Dummy communication step. Re-using atm_busin
     if (lwp) &
     write(numout,*) 'cplatm_xchng: After cplao_xchng recv_flag =',recv_flag, &
                ' -> Dummy communication step. Re-using atm_busin'
    else
     ! Useful communication step. Using atm_busin
     if (lwp) &
     write(numout,*) 'cplatm_xchng: After cplao_xchng recv_flag =',recv_flag, &
                ' -> Useful communication step. Using atm_busin'
   endif 

! Reset variable to zero at the end of cumulation
   IF (reset_busou .and. cplatm_i_avgb == cplatm_n_avgb) THEN
     if (lwp) write(numout,*) 'cplatm_xchng: atm_busou reset'
     cplatm_i_avgb = 0
     atm_busou(:,:,:) = 0.
   ENDIF

   ierror = 0
   return

999 continue

   call MPI_bcast(ierror, 1, MPI_INTEGER, 0, MPI_COMM_OPA, ier )
   if (ierror < 0) call ctl_stop('cplatm_xchng: abort problem')
 
   return
   END SUBROUTINE cplatm_xchng
!!===========================================================================


!!===========================================================================
   SUBROUTINE cplatm_busou( kt_sbc )

   INTEGER :: kt_sbc
   CHARACTER(LEN=7)  :: c_stepno
   CHARACTER(LEN=15) :: clcpu

   INTEGER, PARAMETER :: ikchoix=-1
   INTEGER  ji, jj, ivar, info, k
   REAL(wp),PARAMETER :: mepsil=0.9999, mepsilr=1.0_wp-mepsil, lsmall=1.e-10

   INTEGER  :: icemsk  (jpi,jpj)

!  Prepare exported coupling fields by NEMO
!  Field sequence must be as in the file "coupleur_settings.nml"

!----------------------------------------------------------------------------
! FLUX COUPLING BUS OUT
! Atmosphere independant
! 1- MCP - Ocean model coupling mask  
! 2- ALO - Albedo (ocean)
! 3- ALI - Albedo (ice)
! 4- T4O - Radiation temperature (T**4) (ocean)
! 5- T4I - Radiation temperature (T**4) (ice)
! Ocean model flux calculated
! 6- SHO - Sensible heat flux over water 
! 7- SHI - Sensible heat flux over ice 
! 8- LHO - Latent heat flux over water 
! 9- LHI - Latent heat flux over ice   
!10- TXO - Wind stress x component (water)
!11- TYO - Wind stress y component (water)
!12- TXI - Wind stress x component (ice)
!13- TYI - Wind stress y component (ice)
!14- ZTO - Potential temperature at diagnostic level (water)
!15- ZQO - Specific humidity     at diagnostic level (water)
!16- ZUO - Wind x component      at diagnostic level (water)
!17- ZVO - Wind y component      at diagnostic level (water)
!18- ZTI - Potential temperature at diagnostic level (ice)
!19- ZQI - Specific humidity     at diagnostic level (ice)
!20- ZUI - Wind x component      at diagnostic level (ice)
!21- ZVI - Wind y component      at diagnostic level (ice)
! Atmospheric flux calculation
!22- GLI - Ice concentration
!23- I8I - Ice thickness 
!    Note: if flux coupling: for GEM outputs only
!                      else: passed to RPN physics
!24- SDI - Snow depth
!    Note: if flux coupling: for GEM outputs only
!                      else: passed to RPN physics
!25- TMO - Ocean surface temperature
!26- UUO - Ocean velocity in x direction
!27- VVO - Ocean velocity in y direction
!28- I7I - Ice surface temperature 
!    Note: if flux coupling: for GEM outputs only
!                      else: not used
!29- UUI - Ice velocity in x direction
!30- VVI - Ice velocity in y direction
!31- QSO - Saturation surf specific humidity
!32- QSI - Saturation surf specific humidity
!33- ILO - 1/L (ilmo)
!34- ILI - 1/L (ilmo)
!35- ZMO - Roughness length for momentum (log value)
!36- ZMI - Roughness length for momentum (log value)
!37- ZHO - Roughness length for heat/moisture (log value)
!38- ZHI - Roughness length for heat/moisture (log value)
!----------------------------------------------------------------------------

   cplatm_n_avgb = -9
   IF (cplatm_ptype == 11) cplatm_n_avgb = cplatm_rap_dt
   IF (cplatm_ptype == 22) cplatm_n_avgb = 1

   IF (kt_sbc == nit000 .AND. nit000 > 1) THEN 

      write(clcpu,'(I5.5)') narea
      write(c_stepno,'(I7.7)') kt_sbc - 1
      clcpu  = TRIM(ADJUSTL(clcpu))
      open(555,FILE='restart_atm_busou_'//TRIM(ADJUSTL(clcpu))//'_'//c_stepno,FORM='UNFORMATTED')
      read(555) atm_busou
      read(555) cplatm_i_avgb
      close(555)
      if (lwp) &
      write(numout,*) 'cplatm_busou: atm_busou from restart for kt_sbc-1=', kt_sbc-1

   ELSEIF ( kt_sbc == 1 ) THEN

      ! First step exchange is dummy put
      atm_busou(:,:,:) = 0.
      cplatm_i_avgb = 0
      return

   ELSE

      icemsk(:,:)=0
    
      DO jj=1,jpj
      DO ji=1,jpi
#if defined key_cice
        IF (fr0_i_rpn(ji,jj).gt.mepsilr) icemsk(ji,jj)=1
#endif
      ENDDO
      ENDDO
   
      cplatm_i_avgb = cplatm_i_avgb + 1

      IF ( cplatm_n_fldou /= 38 )  &
        call ctl_stop('cplatm_busou: inconsistent cplatm_n_fldou, will abort')

      do ivar = 1, cplatm_n_fldou

         SELECT CASE ( cplatm_cvou_S(ivar) )

         CASE ( 'MCP' )
            IF (cplatm_mode > 0) THEN
              ! Only compressed points are sent to GEM anyway
              ! Land points will end up 0 in GEM after interpolation
              ! over GEM coupling mask read in file (GEM grid)
              ! This part of atm_busou may be altered before
              ! sending data, see blending section (atm_glb_busou)
              atm_busou(:,:,ivar) = 1.0
            ELSE         ! Forced ocean simulation
              atm_busou(:,:,ivar) = 0.0      
            ENDIF
         CASE ( 'ALO' )
! Hard coded for now 
            alb_o_rpn(:,:) = 0.06
            atm_busou(:,:,ivar) = alb_o_rpn(:,:)*tmask(:,:,1) +   &
            atm_busou(:,:,ivar)
#if defined key_cice
         CASE ( 'ALI' )
! Hard coded for now 
            alb_ice(:,:,1) = 0.97
            atm_busou(:,:,ivar) = (alb_ice(:,:,1)   *icemsk(:,:)+ &
                                   alb_o_rpn(:,:)*(1-icemsk(:,:)))&
                                *tmask(:,:,1)+                    &
            atm_busou(:,:,ivar)
#endif
         CASE ( 'T4O' )
            atm_busou(:,:,ivar) = t4a_o_rpn (:,:)*tmask(:,:,1) +  &
            atm_busou(:,:,ivar)
#if defined key_cice
         CASE ( 'T4I' )
            atm_busou(:,:,ivar) = (t4a_i_rpn(:,:)   *icemsk(:,:)+ &
                                   t4a_o_rpn(:,:)*(1-icemsk(:,:)))&
                                *tmask(:,:,1)+                    &
            atm_busou(:,:,ivar)
#endif
         CASE ( 'SHO' )
            atm_busou(:,:,ivar) = qsb_o_rpn(:,:)*tmask(:,:,1)+    &
            atm_busou(:,:,ivar)
#if defined key_cice
         CASE ( 'SHI' )
            atm_busou(:,:,ivar) = (qsb_i_rpn(:,:)   *icemsk(:,:)+ &
                                   qsb_o_rpn(:,:)*(1-icemsk(:,:)))&
                                *tmask(:,:,1)+                    &
            atm_busou(:,:,ivar)
#endif
         CASE ( 'LHO' )
            atm_busou(:,:,ivar) = qla_o_rpn(:,:)*tmask(:,:,1)+    &
            atm_busou(:,:,ivar)
#if defined key_cice
         CASE ( 'LHI' )
            atm_busou(:,:,ivar) = (qla_ice(:,:,1)   *icemsk(:,:)+ &
                                   qla_o_rpn(:,:)*(1-icemsk(:,:)))&
                                *tmask(:,:,1)+                    &
            atm_busou(:,:,ivar)
#endif
         CASE ( 'TXO' )
            atm_busou(:,:,ivar) = uta_o_rpn(:,:)*umask(:,:,1)+    &
            atm_busou(:,:,ivar)
         CASE ( 'TYO' )
            atm_busou(:,:,ivar) = vta_o_rpn(:,:)*vmask(:,:,1)+    &
            atm_busou(:,:,ivar)
#if defined key_cice
         CASE ( 'TXI' )
            atm_busou(:,:,ivar) = (utau_ice(:,:)   *icemsk(:,:)+  &
                                  uta_o_rpn(:,:)*(1-icemsk(:,:))  &
                                  *umask(:,:,1))                  &
                                  *tmask(:,:,1)+                  &
            atm_busou(:,:,ivar)
         CASE ( 'TYI' )
            atm_busou(:,:,ivar) = (vtau_ice(:,:)   *icemsk(:,:)+  &
                                  vta_o_rpn(:,:)*(1-icemsk(:,:))  &
                                  *vmask(:,:,1))                  &
                                  *tmask(:,:,1)+                  &
            atm_busou(:,:,ivar)
            ! ... note: using utau_ice here as if valid at T-Point (see sbcice_cice.F90)
            ! ... note: using vtau_ice here as if valid at T-Point
            !          (rigourous if CICE, not completely if LIM)
#endif
         CASE ( 'ZTO' )
            atm_busou(:,:,ivar) = to2m_rpn(:,:)*tmask(:,:,1) +    &
            atm_busou(:,:,ivar)
         CASE ( 'ZQO' )
            atm_busou(:,:,ivar) = qo2m_rpn(:,:)*tmask(:,:,1) +    &
            atm_busou(:,:,ivar)
         CASE ( 'ZUO' )
            atm_busou(:,:,ivar) = uo10_rpn(:,:)*tmask(:,:,1) +    &
            atm_busou(:,:,ivar)
         CASE ( 'ZVO' )
            atm_busou(:,:,ivar) = vo10_rpn(:,:)*tmask(:,:,1) +    &
            atm_busou(:,:,ivar)
#if defined key_cice
         CASE ( 'ZTI' )
            atm_busou(:,:,ivar) = (ti2m_rpn(:,:)   *icemsk(:,:) + &
                                   to2m_rpn(:,:)*(1-icemsk(:,:))) &
                                  *tmask(:,:,1)+                  &
            atm_busou(:,:,ivar)
         CASE ( 'ZQI' )
            atm_busou(:,:,ivar) = (qi2m_rpn(:,:)   *icemsk(:,:) + &
                                   qo2m_rpn(:,:)*(1-icemsk(:,:))) &
                                  *tmask(:,:,1)+                  &
            atm_busou(:,:,ivar)
         CASE ( 'ZUI' )
            atm_busou(:,:,ivar) = (ui10_rpn(:,:)   *icemsk(:,:) + &
                                   uo10_rpn(:,:)*(1-icemsk(:,:))) &
                                  *tmask(:,:,1)+                  &
            atm_busou(:,:,ivar)
         CASE ( 'ZVI' )
            atm_busou(:,:,ivar) = (vi10_rpn(:,:)   *icemsk(:,:) + &
                                   vo10_rpn(:,:)*(1-icemsk(:,:))) &
                                  *tmask(:,:,1)+                  &
            atm_busou(:,:,ivar)
         CASE ( 'GLI' )
            atm_busou(:,:,ivar) = (fr0_i_rpn(:,:)  *icemsk(:,:))  &
                                  *tmask(:,:,1)+                  &
            atm_busou(:,:,ivar)
         CASE ( 'I8I' )
            atm_busou(:,:,ivar) = (ht_i(:,:,1)     *icemsk(:,:)) &
                                  *tmask(:,:,1)+                 &
            atm_busou(:,:,ivar)
         CASE ( 'SDI' )
            atm_busou(:,:,ivar) = (ht_s(:,:,1)     *icemsk(:,:)) &
                                  *tmask(:,:,1)+                 &
            atm_busou(:,:,ivar)
#endif
         CASE ( 'TMO' )
            atm_busou(:,:,ivar) = (tsn(:,:,1,jp_tem)+rt0)        &
                                  *tmask(:,:,1)+                 &
            atm_busou(:,:,ivar)
         CASE ( 'UUO' )
            atm_busou(:,:,ivar) = un(:,:,1)*umask(:,:,1)+        &
            atm_busou(:,:,ivar)
         CASE ( 'VVO' )
            atm_busou(:,:,ivar) = vn(:,:,1)*vmask(:,:,1)+        &
            atm_busou(:,:,ivar)
#if defined key_cice
         CASE ( 'I7I' )
            atm_busou(:,:,ivar) = (tn_ice(:,:,1) *icemsk(:,:)+  &
                                  (tsn(:,:,1,jp_tem) + rt0)     &
                                              *(1-icemsk(:,:))) &
                                  *tmask(:,:,1)+                &
            atm_busou(:,:,ivar)
         CASE ( 'UUI' )
            atm_busou(:,:,ivar) = (u_ice(:,:)    *icemsk(:,:))  &
                                  *tmask(:,:,1)+                &
            atm_busou(:,:,ivar)
         CASE ( 'VVI' )
            atm_busou(:,:,ivar) = (v_ice(:,:)    *icemsk(:,:))  &
                                  *tmask(:,:,1)+                &
            atm_busou(:,:,ivar)
            ! ... note: using u_ice here as if valid at T-Point (see sbcice_cice.F90)
            ! ... note: using v_ice here as if valid at T-Point
            !          (rigourous if CICE, not completely if LIM)
#endif
         CASE ( 'QSO' )
            atm_busou(:,:,ivar) = qsu_o_rpn(:,:)*tmask(:,:,1) +       &
            atm_busou(:,:,ivar)
#if defined key_cice
         CASE ( 'QSI' )
            atm_busou(:,:,ivar) = (qsu_i_rpn (:,:) *icemsk(:,:)+  &
                                   qsu_o_rpn (:,:) *(1-icemsk(:,:))) &
                                *tmask(:,:,1)+ &
            atm_busou(:,:,ivar)
         CASE ( 'ILO' )
            atm_busou(:,:,ivar) = ilm_o_rpn(:,:)*tmask(:,:,1) +       &
            atm_busou(:,:,ivar)
         CASE ( 'ILI' )
            atm_busou(:,:,ivar) = (ilm_i_rpn (:,:) *icemsk(:,:)+  &
                                   ilm_o_rpn (:,:) *(1-icemsk(:,:))) &
                                *tmask(:,:,1)+ &
            atm_busou(:,:,ivar)
#endif
         CASE ( 'ZMO' ) !Log values
            atm_busou(:,:,ivar) = alog(max(lsmall,z0m_o_rpn(:,:)))*tmask(:,:,1) +       &
            atm_busou(:,:,ivar)
#if defined key_cice
         CASE ( 'ZMI' ) !Log values
            atm_busou(:,:,ivar) = (alog(max(lsmall,z0m_i_rpn (:,:))) *icemsk(:,:)+  &
                                   alog(max(lsmall,z0m_o_rpn (:,:))) *(1-icemsk(:,:))) &
                                *tmask(:,:,1)+ &
            atm_busou(:,:,ivar)
#endif
         CASE ( 'ZHO' ) !Log values
            atm_busou(:,:,ivar) = alog(max(lsmall,z0h_o_rpn(:,:)))*tmask(:,:,1) +       &
            atm_busou(:,:,ivar)
#if defined key_cice
         CASE ( 'ZHI' ) !Log values
            atm_busou(:,:,ivar) = (alog(max(lsmall,z0h_i_rpn (:,:))) *icemsk(:,:)+  &
                                   alog(max(lsmall,z0h_o_rpn (:,:))) *(1-icemsk(:,:))) &
                                *tmask(:,:,1)+ &
            atm_busou(:,:,ivar)
#endif
         CASE DEFAULT

           call ctl_stop('cplatm_busou: unknown variable name atm_busou')

         END SELECT

      enddo

      IF (kt_sbc == nitend + 1) THEN  
         write(c_stepno,'(I7.7)') kt_sbc-1
         write(clcpu,'(I5.5)') narea
         clcpu  = TRIM(ADJUSTL(clcpu))
         open(555,FILE='restart_atm_busou_'//TRIM(ADJUSTL(clcpu))//'_'//c_stepno,FORM='UNFORMATTED')
         write(555) atm_busou
         write(555) cplatm_i_avgb
         close(555)
         if (lwp) &
         write(numout,*) 'cplatm_busou: atm_busou written on restart for kt_sbc-1=', kt_sbc-1
      ENDIF

   ENDIF

   IF (cplatm_i_avgb == cplatm_n_avgb) THEN
     IF (cplatm_n_avgb /= 1) THEN
       do ivar=1,cplatm_n_fldou
         if ( cplatm_cvou_S(ivar) /= 'MCP' ) then
           atm_busou(:,:,ivar) = atm_busou(:,:,ivar) / cplatm_n_avgb
         endif
       enddo
     ENDIF
    ELSE
     IF (cplatm_ptype == 22) &
      call ctl_stop('cplatm_busou: should not come at this place')
   ENDIF

   END SUBROUTINE cplatm_busou
!!===========================================================================


!!===========================================================================
   SUBROUTINE cplatm_busin( n_atm, sd )

   integer, intent(in) :: n_atm
   TYPE(FLD), INTENT(inout), DIMENSION(:) ::   sd        ! input field related variables

   integer :: kchoix, i, j, k, ind, ivar
   real(wp) :: Tv     

!-----------------------------------------------------------------------------------------------

   ind = n_atm
   IF (n_atm == 0) ind = 1  !Two cases, kt_sbc=1 or kt_sbc=2
                            !In case 2 will be crunched when called for n_atm = 1 
                            !(i.e. dummy get exchange)

   if (cplatm_n_fldin /= 10) call ctl_stop('cplatm_busin: cplatm_n_fldin /= 10 - ABORT')

   do ivar = 1, cplatm_n_fldin

      SELECT CASE ( cplatm_cvin_S(ivar) )

      CASE ( 'FBA' )
         sd(4)%fdta(:,:,1,ind) =atm_busin(:,:,ivar,1) 
      CASE ( 'FIA' )
         sd(5)%fdta(:,:,1,ind) =atm_busin(:,:,ivar,1) 
      CASE ( 'RTA' )
         sd(7)%fdta(:,:,1,ind) =atm_busin(:,:,ivar,1)*1000. ! m/s ==> kg/m**2/s, rho=1000 kg/m**3
      CASE ( 'TTA' )
         sd(6)%fdta(:,:,1,ind) =atm_busin(:,:,ivar,1) 
      CASE ( 'UUA' )
         sd(1)%fdta(:,:,1,ind) =atm_busin(:,:,ivar,1) 
      CASE ( 'VVA' )
         sd(2)%fdta(:,:,1,ind) =atm_busin(:,:,ivar,1) 
      CASE ( 'QQA' )
         sd(3)%fdta(:,:,1,ind) =atm_busin(:,:,ivar,1) 
      CASE ( 'PMA' )
         sd(12)%fdta(:,:,1,ind)=atm_busin(:,:,ivar,1)
      CASE ( 'PTA' )
         sd(11)%fdta(:,:,1,ind)=atm_busin(:,:,ivar,1)
      CASE ( 'P0A' )
         sd(13)%fdta(:,:,1,ind)=atm_busin(:,:,ivar,1)

      CASE DEFAULT
         call ctl_stop('cplatm_busin: unknown variable atm_busin')

      END SELECT

   enddo

!... Set mask for mixt coupling/forcing, assumes cplatm_Nsubgrid=1 for now
!... We put this here so that reset of sd when averaging does not reset the mask
!... to zero, redundant as atm_mxgt is constant, but more practical
!... When atm_mxgt(:,:,1)=1 uses atm. forcing
   sd(14)%fdta(:,:,1,ind)=atm_mxgt(:,:,1)

   do j=1,jpj
   do i=1,jpi

     if ( sd(14)%fdta(i,j,1,ind) < 0.5_wp ) then ! atm_mxgt is by definition 0 or 1

!...   Thickness of atm. first levels (thermo and momentum)
!...   (hydrostatic relation approximating Tv_bar=Tv(layer=1))
       Tv= sd(6)%fdta(i,j,1,ind) * (1.0_wp + DELTA*sd( 3)%fdta(i,j,1,ind)) 
       sd(9 )%fdta(i,j,1,ind) = -RGASD*Tv/grav*log(sd(11)%fdta(i,j,1,ind)/sd(13)%fdta(i,j,1,ind))
       sd(10)%fdta(i,j,1,ind) = -RGASD*Tv/grav*log(sd(12)%fdta(i,j,1,ind)/sd(13)%fdta(i,j,1,ind))

!... Snow precip. put in sd(8) if temperature reaches 0 Celsius
       sd(8)%fdta(i,j,1,ind)=0._wp
       if (sd(6)%fdta(i,j,1,ind) <= rtt) sd(8)%fdta(i,j,1,ind)=sd(7)%fdta(i,j,1,ind)

!... T to potential temperature
       sd(6)%fdta(i,j,1,ind) = sd(6)%fdta(i,j,1,ind)*(sd(13)%fdta(i,j,1,ind)/sd(11)%fdta(i,j,1,ind))**CAPPA

     endif

   end do
   end do

   END SUBROUTINE cplatm_busin
!!===========================================================================

!!===========================================================================
      SUBROUTINE rotate_vector_ninj(u,v,theta,ni,nj)
!--------------------------------------------------------------------------
! Sous-Routine rotate_vector
! Tourne un vecteur (u,v) dans une base orthonormee ayant un angle
! de theta (dans le sens horaire) par rapport a l'ancienne base
!--------------------------------------------------------------------------

! Dans le modele ROM theta=49 degres (il faut transformer en radians)
! si on va de (x,y) vers (est,nord) theta= 49 deg
! si on va de (est,nord) vers (x,y) theta=-49 deg
! Note: comparativement a MoGSL la base ici est a +41
!       au lieu de -49

      implicit none
      integer ni,nj,i,j
      real(wp) u(ni,nj),v(ni,nj),theta(ni,nj),st,ct
      real(wp), allocatable :: utmp(:,:),vtmp(:,:)

      allocate(utmp(ni,nj),vtmp(ni,nj))

      do j=1,nj
      do i=1,ni
        st=sin(theta(i,j))
        ct=cos(theta(i,j))
        utmp(i,j) = ct*u(i,j) - st*v(i,j)
        vtmp(i,j) = st*u(i,j) + ct*v(i,j)
      enddo
      enddo

      u = utmp
      v = vtmp

      deallocate(utmp,vtmp)
      return
      END SUBROUTINE rotate_vector_ninj
!!===========================================================================

!!===========================================================================
      SUBROUTINE rotate_vector_ninj2(u,v,st,ct,ni,nj)
!--------------------------------------------------------------------------
! Sous-Routine rotate_vector
! Tourne un vecteur (u,v) dans une base orthonormee ayant un angle
! de theta (dans le sens horaire) par rapport a l'ancienne base
! sin(theta), cos(theta) pre-calcule
!--------------------------------------------------------------------------

! Dans le modele ROM theta=49 degres (il faut transformer en radians)
! si on va de (x,y) vers (est,nord) theta= 49 deg
! si on va de (est,nord) vers (x,y) theta=-49 deg
! Note: comparativement a MoGSL la base ici est a +41
!       au lieu de -49

      implicit none
      integer ni,nj,i,j
      real(wp) u(ni,nj),v(ni,nj),st(ni,nj),ct(ni,nj)
      real(wp), allocatable :: utmp(:,:),vtmp(:,:)

      allocate(utmp(ni,nj),vtmp(ni,nj))

      do j=1,nj
      do i=1,ni
        utmp(i,j) = ct(i,j)*u(i,j) - st(i,j)*v(i,j)
        vtmp(i,j) = st(i,j)*u(i,j) + ct(i,j)*v(i,j)
      enddo
      enddo

      u = utmp
      v = vtmp

      deallocate(utmp,vtmp)
      return
      END SUBROUTINE rotate_vector_ninj2
!!===========================================================================

#endif

END MODULE sbcblk_rpnc
