MODULE sbccglrrm
   !!======================================================================
   !!                       ***  MODULE rivini   ***
   !! Unstructured River Cond. :   define related variables
   !!======================================================================
   !! History :  1.0  !  2011-04  (F. Dupont)  Original code
   !!                 !  2019-09  (S. MacDermid) Merging into NEMO3.6
   !!----------------------------------------------------------------------
#if defined key_rpne && defined key_cglrrm
   !!----------------------------------------------------------------------
   !!   'key_river' :                    Unstructured River Condition
   !!----------------------------------------------------------------------
   !!   rivcglrrm       : Initialization of unstructured open boundaries
   !!----------------------------------------------------------------------
   USE dom_oce         ! ocean space and time domain
   USE sbc_oce         ! surface boundary condition variables
#if defined key_iceload2
   USE oce, ONLY : sshn, tsb, snwice_mass_n
#else
   USE oce, ONLY : sshn, tsb, snwice_mass_b
#endif
   USE in_out_manager  ! I/O units
!   USE lbclnk          ! ocean lateral boundary conditions (or mpp link)
   USE lib_mpp         ! for mpp_sum  
   USE iom             ! I/O
   USE phycst, ONLY: rau0

   IMPLICIT NONE
   PRIVATE

   PUBLIC   sbc_cglrrm   ! routine called in rivdta.F90

   LOGICAL, PUBLIC :: ln_cglrrm=.FALSE.    ! logical for having the CGLRRM model on
   LOGICAL, PUBLIC :: ln_lake_temp=.FALSE. ! logical for computing river temperature from upstream lake
   CHARACTER(LEN=10), DIMENSION(5) :: &
      name_lakes = (/ 'Superior', 'MichHuron', 'St-Clair', 'Erie', 'Ontario' /)
   REAL(wp), PUBLIC, ALLOCATABLE, &
            SAVE, DIMENSION(:,:,:) :: mask_lakes ! lake mask for each lake (1-5) 
   INTEGER                         :: n_connect
   REAL(wp), DIMENSION(2,5)        :: ar_lake, & ! area of connection points for inlet/outlet
                                       lake_temp ! lake temperatures (from NEMO grid) 
   INTEGER,  PUBLIC, ALLOCATABLE, &
            SAVE, DIMENSION(:)     :: i_connect, &
                                      j_connect, &
                                      l_connect, &
                                      d_connect  ! i/j/l/d connection location
   REAL(wp), PUBLIC, DIMENSION(5)  :: &
      ssh_lakes , &
      rnf_lakes , &
      flow_lakes, &
      ll_lakes  , &
      lake_ar0  , & ! lake areas (from NEMO grid)
! mean levels from IGLD85 [typo for Lake Ontario corrected 27.06.2016, BW]
      lake_ll0 = (/ 183.2_wp, 176.0_wp, 174.4_wp, 173.5_wp, 74.2_wp /)

   INTEGER nite_day
   INTEGER, PUBLIC :: nfreq_cglrrm_print = 10


   !! * Substitutions
#  include "domzgr_substitute.h90"
   !!----------------------------------------------------------------------
   !! NEMO/OPA 3.0 , LOCEAN-IPSL (2008) 
   !! $Id: rivini.F90 1146 2008-06-25 11:42:56Z rblod $ 
   !! Software governed by the CeCILL licence (modipsl/doc/NEMO_CeCILL.txt)
   !!---------------------------------------------------------------------------------

CONTAINS
   
   SUBROUTINE sbc_cglrrm( kt, rnf, rnt, rnk )
      !!----------------------------------------------------------------------
      !!                 ***  ROUTINE sbc_cglrrm  ***
      !!         
      !! ** Purpose :  Computes the river flow in the connecting channels of 
      !!               the Great Lakes following the rules given by CGLRRM
      !!
      !! ** Method  :  Read initialization arrays (mask, indices) to identify 
      !!               the connecting channels
      !!
      !!----------------------------------------------------------------------      
!arguments
      INTEGER :: kt
      REAL(wp), DIMENSION(jpi,jpj) :: rnf ! river runoff on NEMO grid given as kg/m2/s
      REAL(wp), DIMENSION(jpi,jpj) :: rnt ! river temperature on NEMO grid given as kg/m2/s
      INTEGER , DIMENSION(jpi,jpj) :: rnk ! depth of river
!locals
      REAL(wp), DIMENSION(jpi,jpj) :: zlev ! dummy variable for integral calculation
      INTEGER ::   ii, ij, jl, jn, jp
      !!----------------------------------------------------------------------

      IF (kt==nit000) CALL sbc_cglrrm_init(rnf)

      IF( nsec_day == NINT(0.5_wp * rdttra(1)) ) THEN      ! start a new day

         ! compute mean lakelevel
         ! ---------------------------
         IF( nn_ice_embd > 0 ) THEN
#if defined key_iceload2
            zlev(:,:) = sshn(:,:) + snwice_mass_n(:,:) / rau0
#else
            zlev(:,:) = sshn(:,:) + snwice_mass_b(:,:) / rau0
#endif
         ELSE
            zlev(:,:) = sshn(:,:)
         ENDIF
         call sbc_cglrrm_sshint(zlev,ssh_lakes)
         ssh_lakes(:) = ssh_lakes(:) / lake_ar0(:)

         ll_lakes(:) = lake_ll0(:) + ssh_lakes(:) ! CGLRRM requires the total height from IGLD85 datum

         ! Compute runoff into Lake Superior
         ! ---------------------------
         zlev(:,:) = MAX(0.0_wp, rnf(:,:)) / rau0 ! keep only inflow (outflow is negative)
         CALL sbc_cglrrm_sshint(zlev,rnf_lakes)

         ! call to the CGLRRM model
         ! returns new flow lakes
         ! ---------------------------

         IF(lwp) WRITE(numout,*) 'arguments to ccglrrm',ndastp, rnf_lakes(1),&
                   ll_lakes(1), ll_lakes(2), ll_lakes(3), ll_lakes(4), ll_lakes(5), &
                   flow_lakes(1), flow_lakes(2), flow_lakes(3), flow_lakes(4), flow_lakes(5)
         call ccglrrm(ndastp, rnf_lakes(1),&
                   ll_lakes(1), ll_lakes(2), ll_lakes(3), ll_lakes(4), ll_lakes(5), &
                   flow_lakes(1), flow_lakes(2), flow_lakes(3), flow_lakes(4), flow_lakes(5),lwp)

         IF(lwp) THEN
            DO jl=1,5
               WRITE(numout,'(a,i4,2(1x,f13.3))') 'after CGLRRM',jl,flow_lakes(jl),ll_lakes(jl)
            ENDDO
            CALL flush(numout)
         ENDIF

      ENDIF ! compute each day

      ! plug inflows/outflows
      ! ---------------------------
      DO jp = 1, n_connect
         ii=i_connect(jp)
         ij=j_connect(jp)
         jl=l_connect(jp)
         jn=d_connect(jp)
         IF ( jl > 1 .and. jn == 1 ) THEN ! treating inflow, excluding lake superior
            rnf(ii,ij) =   flow_lakes(jl-1) * rau0 / ar_lake(jn,jl)
         ENDIF
         IF ( jn == 2) THEN ! treating outflows with a negative sign
            rnf(ii,ij) = - flow_lakes(jl  ) * rau0 / ar_lake(jn,jl)
        ENDIF
      ENDDO

! overrides river temperature inflow
      IF( ln_lake_temp ) THEN
         CALL sbc_cglrrm_temp(kt, tsb, rnk)
         DO jp = 1, n_connect
            ii=i_connect(jp)
            ij=j_connect(jp)
            jl=l_connect(jp)
            jn=d_connect(jp)
            IF ( jl > 1 .and. jn == 1 ) THEN ! treating inflow, excluding Lake Superior
               rnt(ii,ij) = lake_temp(2,jl-1) ! inflow temperature taken from upstream lake outlet
            ENDIF
         ENDDO
      ENDIF

      ! write regulated flow in restart file
      ! --------------------------------------------------
      IF( lrst_oce )   CALL sbc_cglrrm_rst( kt, 'WRITE', rnf )

   END SUBROUTINE sbc_cglrrm


   SUBROUTINE sbc_cglrrm_init( rnf )
      ! arguments
      REAL(wp), DIMENSION(jpi,jpj) :: rnf ! river runoff on NEMO grid given as kg/m2/s
      ! locals
      INTEGER ::   ji, jj, jl, jn
      INTEGER ::   inum, ios                 ! temporary logical unit
      CHARACTER(LEN=256), DIMENSION(5) :: lake_filemask
      CHARACTER(LEN=356) :: connect_file
      ! connect_map: first index: input/ouput, second index: which lake (Superior, Michigan-Huron, St-Clair, Erie, Ontario)
      REAL(wp), DIMENSION(jpi,jpj) :: zlev, connect_map, connect_mask
      REAL(wp) :: zmax, ztarg
      !!----------------------------------------------------------------------
      !!
      NAMELIST/namsbc_cglrrm/ connect_file, lake_filemask, &
                              ln_lake_temp, nfreq_cglrrm_print
      !!----------------------------------------------------------------------

      IF(lwp) WRITE(numout,*)
      IF(lwp) WRITE(numout,*) 'sbc_cglrrm_init : initialization of the regulation and routing model for the Great Lakes'
      IF(lwp) WRITE(numout,*) '~~~~~~~~'
      !

      ALLOCATE( mask_lakes(jpi,jpj,5) )

      mask_lakes(:,:,:) = 0.0_wp


      ! Read namelist parameters
      ! ---------------------------
      REWIND( numnam_ref )
      READ  ( numnam_ref, namsbc_cglrrm, IOSTAT = ios, ERR = 901 )
901   IF( ios /= 0 ) CALL ctl_nam ( ios , 'namsbc_cglrrm in reference namelist', lwp )

      REWIND( numnam_cfg )
      READ  ( numnam_cfg, namsbc_cglrrm, IOSTAT = ios, ERR = 902 )
902   IF( ios /= 0 ) CALL ctl_nam ( ios , 'namsbc_cglrrm in reference namelist', lwp )


      ! control prints
      IF(lwp) THEN
         WRITE(numout,*) '         namsbc_cglrrm'
         WRITE(numout,*) '~~~~~~~~'
         WRITE(numout,*)
         WRITE(numout,*) 'connection file name                             ', TRIM(connect_file)
         WRITE(numout,*) 'use lake upstream temperature from channel inflow', ln_lake_temp
         WRITE(numout,*) 'frequency of output in time-steps                ', nfreq_cglrrm_print
         WRITE(numout,*) 'lake masks'
         DO jl=1,5
            WRITE(numout,'(a20," : ",a)') name_lakes(jl),TRIM(lake_filemask(jl))
         ENDDO
         WRITE(numout,*)
      ENDIF

      ! Read lake mask arrays
      ! ---------------------------
      DO jl=1,5

         CALL iom_open( lake_filemask(jl), inum )
         CALL iom_get (inum,jpdom_data,'mask',mask_lakes(:,:,jl),1)
         CALL iom_close( inum )

      ENDDO

      ! call to init routine of CGLRRM
      CALL init_flow
      CALL init_sup
      CALL init_surface


      nite_day = NINT( rday / rdt ) ! find the number of iteration in one day

      CALL iom_open ( connect_file, inum )                                 ! open file
      CALL iom_get  ( inum, jpdom_data, 'connect', connect_map )           ! read the river connecting array
      CALL iom_close( inum )                                               ! close file
      !
      ! ---------------------------------------------------
      ! find the total number of connecting points
      ! (global first)
      !
      connect_mask(:,:) = MIN( 1.0_wp, connect_map(:,:) ) * tmask_i(:,:) ! only count points outside haloes
      zmax = SUM(connect_mask) ! sum of all reach points
      IF( lk_mpp )  CALL mpp_sum( zmax );  n_connect = NINT( zmax )
      IF ( n_connect < 10 ) THEN
         CALL ctl_stop( 'sbc_cglrrm: nb of connecting points < 10. Not good!' )   ;   RETURN
      ENDIF
      IF(lwp) THEN
         WRITE(numout,*)
         WRITE(numout,*) 'total number of connecting points found on the NEMO grid', n_connect
      ENDIF

      ! ---------------------------------------------------
      ! find the total number of connecting points
      ! (local now) and allocate
      !
      connect_mask(:,:) = MIN( 1.0_wp, connect_map(:,:) )
      n_connect = SUM(connect_mask) ! sum of all reach points
      n_connect = MAX(n_connect,1) ! make sure that the local number is 1 or more for allocation purposes
      ! allocate 1d fields
      ALLOCATE(d_connect(n_connect), &
               i_connect(n_connect), &
               j_connect(n_connect), &
               l_connect(n_connect))
      d_connect(:) = 0; i_connect(:) = 0; j_connect(:) = 0; l_connect(:) = 0

      ! find how many points are connected the same lake and inlet or outlet
      DO jl = 1, 5
         DO jn = 1, 2
            connect_mask(:,:) = 0.0_wp
            ztarg = 10.0_wp * REAL(jn, wp) + REAL(jl, wp)
            WHERE( ABS(connect_map(:,:)-ztarg) < 1e-12_wp ) connect_mask(:,:) = tmask_i(:,:) * e12t(:,:)
            zmax = SUM(connect_mask) ! extract the maximum count for each lake inlet point
            IF( lk_mpp )  CALL mpp_sum( zmax )
            IF ( zmax == 0._wp ) THEN
               IF(lwp) WRITE(numout,*) 'sbc_glrrm: connecting points < 1 for lake/direction ',TRIM(name_lakes(jl)),' ',jn
               CALL ctl_stop( 'sbc_cglrrm: nb of connecting points < 1. Not good!' )   ;   RETURN
            ENDIF
            ar_lake(jn,jl) = zmax
         ENDDO
      ENDDO

! local management of connection points
      n_connect = 0
      DO jj = 1, jpj
         DO ji = 1, jpi
            IF (connect_map(ji,jj) > 0 ) THEN
               n_connect = n_connect + 1
               jl = NINT(MOD(connect_map(ji,jj)-1.0_wp,10.0_wp)) + 1
               i_connect( n_connect ) = ji
               j_connect( n_connect ) = jj
               l_connect( n_connect ) = jl
               jn = 1
               ztarg = 20.0_wp + REAL(jl, wp)
               IF (ABS(connect_map(ji,jj)-ztarg) < 1e-12_wp ) jn = 2
               d_connect( n_connect ) = jn
            ENDIF
         ENDDO
      ENDDO

      ! read restart file or initialize flow
      ! ---------------------------
      CALL sbc_cglrrm_rst( nit000, 'READ', rnf )
      !

      ! Compute initial lake area from mask
      ! ---------------------------
      zlev(:,:)=1.0_wp
      CALL sbc_cglrrm_sshint(zlev,lake_ar0)

      ! print section
      ! ---------------------------
      IF(lwp) THEN
         WRITE(numout,*)
         WRITE(numout,*) 'lake area from NEMO grid and lake level at rest'
         WRITE(numout,'(a20,a3,a13,a3,a7)') 'LAKE',' | ', 'AREA (m3)',' | ', 'levels from IGLD85'
         DO jl=1,5
            WRITE(numout,'(a20,a3,e13.6,a3,f7.3)') name_lakes(jl),' | ',lake_ar0(jl),' | ',lake_ll0(jl)
         ENDDO
         WRITE(numout,*)
         WRITE(numout,*) 'inlet and outlet areas of each lake'
         DO jl = 1, 5
             WRITE(numout,'(a20,":",2(1x,e13.6))') name_lakes(jl),ar_lake(:,jl)
         ENDDO
         WRITE(numout,*)
         WRITE(numout,*) 'initial outflow for each lake'
         DO jl=1,5
            WRITE(numout,'(a20,f13.3)') name_lakes(jl),flow_lakes(jl)
         ENDDO
         WRITE(numout,*)
         CALL FLUSH(numout)
      ENDIF

   END SUBROUTINE sbc_cglrrm_init


   SUBROUTINE sbc_cglrrm_sshint(ssh,lakint)
   !!---------------------------------------------------------------------------------
   !!   routine doing the mean lake level calculation
   !!---------------------------------------------------------------------------------
   ! arguments
      REAL(wp), DIMENSION(jpi,jpj) ::   ssh    ! area of the interior domain (e1t*e2t*tmask_i)
      REAL(wp), DIMENSION(5)       ::   lakint ! integrated results
   !local
      INTEGER ::  ji, jj, jl ! dummy loop indices
      REAL(wp), DIMENSION(jpi,jpj) ::   e1e2_i    ! area of the interior domain (e1t*e2t*tmask_i)
#if defined key_ddprec
      COMPLEX(wp) :: ctmp                 !MD: tmp var for DDPDD
      INTEGER :: itype                    !MD: tmp var for DDPDD
#endif
      REAL(wp) :: mlevel

      ! compute integrals over lake
      ! ---------------------------
      DO jl=1,5
         e1e2_i(:,:) = e12t(:,:) * tmask_i(:,:) * mask_lakes(:,:,jl)
#if defined key_ddprec
!FD based on MD
         ctmp = cmplx(0.e0,0.e0,wp)
         DO jj = 1, jpj
            DO ji = 1, jpi
               CALL DDPDD( cmplx(e1e2_i(ji,jj)*ssh(ji,jj),0.e0,wp), ctmp, 1, itype)
            END DO
         END DO
         IF ( lk_mpp )   CALL mpp_sum ( ctmp )
         mlevel  = real( ctmp,wp )
#else
         mlevel   = SUM( e1e2_i(:,:) * ssh(:,:) )
         IF( lk_mpp )   CALL  mpp_sum( mlevel )   ! sum over the global domain
#endif
         lakint(jl) = mlevel
      ENDDO

   END SUBROUTINE sbc_cglrrm_sshint

   SUBROUTINE sbc_cglrrm_temp(kt, temp, rnk)
   !!---------------------------------------------------------------------------------
   !!   routine doing the mean river temperature calculation
   !!---------------------------------------------------------------------------------
   ! arguments
      INTEGER :: kt
      REAL(wp), DIMENSION(jpi,jpj,jpk) :: &
                                        temp  ! area of the interior domain (e1t*e2t*tmask_i)
      INTEGER , DIMENSION(jpi,jpj) ::   rnk   ! depth of river
   !local
      INTEGER ::  ji, jj, jk, jp, jl, jn  ! dummy loop indices
      REAL(wp), DIMENSION(jpi,jpj) ::   e1e2_i    ! area of the interior domain (e1t*e2t*tmask_i)
      REAL(wp) :: ml,  mt             ! weigthed mask 
      REAL(wp) :: zmt, zlt            ! accumulators

      lake_temp(:,:) = 0.0_wp
      DO jp = 1, n_connect
         ji = i_connect(jp)
         jj = j_connect(jp)
         jl = l_connect(jp)
         jn = d_connect(jp)
         zmt = 0.0_wp
         zlt = 0.0_wp
         ! mix of the water column heat content
         DO jk = 1, rnk(ji,jj)
            mt = tmask(ji,jj,jk) * fse3t(ji,jj,jk)
            zlt = zlt +                  mt
            zmt = zmt + temp(ji,jj,jk) * mt
         ENDDO
         ml = e12t(ji,jj) * tmask_i(ji,jj)
         lake_temp(jn,jl) = lake_temp(jn,jl) + zmt / zlt * ml
      ENDDO
      !
      DO jl = 1,5
         DO jn = 1,2
            IF( lk_mpp ) CALL mpp_sum ( lake_temp(jn,jl) )
            lake_temp(jn,jl) = lake_temp(jn,jl) / ar_lake(jn,jl)
         ENDDO
      ENDDO
      ! print
      IF ( lwp .and. mod( kt, nfreq_cglrrm_print ) == 0 ) THEN
         WRITE(numout,*)
         WRITE(numout,*) 'Lake temperature inlet/outlet'
         DO jl = 1,5
            WRITE(numout,'(a20,2f10.3)') name_lakes(jl),lake_temp(:,jl)
         ENDDO
         CALL FLUSH(numout)
      ENDIF

   END SUBROUTINE sbc_cglrrm_temp

   SUBROUTINE sbc_cglrrm_rst( kt, cdrw, rnf )
      !!---------------------------------------------------------------------
      !!                   ***  ROUTINE ts_rst  ***
      !!
      !! ** Purpose : Read or write time-splitting arrays in restart file
      !!----------------------------------------------------------------------
      ! arguments
      INTEGER         , INTENT(in) ::   kt         ! ocean time-step
      CHARACTER(len=*), INTENT(in) ::   cdrw       ! "READ"/"WRITE" flag
      REAL(wp), DIMENSION(jpi,jpj) ::   rnf        ! river runoff on NEMO grid given as kg/m2/s
      ! locals
      INTEGER ::  ji, jj, jl, jp, jn      ! dummy loop indices
      REAL(wp) :: zp, ml
      !!----------------------------------------------------------------------
      !
      IF( TRIM(cdrw) == 'READ' ) THEN
         IF( iom_varid( numror, 'cglrrm1', ldstop = .FALSE. ) > 0 )   THEN
            IF(lwp) WRITE(numout,*) 'reading from restart scalar cglrrm1-5'
            CALL iom_get( numror, 'cglrrm1', zp ); flow_lakes(1) = zp
            CALL iom_get( numror, 'cglrrm2', zp ); flow_lakes(2) = zp
            CALL iom_get( numror, 'cglrrm3', zp ); flow_lakes(3) = zp
            CALL iom_get( numror, 'cglrrm4', zp ); flow_lakes(4) = zp
            CALL iom_get( numror, 'cglrrm5', zp ); flow_lakes(5) = zp
         ELSE
            IF(lwp) THEN
               WRITE(numout,*)
               WRITE(numout,*) '!!!!!!!!!!!!!!!!!!!!!!!!!!!'
               WRITE(numout,*) 'caiculating initial lake outflow from runoff'
               WRITE(numout,*) '!!!!!!!!!!!!!!!!!!!!!!!!!!!'
               WRITE(numout,*)
            ENDIF
            ! ---------------------------
            ! find initial outflows from rnf
            ! ---------------------------
            flow_lakes(:) = 0.0_wp
            DO jp = 1, n_connect
               ji = i_connect(jp)
               jj = j_connect(jp)
               jl = l_connect(jp)
               jn = d_connect(jp)
               IF (jn==1) CYCLE
               ml = tmask_i(ji,jj) * e12t(ji,jj)
               flow_lakes(jl) = flow_lakes(jl) &
                       - rnf(ji,jj) * ml / rau0 ! negative sign because rnf is negative for output and we need flow_lakes to be positive
            ENDDO
            DO jl = 1,5
               IF ( lk_mpp ) CALL mpp_sum ( flow_lakes(jl) )
            ENDDO
         ENDIF
      ELSEIF( TRIM(cdrw) == 'WRITE' ) THEN
         zp = flow_lakes(1) ; CALL iom_rstput( kt, nitrst, numrow, 'cglrrm1', zp )
         zp = flow_lakes(2) ; CALL iom_rstput( kt, nitrst, numrow, 'cglrrm2', zp )
         zp = flow_lakes(3) ; CALL iom_rstput( kt, nitrst, numrow, 'cglrrm3', zp )
         zp = flow_lakes(4) ; CALL iom_rstput( kt, nitrst, numrow, 'cglrrm4', zp )
         zp = flow_lakes(5) ; CALL iom_rstput( kt, nitrst, numrow, 'cglrrm5', zp )
      ENDIF
      !
   END SUBROUTINE sbc_cglrrm_rst



#else
    !!---------------------------------------------------------------------------------
   !!   Dummy module                                   NO unstructured river
   !!---------------------------------------------------------------------------------
CONTAINS
   SUBROUTINE sbc_cglrrm( kt, rnf, rnt ) ! dummy routine
   USE dom_oce
      INTEGER :: kt
      REAL(wp), DIMENSION(jpi,jpj) :: rnf ! river runoff on NEMO grid given as kg/m2/s
      REAL(wp), DIMENSION(jpi,jpj) :: rnt ! river temperature on NEMO grid given as kg/m2/s
   END SUBROUTINE sbc_cglrrm
#endif

   !!=================================================================================
END MODULE sbccglrrm
