MODULE sbcr1d
#if defined key_rpne
   !!======================================================================
   !!                       ***  MODULE  sbcr1d  ***
   !! Ocean forcing:  one-d river model
   !!=====================================================================
   !! History :       !  2011/10 (F. Roy and S. Senneville)
   !!                 !  2017/10 (NEMO 3.6 GSL 500, F. Roy 
   !!----------------------------------------------------------------------
!froy - sim
   !!----------------------------------------------------------------------
   !!   sbc_r1d      : adjust emp and emps according to one-d river
   !!   sbc_r1d_init : one-d river initialisation
   !!----------------------------------------------------------------------
   USE dom_oce         ! ocean space and time domain
   USE sbc_oce         ! surface boundary condition variables
   USE oce, ONLY : sshn
   USE iom
   USE fldread         ! read input fields
   USE in_out_manager  ! I/O manager
   USE phycst, ONLY: rau0, rpi, grav
   USE lib_mpp         ! mpp operations (mpi really)

   IMPLICIT NONE
   PRIVATE
   SAVE

   PUBLIC sbc_r1d          ! routine call in sbcrnf.F90 module
   PUBLIC sbc_r1d_init     ! routine call in sncrnf.F90 module
 
   INTEGER                  :: nq, n_reach
!
! array variable definition:
!------------------------------------------------
!  zq,zzq : primary variable water level relative to geoid (z), the extra 'z' duplicates the fields in memory during matrix inversion
!  q,  qq : primary variable river flow (q), the extra 'q' duplicates the fields in memory during matrix inversion
!  datum  : bottom river elevation relative to geoid
!  qtrib  : tributary flow, qtrib is the final value after rescaling
!  qtric  : tributary flow, qtric is read from file
!  ds     : length of river segment
!  k      : bottom friction
!  area,acd,abar : river sectional area, 'area' is the actual value', 'acd' is read from file' and 'bar' denotes a segment average
!  hq,dbar: 'hq' is an extra river depth present when z=datum, 'dbar' is the actual river depth, 'bar' denotes a segment average
!  dtop   : elevation threshold to get over the flood plain
!  b,bt,bcd,bsbar,bbar   : river breadth, 'b,bbar' are the actual values, 'bcd' is the minimum breadth (read from file), 'bt' is flood plain extra (read from file) and 'bar' denotes a segment average
!  nu     : matrix coefficient for elevation in the continuity equation
!  xi     : RHS in the continuity equation
!  eta, theta, mu : coefficients and RHS respectively in the momentum equation
!  as,    cs,     ps,    qs,   : matrix coefficients after partial elimination
!  rs,    ss,     ts,          : "
!  a,     c,      p,     qt,   : "
!  r,     s,      t ,          : "
   REAL(wp),DIMENSION(:), ALLOCATABLE :: &           !: variables of river_dynamic ! nq
                               zq,    zzq,    q,     qq,   &
                               datum, qtrib,  ds,    k,    &
                               nu,    abar,   dbar,  dtop, &
                               area,  bcd,    bt,    hq,   &
                               bsbar, bbar,   acd,   b,    &
                               xi,    eta,    theta, mu,   &
                               as,    cs,     ps,    qs,   &
                               rs,    ss,     ts,          &
                               a,     c,      p,     qt,   &
                               r,     s,      t ,    qtric
   INTEGER,  DIMENSION(:), ALLOCATABLE :: &              !: pointers for each reach ! n_reach
                               segl, segr
   REAL(wp), DIMENSION(:), ALLOCATABLE :: &              !: pointers for each reach ! n_reach
                               ar_reach,                   &
                               q_river, z_downstream
   !! * namsbc_r1d namelist
   INTEGER, ALLOCATABLE, DIMENSION(:,:) :: &
                                ij_1d                    !: indices of position of connecting points on the NEMO grid
   INTEGER                  ::  ndta_1d = 1              !: number of connecting points
   INTEGER                  ::  ndump_1d = 0             !: number of connecting points
   LOGICAL                  ::  ln_r1d_cold = .false.    !: allow a cold start of r1d in case of changes in configuration (dev mode)
   LOGICAL                  ::  ln_r1d_ramp = .false.    !: ramp on river flow
   LOGICAL                  ::  ln_r1d_lslope = .false.  !: linear slope on water level; .false. = datum; .true. = linear slope between datum upstream and 0
                                                         !: at the connexion with NEMO 
   INTEGER                  ::  nn_r1d_branch  = 0       !: solve for network; 0 = none; 1 = one branch;
                                                         !; 2 = two branches, i.e. Quebec
   INTEGER                  ::  nfreq_r1d_diag = 20      !: diagnostics frequency
   REAL(wp)                 ::  ssh_mean_nemo = 0.0_wp   !: mean sea level given by NEMO at Quebec
                                                         !: needed if the far field (OBC) is not zero
   REAL(wp)                 ::  ar_total                 !: total area of connecting points
   INTEGER                  ::  reach_min, reach_max     !: start and end reach connected to NEMO
   REAL(wp)                 :: z_upstream
   REAL(wp)                 :: qnet, q_upstream, qtribt  !: different flow
   REAL(wp)                 :: qclimato                  !: placeholder for the climatological flow

   REAL(wp), ALLOCATABLE, DIMENSION(:) :: &
                                ar1d,   &                !: one-d cell area.
                                tr1d,   &                !: one-d prescribed runoff temp.
                                sr1d,   &                !: one-d prescribed runoff salinity
                                qr1d                     !: one-d computed runoff
   ! FLD read structure
   CHARACTER(len=100)       ::   cn_dir          !: Root directory for location of r1d files
   INTEGER, PARAMETER       ::  nb_r1d_fld =  12 !: total number of fields to be read through fldread 
!$AGRIF_DO_NOT_TREAT
   TYPE(FLD), ALLOCATABLE, &
       DIMENSION(:), TARGET ::   sd_r1d          !: structure of input fields (file informations, fields read)
!$AGRIF_END_DO_NOT_TREAT
   TYPE(FLD_N)              ::   sn_q_r1d        !: information about the flow file to be read
   TYPE(FLD_N)              ::   sn_t_r1d        !: information about the temperature file to be read
   TYPE(FLD_N)              ::   sn_z_r1d        !: information about the bottom elevation file to be read
   TYPE(FLD_N)              ::   sn_a_r1d        !: information about the section area file to be read
   TYPE(FLD_N)              ::   sn_b_r1d        !: information about the section breath file to be read
   TYPE(FLD_N)              ::   sn_k_r1d        !: information about the friction file to be read
   TYPE(FLD_N)              ::   sn_eb_r1d       !: information about the extra breath file to be read
   TYPE(FLD_N)              ::   sn_ed_r1d       !: information about the extra thickness file to be read
   TYPE(FLD_N)              ::   sn_dx_r1d       !: information about the segment length file to be read
   TYPE(FLD_N)              ::   sn_qt_r1d       !: information about the tributary flow file to be read
   TYPE(FLD_N)              ::   sn_qc_r1d       !: information about the flow climatology file to be read
   TYPE(FLD_N)              ::   sn_re_r1d       !: information about the reach file to be read (connectivity table)
   TYPE(FLD_N)              ::   sn_sp_r1d       !: information about the reach file to be read (map of sp)
   INTEGER, PARAMETER &
           ::  i_q_r1d =  1, &
               i_t_r1d =  2, &
               i_z_r1d =  3, &
               i_a_r1d =  4, &
               i_b_r1d =  5, &
               i_k_r1d =  6, &
              i_eb_r1d =  7, &
              i_ed_r1d =  8, &
              i_dx_r1d =  9, &
              i_qt_r1d = 10, &
              i_qc_r1d = 11, &
              i_re_r1d = 12                      !: indices for fld fields
   REAL(wp), DIMENSION(:,:), ALLOCATABLE, PUBLIC &
                            ::   r1d_reach_map

CONTAINS

   SUBROUTINE sbc_r1d( kt, rnf, rnt, rns )
      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE sbc_r1d  ***
      !!       
      !! ** Purpose :   Introduce a climatological run off forcing
      !!
      !! ** Method  :   Set each river mouth with a monthly climatology 
      !!                provided from different data.
      !!                CAUTION : upward water flux, runoff forced to be < 0
      !!
      !! ** Action  :   runoff updated runoff field at time-step kt
      !!----------------------------------------------------------------------
      INTEGER, INTENT(in) ::   kt          ! ocean time step
      REAL(wp), DIMENSION(jpi,jpj), INTENT(OUT) &
                          ::   rnf, rnt, rns
      !
      INTEGER  ::   ierr   ! temporary integer
      INTEGER  ::   ji, jj, idump_1d
      INTEGER ifld
      !!----------------------------------------------------------------------
      !
      ! Update r1d
      ! ---------------
      DO ifld=1,nb_r1d_fld
         IF (sd_r1d(ifld) % nfreqh /= 0 ) THEN
            CALL fld_read ( kt, nn_fsbc, sd=sd_r1d(ifld:ifld) )    ! Read Runoffs data and provide it at kt
            SELECT CASE (ifld)
            CASE(i_q_r1d) ;  qnet       = sd_r1d(ifld) % fnow(1,1,1)
            CASE(i_qc_r1d);  qclimato   = sd_r1d(ifld) % fnow(1,1,1)
            CASE(i_k_r1d) ;  k    (:)   = sd_r1d(ifld) % fnow(:,1,1)
            CASE(i_qt_r1d);  qtric(:)   = sd_r1d(ifld) % fnow(:,1,1)
            END SELECT
         ENDIF
      ENDDO

      CALL stl_oned_river(kt,ierr)
      IF (ierr < 0) THEN
         CALL ctl_stop('Error in stl_oned_river')   ;   RETURN
      ENDIF

! Apply runoff
      DO idump_1d = 1, ndump_1d
         ji = ij_1d(idump_1d,1)
         jj = ij_1d(idump_1d,2)
         rnf(ji,jj) = qr1d(idump_1d)     ! qr1d is in kg.m/s positive if river flows outs (into NEMO)
         rnt(ji,jj) = tr1d(idump_1d)     ! outflow temperature
         rns(ji,jj) = sr1d(idump_1d)     ! outflow sanality (usually zero)
      ENDDO                              ! loop over connecting point index

      IF( lrst_oce ) CALL r1d_rst( kt, 'WRITE' )
      !
   END SUBROUTINE sbc_r1d


   SUBROUTINE stl_oned_river(kt,ierr)
      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE stl_oned_river  ***
      !!       
      !! ** Purpose :   Treat river as in one-d model from St-Lawrence (ISMER)
      !! 
      !! ** Method  :   Almost like ISMER GSL model
      !! ** Action  :   Water enters or go out from the model, absorbing tidal
      !!                energy, only fresh water enters (emps=0 if upstream
      !!                current or in other words emp > 0)
      !!----------------------------------------------------------------------
      !
      ! arguments
      INTEGER                       :: kt
      INTEGER, INTENT(OUT)          :: ierr
      ! locals
      INTEGER                       :: ji, jj, jk
      INTEGER                       :: idump_1d, ir
      REAL(wp)                      :: factor
      !!----------------------------------------------------------------------
      ! 

      ierr = -1

      IF( kt == nit000 ) THEN  
        IF(lwp) WRITE(numout,*) 
        IF(lwp) WRITE(numout,*) 'STL ONED : river model'
        IF(lwp) WRITE(numout,*) '~~~~~~~~~ '
      ENDIF

      ! rescale qtrib in case of large discrepancy between real-time outflow and climatology
      ! so as to avoid potential negative upstream flow (q_upstream)
      ! qtric is the climatology tributary flow
      ! qtrib is the actual tributary flow scaled by the observed/climatology ratio at river head
      !
      qtrib(:) = qtric(:)
      !
      IF ( sd_r1d(i_qt_r1d) % nfreqh == sd_r1d(i_qc_r1d) % nfreqh .AND. &
           sd_r1d(i_qt_r1d) % ln_clim ) THEN
         factor = qnet / qclimato
         qtrib(:) = qtric(:) * factor
      ENDIF

      ! sum contribution of tributaries after rescaling
      qtribt = SUM(qtrib)

      ! define q_upstream as the mean flow at end point minus tributaries = outflow at first segment (i.e., Montreal)
      q_upstream = qnet - qtribt

      qr1d = 0.0_wp;  tr1d(:) = 0.0_wp ;  sr1d(:) = 0.0_wp

!     Compute average SSH to pass to one-d model
!     assuming one-point of one-d model drops onto
!     ndump_1d points from NEMO 

      z_downstream(:) = 0.0_wp
      DO idump_1d=1,ndump_1d
         ji=ij_1d(idump_1d,1)
         jj=ij_1d(idump_1d,2)
         jk=ij_1d(idump_1d,3)
         z_downstream(jk) = z_downstream(jk) + sshn(ji,jj) * ar1d(idump_1d)
      ENDDO
      DO ir = reach_min, reach_max
         IF( lk_mpp)   CALL mpp_sum ( z_downstream(ir) )
         z_downstream(ir) = z_downstream(ir) / ar_reach(ir)     ! normalize the summation by total area
         z_downstream(ir) = z_downstream(ir) - ssh_mean_nemo    ! substract the mean sea level given by NEMO (or correction or datum)
      ENDDO

      CALL dynamic_river(kt, rdt, ierr)
      IF (ierr<0) RETURN

      ! compute the averaged streamflow
      DO idump_1d=1,ndump_1d
         jk=ij_1d(idump_1d,3)
         qr1d(idump_1d) = q_river(jk) * rau0 / ar_reach(jk)
                                              ! Conversion to kg/m**2/s for entry points
      ENDDO
      tr1d(:)  = sd_r1d(i_t_r1d) % fnow(1,1,1)! Climatology temperature
      sr1d(:)  = 0.0_wp                       ! Salinity always zero

      IF (lwp) THEN
        IF ( mod( kt, nfreq_r1d_diag ) == 0 ) THEN
          WRITE(numout,*) 'R1D RIVER RUNOFF MODEL: kt, time   = ', kt, ndastp, nsec_day
          WRITE(numout,*) 'R1D RIVER RUNOFF MODEL: SSH (NEMO) = ', z_downstream(reach_min:reach_max)
          WRITE(numout,*) 'R1D RIVER RUNOFF MODEL: z_upstream = ', z_upstream
          WRITE(numout,*) 'R1D RIVER RUNOFF MODEL: streamflow = ', q_river(reach_min:reach_max)
          WRITE(numout,*) 'R1D RIVER RUNOFF MODEL: dwn/up/tri = ', qnet,q_upstream,qtribt,' (mean flow m3/s)'
        ENDIF
        CALL FLUSH(numout)
      ENDIF

      ierr = 0

   END SUBROUTINE stl_oned_river


   SUBROUTINE sbc_r1d_init( ierr )
      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE sbc_r1d_init  ***
      !!
      !! ** Purpose :   Initialisation of one-d river (memory allocation)
      !!
      !!----------------------------------------------------------------------
      !!
      !!----------------------------------------------------------------------
      ! arguments
      INTEGER, INTENT(OUT) &
               ::   ierr            ! error code
      ! locals
      INTEGER  ::   sbc_r1d_alloc   ! temporary integer
      INTEGER  ::   ios             ! Local integer output status for namelist read
      INTEGER  ::   ir              ! connecting point index
      INTEGER  ::   ji, jj, jk
      TYPE(FLD_N), DIMENSION(nb_r1d_fld) &
               ::   sn_r1d          ! structure of input fields (file informations, fields read)
      CHARACTER(LEN=1052) &
               ::   r1d_file        ! filename for reading the time-independant fields
      INTEGER  ::   ifld, inum, jm  ! temporary integers
      REAL(wp), DIMENSION(jpi,jpj) &
               ::   r1d_reach_mask
      REAL(wp) ::   zmin, zmax

      NAMELIST/namsbc_r1d/ ln_r1d_ramp, nfreq_r1d_diag, &
                           nn_r1d_branch, ln_r1d_lslope,  ssh_mean_nemo, ln_r1d_cold, nq, n_reach, &
                           cn_dir, &
                           sn_q_r1d, sn_t_r1d, sn_z_r1d, sn_a_r1d, sn_b_r1d, sn_k_r1d, &
                           sn_eb_r1d, sn_ed_r1d, sn_dx_r1d, sn_qt_r1d, sn_qc_r1d, sn_re_r1d, &
                           sn_sp_r1d
      !!----------------------------------------------------------------------

      ierr = -1
      sbc_r1d_alloc = 0
      ndta_1d = 0; nq = 0; n_reach = 0

      !                                  ! ============
      !                                  !   Namelist
      !                                  ! ============
      !

      REWIND( numnam_ref )               ! Namelist namsbc_r1d in reference namelist : River1D parameters
      READ  ( numnam_ref, namsbc_r1d, IOSTAT = ios, ERR = 901)
901   IF( ios /= 0 ) CALL ctl_nam ( ios , 'namsbc_r1d in reference namelist', lwp )
      !
      REWIND( numnam_cfg )               ! Namelist namsbc_r1d in configuration namelist : River1D parameters
      READ  ( numnam_cfg, namsbc_r1d, IOSTAT = ios, ERR = 902 )
902   IF( ios /= 0 ) CALL ctl_nam ( ios , 'namsbc_r1d in configuration namelist', lwp )

      IF(lwm) WRITE( numond, namsbc_r1d )

      !                                  ! Control print
      IF(lwp) THEN
         WRITE(numout,*)
         WRITE(numout,*) 'sbc_r1d : one-d river model'
         WRITE(numout,*) '~~~~~~~ '
         WRITE(numout,*) '   Namelist namsbc_r1d'
         WRITE(numout,*) '     ramp on streamflow = ',ln_r1d_ramp
         WRITE(numout,*) '     cold start allowed = ',ln_r1d_cold
         WRITE(numout,*) 'Compute network relation for 1=one branch; 2=two branches'
         WRITE(numout,*) '                        = ',nn_r1d_branch
         WRITE(numout,*) 'Compute network linear slope initial',ln_r1d_lslope
         WRITE(numout,*) '                        = ',nn_r1d_branch
         WRITE(numout,*) '   freqency diagnostic  = ',nfreq_r1d_diag
         WRITE(numout,*) '      ssh_mean_nemo (m) = ',ssh_mean_nemo
         WRITE(numout,*) '         nb of branches = ',n_reach
         WRITE(numout,*) '      total nb of nodes = ',nq
      ENDIF

      IF (n_reach<1) THEN
         CALL ctl_stop( 'sbc_r1d: n_reach<1. Not good!' )   ;   RETURN
      ENDIF

      ALLOCATE( segl(n_reach), segr(n_reach), ar_reach(n_reach), &
             q_river(n_reach), z_downstream(n_reach), STAT=ierr )

      sbc_r1d_alloc = sbc_r1d_alloc + ierr

      IF (nq<1) THEN
         CALL ctl_stop( 'sbc_r1d: nq<1. Not good!' )   ;   RETURN
      ENDIF

      ALLOCATE( zq(nq),    zzq(nq),    q(nq),     qq(nq),   &
                datum(nq), qtrib(nq),  ds(nq),    k(nq),    &
                nu(nq),    abar(nq),   dbar(nq),  dtop(nq), &
                area(nq),  bcd(nq),    bt(nq),    hq(nq),   &
                bsbar(nq), bbar(nq),   acd(nq),   b(nq),    &
                xi(nq),    eta(nq),    theta(nq), mu(nq),   &
                as(nq),    cs(nq),     ps(nq),    qs(nq),   &
                rs(nq),    ss(nq),     ts(nq),              &
                a(nq),     c(nq),      p(nq),     qt(nq),   &
                r(nq),     s(nq),      t(nq) ,              &
                qtric(nq), STAT=ierr )

      sbc_r1d_alloc = sbc_r1d_alloc + ierr

      ! force the value of nn_r1d_branch for known specific cases
      !
      IF ( n_reach == 1 .AND. nn_r1d_branch /= 1 ) THEN
         CALL ctl_warn('nn_r1d_branch forced to 1')
         nn_r1d_branch = 1
      ENDIF

      IF ( n_reach == 3 .AND. nn_r1d_branch /= 2 ) THEN
         CALL ctl_warn('nn_r1d_branch forced to 2')
         nn_r1d_branch = 2
      ENDIF

      ! ===================================================
      ! Find connecting points between r1d and NEMO
      ! ===================================================
      !
      ! ---------------------------------------------------
      ! 1.read map
      !
      ALLOCATE( r1d_reach_map(jpi,jpj), STAT=ierr )

      sbc_r1d_alloc = sbc_r1d_alloc + ierr

      r1d_file = TRIM( cn_dir )//TRIM( sn_sp_r1d % clname )
      IF( .NOT. sn_sp_r1d % ln_clim ) THEN;    WRITE(r1d_file, '(a,"_y",i4)' ) TRIM( r1d_file ), nyear    ! add year 
         IF( sn_sp_r1d % cltype == 'monthly' ) WRITE(r1d_file, '(a,"m",i2.2)') TRIM( r1d_file ), nmonth   ! add month 
         IF( sn_sp_r1d % cltype == 'daily' )   WRITE(r1d_file, '(a,"m",i2.2,"d",i2.2)'  ) &
                                                                               TRIM( r1d_file ), nmonth, nday
      ENDIF
      CALL iom_open ( r1d_file, inum )                                     ! open file
      CALL iom_get  ( inum, jpdom_data, sn_sp_r1d%clvar, r1d_reach_map )   ! read the river mouth array
      CALL iom_close( inum )                                               ! close file
      !
      ! ---------------------------------------------------
      ! 2.find the total number of connecting points
      !
      r1d_reach_mask = MIN( 1.0_wp, r1d_reach_map(:,:) ) * tmask_i(:,:) ! only count points outside haloes
      zmax = SUM(r1d_reach_mask) ! sum of all reach points
      IF( lk_mpp )  CALL mpp_sum( zmax );  ndta_1d = NINT( zmax )

      IF ( ndta_1d < 1 ) THEN
         CALL ctl_stop( 'sbc_r1d: nb of connecting points < 1. Not good!' )   ;   RETURN
      ENDIF
      !
      ! ---------------------------------------------------
      ! 3.allocate 1D list
      !
      ALLOCATE( tr1d(ndta_1d), sr1d(ndta_1d), qr1d(ndta_1d), ar1d(ndta_1d), &
               ij_1d(ndta_1d,3),STAT=ierr)

      sbc_r1d_alloc = sbc_r1d_alloc + ierr
      !
      ! final allocation, now test for problems
      !
      IF( lk_mpp            )   CALL mpp_sum ( sbc_r1d_alloc )
      IF( sbc_r1d_alloc > 0 )   CALL ctl_warn('sbc_r1d: allocation of arrays failed')
      !
      ! ---------------------------------------------------
      ! 4.convert local map to a 1D list
      !
      ndump_1d   = 0
      ij_1d(:,:) = 0
      DO jj = 1, jpj
         DO ji = 1, jpi
            IF ( r1d_reach_map(ji,jj) > 0.0_wp ) THEN
               ndump_1d = ndump_1d + 1
               ij_1d(ndump_1d,1)  = ji
               ij_1d(ndump_1d,2)  = jj
               ij_1d(ndump_1d,3)  = r1d_reach_map(ji,jj)
            ENDIF
         ENDDO
      ENDDO
      ! find min and max of reach value
      IF ( ndump_1d > 0 ) THEN
         zmin = MINVAL( ij_1d(1:ndump_1d,3) )
         zmax = MAXVAL( ij_1d(1:ndump_1d,3) )
      ELSE
         zmin =  999._wp
         zmax = -999._wp
      ENDIF
      IF( lk_mpp )  CALL mpp_max( zmax );  reach_max = NINT( zmax )
      IF( lk_mpp )  CALL mpp_min( zmin );  reach_min = NINT( zmin )
      IF(lwp) WRITE(numout,*) 'min/max of reach connecting points',reach_min,reach_max

      ! at this point ij_1d contains the information of connecting point indices on the local tile
      ! fill value for cell area
      ar1d(:)     = 0.0_wp
      ar_reach(:) = 0.0_wp
      ar_total    = 0.0_wp
      DO ir = 1, ndump_1d
         ji = ij_1d(ir,1)
         jj = ij_1d(ir,2)
         jk = ij_1d(ir,3)
         ar1d(ir) = e12t(ji,jj) * tmask_i(ji,jj) ! tmask_i required to avoid double-counting over haloes
         ar_reach(jk) = ar_reach(jk) + ar1d(ir)
         ar_total     = ar_total     + ar1d(ir)
      ENDDO
      ! sum over all procs if necessary
      IF( lk_mpp)   CALL mpp_sum ( ar_total )
      IF(lwp) WRITE(numout,*) 'Total Reach area: ',ar_total
      DO jk = reach_min, reach_max
         IF( lk_mpp)   CALL mpp_sum ( ar_reach(jk) )
         IF ( ar_reach(jk) == 0.0_wp ) THEN
            IF(lwp) WRITE(numout,*) 'WARNING MODEL WILL CRASH: jk,ar_reach(jk)=', jk,ar_reach(jk)
            CALL ctl_stop('sbcr1d: some One-D points have no wet ocean input points')   ;   RETURN
         ENDIF
         IF(lwp) WRITE(numout,*) 'Reach area #',jk,ar_reach(jk)
      ENDDO


      ! initialize to zero
      zq  = 0.0_wp
      zzq = 0.0_wp
      q   = 0.0_wp
      qq  = 0.0_wp
      bcd = 0.0_wp
      bt  = 0.0_wp
      hq  = 0.0_wp
      ds  = 0.0_wp
      k   = 0.0_wp

      ! fld_read structure
      !
      ALLOCATE( sd_r1d(nb_r1d_fld), STAT=ierr )
      !
      sn_r1d(i_q_r1d) = sn_q_r1d ; sn_r1d(i_t_r1d) = sn_t_r1d ; sn_r1d(i_a_r1d) = sn_a_r1d ;
      sn_r1d(i_b_r1d) = sn_b_r1d ; sn_r1d(i_z_r1d) = sn_z_r1d ; sn_r1d(i_k_r1d) = sn_k_r1d ;
      sn_r1d(i_eb_r1d) = sn_eb_r1d ; sn_r1d(i_ed_r1d) = sn_ed_r1d ; 
      sn_r1d(i_dx_r1d) = sn_dx_r1d ; sn_r1d(i_qt_r1d) = sn_qt_r1d ;
      sn_r1d(i_qc_r1d) = sn_qc_r1d ; sn_r1d(i_re_r1d) = sn_re_r1d
      !
      CALL fld_fill( sd_r1d(1:nb_r1d_fld),  sn_r1d(1:nb_r1d_fld), cn_dir, 'sbc_r1d_init', 'read r1d data', 'namsbc_r1d' )
      !
      ! allocate and read time independent fields
      !
      DO ifld = 1, nb_r1d_fld
         SELECT CASE (ifld)
         CASE(i_q_r1d, i_t_r1d,i_qc_r1d)
            ALLOCATE( sd_r1d(ifld) % fnow( 1,1,1) ); IF( sd_r1d(ifld) % ln_tint ) ALLOCATE( sd_r1d(ifld) % fdta( 1,1,1,2) )
         CASE(i_a_r1d, i_b_r1d, i_z_r1d, i_k_r1d, i_eb_r1d, i_ed_r1d, i_dx_r1d, i_qt_r1d)
            ALLOCATE( sd_r1d(ifld) % fnow(nq,1,1) ); IF( sd_r1d(ifld) % ln_tint ) ALLOCATE( sd_r1d(ifld) % fdta(nq,1,1,2) )
         CASE(i_re_r1d)
            ALLOCATE( sd_r1d(ifld) % fnow(2,n_reach,1) )
         END SELECT
         !
         ! read only time independent fields
         !
         IF (sd_r1d(ifld) % nfreqh == 0 ) THEN
            !
            r1d_file = TRIM( cn_dir )//TRIM( sn_r1d(ifld) % clname )
            !
            IF(lwp) WRITE(numout,*) 'read time-independent field ',TRIM(sd_r1d(ifld)%clvar),' ',TRIM(r1d_file)
            !
            IF( .NOT. sd_r1d(ifld) % ln_clim ) THEN; WRITE(r1d_file, '(a,"_y",i4)' ) TRIM( r1d_file ), nyear    ! add year 
            IF( sd_r1d(ifld) % cltype == 'monthly' ) WRITE(r1d_file, '(a,"m",i2)'  ) TRIM( r1d_file ), nmonth   ! add month 
            IF( sd_r1d(ifld) % cltype == 'daily' )   WRITE(r1d_file, '(a,"m",i2.2,"d",i2.2)'  ) &
                                                                                     TRIM( r1d_file ), nmonth, nday
            ENDIF
            CALL iom_open ( r1d_file, inum )                                 ! open file
            CALL iom_get  ( inum, jpdom_unknown, &
                            sd_r1d(ifld)%clvar, sd_r1d(ifld) % fnow(:,:,1) ) ! read the river mouth array
            CALL iom_close( inum )                                           ! close file
            !
            ! apply specific values
            SELECT CASE (ifld)
            CASE(i_q_r1d) ;  qnet       = sd_r1d(ifld) % fnow(1,1,1)
            CASE(i_qc_r1d);  qclimato   = sd_r1d(ifld) % fnow(1,1,1)
            CASE(i_t_r1d) ! not treated here
            CASE(i_z_r1d) ;  datum(:)   = sd_r1d(ifld) % fnow(:,1,1)
            CASE(i_a_r1d) ;  acd  (:)   = sd_r1d(ifld) % fnow(:,1,1)
            CASE(i_b_r1d) ;  bcd  (:)   = sd_r1d(ifld) % fnow(:,1,1)
            CASE(i_k_r1d) ;  k    (:)   = sd_r1d(ifld) % fnow(:,1,1)
            CASE(i_eb_r1d);  bt   (:)   = sd_r1d(ifld) % fnow(:,1,1)
            CASE(i_ed_r1d);  dtop (:)   = sd_r1d(ifld) % fnow(:,1,1)
            CASE(i_dx_r1d);  ds   (:)   = sd_r1d(ifld) % fnow(:,1,1)
            CASE(i_qt_r1d);  qtric(:)   = sd_r1d(ifld) % fnow(:,1,1)
            CASE(i_re_r1d)
                       segl (:)   = NINT( sd_r1d(ifld) % fnow(1,:,1) )
                       segr (:)   = NINT( sd_r1d(ifld) % fnow(2,:,1) )
            END SELECT
         ENDIF
      ENDDO

      IF(lwp) THEN
         WRITE(numout,*) '  ir   area    stream   storage   dtop  elevation friction segment'
         DO ir=1,nq
         write(numout,'(i4,f9.2,1x,2(f9.3,1x),3(f7.4,1x),f7.1)') &
                 ir,acd(ir),bcd(ir), bt(ir),dtop(ir), datum(ir), k(ir), ds(ir)
         ENDDO
         DO ir=1,n_reach
            WRITE(numout,*) 'segl/segr',segl(ir),segr(ir)
         ENDDO
      ENDIF


      ! initialize from climatology

      IF (ln_r1d_lslope) THEN
         ! Initialize from an hypothetic slope that starts from the datum
         ! upstream and end at 0 at the connexion with NEMO sis001

         zq(1) = datum(1) 
         zzq(1) = datum(1)
         zq(nq) = 0. 
         zzq(nq) = 0.
         do ir=2,nq-1
            zq(ir) = zq(1)*(nq-ir)/nq
            zzq(ir) = zzq(1)*(nq-ir)/nq
         end do
      ELSE
         do ir=1,nq
            zq(ir) = datum(ir)
            if(zq(ir) .lt. 0.0_wp) zq(ir) = 0.0_wp
            zzq(ir) = datum(ir)
            if(zzq(ir) .lt. 0.0_wp) zzq(ir) = 0.0_wp
         end do
      ENDIF

      do ir=1,nq
         hq(ir) = acd(ir) / bcd(ir)
      end do
      do ir=1,nq-1
         bsbar(ir) = 0.5_wp * (bcd(ir) + bcd(ir+1))
      end do
      !
      IF( ln_rstart .AND. .NOT.ln_r1d_cold ) CALL r1d_rst(nit000, 'READ')

      ierr = 0

   END SUBROUTINE sbc_r1d_init


   SUBROUTINE r1d_rst( kt, cdrw )
      implicit none
      ! arguments
      integer, intent(in) :: kt          ! ocean time-step
      character(len=*)    :: cdrw        ! "READ"/"WRITE" flag
      character(len=10)   :: cwk         ! character string for defining 1d vertical array name
      ! locals
      INTEGER             :: nztot, np_r1d
      REAL(wp)            :: zr1d_nq
      REAL(wp), DIMENSION(jpk) &
                          :: wkio_1dz
      REAL(wp), ALLOCATABLE, DIMENSION(:) &
                          :: wkio_1d
      INTEGER             :: jn, jp, jk, jn1

      nztot = 2 * nq
      np_r1d = (nztot - 1) / jpk + 1
      ALLOCATE( wkio_1d(nztot) )

      SELECT CASE ( cdrw )
      CASE ('READ')
         !
         IF(lwp) WRITE(numout,*) '---- r1d_rst ---- read at kt=', kt
         !
         ! load number of nodes and check against internal value
         !
         IF( iom_varid( numror, 'r1d_nq' ) > 0 ) THEN
            CALL iom_get( numror,'r1d_nq', zr1d_nq )
               IF(lwp) THEN
                  WRITE(numout,*) 'code value    : ',nq
                  WRITE(numout,*) 'restart value : ',NINT(zr1d_nq)
               ENDIF
            IF ( ABS(REAL(nq,wp) - zr1d_nq ) > 0.1_wp )  then
               IF(lwp) THEN
                  WRITE(numout,*) 'E R R O R in sbc_r1d'
                  WRITE(numout,*) 'Inconstitent parameter nq' 
                  WRITE(numout,*) 'code value    : ',nq
                  WRITE(numout,*) 'restart value : ',NINT(zr1d_nq)
                  RETURN
               ENDIF
               nstop = nstop + 1
               RETURN
            ENDIF
         ELSE                                                   !* no restart
            IF(lwp) WRITE(numout,*) 'No restart r1d variables found. Continuing with cold-start values.'
            RETURN
         ENDIF ! test on r1d_nq
         !
         ! loop over jp to reconstruct river vectors
         ! assuming that if the first vector is here the rest should follow
         IF( iom_varid( numror, 'r1d_f001' ) > 0 ) THEN
            jn = 0
            DO jp= 1, np_r1d
               WRITE(cwk,'(a,i3.3)') 'r1d_f', jp
               CALL iom_get( numror, jpdom_unknown, TRIM(cwk), wkio_1dz )
               jn1 = MIN( jpk, nztot - (jp - 1) * jpk)
               DO jk = 1, jn1
                  jn = jn + 1
                  wkio_1d(jn) = wkio_1dz(jk)
               ENDDO
            ENDDO
            zzq(1:nq) = wkio_1d(   1:nq  )
             qq(1:nq) = wkio_1d(nq+1:2*nq)
         ENDIF ! test on r1d_f001

      CASE ('WRITE')

       IF(lwp) WRITE(numout,*) '---- r1d_rst ---- write at kt=', kt
       ! write nb of river segment for check
       CALL iom_rstput( kt, nitrst, numrow, 'r1d_nq', REAL(nq,wp) )
       ! write zzq and qq in 1D vertical arrays
       wkio_1d(   1:nq  ) = zzq(1:nq)
       wkio_1d(nq+1:2*nq) =  qq(1:nq)
       jn = 0
       DO jp = 1, np_r1d
          wkio_1dz(:) = 0.0_wp
          jn1 = MIN( jpk, nztot - (jp - 1) * jpk)
          DO jk = 1, jn1
             jn = jn + 1
             wkio_1dz(jk) = wkio_1d(jn)
          ENDDO
          WRITE(cwk,'(a,i3.3)') 'r1d_f', jp
          CALL iom_rstput( kt, nitrst, numrow, TRIM(cwk), wkio_1dz )
       ENDDO

      END SELECT ! cdrw

      DEALLOCATE( wkio_1d )

   END SUBROUTINE r1d_rst


   SUBROUTINE dynamic_river(kstep, dt, ierr)

      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE dynamic_river  ***
      !!       
      !! ** Purpose :   Treat river as in one-d model from St-Lawrence (ISMER)
      !! ** Method  :   Subroutine to update the river transport to its
      !!                value at time kstep*dt after initialization. 
      !!                The tidal elevation at the junction with the ocean
      !!                model, is passed to dynamic_river. dt is the timestep of
      !!                the calling model. On an hourly basis (kstep*dt  = n *
      !!                3600) a restart file is saved. A restart is triggered by
      !!                passing variable initial less than 0.
      !! 
      !! ** Action  :   Water enters or go out from the model, absorbing tidal
      !!                energy, only fresh water enters (emps=0 if upstream
      !!                current or in other words emp > 0)
      !!
      !! ** Modification: Simon Senneville - ISMER - 7 septembre 2010
      !!----------------------------------------------------------------------
      !
      ! I/O
      ! Parametres:
      !       intent(in)   : kstep                               ! Index of the timestep 
      !       intent(in)   : dt                                  ! Timestep of the model
      !       intent(in)   : restart                             ! performing a restart run
      !       intent(inout): ierr
      ! Retour:       n/a              
      !       [ex:    0 :  succes]
      !       [      -1 :  erreur]
      !
      ! [note: ]
      implicit none
      !-----------------------------------------------------------------------
      ! Arguments
      !-----------------------------------------------------------------------
      real(wp), intent(in)   :: dt
      integer, intent(inout) :: ierr   ! Should enter as -1
      !-----------------------------------------------------------------------
      ! Local parameters
      !-----------------------------------------------------------------------
      integer                :: lenstr
      integer                :: kstep
      integer                :: i, ir
      integer                :: iq
      real(wp)               :: div, ramp, g, sigma
      real(wp)               :: term1, term2, term3, term4, storb, dwl
      !!----------------------------------------------------------------------
      ! 
                                                           ! ds(i) = distance between zq(i) and zq(i+1)
      storb = 1.0_wp                                       ! Facteur multiplicatif = 1, donc sans effect actuellement
      g = grav                                             ! gravity acceleration taken from phycst

      do i=1,nq
        zq(i) = zzq(i)
        q(i)  =  qq(i)
      end do

      ramp  = 1.0_wp
      if (ln_r1d_ramp) ramp  = kstep * dt &
                                 / (12.0_wp*3600.0_wp)     ! Rampe sur 12 heures
      if( ramp .gt. 1.0_wp ) ramp = 1.0_wp
      qq(segl(1))  = q_upstream * ramp

      do ir = reach_min, reach_max                         ! After discussing the matter with D. Lefaivre of IML,
                                                           ! it appears that the geiod is the same for NEMO (ssh=0) and r1d
         zzq(segr(ir)) = z_downstream(ir)                  ! Initialise les niveaux de 1D pour les noeud 72 et 78
      enddo                                                ! 2m is the correction required at Quebec relative to datum
                                                           !------------------------------
                                                           ! Compute the cross-section area (m**2)
      do i=1,nq
        dwl = zq(i) - datum(i)
        if(dwl .le. 0.0_wp) then
          b(i) = bcd(i)
          area(i) = acd(i) + bcd(i) * dwl
        end if
        if(dwl .gt. 0.0_wp .and. dwl .le. dtop(i)) then
          b(i) = bcd(i) + storb * (bt(i) - bcd(i)) * dwl/dtop(i)
          area(i) = acd(i) + bcd(i) * dwl + 0.5_wp*(bt(i)-bcd(i))*dwl**2/dtop(i)
        end if
        if(dwl .gt. dtop(i)) then
          b(i) = bcd(i) + storb * (bt(i) - bcd(i))
          area(i) = acd(i) + bcd(i) * dwl + 0.5_wp*(bt(i)-bcd(i))*(dwl-0.5_wp*dtop(i))
        end if
      end do
      do i=1,nq-1
        abar(i) = 0.5_wp * ( area(i) + area(i+1) )
        dbar(i) = 0.5_wp * ( zq(i) +hq(i)  - datum(i) + zq(i+1) + hq(i+1) - datum(i+1) )
        bbar(i) = 0.5_wp * ( b(i) + b(i+1) )
      end do
                                                           ! Compute the major coefficients (nu,xi,eta,theta,mu)
      do i=1,nq-1
        nu(i) = ds(i)*bbar(i)/(2.0_wp*dt)
        xi(i) = nu(i)*(zq(i+1)+zq(i)) + 0.5_wp*(qtrib(i+1) + qtrib(i))
        term1 = ds(i)/(2.0_wp*g*dt*abar(i))
        term2 = (bsbar(i)+bbar(i))*(q(i+1)+q(i))/(2.0_wp*g*bbar(i)*(abar(i)**2))
        term3 = k(i)*ds(i)*abs(q(i+1)+q(i))/(4.0_wp*g*dbar(i)*(abar(i)**2))
        term4 = (q(i+1)+q(i))*(area(i+1)-area(i))/(4.0_wp*g*(abar(i)**3))
        eta(i)   = term1 + term2 + term3 - term4
        theta(i) = term1 - term2 + term3 - term4
        mu(i)    = term1 * (q(i+1)+q(i))
      end do
                                                           ! Compute the minor coefficients, down-river sweep
      do ir=1,n_reach
        ps(segr(ir)) = 0.0_wp
        rs(segr(ir)) = 0.0_wp
        as(segr(ir)) = 1.0_wp
        do i=segr(ir),segl(ir)+1,-1
          div = ps(i)-eta(i-1)
          qs(i) = 1.0_wp/div
          ts(i) = -theta(i-1)/div
          ss(i) = (rs(i)-mu(i-1))/div
          cs(i) = as(i)/div
          sigma = 1.0_wp - nu(i-1)*ps(i)
          div = sigma*qs(i) - nu(i-1)
          ps(i-1) = (sigma*ts(i) + 1.0_wp)/div
          rs(i-1) = (-xi(i-1) + sigma*ss(i) + nu(i-1)*rs(i))/div
          as(i-1) = (sigma*cs(i) + nu(i-1)*as(i))/div
        end do
      end do
                                                           ! Compute the minor coefficients, up-river sweep
      do ir=1,n_reach
        p(segl(ir)) = 0.0_wp
        r(segl(ir)) = 0.0_wp
        a(segl(ir)) = 1.0_wp
        do i=segl(ir)+1,segr(ir)
          div = p(i-1)+theta(i-1)
          qt(i-1) = 1.0_wp/div
          t(i-1) = eta(i-1)/div
          s(i-1) = (r(i-1)+mu(i-1))/div
          c(i-1) = a(i-1)/div
          sigma  = 1.0_wp + nu(i-1)*p(i-1)
          div = sigma*qt(i-1) + nu(i-1)
          p(i) = (sigma*t(i-1) + 1.0_wp)/div
          r(i) = (xi(i-1) + sigma*s(i-1) - nu(i-1)*r(i-1))/div
          a(i) = (sigma*c(i-1) - nu(i-1)*a(i-1))/div
        end do
      end do

      ! Compute the elevations and the volume flow rates
      !
      ! 1-compute the elevation and flow at each end of the reach
      !
      SELECT CASE (nn_r1d_branch)
      CASE(1)
         CALL network_one_branch
      CASE(2)
         CALL network_quebec
      END SELECT
      !
      ! 2-compute the elevation and flow at all other points using arbitrarily one of the directions (here "down-sweep")
      !
      do ir=1,n_reach
        do i=segl(ir)+1,segr(ir)
          zzq(i-1) =                 - ps(i-1)*qq(i-1) + rs(i-1) + as(i-1)*zzq(segr(ir))
          qq (i  ) = -qs(i)*zzq(i-1) - ts(i  )*qq(i-1) + ss(i)   + cs(i  )*zzq(segr(ir))
        end do
      end do

      ! FD the reason for unstaggering the river flow is obscure to me
      do ir = 1, n_reach                                   ! loop over reach
         q_river(ir) = 0.5_wp * &
                          ( qq(segr(ir)) + qq(segr(ir)-1) )! streamflow at extremity of reach (ex:Quebec)
      enddo
      z_upstream = zzq(1)                                  ! save the value of water level upstream referenced to geoid (mean z=0 downstream)

      ierr = 0

      END SUBROUTINE dynamic_river

      SUBROUTINE network_quebec

      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE network  ***
      !!       
      !! ** Purpose :   Hard coded for GSL one-d version 
      !! ** Method  :   gitlab.science.gc.ca/concepts/ncdc/wikis/r1d-discretization
      !! 
      !! ** Action  :  
      !!
      !!----------------------------------------------------------------------
      implicit none
      ! locals
      real(wp) h1,h2,h3,h4,h5,h6,q1,q2,q3,q4,q5,q6 ! water level ref to geiod and river flow
                                                   ! the number corresponds to the index in list_index
      real(wp) a2, a4, a6                          ! upstream sweep coefficient for upriver water level
      real(wp) as1, as3, as4, as5, as6             ! downstream sweep coefficient for downriver water level
      real(wp) r2, r3, r4, r5, r6                  ! R.H.S after upriver sweep 
      real(wp) rs1, rs3, rs4, rs5, rs6             ! R.H.S after downriver sweep
      real(wp) p2, p3, p4, p5, p6                  ! upstream sweep coefficient for downriver flow
      real(wp) ps1, ps3, ps5                       ! downstream sweep coefficient for upriver flow
      integer,dimension(6)  :: list_index

      list_index=(/segl(1), segr(1), segl(2), segr(2), segl(3), segr(3)/)

      h4 = zzq(list_index(4))
      h6 = zzq(list_index(6))
      q1 = qq(list_index(1))

      a2 = a(list_index(2))
      a4 = a(list_index(4))
      a6 = a(list_index(6))

      as1 = as(list_index(1))
      as3 = as(list_index(3))
      as4 = as(list_index(4))
      as5 = as(list_index(5))
      as6 = as(list_index(6))

      r2 = r(list_index(2))
      r3 = r(list_index(3))
      r4 = r(list_index(4))
      r5 = r(list_index(5))
      r6 = r(list_index(6))

      rs1 = rs(list_index(1))
      rs3 = rs(list_index(3))
      rs4 = rs(list_index(4))
      rs5 = rs(list_index(5))
      rs6 = rs(list_index(6))

      p2 = p(list_index(2))
      p3 = p(list_index(3))
      p4 = p(list_index(4))
      p5 = p(list_index(5))
      p6 = p(list_index(6))

      ps1 = ps(list_index(1))
      ps3 = ps(list_index(3))
      ps5 = ps(list_index(5))

      h2 = ps3*ps5*a2*ps1*q1 +p2*ps5*as3*h4 + p2*ps3*as5*h6 &
     & -ps3*ps5*r2 - ps3*ps5*a2*rs1 + p2*ps5*rs3 + p2*ps3*rs5

      h2 = h2  /( p2*ps5 + p2*ps3 -ps3*ps5 + ps3*ps5*a2*as1 )

      h1 = -ps1*q1 + rs1 + as1*h2

      h3 = h2

      h5 = h2

      q3 = - (h3 -rs3 -as3*h4)/ps3

      q4 = -(h4 -r4 - a4*h3)/p4

      q5 = -(h5 -rs5 -as5*h6) /ps5

      q6 = -(h6 -r6 -a6*h5)/p6

      q2 = q3 + q5

      zzq(list_index(1)) = h1

      zzq(list_index(2)) = h2
       qq(list_index(2)) = q2

      zzq(list_index(3)) = h3
       qq(list_index(3)) = q3

       qq(list_index(4)) = q4

      zzq(list_index(5)) = h5
       qq(list_index(5)) = q5

       qq(list_index(6)) = q6

      END SUBROUTINE network_quebec


      SUBROUTINE network_one_branch

      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE network  ***
      !!       
      !! ** Purpose :   General coding for one branch
      !! ** Method  :   1) z_L=rs(1)+as(1).z_R -ps(1).q_L
      !!                2) q_R=1/p(n-1) [ r(n-1) + a(n-1) z_L -z_R ]
      !!       
      !! see also gitlab.science.gc.ca/concepts/ncdc/wikis/r1d-discretization
      !!       
      !! ** Action  :  
      !!
      !!----------------------------------------------------------------------
      implicit none
      ! locals
      real(wp) q1, q2, h1, h2                      ! water level ref to geiod and river flow
                                                   ! the number corresponds to the index in list_index
      real(wp) a2, r2, p2                          ! upstream sweep coefficient for
                                                   ! upriver water level, RHS and downriver flow
      real(wp) as1, rs1, ps1                       ! downstream sweep coefficient for
                                                   ! downriver water level, RHS and upriver flow
      integer,dimension(2)  :: list_index
     
      list_index=(/1, nq/)

      h2  = zzq(list_index(2))
      q1  = qq (list_index(1))

      a2  = a  (list_index(2))
      r2  = r  (list_index(2))
      p2  = p  (list_index(2))

      as1 = as (list_index(1))
      rs1 = rs (list_index(1))
      ps1 = ps (list_index(1))

      h1 = -ps1*q1 + rs1 + as1*h2

      q2 = ( r2 + a2*h1 - h2 ) / p2

      zzq(list_index(1)) = h1
       qq(list_index(2)) = q2

      END SUBROUTINE network_one_branch
#else
   USE par_oce         ! domain size
   INTEGER, PUBLIC          :: ln_r1d = .false.
   CONTAINS
    !! Empty module
   SUBROUTINE sbc_r1d( kt, rnf, rnt, rns )
      !!----------------------------------------------------------------------
      INTEGER, INTENT(in) ::   kt          ! ocean time step
      REAL(wp), DIMENSION(jpi,jpj) &
                          ::   rnf, rnt, rns
      !!----------------------------------------------------------------------
      PRINT *,'should not be there!!!'
      !!----------------------------------------------------------------------
   END SUBROUTINE sbc_r1d
      !                                   
#endif
   
   !!======================================================================
END MODULE sbcr1d
