MODULE t_filt
   !!======================================================================
   !!                       ***  MODULE  t_filt  ***
   !!                              Tidal filter
   !!
   !!   The tidal filter is used for (1) the tidal nudging (see Wang et al., 2021 OM)
   !!   and (2) the isolation of surface tidal currents for calculating tidal ice stress.
   !!======================================================================
   !!----------------------------------------------------------------------
   !!   tfilt_ubar_dmp    : t_filt the diff in ubar between mod and TPXO and do the nudging/damping 
   !!   tfilt_usurf       : t_filt surface currents
   !!   tfilt_init        : initialization of t_filt
   !!----------------------------------------------------------------------
   USE oce            ! ocean: variables
   USE lib_mpp        ! MPP library
   USE lbclnk         ! ocean lateral boundary conditions (or mpp link)
   USE sbctide
   USE tideini
   USE dynspg_oce
   USE sbc_oce, only:  ice_msk

   IMPLICIT NONE
   PRIVATE
   PUBLIC   tfilt_init        ! routine to be called once before any of tfilt_ubar_dmp, tfilt_usurf
   PUBLIC   tfilt_ubar_dmp    ! routine called by dynspg_ts.F90
   PUBLIC   tfilt_usurf       ! routine called by sbcblk_clio.f90   
!   
   INTEGER  ::   ji, jj, jk   ! dummy loop indices
   REAL(wp), ALLOCATABLE,SAVE, dimension(:) :: K_SN
   REAL(wp), ALLOCATABLE,SAVE, dimension(:) :: H_SN
   REAL(wp), ALLOCATABLE,SAVE, dimension(:,:) :: M_SN
   REAL(wp), ALLOCATABLE,SAVE, dimension(:,:) :: KH_SN
   REAL(wp), ALLOCATABLE,SAVE, dimension(:,:) :: Eye_SN
   REAL(wp), ALLOCATABLE,SAVE, dimension(:,:) :: M_update
   REAL(wp), dimension(8) :: omdts

CONTAINS

   SUBROUTINE tfilt_ubar_dmp( kt )
      !!----------------------------------------------------------------------
      !!                   ***  ROUTINE tfilt_ubar_dmp  ***
      !!                  
      !! ** Purpose : t_filt the diff in ubar between mod and TPXO and do the nudging/damping 
      !!----------------------------------------------------------------------
      !
      INTEGER ::   kt                ! ocean time-step index
      REAL(wp) :: a_tide             ! local scalars
      !!----------------------------------------------------------------------

      DO jj = 2, jpjm1   ! vector opt.
         DO ji = 2, jpim1

            if(pot_mask(ji,jj) .gt. 0.0)  then ! apply over deep water only, save cost
               s1_SN(:,ji,jj)=matmul(M_update, s1_SN(:,ji,jj)) + K_SN*(u_tpxo2(ji,jj)-ua_sn(ji,jj))
               a_tide=dot_product(H_SN,s1_SN(:,ji,jj))
               ua_sn(ji,jj)=ua_sn(ji,jj)+a_tide*pot_mask(ji,jj)        

               s2_SN(:,ji,jj)=matmul(M_update, s2_SN(:,ji,jj)) + K_SN*(v_tpxo2(ji,jj)-va_sn(ji,jj))
               a_tide=dot_product(H_SN,s2_SN(:,ji,jj))  
               va_sn(ji,jj)=va_sn(ji,jj)+a_tide*pot_mask(ji,jj)       
            end if

         END DO
      END DO
   END SUBROUTINE tfilt_ubar_dmp

   SUBROUTINE tfilt_usurf( kt)
      !!----------------------------------------------------------------------
      !!                   ***  ROUTINE tfilt_usurf  ***
      !!                  
      !! ** Purpose : t_filt surface currents 
      !!----------------------------------------------------------------------
      !
      INTEGER ::   kt                ! ocean time-step index
      REAL(wp) :: a_tide             ! local scalars
      !!----------------------------------------------------------------------

!   
      DO jj = 2, jpjm1   ! vector opt.
         DO ji = 2, jpim1

            if(ice_msk(ji,jj) .gt. 2.0)  then ! apply for north pole only                
               s3_SN(:,ji,jj)=matmul(M_update, s3_SN(:,ji,jj)) + K_SN*us_sn(ji,jj)
               a_tide=dot_product(H_SN,s3_SN(:,ji,jj))
               us_sn(ji,jj)=a_tide

               s4_SN(:,ji,jj)=matmul(M_update, s4_SN(:,ji,jj)) + K_SN*vs_sn(ji,jj)
               a_tide=dot_product(H_SN,s4_SN(:,ji,jj))
               vs_sn(ji,jj)=a_tide
            end if

         END DO
      END DO
   END SUBROUTINE tfilt_usurf   

   SUBROUTINE tfilt_init(kt)
      INTEGER ::   kt   ! ocean time-step index

      ALLOCATE( K_SN(NX), H_SN(NX) )
      ALLOCATE( M_SN(NX,NX), KH_SN(NX,NX), Eye_SN(NX,NX) , M_update(NX,NX))
      IF(lwp) THEN
         WRITE(numout,*)  kt
         WRITE(numout,*) 't_filt : Initialization'
         WRITE(numout,*) '~~~~~~~~ '
      ENDIF
      K_SN(:)=0.0_wp
      H_SN(:)=0.0_wp
      M_SN(:,:)=0.0_wp
      KH_SN(:,:)=0.0_wp
      Eye_SN(:,:)=0.0_wp
      M_update(:,:)=0.0_wp

      !olh if starting from rest, no restart
      IF (.not. ln_rstart) THEN
         s1_SN(:,:,:)=0.0_wp
         s2_SN(:,:,:)=0.0_wp
         s3_SN(:,:,:)=0.0_wp
         s4_SN(:,:,:)=0.0_wp
      ENDIF
      

   !  K_SN(1)=0_wp*kappa_SN
   !  H_SN(1)=0_wp;
   !  M_SN(1,1)=0._wp           
      omdts(1)=2_wp*rpi/44714.16470_wp*rdt
      omdts(2)=2_wp*rpi/43200.0_wp*rdt
      !43200., 45569.88, 43081.92,  86164.2,  92949.52
      omdts(3)=2_wp*rpi/92949.52_wp*rdt
      omdts(4)=2_wp*rpi/86164.2_wp*rdt
      omdts(5)=2_wp*rpi/45570.081722_wp*rdt
      omdts(6)=2_wp*rpi/43082.041370_wp*rdt
      omdts(7)=2_wp*rpi/96726.0905_wp*rdt
      omdts(8)=2_wp*rpi/86637.1779_wp*rdt

      DO jj = 1, int(NX/2)
         K_SN(2*jj-1)=2_wp*kappa_SN
         H_SN(2*jj-1)=1_wp
         M_SN(2*jj-1, 2*jj-1) =  cos(omdts(jj))
         M_SN(2*jj-1, 2*jj)   = -sin(omdts(jj))
         M_SN(2*jj, 2*jj-1)   =  sin(omdts(jj))
         M_SN(2*jj, 2*jj)     =  cos(omdts(jj))
      ENDDO 

      DO jj = 1, NX
         DO ji = 1, NX
            KH_SN(ji,jj)=K_SN(ji)*H_SN(jj)
         ENDDO     
         Eye_SN(jj,jj)=1._wp
      ENDDO

      M_update = matmul(Eye_SN-KH_SN,M_SN)

   END SUBROUTINE tfilt_init
!
END MODULE t_filt
