MODULE traadv_sl

   USE par_oce          ! ocean parameters
   USE dom_oce          ! ocean space and time domain
   USE oce              ! ocean dynamics and tracers variables
   USE sbc_oce          ! sbc variables
   USE in_out_manager   ! I/O manager
   USE wrk_nemo         ! Memory allocation
   USE lib_mpp          ! distributed memory computing library
   USE lbclnk           ! ocean lateral boundary conditions (or mpp link)
   USE prtctl 
   USE semilag_adv      ! semi-Lagrangian advection core routines
   USE sbcrnf           ! runoff module
   USE phycst, ONLY: r1_rau0           

   IMPLICIT NONE
   PRIVATE

   PUBLIC tra_adv_sl     ! routine called by step.F90

   !! * Substitutions
#  include "domzgr_substitute.h90"
#  include "vectopt_loop_substitute.h90"

CONTAINS

!copyright (C) 2001  MSC-RPN COMM  %%%MC2%%%
!***s/r slag3d -- Upstream interpolation: Semi-Lagrangian Scheme
!*

      subroutine tra_adv_sl( kt, kit000, cdtype, ptb, ptn, pta, kjpt, do_limit, toler )
! arguments
      INTEGER                              , INTENT(in   ) ::   kt              ! ocean time-step index
      INTEGER                              , INTENT(in   ) ::   kit000          ! first time step index
      CHARACTER(len=3)                     , INTENT(in   ) ::   cdtype          ! =TRA or TRC (tracer indicator)
      INTEGER                              , INTENT(in   ) ::   kjpt            ! number of tracers
      LOGICAL                              , INTENT(in   ) ::   do_limit        ! apply limiting
      REAL(wp),                              INTENT(in   ) ::   toler           ! tolerance (do_limit=.true.)
      REAL(wp), DIMENSION(jpi,jpj,jpk,kjpt), INTENT(in   ) ::   ptb, ptn        ! before and now tracer fields
      REAL(wp), DIMENSION(jpi,jpj,jpk,kjpt), INTENT(inout) ::   pta             ! tracer trend
!*
!*
!*AUTHORs       M. Desgagne & C. Girard
!*              P. Pellerin: Solid Object
!*
!*OBJECT
!*
!*    *******************************************************************
!*    *                                                                 *
!*    *               ANDRE ROBERT  SEMI-LAGRANGIAN SCHEME              *
!*    *                                                                 *
!*    *                 CALCULATES upstream values fup                  *
!*    *                                                                 *
!*    *                      of model RHS fields   f                    *
!*    *                                                                 *
!*    *          assuming rectilinear Lagrangian displacements          *
!*    *                                                                 *
!*    *                             equal to                            *
!*    *                                                                 *
!*    *                            xd, yd, zd                           *
!*    *                                                                 *
!*    *        and taking into account the type of grids involved       *
!*    *                                                                 *
!*    * ( displacement fields xd, yd, zd known for the grid of type-p ) *
!*    *                                                                 *
!*    *******************************************************************
!*
!*ARGUMENTS
!*     _________________________________________________________________
!*    |         |                                                 |     |
!*    |  NAME   | DESCRIPTION                                     | I/O |
!*    |---------|-------------------------------------------------|-----|
!*    |         |                                                 |     |
!*    |    xd   | lagrangian displacement along X                 |  i  |
!*    |    yd   | lagrangian displacement along  Y                |  i  |
!*    |    zd   | lagrangian displacement along   Z               |  i  |
!*    |   szd   | lagrangian displacement along   z (true heigth) |  i  |
!*    |_________|_________________________________________________|_____|
!*
!*
! locals
      REAL(wp), POINTER, DIMENSION(:,:,:) :: ztasl ! upstream tendencies
      REAL(wp) :: dt_leapfrog ! dt for leapfrog differencing
      INTEGER  :: ierr, jn, ji, jj, jk
      REAL(wp) :: ptw1, ptw2

      IF(lwp) THEN
         WRITE(numout,*)
         WRITE(numout,*) 'semilag : Semi_Lagragian treatment of active travers'
         WRITE(numout,*) '~~~~~~~~~~~'
      ENDIF

      IF ( kt == kit000 ) THEN
        IF ( .not. allocated( xd ) ) THEN
          ALLOCATE( xd(jpi,jpj,jpk), yd(jpi,jpj,jpk), zd(jpi,jpj,jpk), STAT=ierr )
          IF( ierr /= 0 )   CALL ctl_warn('tra_adv_sl: failed to allocate arrays')
        ENDIF
      ENDIF
      CALL wrk_alloc( jpi, jpj, jpk, ztasl )

      ztasl = 0

      ! By default, we compute a phantom trend based on the difference between
      ! what semi-lagrangian advection would give us after one step and
      ! the *before*-time values for temperature and salinity.  This is
      ! important for stability.
      dt_leapfrog = 2*rdt
      if (kt == kit000 .and. neuler == 0) then
         ! If this is the first timestep and the Euler step has
         ! not yet happened, then set dt_leapfrog to rdt
         dt_leapfrog = rdt
      endif

!***********************************************************************
!*
!* find upstream trajectories
!*     
!***********************************************************************

      print_diag = 0
      if (print_tdiag /= 0) then
         write (numout, *) 'Semilag advection of temperature, diagnostic point'
         write (numout, *) 'local i,j,k =', diag_i, diag_j, diag_k
         write (numout, *) 'global i,j  =', mig(diag_i), mjg(diag_j)
      endif
      if (trajectory_kt /= kt ) then
         ! If the trajectories are not stored at the current timestep, compute them
         CALL traject_sld( kt, dt_leapfrog )
         trajectory_kt = kt
      endif

      ! Advect the ``now'' tracers based on current velocities
      DO jn = 1, kjpt
         !
         if (print_tdiag /= 0) then
            print_diag = 1
            write (numout,*) 'Displacements: x'
            write (numout,'(3E14.5)') xd(diag_i-1:diag_i+1,diag_j-1:diag_j+1,diag_k)
            write (numout,*) 'Displacements: y'
            write (numout,'(3E14.5)') yd(diag_i-1:diag_i+1,diag_j-1:diag_j+1,diag_k)
            write (numout,*) 'Displacements: z'
            write (numout,'(3E14.5)') zd(diag_i-1:diag_i+1,diag_j-1:diag_j+1,diag_k)
            write (numout,*) 'Before ttracer advection: whole field, jn=', jn
            write (numout,'(5F7.3)') ptb(diag_i-2:diag_i+2,diag_j-2:diag_j+2,diag_k,jn)
            !write (numout,'(5F7.3)') ptn(diag_i-2:diag_i+2,diag_j-2:diag_j+2,diag_k,jn)
         endif
         ! Call interpolation with limiting of overshoots
         call upwnd3_new ( ztasl, ptb(:,:,:,jn), tmask, xd, yd, zd, 0.0, 0.0, 0.0, &
      &                   'noflux_between', 'noflux_between', 'noflux_between', do_limit, toler )

         ! Construct a phantom trend by taking the time difference, and add
         ! this to the existing advection trend
         pta(:,:,:,jn) = pta(:,:,:,jn) &
         + (ztasl(:,:,:) - ptb(:,:,:,jn))/dt_leapfrog*tmask(:,:,:)

         ! Modify the tracer trend to account for river runoff.  As of NEMO3.6, river runoff modifies
         ! the ocean volume through a modified phdiv (horizontal divergence).  This divergence is added
         ! just prior to computing w, so the advection routines see a convergent flow.  For compatibility,
         ! we need to remove the modification of horizontal divergence and add the tracer trend explicitely.
         ! Keeping the modification of the horizontal divergence in the non VVL case would possibly
         ! stimulate SL limiters artificially. In the non VVL case this modification is equivalent 
         ! to a virtual flux anyway (as added below).

         ! This section is largely copied from 211-238 of sbcrnf.F90
         ! Variable volume case -- not copied here for now
         ! Else: constant volume case
         IF ( ln_rnf .and. .not. lk_vvl ) THEN
           IF ( .not. ln_rnf_wsl ) CALL ctl_stop('SL tracer advection scheme must be executed with ln_rnf_wsl=T')
!TEST SL
!           do jj = 1, jpj
!           do ji = 1, jpi
!           do jk = 1, nk_rnf(ji,jj)
!              IF ( jn == jp_sal ) THEN
!                 ptw1 = 0.
!                 ptw2 = 0.
!                 IF( ln_rnf_sal ) THEN
!                    ! Re-compute salinity of river input
!                    ! rnf_tsc = sf_s_rnf * rnf * r1_rau0
!                    IF ( rnf(ji,jj) > 0.   ) &
!                         ptw1 = rnf_tsc(  ji,jj,jn) / ( rnf(  ji,jj) * r1_rau0 )
!                    IF ( rnf_b(ji,jj) > 0. ) &
!                         ptw2 = rnf_tsc_b(ji,jj,jn) / ( rnf_b(ji,jj) * r1_rau0 )
!                    ! The runoff virtual volume flux is 0.5*(rnf(ji,jj + rnf_b(ji,jj))/h_rnf(ji,jj)
!                    ! It can be re-converted to a PSU tendency by multiplying by S*r1_rau0
!                 ENDIF
!              ELSEIF ( jn == jp_tem ) THEN
!                 IF ( nn_fsbc /= 1 ) CALL ctl_stop('SL tracer advection scheme assumes nn_fsbc=1')
!                 ! A zero flux is appropriate and more straightforward
!                 ! (as compared to recuperate sst_m, nn_fsbc = 1 at CMC all the time)
!                 ptw1 = ptn(ji,jj,jk,jn)
!                 ptw2 = ptn(ji,jj,jk,jn)
!                 IF( ln_rnf_tem ) THEN
!                    ! Re-compute temperature of river input
!                    ! rnf_tsc = sf_t_rnf * rnf * r1_rau0
!                    IF ( rnf(ji,jj) > 0.   ) &
!                         ptw1 = rnf_tsc(  ji,jj,jn) / ( rnf(  ji,jj) * r1_rau0 )
!                    IF ( rnf_b(ji,jj) > 0. ) &
!                         ptw2 = rnf_tsc_b(ji,jj,jn) / ( rnf_b(ji,jj) * r1_rau0 )
!                    ! The runoff virtual volume flux is 0.5*(rnf(ji,jj) + rnf_b(ji,jj))/h_rnf(ji,jj)
!                    ! It can be re-converted to a T tendency by multiplying by T*r1_rau0
!                 ENDIF 
!              ENDIF
!              pta(ji,jj,jk,jn) = pta(ji,jj,jk,jn) + 0.5 * ( &
!                      ( ptw1 - ptn(ji,jj,jk,jn) ) * rnf(  ji,jj) &
!                    + ( ptw2 - ptn(ji,jj,jk,jn) ) * rnf_b(ji,jj) ) * r1_rau0 / h_rnf(ji,jj)
!           enddo
!           enddo
!           enddo 
         ELSEIF ( lk_vvl ) THEN
           CALL ctl_stop('SL with VVL needs adaptation')
           ! Do not forget re-computation of h_rnf in div_cur for VVL case, now skipped !!
         ENDIF

         if (print_tdiag /= 0) then
            write (numout,*) 'After tracer advection: whole field'
            write (numout,'(3E14.5)') ztasl(diag_i-1:diag_i+1,diag_j-1:diag_j+1,diag_k)
            write (numout,*) 'Trend'
            write (numout,'(3E14.5)') pta(diag_i-1:diag_i+1,diag_j-1:diag_j+1,diag_k,jn)
!            if (diag_k > 1) then
!               write (numout,*) 'Comparable cen2 vertical trend'
!               write (numout,'3E14.5') -wn(diag_i-1:diag_i+1,diag_i-1:diag_i+1,diag_k)*0.5*    &
!               &                        (ptn(diag_i-1:diag_i+1,diag_j-1:diag_j+1,diag_k-1,jn) +    &
!               &                         ptn(diag_i-1:diag_i+1,diag_j-1:diag_j+1,diag_k,jn))  +    &
!               &                        wn(diag_i-1:diag_i+1,diag_i-1:diag_i+1,diag_k+1)*0.5*  &
!               &                        (ptn(diag_i-1:diag_i+1,diag_j-1:diag_j+1,diag_k,jn) +      &
!               &                         ptn(diag_i-1:diag_i+1,diag_j-1:diag_j+1,diag_k+1,jn))
!            endif
            print_diag = 0
         endif

         CALL lbc_lnk( pta(:,:,:,jn), 'T', 1. )

      ENDDO

      CALL wrk_dealloc( jpi, jpj, jpk, ztasl )
!*
!*---------------------------------------------------------------------
      end subroutine tra_adv_sl

end module traadv_sl
