MODULE trasdmp
   !!======================================================================
   !!                       ***  MODULE  trasdmp  ***
   !! Ocean physics: spectral restoring on active tracers (T and S)
   !!======================================================================
#if defined key_trasdmp
   !!----------------------------------------------------------------------
   !!   key_trasdmp                                        spectral damping
   !!----------------------------------------------------------------------
   !!
   !!----------------------------------------------------------------------
   !!   Subroutines
   !!----------------------------------------------------------------------
   !!   init_spnudge       : initializes spectral nudging and bias reduction
   !!   init_filters       : specify the filters to be applied to nudges
   !!   tra_sdmp           : called from step to calculate and add the nudges to the tracer trends
   !!   spnudge_update     : called from tra_sdmp to determine spectral nudge updates
   !!   time_filt          : determine the mean and annual cycle of 3D fields and store in sm3
   !!   space_filt_3d      : spatially smooth any 3D field and return results in the input variable
   !!   space_filt_2d      : spatially smooth any 2D field and return results in the input variable

   !!----------------------------------------------------------------------
   !!   logical parameters to allow namelist choices
   !!----------------------------------------------------------------------
   !!   lk_apply_nudges           : spectral nudges not actually applied unless true
   !!   lk_time_filt_nudge        : determine the mean and annual cycle
   !!   lk_space_filt_nudge       : spatially smooth TR_NUDGE
   !!   lk_recursive_space_filt   : apply the space filter each time sm3 is updated
   !!   lk_seasonal_correction    : include nudging of the seasonal cycle
   !!   lk_std_sponge_layers      : sponge layers without time filtering
   !!   lk_sp_sponge_layers       : sponge layers with time filtering
   !!   lk_north_sponge_layer     : sponge layer along the northern boundary
   !!   lk_south_sponge_layer     : sponge layer along the southern boundary
   !!   lk_east_sponge_layer      : sponge layer along the eastern boundary
   !!   lk_west_sponge_layer      : sponge layer along the western boundary
   !!----------------------------------------------------------------------
 
  !! * Modules used
   USE oce            ! ocean: variables
   USE dom_oce        ! ocean: domain variables
   USE c1d            ! 1D vertical configuration
   USE trd_oce        ! trends: ocean variables
   USE trdtra         ! trends manager: tracers 
   USE zdf_oce        ! ocean: vertical physics
   USE phycst         ! physical constants
   USE restart        ! only for lrst_oce
   USE dtatsd         ! data: temperature & salinity
   USE in_out_manager ! I/O manager
   USE lib_mpp        ! MPP library
   USE daymod         ! calendar                         (day routine)
   USE iom            ! for reading input netcdf and read/write restart
   
   IMPLICIT NONE
   PRIVATE
   SAVE

   !! * Routine accessibility
   PUBLIC tra_sdmp, space_filt_3d, init_spnudge, init_filters, trasdmp_rst

   !! * Share npass so that diawri can call space_filt_3d with the right parameters
! FD   PUBLIC npass

   !! * Shared module variables
   LOGICAL , PUBLIC, PARAMETER ::   lk_trasdmp = .true.    !: spectral damping flag

   !! 2,3 is for four variables with three parameters to represent them
   REAL(wp), PUBLIC, ALLOCATABLE, DIMENSION(:,:,:)     :: restsp   ! 1/spectral restoring time (3D)
   REAL(wp), PUBLIC, ALLOCATABLE, DIMENSION(:,:)       :: reststd  ! 1/standard restoring time (2D)
   REAL(wp), PUBLIC, ALLOCATABLE, DIMENSION(:,:,:,:,:) :: sm3      ! mean and seasonal cycle
   REAL(wp), PUBLIC, ALLOCATABLE, DIMENSION(:,:,:,:)   :: tr_nudge ! actual term applied in tracer trend

   LOGICAL, PUBLIC  :: lk_time_filt_nudge             ! time filter T/F=on/off
   LOGICAL, PUBLIC  :: lk_space_filt_nudge            ! space filter T/F=on/off
   LOGICAL          :: lk_recursive_space_filt        ! recursive space filter T/F=on/off
   LOGICAL          :: lk_apply_nudges                ! apply nudge T/F=on/off
   LOGICAL          :: lk_seasonal_correction         ! apply seasonal correction T/F=on/off
   LOGICAL          :: lk_std_sponge_layers           ! apply standard nudging in sponge layers T/F=on/off
   LOGICAL          :: lk_sp_sponge_layers            ! apply spectral nudging in sponge layers T/F=on/off
   LOGICAL          :: lk_north_sponge_layer          ! apply sponge layer at north wall T/F=on/off
   LOGICAL          :: lk_south_sponge_layer          ! apply sponge layer at south wall T/F=on/off
   LOGICAL          :: lk_east_sponge_layer           ! apply sponge layer east wall T/F=on/off
   LOGICAL          :: lk_west_sponge_layer           ! apply sponge layer at west wall T/F=on/off

   !! * Module variables
   REAL(wp), DIMENSION(3,3) :: Trans3
   REAL(wp), DIMENSION(3)   :: zz3
   REAL(wp), DIMENSION(3)   :: K3

   REAL(WP) :: tyear, days_in_year
   REAL(WP) :: restsp0
   REAL(WP) :: kappa                       ! time filter bandwidth
   REAL(WP) :: max_kappa_inverse           ! specified in years; inverse gives minimum bandwidth
   REAL(wp) :: ndt_per_day                 ! # steps per day
   INTEGER  :: nudge_update_interval       ! # steps between updates of nudges
   INTEGER  :: seasonal_correction_depth   ! limiting depth for the seasonalcorrection in grid cells
   INTEGER, PUBLIC  :: npass                       ! # spatial smoother iterations
   INTEGER  :: M_alpha                     ! alpha/kappa used in recursive filtering
   LOGICAL  :: ln_arctic_mask              ! Arctic masking logical
   REAL(wp) :: rn_arctic_maskdepth         ! Arctic masking maximum depth (m)
   REAL(wp) :: rn_arctic_masklat           ! Arctic masking mimimum latitude (deg)
   LOGICAL  :: ln_read_mask_trasdmp2d      ! logical for reading or not a 2D masking file for spectral nudging
   LOGICAL  :: ln_read_mask_trasdmp3d      ! logical for reading or not a 3D masking file for spectral nudging

   !! * Substitutions
#  include "domzgr_substitute.h90"
#  include "vectopt_loop_substitute.h90"
   !!----------------------------------------------------------------------
   !!   OPA 9.0 , LOCEAN-IPSL (2005) 
   !! $Header: /home/opalod/NEMOCVSROOT/NEMO/OPA_SRC/TRA/trasdmp.F90,v 1.9 2005/09/02 15:45:33 opalod Exp $ 
   !! This software is governed by the CeCILL licence see modipsl/doc/NEMO_CeCILL.txt 
   !!----------------------------------------------------------------------

CONTAINS

   INTEGER FUNCTION trasdmp_alloc()
      !!---------------------------------------------------------------------
      !!                  ***  FUNCTION trasdmp_alloc  ***
      !!---------------------------------------------------------------------
      INTEGER :: ierr(4)

      !!---------------------------------------------------------------------
      ierr(:) = 0
      !
      ALLOCATE( restsp  (jpi,jpj,jpk       ), STAT=ierr(1) )   ! 1/spectral restoring time (3D)
      ALLOCATE( reststd (jpi,jpj           ), STAT=ierr(2) )   ! 1/standard restoring time (2D)
      ALLOCATE( sm3     (jpi,jpj,jpk,jpts,3), STAT=ierr(3) )   ! main array storing the mean and seasonal cycle of jpts tracer field
      ALLOCATE( tr_nudge(jpi,jpj,jpk,jpts  ), STAT=ierr(4) )   ! actual nudge applied in tracer trend
      !
      trasdmp_alloc = MAXVAL( ierr )
      IF( lk_mpp            )   CALL mpp_sum ( trasdmp_alloc )
      IF( trasdmp_alloc > 0 )   CALL ctl_warn('trasdmp_alloc: allocation of arrays failed')
      !
   END FUNCTION trasdmp_alloc

   SUBROUTINE init_spnudge
      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE init_spnudge
      !! 
      !! ** Purpose :   Basic initialization for the spectral damping 
      !!                Initialization of filter details done separately in init_filters
      !!
      !! ** Method  :   read the namsdp namelist and check the parameters
      !!                called in opa.F90 with other initialization routines
      !!
      !! History :
      !!----------------------------------------------------------------------

      !! * Local declarations
 
      implicit none
      INTEGER  :: ji, jj, jk               ! dummy loop indices
      INTEGER  :: sponge_layer_width       ! width of the sponge layers in grid cells
      REAL(wp) :: sponge_layer_tau_min     ! minimum restoring time applied at the outer edges of the sponge layers
      REAL(wp) :: tausl                    ! temporary variable to hold minimum restoring time
      REAL(wp) :: tausp                    ! damping time scale (days) used locally
      INTEGER  ::  inum                    ! temporary logical unit
      INTEGER  ::  ios                     ! Local integer for output status of namelist read

      !!----------------------------------------------------------------------
      NAMELIST/namsdp/ lk_apply_nudges,       &
                       lk_std_sponge_layers,  &
                       lk_sp_sponge_layers,   &
                       lk_north_sponge_layer, &
                       lk_south_sponge_layer, &
                       lk_east_sponge_layer,  &
                       lk_west_sponge_layer,  &
                       sponge_layer_width,    &
                       sponge_layer_tau_min,  &
                       tausp,                 &
                       ln_arctic_mask,        &
                       rn_arctic_maskdepth,   &
                       rn_arctic_masklat,     &
                       ln_read_mask_trasdmp2d, &
                       ln_read_mask_trasdmp3d
      !!----------------------------------------------------------------------
 
      ! Set default values
      ! --------------------
      lk_apply_nudges = .false.
      lk_std_sponge_layers = .false.
      lk_sp_sponge_layers = .false.
      lk_north_sponge_layer = .false.
      lk_south_sponge_layer = .false.
      lk_east_sponge_layer = .false.
      lk_west_sponge_layer = .false.
      sponge_layer_width = 24
      sponge_layer_tau_min = 2.0_wp
      tausp = 20.0_wp
      ln_arctic_mask      = .false.
      rn_arctic_maskdepth = 200.0_wp
      rn_arctic_masklat   = 70.0_wp
      ln_read_mask_trasdmp2d = .false.
      ln_read_mask_trasdmp3d = .false.
      

      ! Read Namelist namsdp : temperature and salinity damping term
      ! --------------------
      REWIND( numnam_ref )   ! Namelist namtra_dmp in reference namelist : T & S relaxation
      READ  ( numnam_ref, namsdp, IOSTAT = ios, ERR = 901)
901   IF( ios /= 0 ) CALL ctl_nam ( ios , 'namsdp in reference namelist', lwp )
      !
      REWIND( numnam_cfg )   ! Namelist namtra_dmp in configuration namelist : T & S relaxation
      READ  ( numnam_cfg, namsdp, IOSTAT = ios, ERR = 902 )
902   IF( ios /= 0 ) CALL ctl_nam ( ios , 'namtra_dmp in configuration namelist', lwp )
      IF(lwm) WRITE ( numond, namsdp )
      !
      tausl = sponge_layer_tau_min

      ! Parameter control and print
      ! ---------------------------
      IF(lwp) THEN
         WRITE(numout,*)
         WRITE(numout,*) '  tra_sdmp : T and S spectral damping'
         WRITE(numout,*) '  ~~~~~~~~'
         WRITE(numout,*) '  Namelist namsdp : set spectral damping parameters'
         WRITE(numout,*)
         WRITE(numout,*) '  Surface spectral damping time scale (days)  = ', tausp
         IF ( lk_apply_nudges ) THEN
            WRITE(numout,*) '  Nudges will be computed and applied: lk_apply_nudges = ', lk_apply_nudges
         ELSE
            WRITE(numout,*) '  Nudges will be computed but NOT applied: lk_apply_nudges = ', lk_apply_nudges
         ENDIF
         WRITE(numout,*)
         IF ( lk_std_sponge_layers ) THEN
            WRITE(numout,*) '  Sponge layers included without time filtering'
            WRITE(numout,*) '  Sponge layer width is ', sponge_layer_width, ' grid cells'
         ENDIF
         IF ( lk_sp_sponge_layers ) THEN
            WRITE(numout,*) '  Sponge layers included with time filtering'
            WRITE(numout,*) '  Sponge layer width is ', sponge_layer_width, ' grid cells'
         ENDIF
         WRITE(numout,*)
      ENDIF

      IF( trasdmp_alloc() /= 0 )   CALL ctl_stop( 'STOP', 'init_spnudge : unable to allocate trasdmp arrays' )
      restsp  = 0.0_wp
      reststd = 0.0_wp

 
      IF(lwp) THEN
!          WRITE(numout,*) '  rday                   = ', rday
         WRITE(numout,*) '  tausp                  = ', tausp
         WRITE(numout,*) '  lk_north_sponge_layer  = ', lk_north_sponge_layer
         WRITE(numout,*) '  lk_south_sponge_layer  = ', lk_south_sponge_layer
         WRITE(numout,*) '  lk_east_sponge_layer   = ', lk_east_sponge_layer
         WRITE(numout,*) '  lk_west_sponge_layer   = ', lk_west_sponge_layer
         WRITE(numout,*) '  sponge_layer_width     = ', sponge_layer_width
         WRITE(numout,*) '  sponge_layer_tau_min   = ', sponge_layer_tau_min
         WRITE(numout,*) '  ln_arctic_mask         = ', ln_arctic_mask
         WRITE(numout,*) '  rn_arctic_maskdepth    = ', rn_arctic_maskdepth
         WRITE(numout,*) '  rn_arctic_masklat      = ', rn_arctic_masklat
         WRITE(numout,*) '  ln_read_mask_trasdmp2d = ', ln_read_mask_trasdmp2d
         WRITE(numout,*) '  ln_read_mask_trasdmp3d = ', ln_read_mask_trasdmp3d
      ENDIF
      
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!     Note that the following code implements vertically and horizontally uniform restoring but         !
!     you can modify restsp to have vertical structure or even eliminate the nudging in certain areas   !
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      
      restsp0 = 1.0_wp/(tausp*rday)
      DO jk = 1, jpkm1
         DO jj = 1, jpj
            DO ji = 1, jpi
               restsp(ji,jj,jk) =  restsp0
             END DO
         END DO
      END DO

! FD add a mask [0:1] for more complex geometry reduction
      IF ( ln_read_mask_trasdmp2d ) THEN
         CALL iom_open( 'mask_trasdmp2d.nc', inum )  
         CALL iom_get ( inum, jpdom_data, 'mask', reststd )
         CALL iom_close (inum)
         DO jk = 1, jpkm1
            restsp(:,:,jk) =  restsp0 * reststd(:,:)
         END DO
         reststd(:,:) = 0.0_wp
      ENDIF
      IF ( ln_read_mask_trasdmp3d ) THEN
         CALL iom_open( 'mask_trasdmp3d.nc', inum )  
         CALL iom_get ( inum, jpdom_data, 'mask', restsp )
         CALL iom_close (inum)
      ENDIF

! FD additional masking for Arctic
      IF ( ln_arctic_mask ) THEN
         DO jk=1,jpk
            WHERE( gphit(:,:) .gt. rn_arctic_masklat .AND. &
              (gdept_0(:,:,jk).lt. rn_arctic_maskdepth) ) restsp(:,:,jk) =  0.0_wp
         END DO
      ENDIF

      IF ( lk_sp_sponge_layers ) THEN
         !Allow for spectral sponge layers with enhanced restoring by the open boundaries
         DO jk = 1, jpkm1
            DO ji = 1, sponge_layer_width
               IF ( lk_west_sponge_layer .and. mi0(ji).eq.mi1(ji) )                          &
                  restsp(mi0(ji),:,jk) = MAX(restsp(mi0(ji),:,jk),                           &
                     (REAL(sponge_layer_width-ji,wp)+1.0)                                    &
                        / (REAL(sponge_layer_width,wp)*tausl*rday)) ! west sponge layer
               IF ( lk_south_sponge_layer .and. mj0(ji).eq.mj1(ji) )                         &
                  restsp(:,mj0(ji),jk) = MAX(restsp(:,mj0(ji),jk),                           &
                     (REAL(sponge_layer_width-ji,wp)+1.0)                                    &
                        / (REAL(sponge_layer_width,wp)*tausl*rday)) ! south sponge layer 
               IF ( lk_east_sponge_layer .and. mi0(jpiglo-ji+1).eq.mi1(jpiglo-ji+1) )        &
                  restsp(mi1(jpiglo-ji+1),:,jk) = MAX(restsp(mi1(jpiglo-ji+1),:,jk), &
                     (REAL(sponge_layer_width-ji,wp)+1.0)                                    &
                       / (REAL(sponge_layer_width,wp)*tausl*rday)) ! east sponge layer 
               IF ( lk_north_sponge_layer .and. mj0(jpjglo-ji+1).eq.mj1(jpjglo-ji+1) )       &
                  restsp(:,mj1(jpjglo-ji+1),jk) = MAX(restsp(:,mj1(jpjglo-ji+1),jk),         &
                     (REAL(sponge_layer_width-ji,wp)+1.0)                                    &
                       / (REAL(sponge_layer_width,wp)*tausl*rday)) ! north sponge layer
            END DO
         END DO
      ENDIF

      IF ( lk_std_sponge_layers ) THEN
         !Allow for standard sponge layers with enhanced restoring by the open boundaries
         DO ji = 1, sponge_layer_width
            IF ( lk_west_sponge_layer .and. mi0(ji).eq.mi1(ji) ) THEN
               reststd(mi0(ji),:) = MAX(restsp0,       &
                  (REAL(sponge_layer_width-ji,wp)+1.0) &
                 /(REAL(sponge_layer_width,wp)*tausl*rday)) ! west sponge layer
            ENDIF
            IF ( lk_south_sponge_layer .and. mj0(ji).eq.mj1(ji) ) THEN
               reststd(:,mj0(ji)) = MAX(restsp0,       &
                  (REAL(sponge_layer_width-ji,wp)+1.0) &
                 /(REAL(sponge_layer_width,wp)*tausl*rday)) ! south sponge layer
            ENDIF
            IF ( lk_east_sponge_layer .and. mi0(jpiglo-ji+1).eq.mi1(jpiglo-ji+1) ) THEN
               reststd(mi1(jpiglo-ji+1),:) = MAX(restsp0,  &
                  (REAL(sponge_layer_width-ji,wp)+1.0)     &
                 /(REAL(sponge_layer_width,wp)*tausl*rday)) ! east sponge layer
            ENDIF
            IF ( lk_north_sponge_layer .and. mj0(jpjglo-ji+1).eq.mj1(jpjglo-ji+1) ) THEN
               reststd(:,mj1(jpjglo-ji+1)) = MAX(restsp0,  &
                  (REAL(sponge_layer_width-ji,wp)+1.0)     &
                 /(REAL(sponge_layer_width,wp)*tausl*rday)) ! north sponge layer
            ENDIF
         END DO
      ENDIF

      !Mask 3D restsp and 2D reststd arrays
      restsp = restsp * tmask
      reststd = reststd * tmask(:,:,1)

   END SUBROUTINE init_spnudge

!***********************************************************************

   SUBROUTINE init_filters
      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE init_filters  ***
      !! 
      !! ** Purpose :   Initialize the time and space filter parameters
      !!                used in spectral damping 
      !!
      !! ** Method  :   read the nam_filters namelist and set parameters
      !!
      !!----------------------------------------------------------------------
      !! * Local declarations

      implicit none
      REAL(wp) :: omdt
      INTEGER  ::  ios                     ! Local integer for output status of namelist read

      !!----------------------------------------------------------------------
      NAMELIST/nam_filters/ lk_time_filt_nudge,  &
                            max_kappa_inverse, &
                            lk_seasonal_correction, &
                            seasonal_correction_depth, &
                            nudge_update_interval, &
                            lk_space_filt_nudge, &
                            lk_recursive_space_filt, &
                            npass, &
                            M_alpha
      !!----------------------------------------------------------------------

      !Set default values
      lk_time_filt_nudge = .false.
      lk_space_filt_nudge = .false.
      lk_recursive_space_filt = .false.
      lk_seasonal_correction = .false.
      seasonal_correction_depth = jpkm1
      nudge_update_interval = 1
      max_kappa_inverse = 10.0   !default to an moderately small bandwidth of 1/(10 years)
      npass = 1
      M_alpha = 100
      ! calendar parameters
      SELECT CASE ( nleapy )        ! Choose calendar for spectral window
      CASE ( 1)   ;   days_in_year = 365.25_wp
      CASE ( 0)   ;   days_in_year = 365.0_wp
      CASE (30)   ;   days_in_year = 360.0_wp
      END SELECT
      ndt_per_day = rday/rdttra(1)

      ! Read Namelist nam_filters : determine spectral nudging
      ! --------------------
      REWIND( numnam_ref )   ! Namelist nam_filters in reference namelist : T & S spectral relaxation
      READ  ( numnam_ref, nam_filters, IOSTAT = ios, ERR = 903)
903   IF( ios /= 0 ) CALL ctl_nam ( ios , 'nam_filters in reference namelist', lwp )
      !
      REWIND( numnam_cfg )   ! Namelist namtra_filters in configuration namelist : T & S spectral relaxation
      READ  ( numnam_cfg, nam_filters, IOSTAT = ios, ERR = 904 )
904   IF( ios /= 0 ) CALL ctl_nam ( ios , 'nam_filters in configuration namelist', lwp )
      IF(lwm) WRITE ( numond, nam_filters )
      !
      seasonal_correction_depth = min(seasonal_correction_depth, jpkm1)

      ! Parameter control and print
      ! ---------------------------
      IF(lwp) THEN
         WRITE(numout,*)
         WRITE(numout,*) '  Namelist nam_time_filt : spectral damping parameters'
         WRITE(numout,*) '  Tracer damping throughout the water column'
         WRITE(numout,*)
         WRITE(numout,*) '  days_in_year              = ', days_in_year
         WRITE(numout,*) '  ndt_per_day               = ', ndt_per_day
         WRITE(numout,*) '  rdttra(1)                 = ', rdttra(1)
         WRITE(numout,*) '  lk_time_filt_nudge        = ', lk_time_filt_nudge
         WRITE(numout,*) '  max_kappa_inverse         = ', max_kappa_inverse
         WRITE(numout,*) '  lk_seasonal_correction    = ', lk_seasonal_correction
         WRITE(numout,*) '  seasonal_correction_depth = ', seasonal_correction_depth
         WRITE(numout,*) '  nudge_update_interval     = ', nudge_update_interval
         WRITE(numout,*) '  lk_space_filt_nudge       = ', lk_space_filt_nudge
         WRITE(numout,*) '  lk_recursive_space_filt   = ', lk_recursive_space_filt
         WRITE(numout,*) '  npass                     = ', npass
         WRITE(numout,*) '  M_alpha                   = ', M_alpha
         WRITE(numout,*)
      ENDIF

      !Define omega * dt where omega = (2*pi)/(annual period)
      omdt = 2.0_wp*rpi/days_in_year/ndt_per_day

      !Define transition matrix to advance (mean, sin(omega*t+delta), cos(omega*t+delta))
      !to (mean, sin(omega*(t+dt)+delta), cos(omega*(t+dt)+delta))
      Trans3 = 0.0_wp
      Trans3(1,1) = 1.0_wp
      Trans3(2,2) =  cos(omdt)
      Trans3(2,3) =  sin(omdt)
      Trans3(3,2) = -sin(omdt)
      Trans3(3,3) =  cos(omdt)

      !Select the mean and sine components. Cosine is an auxiliary variable.
      zz3(1) = 1.0_wp
      zz3(2) = 1.0_wp
      zz3(3) = 0.0_wp

      !Gain factors used in updating the spectral nudging components.
      K3(1) = 1.0_wp
      K3(2) = 2.0_wp
      K3(3) = 0.0_wp

   END SUBROUTINE init_filters

!***********************************************************************

   SUBROUTINE tra_sdmp(kt)
      
!-----------------------------------------------------------------------
!
!     add the spectral nudge to the tracer trends stored in ta and sa
!       
!-----------------------------------------------------------------------

      implicit none
      INTEGER :: kt       ! time index
      INTEGER :: jk

      IF( kt == nit000 ) THEN
         !
         IF(lwp) THEN
            WRITE(numout,*)
            WRITE(numout,*) 'tra_sdmp : T and S spectral damping'
            WRITE(numout,*) '~~~~~~~~'
         ENDIF

         CALL trasdmp_rst( nit000, 'READ' )   ! read or initialize fields
      ENDIF

      call spnudge_update(kt) ! based on tsb fields for stability

      IF( lk_apply_nudges ) THEN
        tsa(:,:,:,:) = tsa(:,:,:,:) + tr_nudge(:,:,:,:)
      ENDIF

      ! write spectral nudging variables in restart file
      ! --------------------------------------------------
      IF( lrst_oce )   CALL trasdmp_rst( kt, 'WRITE' )

   END SUBROUTINE tra_sdmp

!***********************************************************************

   SUBROUTINE spnudge_update(kt)
      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE spnudge_update  ***
      !!
      !! ** Method: Determine instantaneous nudges and THEN filter them.
      !!
      !! Note that the nudges calculated here are nudges per unit time.
      !!
      !! History :
      !!----------------------------------------------------------------------

      IMPLICIT NONE
      ! arguments
      INTEGER, INTENT(in)                   ::   kt                    ! time index
      ! locals
      REAL(wp), DIMENSION(jpi,jpj,jpk,jpts) ::  zts_dta 
      REAL(wp), DIMENSION(jpi,jpj,jpk,jpts) ::  znudge
      INTEGER  ::   ji, jj, jk, jt            ! dummy loop indices
      REAL(wp), DIMENSION(2) :: ts_limit = (/ -1.8_wp, 0.0_wp /) ! limiters for the final nudge term
      !!----------------------------------------------------------------------

      CALL dta_tsd( kt, zts_dta )            ! read and interpolates T-S data at kt
      !

      !!
      DO jt = 1, jpts
         znudge(:,:,:,jt) = restsp(:,:,:) * (zts_dta(:,:,:,jt)-tsb(:,:,:,jt) )
      ENDDO

      ! First update sm3
      IF( lk_time_filt_nudge ) CALL time_filt(kt, znudge)

      ! Determine TR_NUDGE with or without time filtering
      IF( mod(kt, nudge_update_interval) == 0 )THEN
         !
         IF( lk_time_filt_nudge ) THEN
            !
            tr_nudge(:,:,:,:) = sm3(:,:,:,:,1)   ! Just the mean
            !
            IF( lk_seasonal_correction ) THEN
               !------------------------------------------------------------------
               ! The seasonal cycle is only nudged down to the level set by the
               ! parameter seasonal_correction_depth.  This parameter is included 
               ! to allow the user to avoid udging the seasonal cycle at depths 
               ! where it may be undetermined or poorly determined in the available 
               ! climatologies. 
               !------------------------------------------------------------------
               DO jt = 1, jpts
                 DO jk=1, seasonal_correction_depth
                    tr_nudge(:,:,jk,jt) = tr_nudge(:,:,jk,jt) + sm3(:,:,jk,jt,2)  ! add the seasonal cycle
!                     tr_nudge(:,:,jk,jt) = tr_nudge(:,:,jk,jt) + sm3(:,:,jk,jt,3)  ! add the seasonal cycle ! FD might not need the 3rd component
                 ENDDO
               ENDDO
            ENDIF
         ELSE ! if no time filtering, just take the difference as a starting point
            tr_nudge(:,:,:,:) = znudge(:,:,:,:)
         ENDIF

         IF( lk_std_sponge_layers ) THEN
            DO jt = 1, jpts
               DO jk=1,jpkm1
                  tr_nudge(:,:,jk,jt) = tr_nudge(:,:,jk,jt) + reststd(:,:) * (zts_dta(:,:,jk,jt)-tsb(:,:,jk,jt) )
               ENDDO
            ENDDO
         ENDIF

         ! reduces SN to standard nudging during first month of time filter
         IF(lk_time_filt_nudge .AND. tyear .lt. 0.1_wp) THEN
            DO jt = 1, jpts
               tr_nudge(:,:,:,jt) = restsp(:,:,:) * (zts_dta(:,:,:,jt)-tsb(:,:,:,jt) )
            ENDDO
         ENDIF

         ! Determine the spatially filtered TR_NUDGE
         IF( lk_space_filt_nudge .and. .not.lk_recursive_space_filt ) THEN
            DO jt = 1, jpts
               CALL space_filt_3d(tr_nudge(:,:,:,jt), npass)
            ENDDO
         ENDIF

! DW     I included this just so that the nudges can't result in sign changes.
!        This is particulary important for salinity where the dynamics can result
!        in S becoming very small and the nudges don't know anything about the
!        requirement for positive S or short time scale variations that could
!        push you close to S = 0.
         DO jt = 1, jpts
            tr_nudge(:,:,:,jt) = SIGN( MIN( ABS(tr_nudge(:,:,:,jt)),  &
            5.0_wp*abs(tsb(:,:,:,jt)-ts_limit(jt))/rday),tr_nudge(:,:,:,jt))
         ENDDO

      ENDIF

      ! XIOS/IOM output
      CALL iom_put("nudget", tr_nudge(:,:,:,jp_tem))
      CALL iom_put("nudges", tr_nudge(:,:,:,jp_sal))

   END SUBROUTINE spnudge_update

!***********************************************************************

   SUBROUTINE time_filt(kt, y)
      
!-----------------------------------------------------------------------
!
!     update the estimates of mean and annual cycle of the nudges
!     y is the instantaneous nudge which can be used for robust diag calculations
!     sm3 holds the mean, sine and cosine components of the 3D climatological 
!     nudges which are used to determine TR_NUDGE in spnudge_update.
!
!     sm3 has dimensions jpi,jpj,jpk,2,3
!         =>jpi, jpj, jpk are the x, y and z indices
!         =>The 4th argument distinguishes between temperature and salinity
!         =>The 5th argument distinguishes between mean, sine and cosine components
!           of the time filtered nudges
!
!         =>Note that sm3 is not normally spatially filtered; TR_NUDGE is. However, 
!           spatial smoothing is applied to sm3 when a recursive filtering is used.
!           The latter permits strong spatial filtering with low CPU cost.
!     
!-----------------------------------------------------------------------

      implicit none

      INTEGER, INTENT(IN) :: kt
      REAL(wp), dimension(jpi,jpj,jpk,jpts), INTENT(IN) :: y

      REAL(wp), dimension(3) :: tsm3_temp
      REAL(wp), dimension(3) :: sm3_temp
      REAL(wp) :: y_diff
                      
      INTEGER :: ji, jj, jk, ii, jt

!-----------------------------------------------------------------------
!     In the code below, I increase 1/kappa like time until 1/kappa
!     equals max_kappa_inverse years.  
!-----------------------------------------------------------------------
     
      tyear = adatrj/days_in_year
      kappa = 1.0_wp/((10.0_wp+min(max_kappa_inverse,tyear)*days_in_year)*ndt_per_day)

      DO jt = 1, jpts
         DO jk = 1, jpk
            DO jj = 1, jpj
               DO ji =1, jpi
                  !Isolate the mean, sine and cosine components for a
                  !particular variable at a particular location
                  sm3_temp(:) = sm3(ji,jj,jk,jt,:)
                  DO ii=1,3
                     !Update the estimates of each component to the new time level
                     tsm3_temp(ii) = SUM(Trans3(ii,:) * sm3_temp(:))
                  ENDDO
                  !Determine data-model misfit at the current time.
                  !The model here is the estimated mean plus annual cycle.
                  y_diff = y(ji,jj,jk,jt) - SUM(zz3(:)*tsm3_temp(:))
                  DO ii=1,3
                     !Update the mean, sine and cosine components based on the misfit.
                     sm3(ji,jj,jk,jt,ii)=tsm3_temp(ii) + kappa * K3(ii) * y_diff
                  ENDDO
               ENDDO
            ENDDO
         ENDDO
      ENDDO

      !For the special case of a recursive spatial filter, apply a single pass of
      !the spatial filter to each component of the nudges.
      IF(lk_recursive_space_filt .and. (mod(kt, nudge_update_interval) == 0) ) THEN
         DO jt = 1, jpts
            CALL space_filt_3d(sm3(:,:,:,jt,1), 1)
            CALL space_filt_3d(sm3(:,:,:,jt,2), 1)
            CALL space_filt_3d(sm3(:,:,:,jt,3), 1)
         ENDDO
      ENDIF

   END SUBROUTINE time_filt

!***********************************************************************

   SUBROUTINE space_filt_3d(fldin, niter)

!  DESCRIPTION:  Smooth a 3D field by calling space_filt_2d for each level

   implicit none
   integer :: jk          ! dummy loop indices
   integer, intent(in) :: niter
   real(wp), dimension(jpi,jpj,jpk), intent(inout) :: fldin
   real(wp), dimension(jpi,jpj) :: fld, wgt

   IF(niter == 0) return

   DO jk = 1, jpkm1
      fld(:,:) = fldin(:,:,jk)
      wgt(:,:) = fse3t_n(:,:,jk) * tmask(:,:,jk)
      CALL space_filt_2d(fld, wgt, niter)
      fldin(:,:,jk) = fld
   END DO

 end subroutine space_filt_3d

!***********************************************************************

   SUBROUTINE space_filt_2d(fld, hb0, niter)

!  !DESCRIPTION:  Smooth a 2D field
!
!  FLD is smoothed by niter applications of a
!  9-point averaging stencil:
!
!           1/16 alpha -- 2/16 alpha -- 1/16 alpha
!           |                  |                 |
!           2/16 alpha --     wtc    -- 2/16 alpha
!           |                  |                 |
!           1/16 alpha -- 2/16 alpha -- 1/16 alpha
!
!  Elements of the stencil are multiplied by the local cell thickness h_i
!  and wtc = 1 - [ sum over eight surrounding cells (h_i * wt_i)]/h_c, where
!  h_c is the thickness of the center cell.
!  Land cells are given zero thickness so they are treated the same as other 
!  points but given no weight in the averaging proceedure.
!  Note that only the center weight is modIFied so that exchanges between
!  cells remain balanced.  The normal practice of changing all weights
!  when one or more cells are land points can result in dIFferent weights being 
!  used when the cell is giving (i.e. on the boundaries of the stencil) from 
!  when it is receiving (i.e., at the center of the cell) and this can result
!  in spatial smoothing changing the net heat and salt sources and sinks. 
!  The present approach avoids this problem.
!
!  For a function of the form cos(kx)*cos(ly), a single pass of this 
!  spatial smoother in a region with unIForm cell thicknesses will have 
!  a gain of approximately 1 - 0.25*alpha*((k dx)**2 + (l dy)**2).
!  For a feature with lambda_x/dx = lambda_y/dy = 2*n (k*dx=l*dy=pi/n) 
!  the gain is 1 - 0.5*alpha*(pi/n)**2  --  for a single pass of the filter.
 
!  For the recursive filter applied to the mean (omega_c = 0) climatological
!  conditions, the complex gain for variations at frequency omega is approximately
!  1/[1 + 0.5*(alpha/Kappa)*(pi/n)**2 + i * omega*dt/(2*Kappa)].
!  If alpha = M_alpha * Kappa, the complex gain is
!  1/[1 + 0.5*M_alpha*(pi/n)**2 + (-1)^0.5 * omega*dt/(2*Kappa)].
!  At very low frequencies, the gain reduces to 1/[1+0.5*M_alpha*(pi/n)**2].
!  The reduction in this limit is due entirely to the spatial filter.
!  Transient eddies with time scales short compared to 1/Kappa are damped 
!  much less since the filtered nudge has small amplitude.
!  The reduced gain in this case is due to the time filter.

!  For a feature with a diameter of 10*dx (n=10), and using M_alpha = 100, 
!  the gain at  omega = 0 is approximately 1/6. With a diameter of 5*dx 
!  the gain with M_alpha = 100 reduces to approximately 1/21.
!-----------------------------------------------------------------------

   IMPLICIT NONE
   ! arguments
   INTEGER, INTENT(IN) :: niter
   REAL(wp), DIMENSION(jpi,jpj), INTENT(INOUT) :: fld
   REAL(wp), DIMENSION(jpi,jpj), INTENT(IN)    :: hb0

!-----------------------------------------------------------------------
!
! local or common variables:
!
!-----------------------------------------------------------------------

   INTEGER :: ji,jj,iter          ! dummy loop indices
   REAL(wp) :: hb00, alpha0
   REAL(wp), DIMENSION(jpi, jpj) ::&
      izwork,             &! num points contributing to 9pt avg
      wtc

   REAL (wp), DIMENSION(jpi, jpj) :: zwork       ! local real work space
  !-----------------------------------------------------------------------
  !
  !
   CALL lbc_lnk(fld,'T',1.0_wp) ! FD maybe not necessary

   IF(niter == 0) RETURN

   IF (lk_recursive_space_filt)THEN
      alpha0 = min(0.1_wp, M_alpha * nudge_update_interval * kappa) 
   ELSE
      alpha0 = 1.0_wp
   ENDIF

      zwork = 0.0_wp
      izwork = 0.0_wp
      wtc = 0.0_wp

      DO jj=2,jpj-1
         DO ji=2,jpi-1
            hb00=hb0(ji,jj)
            izwork(ji,jj) =  alpha0 *                  &
                  ((hb0(ji+1,jj  ) &
                  + hb0(ji-1,jj  )        &
                  + hb0(ji  ,jj+1)        &
                  + hb0(ji  ,jj-1))*2.0_wp   &
                  + hb0(ji+1,jj+1)      &
                  + hb0(ji-1,jj+1)      &
                  + hb0(ji+1,jj-1)      &
                  + hb0(ji-1,jj-1))     &
                  / 16.0_wp
            wtc(ji,jj) = hb0(ji,jj) - izwork(ji,jj)
         ENDDO
      ENDDO

      DO iter = 1, niter
   
         DO jj = 2, jpj - 1
            DO ji = 2, jpi - 1
               hb00=hb0(ji,jj)
               zwork(ji,jj) = wtc(ji,jj)*fld(ji,jj) &
                           + alpha0 *   &
                  ((fld(ji+1,jj  ) * hb0(ji+1,jj  ) &
                  + fld(ji-1,jj  ) * hb0(ji-1,jj  )        &
                  + fld(ji  ,jj+1) * hb0(ji  ,jj+1)        &
                  + fld(ji  ,jj-1) * hb0(ji  ,jj-1))*2.0_wp   &
                  + fld(ji+1,jj+1) * hb0(ji+1,jj+1)      &
                  + fld(ji-1,jj+1) * hb0(ji-1,jj+1)      &
                  + fld(ji+1,jj-1) * hb0(ji+1,jj-1)      &
                  + fld(ji-1,jj-1) * hb0(ji-1,jj-1))     &
                  / 16.0_wp
            ENDDO
         ENDDO

!-----------------------------------------------------------------------
!
!        new 2d field
!
!-----------------------------------------------------------------------

         DO jj = 2, jpj-1
            DO ji = 2, jpi-1
               IF(hb0(ji,jj).gt.1.e-10_wp) THEN
                  fld(ji,jj) = zwork(ji,jj)/hb0(ji,jj)
               ELSE
                  fld(ji,jj) = 0.0_wp
               ENDIF
            ENDDO
         ENDDO

         ! update the overlapping 'ghost cell' regions.
         ! need to verIFy that lbc_lnk is the right routine for this.
         ! I think so but haven't checked really carefully.

         CALL lbc_lnk(fld,'T',1.0_wp)

      ENDDO

 END SUBROUTINE space_filt_2d


   SUBROUTINE trasdmp_rst( kt, cdrw )
      !!---------------------------------------------------------------------
      !!                   ***  ROUTINE ts_rst  ***
      !!
      !! ** Purpose : Read or write time-splitting arrays in restart file
      !!----------------------------------------------------------------------
      INTEGER         , INTENT(in) ::   kt         ! ocean time-step
      CHARACTER(len=*), INTENT(in) ::   cdrw       ! "READ"/"WRITE" flag
      !
      INTEGER ::  ji, jk        ! dummy loop indices
      !!----------------------------------------------------------------------
      !
      IF( TRIM(cdrw) == 'READ' ) THEN
         ! tr_nudge is needed in the restart file if the updating is not done at every time step
         ! default is to have it in any case
         IF( iom_varid( numror, 'nudget', ldstop = .FALSE. ) > 0 ) THEN
            CALL iom_get(numror,jpdom_autoglo,'nudget',tr_nudge(:,:,:,jp_tem))
            CALL iom_get(numror,jpdom_autoglo,'nudges',tr_nudge(:,:,:,jp_sal))
            CALL iom_get(numror,jpdom_autoglo,'sm3_T1',sm3(:,:,:,jp_tem,1))
            CALL iom_get(numror,jpdom_autoglo,'sm3_T2',sm3(:,:,:,jp_tem,2))
            CALL iom_get(numror,jpdom_autoglo,'sm3_T3',sm3(:,:,:,jp_tem,3))
            CALL iom_get(numror,jpdom_autoglo,'sm3_S1',sm3(:,:,:,jp_sal,1))
            CALL iom_get(numror,jpdom_autoglo,'sm3_S2',sm3(:,:,:,jp_sal,2))
            CALL iom_get(numror,jpdom_autoglo,'sm3_S3',sm3(:,:,:,jp_sal,3))
         ELSE
            sm3(:,:,:,:,:)    = 0.0_wp
            tr_nudge(:,:,:,:) = 0.0_wp
         ENDIF
      ELSEIF( TRIM(cdrw) == 'WRITE' ) THEN
         CALL iom_rstput( kt, nitrst, numrow, 'nudget' , tr_nudge(:,:,:,jp_tem))
         CALL iom_rstput( kt, nitrst, numrow, 'nudges' , tr_nudge(:,:,:,jp_sal))
         CALL iom_rstput( kt, nitrst, numrow, 'sm3_T1' , sm3(:,:,:,jp_tem,1))
         CALL iom_rstput( kt, nitrst, numrow, 'sm3_T2' , sm3(:,:,:,jp_tem,2))
         CALL iom_rstput( kt, nitrst, numrow, 'sm3_T3' , sm3(:,:,:,jp_tem,3))
         CALL iom_rstput( kt, nitrst, numrow, 'sm3_S1' , sm3(:,:,:,jp_sal,1))
         CALL iom_rstput( kt, nitrst, numrow, 'sm3_S2' , sm3(:,:,:,jp_sal,2))
         CALL iom_rstput( kt, nitrst, numrow, 'sm3_S3' , sm3(:,:,:,jp_sal,3))
      ENDIF
      !
   END SUBROUTINE trasdmp_rst


!***********************************************************************

#else
   !!----------------------------------------------------------------------
   !!   Default key                                     NO spectral damping
   !!----------------------------------------------------------------------
   LOGICAL , PUBLIC, PARAMETER ::   lk_trasdmp = .FALSE.   !: spectral damping flag
CONTAINS
   SUBROUTINE spnudge_update(kt)        ! Empty routine
      INTEGER, INTENT( in ) ::   kt       ! time-step index
      WRITE(*,*) 'spnudge_update: You should not have seen this print! An error?'
   END SUBROUTINE spnudge_update

   SUBROUTINE init_spnudge
      WRITE(*,*) 'init_spnudge: You should not have seen this print! error?'
   END SUBROUTINE init_spnudge

   SUBROUTINE init_filters
      WRITE(*,*) 'init_filters: You should not have seen this print! error?'
   END SUBROUTINE init_filters

!***********************************************************************

   SUBROUTINE tra_sdmp(kt)
      INTEGER, INTENT( in ) ::   kt       ! time-step index
      WRITE(*,*) 'tra_sdmp: You should not have seen this print! An error?'
   END SUBROUTINE tra_sdmp

#endif

   !!======================================================================
END MODULE trasdmp
