      module lakesup_mod

      use par_kind

      implicit none

      real(wp) :: SupForceFlow
!                   The St. Marys River flow to use if solving for
!                   Lake Superior levels, but forcing an outflow.
!
      real(wp) :: SupSideChannel, ComputedNonGate,                     &
     &                 LastComputedNonGate
!                   The combined flow NOT going through the Compensating
!                   Works.  ComputedNonGate is the value determined by  
!                   the regulation plan, and the value persists until the 
!                   regulation plan cycles again (i.e., next month).  
!                   SupSideChannel starts out as the input read from the
!                   time series data.  If the input is -9999, the value
!                   of SupSideChannel is taken as ComputedNonGate.  If
!                   SupSideChannel comes in as anything else, it retains 
!                   that value as a forced amount. If solving for Lake 
!                   Superior levels using an outflow equation, neither is
!                   used.   LastComputedNonGate stores the value of 
!                   ComputedNonGate in case the latter is overriden by a
!                   forced sidechannel.

      real(wp) :: NominalStMarys
!                   Nominal Lake Superior outflow for the regulation 
!                   period, as determined by a regulation plan (Plan
!                   1977A, 1955 Modified Rule of '49).  Note that a
!                   regulation plan may have a different period than
!                   the routing interval.  For instance, you could
!                   route daily or quarter-monthly NBS through Lake
!                   Superior in conjunction with Plan 1977A-determined
!                   monthly outflows.  The outflow remains constant 
!                   for each month, but the level will fluctuate a
!                   little each day or quarter-month.  Note that the
!                   model completely ignores the bigger question of 
!                   whether this type of calculation has any validity.
!                   We modellers are supposed to know if what we are 
!                   doing makes sense; after all, that's why they pay
!                   us the big bucks (ROTFL).  Note that these storages 
!                   are actually aliased to memory in FWRK.

      INTEGER       BOMRound, FlowRound, LevelRound
      INTEGER       MidLakeSolution, UStLawrSolution
      INTEGER       StMarysSolution, P77ACompat
!                   1=outflow eqn, 2=1955 mod rule, 3=plan77a
!                   -1 = virtcarl, 0 = read-in, 1 = 6-month operational

      INTEGER       RoutingInterval(3), RoutingIncrement(3)
!                   Routing interval to use on Superior, middle lakes,
!                   and Ontario (1=daily, 2=weekly, 3=qmonthly, 
!                   4=monthly), and the number of increments within each 
!                   interval.

      REAL          PreviousGates

      INTEGER       MaxSGSetting
!                   See descriptions in Block Data Genral
      real(wp) :: SPK,SPYM,SPA,SPB,SPWT,SPC
      real(wp) :: SGX(17), SGY(17), SGZ(17)
!                   See descriptions in CHECKPAR and OUTFLOW
      REAL          SupGatesOpen
!                   The number of Superior gates open in current routing 
!                   period.

      real(wp) :: SPErrorFlow
!                   Used for tracking iterative convergence
      real(wp) :: SPSolutionCheck
!                   Scratch variables.  SPSolutionCheck contains the 
!                   the accuracy of superior routing (under some 
!                   methods).  LastFlow is used for convergence
!                   testing during iterative solutions. 
!                   Note that these storages are actually aliased to
!                   memory in FWRK.
      INTEGER       Day(3), Q(4,3)
!                   Start date (of month) for the period being routed.
!                   Q is an array holding the end date of each quarter-
!                   month within the month (so Q(4,L) tells month length
!                   for lake L).  
!
      INTEGER       DaysSinceSaturday
!                   See description in subroutine RUNIT
!
      real(wp) :: dT(3)
!                   "delta time" associated with routing interval (or
!                   increment).  Used in storage-change calculations.
!                   First element is for Lake Superior, second for
!                   middle lakes, third for Lake Ontario.
!
      LOGICAL       NumIncsVary(3) 
!                   True if number of increments gets adjusted on the fly 
!                   during monthly or qmonthly routing, using actual
!                   month length, to make dT come out to 1 hour. 

      INTEGER       PeriodEnd(6,3)
!                   Tells the date that the current routing period ends
!                   on for each routing interval.  PeriodEnd(1,?) gives
!                   the day, PeriodEnd(2,?) gives the quartermonth number, 
!                   PeriodEnd(3,?) gives the month, PeriodEnd(4,?) gives 
!                   the year, PeriodEnd(5,?) gives the number of elapsed
!                   days at the final day of the interval.  PeriodEnd(6,?)
!                   gives the elapsed days for the end of the next period 
!                   (for loop control).  PeriodEnd(?,1) is for the Lake 
!                   Superior routing interval, PeriodEnd(?,2) is for the
!                   middle lakes, and PeriodEnd(?,3) is for Lake Ontario.

      CHARACTER MonthAbbrev(12)*3

      real(wp) :: SPSTLV, MHSTLV, SCSTLV, ERSTLV, CGSTLV, ONSTLV
!               See descriptions in Block Data Genral
!     
      real(wp) :: SPBOMLev, MHBOMLev, SCBOMLev, ERBOMLev
!               The beginning-of-month lake levels used by Plan 1977A
!
      INTEGER   PrevStMarysFlow
!               Initial Lake Superior outflow (average for month 0)
! 
      REAL      PrevSupGates 
!               Initial gates open at Comp Works
! 
      real(wp) :: MHBOP, MHEOP, SCBOP, SCEOP, ERBOP, EREOP,              &
     &              SPBOP, SPEOP, ONBOP, ONEOP
      real(wp) :: SPBOI, SPEOI, SPNTS, SMICE, SPMLV,                     &
     &              STMRIVFLOW, SPOutI, SolutionTest
!                   used in natural plan
!           NOTE: SPNTS is actually the previous month's Net Total
!           Supply, but the called function only uses it to calculate 
!           the EOP level (and mean level), which we're not concerned 
!           with here.  Eventually, it may be cleaner (& computationaly
!           faster) to write a special subroutine for these 
!           calculations rather than reusing this one.
      real(wp) :: NIZ
!                   Variable used for two-gage equation on Niagara 
!                   River.  NIZ is mean level for previous increment.  
!                   Note that this storage is actually aliased to
!                   memory in FWRK.

      real(wp) :: NumSupForceGates, NumOntForceGates
!                   Variables used to store the number of gates open 
!                   at the Compensating Works or Long Sault Dam River,
!                   if gate settings are forced.  Note that this 
!                   storage is actually aliased to memory in FWRK.
      real(wp) :: SupDevFlow, SupForceEOPLev
!                   Deviations from Lake Superior regulated discharge,
!                   and optional level to replace the modeled BOP level.

      INTEGER       IntervalNameLen(4)
!                   Number of letters in each element of IntervalName
      INTEGER       MonthLength
      LOGICAL       LakesToSolve(8) 
      INTEGER       StartYear, StartMonth, StartDay,                  &
     &              EndYear, EndMonth, EndDay

      INTEGER       SPSolveErrors       
!                   Counts how often the simultaneous solution of the
!                   routing equations failed to achieve a certain
!                   accuracy.

      CHARACTER*9  MLSolutionName(2)
!               Strings set to 'interative' and 'matrix', used for output.
      CHARACTER*16 SMSolutionName(5), USLSolutionName(5)
      CHARACTER*8 IntervalName(4)
      CHARACTER*8  MonthLengthName(2)

      DATA BOMRound , FlowRound, LevelRound /3 * -9999/

      DATA SGX /14.03_wp, 33.12_wp, 66.13_wp,                           &
     &        99.05_wp, 140.17_wp, 181.23_wp,                           &
     &        227.17_wp, 272.27_wp, 315.35_wp,                          &
     &        340.25_wp, 355.4_wp, 372.39_wp,                           &
     &        383.33_wp, 397.8_wp, 413.62_wp,                           &
     &        427.08_wp, 437.69_wp/
!
      DATA SGY /1._wp, 1._wp, 1.0_wp, 1._wp,                            &
     &        1._wp, 1._wp, 1._wp, 0.951_wp,                            &
     &        0.954_wp, 0.951_wp, 0.946_wp,                             &
     &        0.951_wp, 0.949_wp, 0.944_wp,                             &
     &        0.946_wp, 0.944_wp, 0.944_wp/
!
      DATA SGZ /180.750_wp, 180.750_wp,                                 &
     &        180.619_wp, 180.692_wp, 180.856_wp,                       &
     &        181.006_wp, 181.137_wp, 172.270_wp,                       &
     &        172.918_wp, 172.401_wp,                                   &
     &        171.485_wp, 172.407_wp, 172.033_wp,                       &
     &        171.113_wp, 171.497_wp,                                   &
     &        171.132_wp, 171.122_wp/
!
      DATA MaxSGSetting /17/

      DATA MonthAbbrev /'Jan','Feb','Mar','Apr','May','Jun','Jul','Aug',&
     &             'Sep','Oct','Nov','Dec'/

      DATA SPSTLV,MHSTLV,SCSTLV,ERSTLV,CGSTLV,ONSTLV / 6 * -9999._wp /

      DATA PrevStMarysFlow / -9999 /
      DATA PrevSupGates  / -9999. /
      DATA SPBOMLev, MHBOMLev, SCBOMLev, ERBOMLev  / 4 * -9999._wp /


      DATA MonthLengthName /'uniform ', 'actual  '/
      DATA MonthLength / -9999/
      DATA LakesToSolve / 8 * .FALSE. /
      DATA IntervalNameLen /5,6,8,7/
      DATA IntervalName /'daily','weekly','qmonthly','monthly'/
      DATA MLSolutionName /'iterative', 'matrix   '/
      DATA SMSolutionName /'outflow equation', '1955 Mod. Rule  ',      &
     &    'Plan 1977A      ', 'Sup For Sup ', 'Sup For Michuron'/
      DATA USLSolutionName /'outflow equation', 'Plan 1958D      ',     &
     &    'IS Plan         ','Plan 1998       ','Preproject      '/
      DATA StartYear, StartMonth, StartDay  /3 * -9999/
      DATA EndYear, EndMonth, EndDay  /3 * -9999/ 
      DATA StMarysSolution, P77ACompat, MidLakeSolution,                &
     &     UStLawrSolution  / 4 * -9999/
      DATA RoutingIncrement /3 * -9999/
      DATA RoutingInterval /3 * -9999/
      DATA NumSupForceGates /-9999_wp/
      DATA SupDevFlow /-9999_wp/
      DATA SupForceEOPLev /-9999_wp/
      DATA SupForceFlow /-9999_wp/
      DATA SupSideChannel /-9999_wp/

      end module lakesup_mod
