!
!
!----&------------------------------------------------------------------
!
      SUBROUTINE P77AMidLakeRoute (SecondsInPeriod, MHNTS, SCNTS, ERNTS,&
     &   MHBOP, SCBOP, ERBOP, SCRRetard, DETRetard, NIARetard,          &
     &   Frnd, Lrnd, Mrnd, NumInc, MHEOP, SCEOP, EREOP, MHMLV,          &
     &   verbose)
!      
!     This function does middle lakes rounding as described by Plan
!     1977A.  It is a simpler, less robust method than IterativeRoute,
!     but doesn't rely as much on COMMON block data.  Is used for both
!     Plan 1977A routing, and Criterion checks.  Only the Plan 1977A
!     discharge equation constants are used from the Plan 1977A COMMON
!     block; all else must be specified as parameters.  This is a 
!     confusing approach, but that's because the existing practice for
!     Criterion checks has very messy logic.  Believe it or not, this
!     is actually a much clearer computer implementation than existing 
!     models.
!     
! modules
      USE par_kind
      USE outflow_mod

      IMPLICIT NONE
!
!     Declarations regarding subroutine arguments:
!      
      real(wp) :: SecondsInPeriod     
!                      (input) The length of the routing interval.
!
      real(wp) :: MHBOP, SCBOP, ERBOP
!                      (input) Beginning-of-Period lake levels 
!    
      INTEGER          MHNTS, SCNTS, ERNTS
!                      (Input) Net Total Supply to use for the lakes 
!                      (including diverions).
!
      INTEGER          SCRRetard, DETRetard, NIARetard
!                      (input) Retardation amounts to use
!    
      INTEGER          Frnd, Lrnd, Mrnd, NumInc
!                      (Input) Rounding levels for flows and levels,
!                      expessed as powers of ten (so that -2 means cm),
!                      and end-of-month level rounding factor,
!                      and the number of increments to use in the month.
!
      real(wp) :: MHEOP, SCEOP, EREOP, MHMLV
!                      (output) End-of-Period lake levels, and average
!                      MH level for the period.
      integer verbose
!
!                       
!
!
!     Declarations regarding local variables or functions:
!
      INTEGER          I, K
!                      Scratch variables
!
      real(wp) :: SCRIncFlow, DETIncFlow, NIAIncFlow, LMHIncLevel,      &
     &                 LSCIncLevel, LERIncLevel, MHBOI, SCBOI, ERBOI
!                      Intermediate routing terms (mean levels and
!                      flows for an increment, beginning-of-increment
!                      levels)
!
      CHARACTER*(*)    Prefix
!                      Used as prefix on lines of detailed.log 
!
      real(wp) :: Round
!                      Function for rounding.
!
      real(wp) :: GETAREA
!                      Function that returns surface area for a lake
!
      real(wp) :: OUTFLOW
!                      Function that returns connecting channel flow, 
!                      given the parameters for the discharge equation.
!
!
      PARAMETER (Prefix = 'P77A MidLake Routing')
!
      IF( verbose .GT. 0 ) THEN
         WRITE(*,11116) Prefix, ' MHBOP=',MHBOP,              &
     &      ' SCBOP=', SCBOP, ' ERBOP=', ERBOP
         WRITE(*,11117) Prefix, ' MH NTS=', MHNTS,            &
     &      ' StC NTS=', SCNTS, ' Erie NTS=', ERNTS
         WRITE(*,11117) Prefix, ' IceMH=', SCRRetard,         &
     &      ' IceSC=', DETRetard, ' IceER=', NIARetard
      ENDIF
!
!
      SCRIncFlow = 0._wp
      DETIncFlow = SCRIncFlow 
      NIAIncFlow = SCRIncFlow 
!
      MHEOP = MHBOP
      SCEOP = SCBOP
      EREOP = ERBOP
!
      MHMLV = 0._wp
!
      DO 5210,  I = 1, NumInc
!
!
         MHBOI = MHEOP
         SCBOI = SCEOP
         ERBOI = EREOP
!
!
!        Approximate the mean level for the increment as BOI level
         LMHIncLevel = MHBOI
         LSCIncLevel = SCBOI 
         LERIncLevel = ERBOI 
!
!        Iterate five times (maybe in future remove rounding, and
!        iterate to a tolerance?)
         DO 5230, K = 1, 5            
!
            SCRIncFlow = Round( OUTFLOW( LMHIncLevel, LSCIncLevel,      &
     &         P77AMHK, P77AMHYM, P77AMHA,P77AMHB, P77AMHWT, P77AMHC)   &
     &         - SCRRetard, Frnd ) 
            DETIncFlow = Round( OUTFLOW( LSCIncLevel, LERIncLevel,      &
     &         P77ASCK, P77ASCYM, P77ASCA, P77ASCB, P77ASCWT, P77ASCC)  &
     &         - DETRetard, Frnd ) 
            NIAIncFlow = Round( OUTFLOW( LERIncLevel, P77ACGIP, P77AERK,&
     &         P77AERYM, P77AERA, P77AERB, P77AERWT, P77AERC)           &
     &         - NIARetard, Frnd ) 
!
            MHEOP = Round( MHBOI + (MHNTS - SCRIncFlow) *               &
     &         SecondsInPeriod/(NumInc*GetArea(2,LMHIncLevel)), Lrnd )
            SCEOP = Round( SCBOI + (SCNTS + SCRIncFlow - DETIncFlow) *  &
     &         SecondsInPeriod/(NumInc*GetArea(5,LSCIncLevel)), Lrnd)
            EREOP = Round( ERBOI + (ERNTS + DETIncFlow - NIAIncFlow) *  &
     &         SecondsInPeriod/(NumInc*GetArea(6,LERIncLevel)), Lrnd )
!
!           Compute mean levels for the iteration
            LMHIncLevel = ( MHBOI + MHEOP ) / 2
            LSCIncLevel = ( SCBOI + SCEOP ) / 2
            LERIncLevel = ( ERBOI + EREOP ) / 2 
 5230       CONTINUE
!
! 
! 
!
         IF( verbose .GT. 0 ) THEN
            WRITE(*,11111) Prefix, I, ' OutMH=',SCRIncFlow,   &
     &         ' OutSC=', DETIncFlow,' OutER=', NIAIncFlow
            WRITE(*,11111) Prefix, I, ' MHBOI=',MHBOI,        &
     &         ' SCBOI=', SCBOI, ' ERBOI=', ERBOI
            WRITE(*,11111) Prefix, I, ' MHEOI=',MHEOP,        &
     &         ' SCEOI=', SCEOP, ' EREOI=', EREOP
            WRITE(*,11111) Prefix, I, ' MHMLV=',LMHIncLevel,  &
     &         ' SCMLV=',LSCIncLevel,' ERMLV=', LERIncLevel
         ENDIF
!
11111    FORMAT(A,' Inc ', I2, 3(A,F15.8))
11117    FORMAT(A,3(A,I8))
11116    FORMAT(A,3(A,F15.8))
!
!
 5210    MHMLV = MHMLV + LMHIncLevel
!
      MHEOP = Round( MHEOP, Mrnd )
      SCEOP = Round( SCEOP, Mrnd )
      EREOP = Round( EREOP, Mrnd )
!
      MHMLV = Round( MHMLV / NumInc, Mrnd )
!
      IF( verbose .GT. 0 ) THEN
         WRITE(*,11116) Prefix, ' MHEOP=',MHEOP,          &
     &      ' SCEOP=', SCEOP, ' EREOP=', EREOP
         WRITE(*,'(A,A,F15.8)') Prefix, ' MHMLV=', MHMLV 
      ENDIF
!
!
      END
!
!
!----&------------------------------------------------------------------
!----&------------------------------------------------------------------
!----&------------------------------------------------------------------
!----&------------------------------------------------------------------
!----&------------------------------------------------------------------
      SUBROUTINE P77ALimitations( SecsInPeriod, PreviousFlow, InSupNTS, &
     &   SuperiorBOPLevel,MicHuronBOPLevel,StClairBOPLevel,ErieBOPLevel,&
     &   SuperiorEOPLevel,MicHuronEOPLevel,StClairEOPLevel,ErieEOPLevel,&
     &   StMarysFlow, Gates, Month, AdjustmentCodes, OutputPrefix,      &
     &   verbose)
!
!     This function takes a preliminary flow, from either the balancing
!     equation or the average of forecasted plan flows, and applies the 
!     various outflow limitations described in Plan 1977A.  These are:
!
!        Requirement A:  Maximum outflow May-Nov = 2320 m3s + 16 gates
!        Requirement B:  Maximum outflow Dec-Apr = 2410 m3s
!        Requirement C:  Minimum outflow = 1560 m3s
!        Requirement D:  (not applicable in this context)
!        Requirement E:  (not applicable in this context)
!        Requirement F:  (not applicable in this context)
!        Requirement G:  (not applicable in this context)
!        Requirement H:  i don't think there is one
!        Requirement I:  Maximum change between months = 850 m3s
!        Requirement J:  Minimum gates open = 0.5 m3s
!        Requirement K:  Maximum outflow Oct = 4110 m3s, Nov = 3260 m3s
!
!
!
! modules
      USE par_kind
      USE lakesup_mod
      USE outflow_mod

      IMPLICIT NONE

!     Declarations regarding subroutine arguments:
!      
      INTEGER          Month
!                      (input) The month number to compute outflow for.
!
      INTEGER          InSupNTS
!                      (input) The Superior net total supply to use 
!                      for computing mean level/outflow for a gate
!                      setting.  If -9999, indicates use the Plan 1977A
!                      supplies based on trigger level.  Otherwise, 
!                      call to this function was made after averaging 
!                      outflows from Plan77A forecast months.
!
      real(wp) :: SecsInPeriod
!                      (input) The length of the routing interval.

      real(wp) :: PreviousFlow
!                      (input) The outflow last routing interval.
!
      real(wp) :: SuperiorBOPLevel, MicHuronBOPLevel, &
     &                 StClairBOPLevel, ErieBOPLevel
!                      (input) Beginning-of-Period lake levels 
!    
      CHARACTER*(*)    OutputPrefix
!                      (input) String to use at beginning of lines 
!                      written to detailed.log, including indent.
!    
      real(wp) :: StMarysFlow 
!                      (input/output) The Superior outflow for the 
!                      routing interval.  Initially given as some kind
!                      of target, either from the balancing equation or
!                      forecast averaging.  This routine adjusts and 
!                      modifies this flow according to the various Plan 
!                      77A limitations, and returns the adjusted value.
!
      REAL             Gates
!                      (output) The number of gates open in Month.
!
      CHARACTER*(24)   AdjustmentCodes
!                      (input/output) String containing code letters 
!                      describing what adjustments were made to come
!                      with the Plan flow.
!
      real(wp) :: SuperiorEOPLevel, MicHuronEOPLevel, &
     &                 StClairEOPLevel, ErieEOPLevel
!                      (output) End-of-Period lake levels
      integer verbose
!
!                       
!
!
!     Declarations regarding local variables or functions:
!
      CHARACTER*(3)    CodeChar
      real(wp) :: tmpflow, HalfGateFlow, FullGateFlow
      INTEGER          I, K, L 
!                      Scratch variables
!
      INTEGER          PL
!                      Length of OutputPrefix
!
      real(wp) :: SupNTS, MHNTS
!                      The Net Total Supply to use for Lakes Superior
!                      and Michigan-Huron (considering trigger levels, 
!                      and diverions)
!
      real(wp) :: SuperiorAveLevel, MicHuronAveLevel
!                      Mean levels for the routing period, as computed
!                      within Plan 1977A.  NOT necessarily same results
!                      as routing through these lakes after Plan 1977A 
!                      outflow is determined.
!
      real(wp) :: Round, GetArea
!                      Function for rounding
!
!
!
!
!     Log the entering data
!
      IF( verbose .GT. 0) THEN
         WRITE(*, 6543) TRIM(OutputPrefix), MonthAbbrev(Month)
         WRITE(*, 6544) TRIM(OutputPrefix), StMarysFlow,          &
     &      AdjustmentCodes
         WRITE(*, 6545) TRIM(OutputPrefix), PreviousFlow,         &
     &      SuperiorBOPLevel, MicHuronBOPLevel, StClairBOPLevel,  &
     &      ErieBOPLevel
      ENDIF
 6543 FORMAT(A,'Checking and adjusting Plan 1977A outflow for ',A3) 
 6544 FORMAT(A,'Unadjusted flow=', F9.1, '  AdjustmentCodes=', A) 
 6545 FORMAT(A,'Previous flow=', F9.1,' BOP Levels=', 4F8.3) 
!
!
      IF(InSupNTS .EQ. -9999) THEN
!        Check trigger levels and determine the supplies to use
!        for routing this period.  Just put the NBS values in the
!        NTS variables for now; will add in the diversions after
!        write to logs.  Also record an indicator letter into 2nd
!        digit of AdjustmentCodes for which supplies being used.
!
         IF( SuperiorBOPLevel .LE. P77ALowTrigger ) THEN 
            SupNTS = P77ASup95NBS(Month) 
            MHNTS  = P77AMH95NBS(Month) 
            IF( verbose .GT. 0)                                    &
     &       WRITE(*, 6529) TRIM(OutputPrefix), SuperiorBOPLevel,  &
     &         'less', P77ALowTrigger, '95'
            AdjustmentCodes(4:6) = 'LS '
         ELSE IF( SuperiorBOPLevel .GE. P77AHighTrigger ) THEN 
            SupNTS = P77ASup05NBS(Month) 
            MHNTS  = P77AMH05NBS(Month) 
            IF( verbose .GT. 0)                                    &
     &       WRITE(*, 6529) TRIM(OutputPrefix), SuperiorBOPLevel,  &
     &         'greater', P77AHighTrigger, '5'
            AdjustmentCodes(4:6) = 'HS '
         ELSE
            SupNTS = P77ASup50NBS(Month) 
            MHNTS  = P77AMH50NBS(Month) 
            IF( verbose .GT. 0)                                    &
     &       WRITE(*, 6530) TRIM(OutputPrefix), SuperiorBOPLevel,  &
     &         P77ALowTrigger, P77AHighTrigger
            AdjustmentCodes(4:6) = 'NS '
         ENDIF
!
!        Write info to log
!                               `
         IF( verbose .GT. 0) THEN
            WRITE(*, 6531) TRIM(OutputPrefix), 'Superior',         &
     &         SupNTS + P77ALLOgokiDiv(Month),                     &
     &         NINT(SupNTS), P77ALLOgokiDiv(Month)
            WRITE(*, 6531) TRIM(OutputPrefix), 'Michigan-Huron',   &
     &         MHNTS + P77AChicagoDiv(Month),                      &
     &         NINT(MHNTS), P77AChicagoDiv(Month)
         ENDIF
 6529    FORMAT(A,'BOP level ',F7.3,1X,A,' than trigger level (', F7.3, &
     &      ') so using ', A, '% supplies.' )
 6530    FORMAT(A,'BOP level ',F7.3,' greater than low trigger level (',&
     &      F7.3, '), and less than high trigger level (', F7.3,        &
     &      '), so 50% supplies.' )
 6531    FORMAT(A, A,' NTS=', F8.1, ' (', I5, ' + ', I4, ')')
!
!
!        Add in diversions.
!
         SupNTS = SupNTS + P77ALLOgokiDiv(Month) 
         MHNTS  = MHNTS + P77AChicagoDiv(Month)
!
!
!
      ELSE
         SupNTS = InSupNTS
         IF( verbose .GT. 0)                                            &
     &     WRITE(*, 6528) TRIM(OutputPrefix), SupNTS
 6528    FORMAT(A,'Using operational Superior net total supply: ',F17.3,&
     &      A1 )
      ENDIF
!
!
!     ===> Requirements C, J, and A
!
!     Check to see if incoming flow (from balancing equation or 
!     average of forecast flows) is less than minimum.  
!
      IF( verbose .GT. 0) THEN
         WRITE(*, 6541) TRIM(OutputPrefix)
 6541    FORMAT(A,'Checking preliminary flow against minimum:')
      ENDIF 
!
      IF( NINT(StMarysFlow) .LE. P77AMinimumFlow(Month)) THEN
!
!        Flow is too low.
         IF( verbose .GT. 0) THEN
            WRITE(*, 6533) TRIM(OutputPrefix), StMarysFlow,     &
     &         P77AMinimumFlow(Month)
 6533       FORMAT(A,'Preliminary flow of ', F9.1,              &
     &         ' m3s below minimum; setting it to ', I6,' m3s')
         ENDIF 
!    
         AdjustmentCodes(7:12) = 'RC RJ '
         StMarysFlow = P77AMinimumFlow(Month)
         Gates = 0.5
         CALL P77ASolveSup(SuperiorBOPLevel, SupNTS, SecsInPeriod,      &
     &      -1.0, P77AFlowRound, P77AMonthRound, SuperiorEOPLevel,      &
     &      StMarysFlow, SuperiorAveLevel,                              &
     &      verbose)
      ELSE
!
!        Flow is above minimum.  If desired, log a message.
         IF( verbose .GT. 0) THEN
            WRITE(*, 6537) TRIM(OutputPrefix)
         ENDIF 
!     
!        Find out gate position and flow (assuming maximum sidechannel) 
!        closest to entering flow, and return it in tmpflow.
         CALL SupClosestFlow( StMarysFlow, SuperiorBOPLevel,            &
     &      MicHuronBOPLevel, SupNTS, SecsInPeriod, P77AMonthRound,     &
     &      P77AFlowRound, tmpflow, Gates, SuperiorEOPLevel,            &
     &      verbose)
!
         IF( verbose .GT. 0) THEN
            WRITE(*, 6532) TRIM(OutputPrefix), tmpflow, Gates
 6532       FORMAT(A,'Closest flow (assuming maximum sidechannel)=',    &
     &         F8.1, ' (', F5.2,' gates)')
         ENDIF 
!       
!
!        XXX Carl-compatibility fix.  Somehow Carl's model sets the plan
!        XXX flow to permit full sidechannel flow at 1/2 gate, if the 
!        XXX entering flow is within 1 m3s of the 1/2 gate full 
!        XXX sidechannel flow.  Nuke this line and uncomment the next
!        XXX one when backward compatibility not an issue regrading P77A
         IF(tmpflow .GT. (StMarysFlow +1) .AND.NINT(Gates*2).EQ.1)THEN
!        XXX Matt's preference:
!        IF( tmpflow .GT. StMarysFlow .AND. NINT(Gates * 2) .EQ. 1) THEN
!           
!           If closest flow is greater than entering flow, and only half
!           gate open, then can't have maximum sidechannel flow.  Since
!           entering flow is above the minimum, and less than 1/2 gate
!           plus full sidechannel, it is a valid flow itself, so just
!           leave it unadjusted in StMarysFlow.  
!
            IF( verbose .GT. 0) THEN
              WRITE(*, 6542) TRIM(OutputPrefix)
 6542         FORMAT(A,'Unnecessary to adjust flow according to even ', &
     &           'gate setting (sidechannel less than maximum).')
            ENDIF 
!
            CALL P77ASolveSup(SuperiorBOPLevel, SupNTS, SecsInPeriod,   &
     &         -1.0, P77AFlowRound, P77AMonthRound, SuperiorEOPLevel,   &
     &         StMarysFlow, SuperiorAveLevel,                           &
     &         verbose)
            AdjustmentCodes(7:12) = '   RJ '
!
!        Note that older model simply presumes that total flow for 
!        1/2 gate is always 2390 m3s.  So to be compatible with older
!        answers, must make same assumption.  If 2390 is greater than 
!        entering flow, and only half gate open, then can't have
!        maximum sidechannel flow, so just leave StMarysFlow unadjusted.  
!        XXX  comment this clause out when don't need backward
!        XXX  compatibility with old bugs!
         ELSE IF( StMarysFlow .LT. 2390 ) THEN
            IF( verbose .GT. 0)                                         &
     &            WRITE(*, 6601) TRIM(OutputPrefix)
 6601       FORMAT(A,'Not adjusting flow according to nearest gate ',   &
     &         'setting in order to replicate bug in the old model.')
            IF( verbose .GT. 0)                                         &
     &            WRITE(*, 6602) TRIM(OutputPrefix)
 6602       FORMAT(A,'For more information see the implementation of ', &
     &         'Requirements C, J, and A in sub P77ALimitations of ',   &
     &         'SUPERIOR.FOR.')
            CALL P77ASolveSup(SuperiorBOPLevel, SupNTS, SecsInPeriod,   &
     &         -1.0, P77AFlowRound, P77AMonthRound, SuperiorEOPLevel,   &
     &         StMarysFlow, SuperiorAveLevel,                           &
     &         verbose)
            AdjustmentCodes(7:12) = '   RJ '
!        XXX End of clause to remove someday
! 
         ELSE
!            
            IF( NINT(Gates) .EQ. 16 .AND. StMarysFlow .GT. tmpflow)THEN
!              Entering flow is greater than capacity! (Requirement A) 
               AdjustmentCodes(16:18) = 'RA '
            ELSE
               AdjustmentCodes(16:18) = '   '
            ENDIF
!              
!           Entering flow high enough for maximum sidechannel flow with
!           half a gate.  Use the closest flow.
!           
            StMarysFlow = tmpflow
            AdjustmentCodes(7:12) = '      '
!
!           XXX Another older model compatibility fix.  In order to replicate
!           XXX rounding in older model, must run subroutine again.
!           XXX Nuke this block when backward compatibility not an issue.
            CALL P77ASolveSup(SuperiorBOPLevel, SupNTS, SecsInPeriod,   &
     &         -1.0, P77AFlowRound, P77AMonthRound, SuperiorEOPLevel,   &
     &         StMarysFlow, SuperiorAveLevel,                           &
     &         verbose)
!           XXX end of fix
!
         ENDIF
      ENDIF
!
!        
!
!
!
!     Check rate-of-change limitation (Requirement I)
!
      IF( verbose .GT. 0) THEN
         WRITE(*, 6539) TRIM(OutputPrefix)
      ENDIF 
!
!     XXX Hardcoded rounding in Plan1977A:
      IF( IABS(NINT(StMarysFlow) - NINT(PreviousFlow))              &
     &   .GE. P77AMaxChange) THEN
!
         IF( verbose .GT. 0) THEN
            WRITE(*, 6536) TRIM(OutputPrefix), StMarysFlow, Gates,  &
     &         P77AMaxChange, PreviousFlow
         ENDIF 
!        Flow change is too great.  Loop around until jump out.
         DO 5734, I=1,20
!
            IF( verbose .GT. 0) THEN
               WRITE(*, '(A, A, F5.2)') TRIM(OutputPrefix),         &
     &             ' Checking with gates= ', Gates
            ENDIF 
!           Increment/decrement gate setting
            IF(StMarysFlow .GT. PreviousFlow) THEN
!              Flow rising too much this month, back off a gate
               Gates= Gates - 1
               IF(NINT(Gates) .EQ. 0) Gates = 0.5
! 
!              Really bizarre numbers mean a bust somewhere
               IF(StMarysFlow.LT.50 .OR. PreviousFlow.LT.50)STOP 'Lucy!'
!
               IF(Gates .LT. 0.0) THEN
!               
!                 Hmmm.  Max sidechannel plus 1/2 gate is more than 850 
!                 higher than previous flow.  So can't have max 
!                 sidechannel.  Set flow to 850 plus previous flow, and
!                 recompute EOP level.
                  Gates = 0.5
                  StMarysFlow = NINT(PreviousFlow) + P77AMaxChange
!                 Check for inability to reconcile Requirements C, I, 
!                 and J.  Per Nanette's advice, let C control over I 
!                 (stick with 1560 m3s, don't worry about 850 change).   
                  IF( NINT(StMarysFlow) .LT. P77AMinimumFlow(Month))    &
     &               StMarysFlow = P77AMinimumFlow(Month)
                  CALL P77ASolveSup(SuperiorBOPLevel, SupNTS,           &
     &               SecsInPeriod, -1.0, P77AFlowRound, P77AMonthRound, &
     &               SuperiorEOPLevel, StMarysFlow, SuperiorAveLevel,   &
     &               verbose)
                  IF( verbose .GT. 0) THEN
                     WRITE(*,6102) TRIM(OutputPrefix),                  &
     &                  NINT(StMarysFlow)
                  ENDIF
!                 Now leave loop                    
                  GOTO 5735 
               ENDIF
            ELSE
!              Flow dropping too much this month
!              
               IF(NINT(Gates - 0.1) .EQ. 0) THEN
!
!                 See if previousflow - 850 is between 1/2 and 1 gate
!                 If so, set it at previousflow - 850 
!
                  IF( verbose .GT. 0) THEN
                     WRITE(*, 6298) TRIM(OutputPrefix), '0.5'
                  ENDIF 
                  CALL P77ASolveSup(SuperiorBOPLevel, SupNTS,           &
     &               SecsInPeriod, 0.5, P77AFlowRound, P77AMonthRound,  &
     &               SuperiorEOPLevel, HalfGateFlow, SuperiorAveLevel,  &
     &               verbose)
!
                  IF( verbose .GT. 0) THEN
                     WRITE(*, 6298) TRIM(OutputPrefix), '1.0'
                  ENDIF 
                  CALL P77ASolveSup(SuperiorBOPLevel, SupNTS,           &
     &               SecsInPeriod, 1.0, P77AFlowRound, P77AMonthRound,  &
     &               SuperiorEOPLevel, FullGateFlow, SuperiorAveLevel,  &
     &               verbose)
                  IF( verbose .GT. 1) THEN
                     WRITE(*,6104) TRIM(OutputPrefix), HalfGateFlow,    &
     &                  FullGateFlow
                  ENDIF
!
!                 XXX Fix to accomodate bug in older model, which
!                 XXX allows the flow to change more than 850 m3s
!                 XXX if 1/2 gate with full sidechannel is closer to
!                 XXX target (PreviousFlow -P77AMaxChange) than 1 gate
!                 XXX with full sidechannel, when the target flow is 
!                 XXX the max for the month, rounded to the nearest 10m3s
!                 XXX Simply remove this block to remove the "fix":
!
                  IF( Month .LE. 4 .OR. Month .EQ. 12) THEN
                     L = P77AWinterMax
                  ELSE
                     L = (12 - Month) * P77AMaxChange + P77AWinterMax
                  ENDIF
                  IF(ABS( (NINT(PreviousFlow) - P77AMaxChange) -        &
     &               NINT( HalfGateFlow ) )  .LT.                       &
     &               ABS( (NINT(PreviousFlow) - P77AMaxChange) -        &
     &               NINT( FullGateFlow ) ) .AND.                       &
     &               NINT( (PreviousFlow - P77AMaxChange) / 10.0_wp )   &
     &               * 10 .EQ. L ) THEN
!                    1/2 gate flow with full sidechannel is closer to 
!                    last month's flow minus 850, and full gate flow
!                    exceeds max for month.                       
                     StMarysFlow = HalfGateFlow
                     IF( verbose .GT. 0) THEN
                        WRITE(*, 6108) TRIM(OutputPrefix),              &
     &                     ' Running P77ASolveSup in Req I compat fix'//&
     &                     ' to determine new mean level'
                     ENDIF
                     CALL P77ASolveSup(SuperiorBOPLevel, SupNTS,        &
     &                  SecsInPeriod, -1., P77AFlowRound,P77AMonthRound,&
     &                  SuperiorEOPLevel, StMarysFlow, SuperiorAveLevel,&
     &                  verbose)
!                    Now leave loop                    
!                    Leave gates = 0.5
                     GOTO 5735 
                  ELSE
                     IF( verbose .GT. 0) THEN
                        WRITE(*, 6108 ) TRIM(OutputPrefix),             &
     &                     ' Req I compat fix not required'
                     ENDIF
                  ENDIF
!                 XXX End of "fix" block
!
!                 Say that StMarysFlow 
!
                  StMarysFlow = FullGateFlow
                  Gates = 1
!
!
!
                  IF((PreviousFlow - P77AMaxChange).LT.StMarysFlow)THEN
!
                     IF( verbose .GT. 0) THEN
                        WRITE(*, 6834) TRIM(OutputPrefix),              &
     &                     StMarysFlow, Gates, 'now', P77AMaxChange,    &
     &                     NINT(PreviousFlow)
                        WRITE(*, '(A,A,I6)' ) TRIM(OutputPrefix),       &
     &                     ' Setting StMarysFlow to PreviousFlow - ',   &
     &                     P77AMaxChange
                     ENDIF
                     StMarysFlow = NINT(PreviousFlow) - P77AMaxChange
!
!                    XXX Yet another odd bit logic intended to replicate 
!                    XXX answers from the older models.
                     IF( StMarysFlow .LT. 2390 .AND.                    &
     &                  StMarysFlow .LE. HalfGateFlow) THEN 
                        Gates = 0.5
                     ELSE
                        Gates = 1.0
                        IF( verbose .GT. 0) THEN
                           WRITE(*, 6108 ) TRIM(OutputPrefix),          &
     &                        ' Running P77ASolveSup after setting '//  &
     &                        '1.0 gate for 2390 m3/s compat fix '
                        ENDIF
                        CALL P77ASolveSup(SuperiorBOPLevel, SupNTS,     &
     &                     SecsInPeriod, Gates, P77AFlowRound,          &
     &                     P77AMonthRound, SuperiorEOPLevel,            &
     &                     StMarysFlow, SuperiorAveLevel,               &
     &                     verbose)
                     ENDIF
!                    XXX End of "fix" block
                     IF( verbose .GT. 0) THEN
                        WRITE(*, '(A,A,F5.2,A)') TRIM(OutputPrefix),    &
     &                     ' Gates set to ', gates
                     ENDIF
                     IF( verbose .GT. 0) THEN
                        WRITE(*, 6108 ) TRIM(OutputPrefix),             &
     &                     ' Running P77ASolveSup after Req I compat '//&
     &                     'fix to determine new mean level'
                     ENDIF
                     CALL P77ASolveSup(SuperiorBOPLevel, SupNTS,        &
     &                  SecsInPeriod, -1., P77AFlowRound,P77AMonthRound,&
     &                  SuperiorEOPLevel, StMarysFlow, SuperiorAveLevel,&
     &                  verbose)
!                    Now leave loop                    
                     GOTO 5735 
                  ELSE
                     IF( verbose .GT. 0) THEN
                        WRITE(*, 6834) TRIM(OutputPrefix),              &
     &                     StMarysFlow, Gates, 'not', P77AMaxChange,    &
     &                     NINT(PreviousFlow)
                     ENDIF
                  ENDIF
!
               ENDIF
!               
!              Increase a gate and try again
 5727          Gates= NINT(Gates - 0.1) + 1
               IF(Gates .GT. 0 .AND. verbose .GT. 0) THEN
                  WRITE(*, 6519)TRIM(OutputPrefix),                     &
     &               MonthAbbrev(Month), SuperiorAveLevel, Gates,       &
     &               NINT(StMarysFlow), NINT(PreviousFlow)
               ENDIF
            ENDIF
!
!           Get new mean outflow for adjusted gate setting
            IF( verbose .GT. 0) THEN
               WRITE(*, 6108 ) TRIM(OutputPrefix),                      &
     &            ' Running P77ASolveSup to compute new outflow for '// &
     &            ' adjusted gate setting . . .'
            ENDIF
            CALL P77ASolveSup(SuperiorBOPLevel, SupNTS, SecsInPeriod,   &
     &         Gates, P77AFlowRound, P77AMonthRound,                    &
     &         SuperiorEOPLevel, StMarysFlow, SuperiorAveLevel,         &
     &         verbose)
!
!           XXX Another older model compatibility fix.  In order to replicate
!           XXX rounding in older model, must run subroutine again. 
!           XXX Nuke this block when backward compatibility with earlier
!           XXX models regarding P77A is not an issue.
            IF( verbose .GT. 0) THEN
               WRITE(*, 6108 ) TRIM(OutputPrefix),                      &
     &            ' Running P77ASolveSup again to re-compute mean '//   &
     &            ' level, as done in old code . . .'
            ENDIF
            CALL P77ASolveSup(SuperiorBOPLevel, SupNTS,                 &
     &         SecsInPeriod, -1., P77AFlowRound,P77AMonthRound,         &
     &         SuperiorEOPLevel, StMarysFlow, SuperiorAveLevel,         &
     &         verbose)
!           XXX End of "fix" block
!
!
!        XXX hardcoded rounding to nearest integer flow replicates 
!        XXX previous model, but is not flexible.
 5734    IF( IABS(NINT(StMarysFlow) - NINT(PreviousFlow))               &
     &      .LE. P77AMaxChange) GOTO 5735
!
         STOP 'Little Ricky!'
 5735    IF( verbose .GT. 0) THEN
            WRITE(*,6535) TRIM(OutputPrefix), StMarysFlow, Gates
         ENDIF 
         AdjustmentCodes(13:15) = 'RI '
      ELSE
         AdjustmentCodes(13:15) = '   '
         IF( verbose .GT. 0) THEN
            WRITE(*, 6537) TRIM(OutputPrefix)
         ENDIF 
      ENDIF
 6102 FORMAT( A, 'Unable to allow maximum sidechannel flow;',           &
     &   ' Req. I limits flow to ', I7, ' m3s')      
 6104 FORMAT( A, 'Flow for .5 and 1 gates with max sidechannel flow: ', &
     &   2F8.1)      
 6108 FORMAT( A, A)      
 6298 FORMAT (A, ' Running P77ASolveSup() to determine ', A,            &
     &   ' gate flow (full sidechan) . . .')

 6536 FORMAT(A,'Preliminary flow of ', F6.1,' m3s (', F5.2,             &
     &   ' gates) changed more than ', I4,                              &
     &   ' from previous period (', F6.1, ')')
 6539 FORMAT(A,                                                         &
     &   'Checking preliminary flow against rate-of-change limitation:',&
     &    A1)
 6834 FORMAT(A,' StMarysFlow of ',F10.3,' with ',F5.2,' gates open is ',&
     &       A,' within ',I4,' m3/s of previous flow (',I4,')') 
!
!
!     Check maxmimum flow limitations (Requirements B, K) 
!     (Requirement A checked earlier)
!
!        Requirement B:  Maximum outflow Dec-Apr = 2410 m3s
!        Requirement K:  Maximum outflow Oct = 4110 m3s, Nov = 3260 m3s
!
      IF( verbose .GT. 0) THEN
         WRITE(*, 6540) TRIM(OutputPrefix)
      ENDIF 
!
      IF( Month .LE. 4 .OR. Month .EQ. 12) THEN
!        It's winter.  Set maximum to P77AWinterMax
         L = P77AWinterMax
         CodeChar = 'RB '
      ELSE
!        It's not winter.  Set maximum to whatever we have to stay
!        under in order to get to P77AWinterMax by December, without
!        violating P77AMaxChange.
         L = (12 - Month) * P77AMaxChange + P77AWinterMax
         CodeChar = 'RK '
!         WRITE(*,'(A,I4)') 'Max = ', L
      ENDIF
      IF( verbose .GT. 0) THEN
         WRITE(*,6538)TRIM(OutputPrefix), MonthAbbrev(Month), L
      ENDIF 
 6538 FORMAT(A,'Maximum flow for ', A3, ': ', I8, ' m3s')
!
      IF( NINT(StMarysFlow) .GT. L) THEN
!        Flow is too high.  Loop around until jump out.
         IF( verbose .GT. 0) THEN
            WRITE(*, 6534)TRIM(OutputPrefix), StMarysFlow,Gates,L
         ENDIF 
         K = 0
         DO 5434, I=1,99
!           Decrement gate setting
            Gates= Gates - 1
            IF(NINT(Gates) .LE. 0) THEN
                Gates = 0.5
!               Set K so that we will know if loop back here again
                K = K + 1
                IF ( K .EQ. 2 ) THEN
!                  Last time through was half gate also, and flow was too
!                  high for Requirements B or K.  So have conflict between
!                  Req J and B/K.  Per David Fay on 7Aug03, let the 1/2 gate
!                  Requirement control.
!                  
            IF( verbose .GT. 0)                                   &
     &                   WRITE(*, 6546)TRIM(OutputPrefix),        &
     &                MonthAbbrev(Month), SuperiorAveLevel,       &
     &                NINT(StMarysFlow), L
            IF( verbose .GT. 0)                                   &
     &                   WRITE(*, '(A,A,A)')TRIM(OutputPrefix),   &
     &                ' Requirement J controls; ',                &
     &                'maximum winter flow disregarded.'
!                  Now jump back in at end of Requirement checking
                   GOTO 5435
                ENDIF
            ENDIF
!
!           Get new mean outflow for adjusted gate setting
            CALL P77ASolveSup(SuperiorBOPLevel, SupNTS, SecsInPeriod,   &
     &         Gates, P77AFlowRound, P77AMonthRound,                    &
     &         SuperiorEOPLevel, StMarysFlow, SuperiorAveLevel,         &
     &         verbose)
!
!           If make it below outflow ceiling jump out of gate-closing loop 
 5434       IF( NINT(StMarysFlow) .LE. L) GOTO 5435
!
!        Should never get here - either get below outflow ceiling or hit 
!        1/2 gate twice in a row.
         STOP 'Trixie'
!
 5435    IF( verbose .GT. 0) THEN
            WRITE(*,6535) TRIM(OutputPrefix), StMarysFlow, Gates
         ENDIF 
         AdjustmentCodes(16:18) = CodeChar
      ELSE
         IF( verbose .GT. 0) THEN
            WRITE(*, 6537) TRIM(OutputPrefix)
         ENDIF 
      ENDIF
 6534 FORMAT(A,'Preliminary flow of ', F6.1, ' m3s (', F5.2,            &
     &   ' gates) above maximum of ', I8,' m3s')
 6540 FORMAT(A,'Checking preliminary flow against maximum:')
 6546 FORMAT(A,' Month: ', A3, ' working Sup level: ', F6.2,            &
     &   ' Req J (1/2 gate) flow:', I5,                                 &
     &   ' in conflict with Req B/K max flow:', I5)
 6519 FORMAT(A,' Month: ', A3, ' working Sup level: ', F8.2,            &
     &   ' Gates: ', F5.2, ' St. Marys flow:', I9,                      &
     &   ' Req I (prev) flow:', I9)
!
!
!     XXX Another ugly re-compute necessary to exactly match rounding
!     XXX practices in older model.  Someday just remove it.
      SuperiorEOPLevel = Round( SuperiorBOPLevel, P77AMonthRound ) +    &
     &   Round( (SupNTS - StMarysFlow) * SecsInPeriod /                 &
     &   GetArea(1,SuperiorBOPLevel), P77AMonthRound )
!     XXX End of fix clause 
!
!
!     Now route the flow downstream, so can have a MH level for the
!     next period of forecast, using the method defined in Plan1977A.
!     If called to do check on average of forecasted flows, don't
!     need to do this.
      IF(InSupNTS .EQ. -9999) CALL P77AMidLakeRoute ( SecsInPeriod,     &
     &   NINT(Round(MHNTS + StMarysFlow,P77AFlowRound)),                &
     &   P77AStC50NBS(Month),                                           &
     &   P77AErie50NBS(Month) + P77AWellandFlow(Month),                 &
     &   MicHuronBOPLevel,StClairBOPLevel,ErieBOPLevel,                 &
     &   P77AStClairRetrd(Month),                                       &
     &   P77ADetroitRetrd(Month), P77ANiagaraRetrd(Month),              &
     &   P77AFlowRound, P77ALevelRound, P77AMonthRound, P77AMidLakesInc,&
     &   MicHuronEOPLevel,StClairEOPLevel,ErieEOPLevel,MicHuronAveLevel,&
     & verbose)
! 
!
! 6525 FORMAT( A,'Previous month outflow=', F20.3, ', gates open=', 
!     &   F5.2, '.') 
 6535 FORMAT(A,'Adjusted preliminary flow to ', F6.1, ' m3s (', F5.2,   &
     &   ' gates)')
 6537 FORMAT(A, ' . . . passed check OK.')
      END
!
!
!
!----&------------------------------------------------------------------
!----&------------------------------------------------------------------
!
      SUBROUTINE SupCriterions( Year, Month, SuperiorNWB, MichuronNWB,  &
     &   StClairNWB, ErieNWB, SCRivRet, DetRivRet, NiaRivRet,           &
     &   SuperiorBOPLevel, MicHuronBOPLevel, StClairBOPLevel,           &
     &   ErieBOPLevel, SecsInPeriod, StMarysFlow, SWPier,               &
     &   NonGatedFlow, Gates, AdjustmentCodes, verbose )
!
!     This function takes a preliminary flow, from either Plan 1977A (or
!     potentially in the future from the 1955 Modified Rule of '49), and 
!     checks/modifies the flow according to Criterions B and C of the 
!     Orders of Approval (at least according to the operational 
!     practices in effect in October 2000).
!
!          Criterion A:  (not applicable in this context)
!          Criterion B:  Prevent levels at US Slip greater than 177.94m
!          Criterion C:  Outflows capped at pre-project below 183.40m
!
!
! modules
      USE par_kind
      USE lakesup_mod
      USE outflow_mod

      IMPLICIT NONE
!
!     Declarations regarding subroutine arguments:
!      
      INTEGER          Year, Month
!                      (input) The date of the outflow.
!
      INTEGER          SuperiorNWB
!                      (input) The net water balance (NBS + LLO) to 
!                      use for Lake Superior change-in-storage 
!                      calculations.  Could be determined in three 
!                      different ways, depending on value of P77ACompat.
      INTEGER          MichuronNWB
!                      (input) The net water balance (NBS + ChiDiv), 
!                      calculated according to value of P77ACompat.
      INTEGER          StClairNWB
!                      (input) The net water balance (NBS), 
!                      calculated according to value of P77ACompat.
      INTEGER          ErieNWB
!                      (input) The net water balance (NBS - Welland), 
!                      calculated according to value of P77ACompat.
      INTEGER          SCRivRet, DetRivRet, NiaRivRet
!                      (input) Flow retardation values to use, 
!                      calculated according to value of P77ACompat.
!
      real(wp) :: SuperiorBOPLevel, MicHuronBOPLevel,     &
     &                 StClairBOPLevel, ErieBOPLevel
!                      (input) Beginning-of-Period lake levels 
!    
      real(wp) :: SecsInPeriod     
!                      (input) The length of the routing interval.
!
      real(wp) :: StMarysFlow
!                      (input/output) The Superior outflow for the 
!                      routing interval.  Initially given as some kind
!                      of target, either from the balancing equation or
!                      forecast averaging.  This routine adjusts and 
!                      modifies this flow according to Criterions B and
!                      C, and returns the adjusted value in StMarysFlow.
!
      real(wp) :: SWPier, NonGatedFlow
!                      (output) The SWPier level as the old models would
!                      compute it, which yields a "gated flow" which
!                      yields the 'sidechannel' flow.
!
      REAL             Gates
!                      (input/output) The number of gates open in Month.
!
      CHARACTER*(24)   AdjustmentCodes
!                      (input/output) String containing code letters 
!                      describing what adjustments were made.
!    
      INTEGER verbose
!
!                       
!
!
!     Declarations regarding local variables or functions:
!
      real(wp) :: tmplev, gatedflow
      INTEGER          I, L, Flow2ItsAgo
!                      Scratch variables
!
      real(wp) :: SuperiorAveLevel, MicHuronAveLevel
!                      Mean levels for the routing period, as computed
!                      within Plan 1977A.  NOT necessarily same results
!                      as routing through these lakes after Plan 1977A 
!                      outflow is determined.
!
      real(wp) :: SuperiorEOPLevel, MicHuronEOPLevel,    &
     &                 StClairEOPLevel, ErieEOPLevel
!                      End-of-Period lake levels 
!    
!
      CHARACTER*(*)    OutputPrefix
!                      Used as prefix on lines of detailed.log 
!
      real(wp) :: Round
!                      Function for rounding.
!
      real(wp) :: GetArea
!                      Returns area of lake
!
      real(wp) :: USS
!                      Function that estimates level at US Slip based
!                      on St Marys flow and period average level of
!                      Michigan-Huron
!
      real(wp) :: Q1887
!                      Pre-project flow
      CHARACTER*20     CWRK
!
!
!
!
!
      PARAMETER (OutputPrefix = 'Superior Criterion check: ')
! 
!
!     Log the entering data
!
      IF( verbose .GT. 0) THEN
         WRITE(*, 6546) OutputPrefix
         WRITE(*, 6542) OutputPrefix,MonthAbbrev(Month),Year
         WRITE(*, 6544) OutputPrefix, StMarysFlow, Gates,      &
     &      AdjustmentCodes
         WRITE(*, 6545) OutputPrefix, SuperiorBOPLevel,        &
     &      MicHuronBOPLevel, StClairBOPLevel, ErieBOPLevel,   &
     &      SecsInPeriod
         WRITE(*, 6541) OutputPrefix, SuperiorNWB
         WRITE(*, 6543) OutputPrefix, MichuronNWB,             &
     &      StClairNWB, ErieNWB
         WRITE(*, 6540) OutputPrefix, SCRivRet, DetRivRet,     &
     &      NiaRivRet
      ENDIF
 6541 FORMAT(A,'Superior NTS=', I6) 
 6542 FORMAT(A,'Evaluating plan flow for ',A3,I5) 
 6543 FORMAT(A,'MH/SC/ER net water balances: ',3I7) 
 6540 FORMAT(A,'MH/SC/ER outflow retardations: ',3I7) 
 6544 FORMAT(A,'Plan flow=', F9.1, ' (', F5.2, ' gates)',      &
     &   '  AdjustmentCodes=', A) 
 6545 FORMAT(A,'BOP Levels=', 4F8.3, ' dT=', F14.3) 
 6546 FORMAT(A,'Checking preliminary flow against level at US Slip:')
!
!
      SuperiorBOPLevel = Round( SuperiorBOPLevel, CritBOMRound )
!
!     Check Criterion C
!
!     Pre-existing practice is to estimate the pre-project flow using the
!     mean monthly Superior level associated with the outflow from Plan
!     1977A/Criterion B.  
!
!     More proper computation would be to solve 3 simultaneous equations: 
!     pre-project eqn for outflow, mean level,
!     and change-in-storage.  
!
!
!     Env Canada code:
!
!     Apply criterion C of IJC Orders here.  If SUPBOP < 183.40m
!     then compute 1887 relation and test against SUPFLOW.
!     Use the Point Iroquois mtric IGLD 1985 relation
!
!     IF(SUPMEAN.LT.DBLE(183.40))THEN
!
!        if in range of equation, else set flow=10 and set flag=F
!
!        IF(SUPMEAN.GT.DBLE(181.43))THEN
!           Q1887=ROUND((DBLE(82.47)*(SUPMEAN-DBLE(181.43))**DBLE(1.5)),DBLE(10))
!        ELSE
!           Q1887=10.
!           CRTRNC=3
!        ENDIF
!
!        IF(SUPFLOW.GT.Q1887)THEN
!           criterion C applies
!           CRTRNC=1
!           CALL QSUP(Q1887, SUPBOP, SUPNTS, X, Y, Z, QFLOW,
!    1         SIDECHAN, GATESET, MONTH, 1)
!
!           compute final criterion C values for Lake Superior
!
!           SUPFLOW = QFLOW
!           SUPEOP = ROUND(SUPBOP + (SUPNTS - SUPFLOW)/3138,DBLE(100))
!           SUPMEAN = ROUND((SUPBOP + SUPEOP)/2, DBLE(100))
!           DATUM  = SUPMEAN - SUPDATUM
!        ENDIF
!     ENDIF
!
!
!     First we need to figure out Superior mean level (and possibly 
!     outflow) based on Plan 1977A.  
!
      IF( INDEX(AdjustmentCodes, 'RJ') .EQ. 0) THEN
!        Full sidechannel capacity.  Calculate P77A mean level
!        and outflow based on gates.
         CALL P77ASolveSup(SuperiorBOPLevel,                        &
     &      DBLE(SuperiorNWB), SecsInPeriod, Gates, CritFlowRound,  &
     &      CritBOMRound, SuperiorEOPLevel, StMarysFlow,            &
     &      SuperiorAveLevel,                                       &
     &      verbose)
      ELSE
!        Partial sidechannel capacity.  Calculate P77A mean level
!        based on StMarysFlow
         CALL P77ASolveSup(SuperiorBOPLevel,                        &
     &      DBLE(SuperiorNWB), SecsInPeriod, -1.0, CritFlowRound,   &
     &      CritBOMRound, SuperiorEOPLevel, StMarysFlow,            &
     &      SuperiorAveLevel,                                       &
     &      verbose)
      ENDIF  
!
      IF( verbose .GT. 0) THEN
         WRITE(*,6512) OutputPrefix, SuperiorAveLevel
      ENDIF 
 6512 FORMAT( A,'trying Criterion C, with mean Sup level= ', F12.6)
!
!     XX TODO: create logic to punt and return some flow if elev too 
!     XX low for 1887 eqn
      IF(SuperiorAveLevel .LT. 181.43_wp) STOP ' Mr. Howell!'
!
      IF( Q1887Method .EQ. 1) THEN
         Q1887= Round(824.7_wp*(SuperiorAveLevel-181.43_wp)**1.5_wp,    &
     &      CritFlowRound)
      ELSE IF( Q1887Method .EQ. 2) THEN
         CALL GetSupPreProject( SuperiorBOPLevel, DBLE(SuperiorNWB),    &
     &      SecsInPeriod, verbose, Month, Q1887, tmplev)
         Q1887= Round(Q1887, CritFlowRound)
      ENDIF 
!
      IF( verbose .GT. 0) THEN
         WRITE(*,'(A, A3,I5, A, F15.10)')' Pre-project flow for ',      &
     &      MonthAbbrev(Month), Year, ': ',  Q1887
      ENDIF 
!
      WRITE(SupPreProject, '(I6)') NINT(Q1887) 
!
      IF( SuperiorAveLevel .LT. CritCThresholdEl) THEN
!
         IF( verbose .GT. 0) THEN
            WRITE(*,'(A, A3,I5, A, F15.10,A,F10.5,A)')                  &
     &         '   Must consider Crit C for ', MonthAbbrev(Month), Year,&
     &         ' because Superior level for Crit C test (',             &
     &         SuperiorAveLevel, 'm) is below threshold of ',           &
     &         CritCThresholdEl, 'm.'
         ENDIF 
!
         DO 845, I = 1, 999
!
!           Jump out if not a problem 
            IF(StMarysFlow .LE. Q1887) GOTO 9763
!
!           Violated Criterion C!
            AdjustmentCodes(22:24) = 'CC '
            IF( verbose .GT. 0) THEN
               WRITE(*,6550) OutputPrefix
            ENDIF
!
            IF( NINT(Gates + 0.51) .EQ. 1 ) THEN
!              Only half a gate open!  Can't close any more! 
               StMarysFlow = Q1887
               IF( verbose .GT. 0) THEN
                  WRITE(*,'(A)') '  Re-computing mean Superior '//      &
     &                'level changing regulated flow to '//             &
     &                'pre-project flow.'
               ENDIF
               CALL P77ASolveSup( SuperiorBOPLevel, DBLE(SuperiorNWB),  &
     &            SecsInPeriod, -1.0, CritFlowRound, CritBOMRound,      &
     &            SuperiorEOPLevel, StMarysFlow, SuperiorAveLevel,      &
     &            verbose)
               IF( verbose .GT. 0) THEN
                  WRITE(*,6551) OutputPrefix, StMarysFlow
               ENDIF
               GOTO 9764
            ENDIF
!
!           Decrement gate setting
            Gates = Gates - 1
            IF(NINT(Gates) .EQ. 0) Gates = 0.5
            IF(Gates .LT. 0.0) STOP ' Professor'
!
!           Get new mean outflow for adjusted gate setting, loop again
  845       CALL P77ASolveSup(SuperiorBOPLevel,                         &
     &         DBLE(SuperiorNWB), SecsInPeriod, Gates, CritFlowRound,   &
     &         CritBOMRound, SuperiorEOPLevel, StMarysFlow,             &
     &         SuperiorAveLevel,                                        &
     &         verbose)
!
         STOP ' Skipper'   
!
         ELSE 
            IF( verbose .GT. 0) THEN
            WRITE(*,'(A, A3,I5, A, F15.10,A,F10.5,A)')                  &
     &         '   Not considering Crit C for ', MonthAbbrev(Month),    &
     &         Year, ' because Superior level for Crit C test (',       &
     &         SuperiorAveLevel, 'm) is above threshold of ',           &
     &         CritCThresholdEl, 'm.'
         ENDIF 
!
      ENDIF
!
 9763 IF(verbose .GT. 0 .AND. AdjustmentCodes(22:24) .NE.'CC ')THEN
         WRITE(*,6537) OutputPrefix, 'C'
      ENDIF 

!     XX Re-compute mean so that can reproduce rounding effects of 
!     XX earlier models.
!     XX Don't bother if no rounding, or if using new Crit C method,
!     XX or if didn't close a gate because of Criterion C.   
!     XX Remove if compatibility with older models not required.
      IF(AdjustmentCodes(22:24) .EQ. 'CC ' .AND. Q1887Method .EQ. 1     &
     &   .AND. CritBOMRound .NE. -99) THEN
         SuperiorAveLevel = Round(SuperiorBOPLevel, CritBOMRound) +     &
     &      Round((DBLE(SuperiorNWB) - StMarysFlow) * SecsInPeriod *    &
     &      0.5_wp / GetArea(1, SuperiorAveLevel), CritBOMRound)
      ENDIF
!     XX End of fix
!
!
 9764 CONTINUE
!     Check Criterion B level
!
!        There is a problem replicating existing practice for 
!        implementing criterions B and C.  The existing implementations
!        use monthly mean lake levels for Superior and Michigan-Huron,
!        based on supplies for the next month.  This is handled by 
!        the parameter P77A Compat.
!
!         =========================================================
!                      EC code
!
!            USSLIP(YEAR,MONTH) = ROUND(USS(SUPFLOW, MHUMEAN),
!    1                                        DBLE(100))
!
!C                If the US Slip level is too high, decrease the Lake
!C                Superior gate setting and compute a new plan flow
!
!             IF (USSLIP(YEAR,MONTH) .GT. DBLE(177.94)) THEN
!                GATESET = GATE(GATESET, -1)
!                CALL QSUP(DBLE(0), DBLE(0), DBLE(0), X, Y, Z,
!    1              DBLE(0), SIDECHAN, GATESET, MONTH, 2)
!                PLANFLOW = ROUND(FLOW(SUPBOP, SUPNTS, SIDECHAN,
!    1              X, Y, Z), DBLE(10))
!                IF (ROUND(PLANFLOW, DBLE(1000)) .EQ.
!    1              ROUND(SIDECHAN, DBLE(1000))) THEN
!                   PLANFLOW = ROUND(SUPFLOW - 1, DBLE(1))
!                END IF
!                QFLOW = PLANFLOW
!             END IF
!
!             IF (ROUND(USSLIP(YEAR,MONTH), DBLE(1000)) .GT.
!C   1           ROUND(DBLE(582.9), DBLE(1000))) GOTO 10
!    1        ROUND(DBLE(177.94), DBLE(1000))) GOTO 10
!C            UNTIL USSLIP <= 177.94
!
!C              Adjust the gate setting
!
!             IF (ROUND(GATESET, DBLE(100)) .EQ. DBLE(0)) THEN
!                GATESET = DBLE(0.5)
!             END IF
!
!         =========================================================
!
!
!
!
!        Get the estimated MH mean for planned St Marys flow, using
!        the operational NBS's etc.
!
      DO 855, I = 1, 999
!
!        Route St Marys flow thru middle lakes, to get ave MH level
!
!        XX 40 increments hardcoded here
         CALL P77AMidLakeRoute ( SecsInPeriod,               &
     &      NINT(Round(StMarysFlow, CritFlowRound)) +        &
     &      MichuronNWB, StClairNWB, ErieNWB,                &
     &      Round(MicHuronBOPLevel,CritBOMRound),            &
     &      Round(StClairBOPLevel,CritBOMRound),             &
     &      Round(ErieBOPLevel,CritBOMRound),                &
     &      SCRivRet, DetRivRet, NiaRivRet,                  &
     &      CritFlowRound, CritLevelRound, CritBOMRound, 40, &
     &      MicHuronEOPLevel, StClairEOPLevel, ErieEOPLevel, &
     &      MicHuronAveLevel, verbose)
! 
!
         IF( CritBMethod .EQ. 1) THEN
!           XX Old models round to 10 m3s here.  Go figure.
            tmplev = Round(USS ( NINT(Round(StMarysFlow, 1)),  &
     &         MicHuronAveLevel), CritBOMRound)
         ELSE
!           XX If using new Crit B method, don't perpetuate the 
!           XX inconsistency.  (but converting to int is not proper either)
            tmplev= Round(USS (NINT(Round(StMarysFlow,CritFlowRound)),  &
     &         MicHuronAveLevel), CritBOMRound)
         ENDIF 
!
         IF( verbose .GT. 0) THEN
            WRITE(*,6547) OutputPrefix, tmplev,                 &
     &         MicHuronAveLevel, StMarysFlow, Gates
         ENDIF 
 6547    FORMAT(A,'US Slip= ',F7.3, ' with MH level= ', F7.3,   &
     &      ' and St Marys flow= ', F6.1, ' (', F4.1,')')
!
!
!        If not a problem with US Slip, jump out
         IF( tmplev .LE. 177.94_wp ) THEN
            IF( verbose .GT. 0) THEN
               WRITE(*,6537) OutputPrefix, 'B'
            ENDIF 
            GOTO 9753
         ENDIF
!
!        Violated Criterion B!
         AdjustmentCodes(19:21) = 'CB '
         IF( verbose .GT. 0) THEN
            WRITE(*,6548) OutputPrefix, tmplev
 6548       FORMAT(A,'Criterion B violation!  US Slip level (',F7.3,    &
     &         ') is greater than 177.94. Must close a gate . . .')
         ENDIF
!             
!
         IF( CritBMethod .EQ. 2) THEN
!           Don't allow discharge less than pre-project flow
            IF( StMarysFlow .LE. Q1887 ) THEN
               IF( verbose .GT. 0) THEN
                  IF( I .EQ. 1 ) THEN 
                     CWRK(1:1) = ' '
                     L = 1
                  ELSE
                     CWRK = ' further '
                     L = 9
                  ENDIF
                  WRITE(*,6529)CWRK(1:L),MonthAbbrev(Month),Year
                  WRITE(*,6529)CWRK(1:L),MonthAbbrev(Month),Year
                  WRITE(*,6530) Q1887,StMarysFlow, Gates 
               ENDIF
               GOTO 9753
            ENDIF
         ENDIF
 6529    FORMAT(' Unable to', A, 'reduce ',A,I6,                        &
     &      ' Crit B outflow and remain above pre-project flow!')
 6530    FORMAT(' Pre-project flow: ', F12.1, '   violating flow: ',    &
     &      F12.1, '  violating gates: ', F5.2)
!
         IF( NINT(Gates + 0.51) .EQ. 1 ) THEN
!           Only half a gate open!  Can't close any more!  Logic
!           undefined for existing practice, so just squawk and 
!           move on.  XX Maybe should cut sidechannel flows to 0?
            WRITE(*,6549) MonthAbbrev(Month),Year
 6549       FORMAT(' Unable to comply with Criterion B during ',A,      &
     &         I5,'!')
            GOTO 9753
         ENDIF
!
!        Decrement gate setting
         Gates = Gates - 1
         IF(NINT(Gates) .EQ. 0) Gates = 0.5
!
!        XX This may be possible in weird scenario with MH very high,
!        XX so should come up with logic someday to handle it.
         IF(Gates .LT. 0.0) STOP ' Ginger'
!
!        Get new mean outflow for adjusted gate setting, loop again.
!        Put it straight into StMarysFlow, if were at 1/2 gate 
!        shouldn't have gotten here anyway.  Be aware that this 
!        leaves a hole in the logic regarding minimum flow situations.
         CALL P77ASolveSup(SuperiorBOPLevel, DBLE(SuperiorNWB),         &
     &      SecsInPeriod, Gates, CritFlowRound, CritBOMRound,           &
     &      SuperiorEOPLevel, StMarysFlow, SuperiorAveLevel,            &
     &      verbose)
!
!        Knock accuracy down to cm or whatever
  855    SuperiorEOPLevel = Round( SuperiorEOPLevel, CritBOMRound )
!
!     Get here only by falling out of loop.  How?
      STOP ' Mary Ann' 
!
!
 9753 CONTINUE 
!
!
!     Sidechannel (i.e., non-gated) flow determination
! 
!     Compute non-gated flow for the month.  Need to determine GateFlow,
!     and subtract it from the nominal total outflow.  This requires a
!     SW Pier level, computed from the current estimate of the Superior 
!     monthly mean level.  Existing practice was to perform these  
!     calculations by hand, so all levels are rounded to cm.  
!
!     This is set up for iterative solution of simultaneous equations in
!     if want to use new EC equation for LS:SWP.  The equations are 
!     simultaneous because SWPier level if a function of both Superior
!     level and outflow, while outflow is a function of Superior level.
!     So solve for SWP, get NonGatedFlow, get SWP, get NonGatedFlow, etc
!     until converge.  
!
!     Traditional practice uses the old SWP:LS relationship, which is not
!     a function of outflow, so there is no need to solve simultaneous 
!     equations.  When the old SWP:LS relationship is selected, the 
!     loop cycles twice (need at least two iterations to tets for 
!     convergence), computing the same answer both times.
!              
!     Seed NonGatedFlow with some number
      NonGatedFlow = 2320
!
!
!
      IF(verbose .GT. 0 ) THEN
         WRITE(*,'(A)') ' Solving for sidechannel flow . . .'
      ENDIF
!
      L = -9999
      Flow2ItsAgo = L
!
      DO 4778, I=1,999
         CALL GetSWPier( SuperiorAveLevel, Month, Gates,             &
     &      NINT(NonGatedFlow), CritBOMRound, LStoSWP, SWPier,       &
     &      gatedflow, verbose)
!
!        This is the method that Matt proposes, not the way it
!        is calculated in "existing practices". 
         NonGatedFlow = NINT(StMarysFlow - gatedflow)
!        XX Backward compat fix:
!        XX This following line replicates the goofy mixed rounding of
!        XX of existing practices.  If we're rounding flows, we'll do
!        XX it the old fashioned way. Remove it when allowed.
         IF(CritFlowRound .NE. -99 ) NonGatedFlow =                  &
     &      Round(StMarysFlow,1) - Round(gatedflow,CritFlowRound)
!        XX End of fix block
!
         IF( L .EQ. INT(gatedflow * 100000._wp) ) GOTO 4779
!        Check to see if oscillating between two values 
         IF(CritBOMRound .NE. -99 )THEN
            IF(INT(gatedflow*100000._wp) .EQ. Flow2ItsAgo) GOTO 4779
            Flow2ItsAgo = L
         ENDIF
 4778    L = INT(gatedflow * 100000._wp)
!
 4779 IF(verbose .GT. 0 ) THEN
         WRITE( *, 2310) NonGatedFlow
         WRITE( *, 2309) StMarysFlow, SuperiorAveLevel,     &
     &      SWPier, gatedflow
      ENDIF
!
      WRITE(SupSideChanFlow,'(I6)')                         &
     &   NINT(Round(NonGatedFlow,CritFlowRound))
      IF(verbose .GT. 0 ) THEN
         WRITE( *, '(A)') ' finished SupCriterions'
      ENDIF
!
 2309 FORMAT( '   (based on nominal total flow of ', F9.4,              &
     &   ' Sup/SW Pier levels of ', 2F9.4,', and gate flow of', F9.4,   &
     &   ')')
 2310 FORMAT( '   Computed "Official" non-gated flow as ', F9.4)
!
 6550 FORMAT(A,                                                         &
     &   'Criterion C violation!  Will try to close a gate . . .')
 6551 FORMAT(A,                                                         &
     &   'Cannot close any more gates, setting regulated flow to',      &
     &   F8.2, ' m3s')
 6537 FORMAT(A, ' . . . passed Criterion ', ' check OK.')

      END


!----&------------------------------------------------------------------
!----&------------------------------------------------------------------
!
      SUBROUTINE SupClosestFlow( TargetFlow, SupBOPLevel, MHLevel,      &
     &   NTS, dTm, LevelRnd, FlowRnd, ClosestFlow, Gates, SupEOPLevel,  &
     &   verbose)
!
!     This module takes a specified flow (TargetFlow) and figures out
!     the gate setting that yields the closest period-average flow.
!
! modules
      USE par_kind
      USE lakesup_mod
      USE outflow_mod

      IMPLICIT NONE 
!
!     Declarations regarding subroutine arguments:
!      
!
      real(wp) :: TargetFlow, SupBOPLevel, MHLevel
!                   (Input)  The desired flow and levels of lakes
!                   Superior and Michigan-Huron.  MHLevel isn't really
!                   used at this time, so put in anything you want
!                   when calling this function.  Just have it in place
!                   in case need to pass MH level to NonCompWorksFlow
!                   because made sidechannel flow a function of
!                   effective head.
!
      real(wp) :: NTS, dTm
!                   (Input)  The Net Total Supply, and length of the
!                   period in seconds.
!
      INTEGER       LevelRnd, FlowRnd
!                   (Input)  What power of ten the levels and flows
!                   must be rounded to.  For compatibility with
!                   traditional practice, or looking at better ways.
!
      REAL          Gates
!                   (Output) The number of gates that results in flow
!                   closest to TargetFlow
!
      real(wp) :: ClosestFlow, SupEOPLevel
!                   (Output) The period-average flow and End-of-Period 
!                   level resulting from Gates.
      INTEGER verbose
!
!
!     Declarations regarding local variables or functions:
!
      INTEGER       Setting
      real(wp) :: SettingFlow, MeanSupLevel
!                   Scratch variables
!
!
      SettingFlow = 0.0_wp
!
      DO  3923, Setting = 1, MaxSGSetting
!
!        Save flow from previous iteration in ClosestFlow
         ClosestFlow = SettingFlow
!
!        Figure out how many gates to pass to P77ASolveSup. Use variable
!        Gates for now, although its value will be reassigned at end.
         Gates = Setting - 1.0
         IF( Setting .EQ. 1) Gates = 0.5
!
!        Compute flow for this gate setting
         CALL P77ASolveSup(SupBOPLevel, NTS, dTm, Gates, FlowRnd,      &
     &      LevelRnd, SupEOPLevel, SettingFlow, MeanSupLevel,          &
     &      verbose)
!
!        Jump out if we get a higher flow 
 3923    IF( SettingFlow .GE. TargetFlow ) GOTO 5558
!
!
!     Setting is > MaxSGSetting (unable to pass TargetFlow)
      Setting = MaxSGSetting 
      GOTO 6232
!      
!     Pass through on Setting at minimum number (TargetFlow too small for gates)
!
 5558 IF( Setting .GT. 1 .AND. ABS(SettingFlow - TargetFlow)           &
     &   .GT. ABS(ClosestFlow - TargetFlow)) THEN
!        Since Setting > 1, Setting was incremented so that ClosestFlow
!        currently less than TargetFlow, and SettingFlow is greater than 
!        TargetFlow.  If ClosestFlow is closer, then need to decrement 
!        Setting and assign previous value (currently in ClosestFlow) to
!        SettingFlow.
!
         SettingFlow = ClosestFlow
         Setting = Setting - 1
!        Also need new EOP level 
         CALL P77ASolveSup(SupBOPLevel, NTS, dTm, -1.0, FlowRnd,       &
     &      LevelRnd, SupEOPLevel, ClosestFlow, MeanSupLevel,          &
     &      verbose)
      ENDIF
!
!     The flow and gate settings are figured out now.
 6232 ClosestFlow = SettingFlow
      Gates = Setting - 1.0
      IF(Setting .EQ. 1) Gates = 0.5
!
!
!
!      
      IF( verbose .GT. 0) THEN
         WRITE( *, 6555) SupBOPLevel, MHLevel
         WRITE( *, 6556) TargetFlow, ClosestFlow, Gates
      ENDIF
 6555 FORMAT(10X,'==> With Superior at ', F8.4,                        &
     &   ' and Michigan-Huron at ', F8.4, ',')
 6556 FORMAT(10X,'==> the closest flow to ', F11.4,' m3s is ', F9.4,   &
     &   ' m3s (', F5.2, ' gates)')
      END
!


!----&------------------------------------------------------------------
!----&------------------------------------------------------------------
!
!
!
!
      SUBROUTINE GetSupPreProject( SupBOP, SupNTS, PerLen, verbose, Mon,&
     &   PreProjFlow, PreProjMean)
!
! modules
      USE par_kind
!
      IMPLICIT NONE
!
      real(wp) :: SupBOP, SupNTS, PerLen
!              (Input) The Superior beginning of period level, the net 
!              of the non-outflow water balance terms, and the number 
!              of seconds in the routing period.
!
      INTEGER  Mon
!              (Input) The month that the flow is computer for.
!
      INTEGER  verbose
!              (Input) The level of debugging output to write.
!
      real(wp) :: PreProjFlow, PreProjMean
!              (Output) The mean St. Mary's flow and Superior level 
!              for the routing period.
!
      INTEGER  retard(12)
!              monthly retardation values
!
      INTEGER  incs
!              Number of increments to integrate storage change over
!
      INTEGER  MaxIterations
!              Upper limit for solving simutaneous equations
!
      INTEGER  i, k
      real(wp) :: Qi, BOI, EOI, LastFlow, dT, DScratch
!
!
      CHARACTER    CWRK*80
!                  See descriptions in Block Data Genral
!
      real(wp) :: GetArea, tmpA
!                      Returns area of lake
!
!
!
!     From:           	David_Fay@pch.gc.ca
!     Date sent:      	Mon, 2 Oct 2000 14:03:12 -0400
!     Just talked to Chuck, who apparently did some of the work with Nanette. 
!     He pointed me to the June 1970 report of the Coordinating Committee on
!     Superior flows.   It gives an estimated average Jan-Apr ice retardation
!     for 1871 -1887 to be 4 tcfs or about 110 m3/s based on the differences
!     between the Marquette and SWP discharge equations.  They note that this
!     may include effects of change in slope between Marquette and SWP as well
!     as ice in the upper river.
!
!     From:	Woodruff, Carl L LRE 
!     Sent:	Tuesday, October 03, 2000 9:24 PM
!     Subject:	Ice retardation factor
!     As I now understand it after talking with Nanette the ice retardation / 
!     Criteria "c" concern arose from what was the ice retardation that existed
!     prior to 1887 (Pre-project).  With regulation ice retardation is not a 
!     concern as the outflow is deetermined for the coming month and it is 
!     flowed.  During preproject times there were no structures and natural 
!     conditions prevailed which could have included restrictions due to ice 
!     and weed retardation in the St. Marys River.
!
!     Quoting from  Paragraph 21, Page 9 of"Lake Superior Outflow 1860 - 1968" 
!     by the Coordinating Committee on Great lakes Basic Hydraulic and 
!     Hydrologic Data, June 1970:
!
!     "21.  Effects of ice and weed retardation on stage discharge 
!     relationships.   The stage-discharge relationships at Southwest Pier are 
!     considered to be unaffected by the presence of ice and weeds in the 
!     relatively short reach from the gauge to the head of the rapids.  For the 
!     1860-1870 period, an average winter flow retardation due to ice of 4,000 
!     cfs was applied to the January through April flows from the Marquette 
!     stage-discharge relationship.  The 4,000 cfs retardation is the average 
!     difference between winter discharges from recorded Marquette and Southwest 
!     Pier stages in the 1871-1887 period and the appropriate stage-discharge 
!     relationships.  Thus the retardation used includes the effects of such 
!     factors as slope between Marquette and the rapids as well as the effects 
!     of ice.  The May through December relationship between Marquette and the 
!     discharge, computed from the Southwest Pier equation, contains any weed 
!     effect that occurs."
!
!     But it seems to make a big difference using the retardation, so leave
!     set to zero for now.
      DATA retard / 12 * 0  /
!
      DATA MaxIterations / 999 /
!
!     Use DScratch just to hush up SparcCompiler caution about integer division
      DScratch =  PerLen / DBLE(43200) 
      K = INT( DScratch )
      IF(K .LE. 1) THEN
!        Trial-and-error experience shows that time slices finer than
!        about 12 hours don't improve the Superior change-in-storage 
!        computation.  But if interval is > than 12 hours, just use it.
         incs = 1          
         dT = PerLen
!        
      ELSE
!        PerLen given as argument is longer than 1/2 day.  Find out the 
!        least number of increments contained within PerLen so that they 
!        are no more than 12 hours each.          
         IF( MOD( PerLen, 43200.0_wp ) .EQ. 0.0_wp ) THEN
!            PerLen is an whole multiple of 12 hours
             incs = k   
         ELSE
!            PerLen is not a whole multiple of 12 hours; round up
             incs = k + 1   
         ENDIF
         dT = PerLen / incs
      ENDIF
!
      IF( verbose .GT. 0) THEN
         WRITE(*, '(A,I3,A,F13.1, A)') ' Integrating Superior '//       &
     &      'pre-project outflow over ',incs, ' increments of ', dT,    &
     &      ' seconds each'
         WRITE(*, '(A,F10.5,A,F10.5,A,F13.3)')  ' BOP=', SupBOP,        &
     &      '  net water balance=', SupNTS, '  period len=', PerLen
      ENDIF
!
      PreProjFlow = 0.0_wp
      PreProjMean = SupBOP / 2
!
      EOI = SupBOP
!
      DO 1350 I=1, incs            
!
!        Initialize BOI levels as EOI levels from prev increment
         BOI = EOI
!
!        Dump some debugging info if desired:
         IF(verbose .GT. 0 ) THEN
            WRITE(*,'(A,I3,3(A,F15.8),A,F15.2)')                        &
     &         ' Increment ', I, ' BOI=', BOI, ' EOI=', EOI,            &
     &         ' MLV=', (EOI + BOI)/2, ' Area=',                        &
     &         (EOI + BOI)/2 
         ENDIF
!
!        Will do iterative solution of 2 simultaneous eqns:
!           mean level for increment = (SPBOI + SPEOI)/2
!           SPOutI = f(SPBOI + SPEOI)/2
!           (SPEOI-SPBOI)*Area/dT = Net Total Supply - outflow
         DO 1310, K = 1, MaxIterations
!           Save previous outflow for comparison to tolerance
            LastFlow = Qi
!       
!           Approximate the outflow for the iteration
            Qi=824.7_wp * ((EOI + BOI)/2 - 181.43_wp)**1.5_wp   &
     &         - retard(mon)
!                 
            tmpA = GetArea(1,(EOI + BOI)/2)
            EOI = BOI + (SupNTS - Qi) * dT / tmpA
!
!           Dump some debugging info if desired:
            IF(verbose .GT. 0 ) THEN
               WRITE(*,'(2(A,I3),3(A,F15.8))') ' Increment ',           &
     &            I, ' Iteration ', K, ' Approx. mean=', (EOI + BOI)/2, &
     &            ' Approx. outflow=', Qi, ' delta S=',                 &
     &            (SupNTS - Qi) * dT / tmpA
            ENDIF
!
 1310       IF( ABS(Qi - LastFlow) .LE. 0.000001_wp) GOTO 1330
!
!        If fall to here, must have exceeded MaxIterations, so
!        failed to converge.
         STOP 'Troy McClure'
!
! 
 1330    IF( verbose .GT. 0 ) THEN
            WRITE(*,605)I, BOI, EOI
            WRITE(*,606)I, Qi, (EOI + BOI)/2
            WRITE(*,612)I, K, 0.000001_wp
            WRITE(*,607)I, PreProjFlow, PreProjMean
         ENDIF
!            
!        Accumulate incremental results
         PreProjMean = PreProjMean + EOI
 1350    PreProjFlow = PreProjFlow + Qi
!
      PreProjMean = (PreProjMean - EOI / 2) / incs
      PreProjFlow = PreProjFlow / incs
!           
      IF( verbose .GT. 0) THEN
         WRITE(*,608)PreProjFlow, PreProjMean
      ENDIF
  605 FORMAT(' Sup pre-project increment ',I3,                          &
     &            ' beginning- and end-of-increment levels: ',2F14.8)
  606 FORMAT(' Sup pre-project increment ',I3,                          &
     &            '    mean outflow and mean level for inc: ',2F14.8)
  607 FORMAT(' Sup pre-project increment ',I3,                          &
     &            '    accumulated outflow and mean level: ',2D21.14)
  608 FORMAT(' Sup pre-project outflow and mean level: ', 2F14.8)
  612 FORMAT(I3, ' Required ', I3,                                      &
     &               ' iterations to converge within ',F7.3,' m3s.')
!
      END
!----&------------------------------------------------------------------
!----&------------------------------------------------------------------


!
!----&------------------------------------------------------------------
!----&------------------------------------------------------------------
!
      SUBROUTINE P77ASolveSup(BOPLevel, NTS, dTm, Gates, CFlowRound,    &
     &   CLevelRound, EOPLevel, MeanOutflow, MeanLevel,                 &
     &   verbose)
!     
!     Solves simultaneous equations for Lake Superior outflow,
!     mean level, and change-in-storage, where outflow consists of
!     flow through the compensating works and sidechannels.  Beefed-up 
!     version of EC function "FLOW" or Corps subroutine "MNLVL". 
!
!     Carries out steps 6 - 10 on pages 27 - 28 of Plan 77.  A comment
!     in the old Environment Canada model claims "A fixed number of 
!     iterations is used for coordination with other programs."
!     As far as Matt can tell, this means that four iterations are used
!     because the blue book shows an example doing it that way.  Matt 
!     thinks we should eliminate the rounding in this routine, and 
!     iterate to a tolerance.  
!
!     Any changes in this routine will affect backward-compatibility
!     with the older models.
!     
!     SUBROUTINE MNLVL(S,SM,SN,O,A2,A3)
!     Revised to metric and IGLD 85, Apr 1993
!     IMPLICIT real(wp) :: (A-H,O-Z)
!     O=(A1*(A2*S-A3)**1.5_wp)*0.1_wp+232.0_wp
!     DO 13 L=1,4
!     CALL ROUND(((SN-O)/31.38_wp),ISM)
!     CALL ROUND(S*100.0_wp+ISM*0.5_wp,ISM2)
!     SM=ISM2*0.01_wp
!     O=(A1*(A2*SM-A3)**1.5_wp)*0.1_wp+232.0_wp
!     CALL ROUND(O*10.0_wp,ISO)
!     O=ISO*0.1_wp
!  13 CONTINUE
!     RETURN
!
! modules
      USE par_kind
      USE lakesup_mod
      USE outflow_mod

      IMPLICIT NONE
!
!
!     Declarations regarding subroutine arguments:
!      
!
      real(wp) :: BOPLevel, NTS, dTm 
!                   (Input)  The Beginning-of-Period Superior level, 
!                   Net Total Supply (including diversions), and number
!                   of seconds in the month.  
!
      REAL          Gates
!                   (Input)  The number of gates open.  If gates given 
!                   as a negative value, then MeanOutflow is an input.
!                    
      INTEGER       CFlowRound, CLevelRound
!                   (Input)  The decimal place to round flows and levels
!                   to ( -2 means hundredths, 1 means 10's).
!
      real(wp) :: MeanOutflow
!                   (Input/Output) The mean outflow for period. If gates
!                   is less than zero, then determine EOP and mean level
!                   for Lake Superior based on MeanOutflow.  If gates is
!                   positive, determine EOP and mean level based on gate
!                   equations, and return average outflow in MeanOutflow
!
      real(wp) :: EOPLevel, MeanLevel
!                   (Output) The End-Of-Period and period-average level.
      INTEGER verbose
!            
!
!
!     Declarations regarding local variables or functions:
!
!
      INTEGER       I, G, MaxIterations
      real(wp) :: TmpF, TmpA
!                   Scratch
!
      real(wp) :: NonCompWorksFlow, Round, GetArea
!                   Function returns sidechannel flow given lake levels,
!                   function to round, and function to return area of
!                   lake given its elevation.
      CHARACTER*100 CWRK
!
!
!     XX This hardcoded limit should be changed if have the chance to 
!     XX tweak Plan 1977A.  Better to iterate to a tolerance.  That 
!     XX itself is easier to do if no rounding.
      DATA MaxIterations /4/
!
!     Init MeanLevel
      MeanLevel = BOPLevel
!
!     Compute first approximation
!
      IF(Gates .GT. 0.0) THEN 
!        Save index to SGX, SGY, SGZ in G 
         G = NINT(Gates + 0.51)
         IF(SGY(G) * MeanLevel - SGZ(G) .LT. 0.0_wp .AND. verbose .GT. 0) THEN
            WRITE( *, '(A,F7.3,A,F6.2)') ' Superior Level=',          &
     &         MeanLevel, '   Gates=', Gates
         ENDIF
!        original
         MeanOutflow = Round(NonCompWorksFlow(MeanLevel, 0._wp) +     &
     &     SGX(G) * (SGY(G) * MeanLevel - SGZ(G)) ** 1.5_wp, CFlowRound)
      ENDIF
!
!
      IF( verbose .GT. 0) THEN
         TmpA = GetArea(1,MeanLevel)
         Write(*, '(A,F10.5,A,D21.14,A,F13.2,A,F9.2)')                  &
     &      ' Entering P77ASolveSup, BOP Level= ', BOPLevel, '  Area= ',&
     &      TmpA, ' dT= ',dTm, '   NTS=', NTS
      ENDIF
!      
!     Repeat steps 6 - 10
      DO 10, I = 1, MaxIterations
!         IF( verbose .GT. 0) THEN
!            Write(*, '(2(A,I2),3(A,F10.5))')' Iteration ', 
!     &          I, ' of ',MaxIterations,
!     &          '  EOPLevel=', EOPLevel, '  MeanLevel=', MeanLevel, 
!     &          ' MeanOutflow=', MeanOutflow
!         ENDIF
         EOPLevel= Round(dTm * (NTS - MeanOutflow)   &
     &   /(GetArea(1,MeanLevel))                     &
     &      + BOPLevel, CLevelRound)
         MeanLevel = Round((BOPLevel + EOPLevel) * 0.5_wp, CLevelRound)
         IF( verbose .GT. 0) THEN
            TmpA = GetArea(1,MeanLevel)
            Write(*,'(A,F16.10,A)')'      Unrounded stage change= ',    &
     &         dTm * (NTS - MeanOutflow)/(TmpA)
            Write(*,'(A,F16.10,A)')'      New EOPLevel=',EOPLevel
            Write(*, '(A,F16.10)')                                      &
     &         '      New MeanLevel=', MeanLevel
         ENDIF
!
!        Use Pt Iroquois Eq's because that's what Plan 1977A does.
!        XX Maybe we can add an additional argument to this function
!        XX for optionally using SW Pier, is calling this function 
!        XX for calculations outside Plan1977A.
! 
!
         IF(Gates .GT. 0.0)MeanOutflow= Round(                          &
     &      NonCompWorksFlow(MeanLevel, 0._wp) +                        &
     &     SGX(G) * (SGY(G) * MeanLevel - SGZ(G)) ** 1.5_wp, CFlowRound)
!
         IF( verbose .GT. 0) THEN
            IF(Gates .GT. 0.6) THEN
               WRITE(CWRK, '(A,3F9.4,A,F12.7)')            &
     &           ' X,Y,Z= ',SGX(G),SGY(G),SGZ(G),          &
     &          ' SGX*(SGY*MeanLevel - SGZ)= ',            &
     &          SGX(G) * (SGY(G) * MeanLevel-SGZ(G))
            ELSE IF(Gates .LT. 0.1) THEN
               WRITE(CWRK, '(A)') ' '
            ELSE 
               WRITE(CWRK, '(A)') ', 1/2 gate open, '
            ENDIF
!           Mysterious behavior here.  Embedding the call to NonCompWorksFlow
!           directly into the following WRITE corrupted the string.  
!           Assigning to a temporary value first works OK.  Is there some
!           missing typecasting?  A compiler bug (MS Powerstation)?  Or is 
!           this just the manifestation of a subtle bug somewhere else?
!           XX Try getting rid of TmpF and embedding call in WRITE again.
            TmpF = NonCompWorksFlow(MeanLevel, 0._wp)
            WRITE(*, '(A,F10.5,A,A,F10.5)')              &
     &         '      NonCompWorksFlow= ', TmpF,         &
!     &      '      NonCompWorksFlow= ',NonCompWorksFlow(MeanLevel,0._wp), 
     &         CWRK(1:INDEX(CWRK,' ')-1),                &
     &         ' New Mean Outflow=', MeanOutflow
         ENDIF
!
   10 CONTINUE
!      
      IF( verbose .GT. 0) THEN
         WRITE( *, 6556) Gates, MeanOutflow, MeanLevel
         WRITE( *, 6555) BOPLevel, EOPLevel, NTS, dTm
      ENDIF
 6555 FORMAT(10X,'Superior BOP level: ', F8.4, ' Superior EOP level: ', &
     &    F8.4, ' net total supply: ', F9.3, ' seconds in period: ',    &
     &    F9.1)
 6556 FORMAT(10X, 'Gates open: ', F6.2,                                 &
     &   ' mean outflow:', F8.3, ' mean level: ', F8.4)
!  
      END




      SUBROUTINE AdjustDueToCompOvertop(DayOfMon, Mon, Year, PeriodLen, &
     &   verbose, SPNWB, MaxSideChannel, Gates, SideChan,               &
     & CodeStr)
!
!     returns the adjusted gates and Side Channel flows in situation where 
!     Compensating Works may be overtopped
!      
! modules
      USE par_kind
      USE lakesup_mod
      USE outflow_mod

      IMPLICIT NONE 
!
!     Declarations regarding subroutine arguments:
!      
      INTEGER          DayOfMon, Mon, Year, verbose
!                      (input) The date, month number, year, and debug message level
!
      INTEGER          MaxSideChannel
!                      (input) Total capacity of all outflow not through gates
!
      real(wp) :: PeriodLen, SPNWB
!                      (input) The number of seconds in routing period, Lake
!                      Superior beginning of period level and the sum of the
!                      water balance terms
!
      real(wp) :: SideChan
!                      (input/output) The total side channel flow when the 
!                      overtopping condition arose.  This value will be modified.
!
      REAL             Gates
!                      (input/output) The total gates open when the overtopping
!                      condition arose.  This value will be modified.
!
      CHARACTER*24     CodeStr
!                      (input/output) The codes used for this regulation decision, 
!                      which will be clobbered if reg plan is overridden
!
!     Declarations regarding local variables or functions:
!
      INTEGER          NumGates
!                      Scratch variable
!
      real(wp) :: PreProjFlow, tmplev,                                  &
     &                 tmpEOP, tmpMLvl, tmpSWP, gatedFlow, totalFlow
!
!     Dump input to log if desired  
      IF( verbose .GT. 0) THEN
         WRITE(*,'(A,I2,A3,I5,A,I2)')                                   &
     &      '    AdjustDueToCompOvertop: ',DayOfMon,MonthAbbrev(Mon),   &
     &      Year, '  debug lvl=', verbose
         WRITE(*,'(A,F12.6,A,F11.4,A,F12.3,A,I7,A)')                    &
     &      '         Superior BOP ',SPBOP, 'm,  net inflow: ',         &
     &      SPNWB, ' m3/s,   routing period length: ', PeriodLen,       &
     &      ' seconds   Maximum Side Channel: ' , MaxSideChannel,       &
     &      ' m3/s'
         WRITE(*,'(A,F6.2,A,F11.4,A)')                                  &
     &      '         beginning gates ',Gates, ',  sidechannel ',       &
     &      SideChan, ' m3/s'
      ENDIF
!
!
! From: Fan,Yin [Ontario] [mailto:Yin.Fan@ec.gc.ca]
! Sent: Wednesday, December 05, 2007 1:25 PM
!  To: McPherson, Matthew M HEC
!  Cc: Fay,David [Ontario]; Kropfreiter, Melissa A LRE; Dahl, Travis A LRE
!  Subject: 
!
! Matt,
!  I tried some tests by changing P77AMHRangeFact, I got very high Lake Superior 
!  levels (over 185m) while Lake Superior outflow was less than 2000 cms. As we 
!  know the top of the lowest 4 gates of Comp Work is only 184.0 m, so the flow 
!  and Lake Superior level are not correct. I talked with David about this. We 
!  would like to know if it is possible to code something in CGLRRM like if Lake 
!  Superior level is greater than 184.0 m than the flow should be not less than 
!  preproject flow. To develop our fencepost plan we want to use Lake Superior as 
!  a reservoir for Lake MH, but the levels and flows still need to be physically 
!  real.
!
!  Experimental results indicate that using Criterion B  would allow a greater 
!  outflow than the pre-project flow used here. The implementation here based on 
!  pre-project flows is an arbitrary assumption. (from Yin Fan, 8 Oct 2008)
!
!     First determine pre-project flow.  
!  XX  todo: verify this is using same SPNWB as criterion checks
!     Verbosity hardcoded because we really don't need all the messages in sub
      CALL GetSupPreProject( SPBOP, SPNWB, PeriodLen, 1, Mon,           &
     &   PreProjFlow, tmplev)
!
!     Establish value of mean outflow if already above 
!     XXHardcoded to use old SWP equations for now instead of yr 2000 eq's
!     XXNeed to change this to use the function.
      CALL SolveGatesSide( Gates, NINT(SideChan), Mon, SPBOP, SPNWB,    &
     &   PeriodLen, 1, tmpEOP, tmpMLvl, tmpSWP, gatedFlow, totalFlow)
!
      IF( verbose .GT. 0 ) THEN
         WRITE(*,'(A,F12.6,A,F11.4,A,F11.4,A)')                         &
     &      '        PreProjFlow: ',PreProjFlow,                        &
     &      ' m3/s,  period-ave regulated flow ',totalFlow,             &
     &      ' m3/s,  period-ave gated flow ',gatedFlow, ' m3/s'
      ENDIF
!
!     So if total expected flow is less than pre-project, raise sidechannel
!     to max and open gates until flow higher than pre-project.
!
!     See if already flowing greater than pre-project
      IF( PreProjFlow .LE. totalFlow) THEN
         WRITE(*,'(2(A,F12.6),A)')                                  &
     &      '         Regulated flow ', totalFlow,                  &
     &      ' m3/s is already higher than Pre-project flow ',       &
     &      PreProjFlow,                                            &
     &      ' m3/s, so no adjustments made to avoid overtopping.'
      ELSE
!        Somehow need to raise outflow above Pre-Project
         CodeStr = '    Overtop Adjustment  '
!
         IF( PreProjFlow .LE. (gatedFlow + MaxSideChannel) ) THEN
!           Simply increasing sidechannel is enough.  Leave
!           gates as before, but bump up SideChannel so that
!           total flow equals Pre-Project flow
            SideChan = PreProjFlow - gatedFlow    
            WRITE(*,'(A,F6.2,A,F12.6,A)')                               &
     &         '         kept ', gates,                                 &
     &         ' gates but increased sidechannel flow to ', SideChan,   &
     &         ' m3/s to avoid overtopping.'
         ELSE
!           Set to sidechannel to max and opening gates until get 
!           above preproj flow
            SideChan = MaxSideChannel
!           
!           0.5 gates becomes 0 for purposes of incrementing to next
            DO 352, NumGates = NINT(gates - .1) + 1, MaxSGSetting - 1
               CALL SolveGatesSide( REAL(NumGates), MaxSideChannel,     &
     &            Mon, SPBOP, SPNWB, PeriodLen, 1, tmpEOP, tmpMLvl,     &
     &            tmpSWP, gatedFlow, totalFlow )
!
               IF( verbose .GT. 0) THEN
                  WRITE(*,'(A,I2,3(A,F11.4),A)')                        &
     &                '         trying ',Numgates,                      &
     &                ' gates and sidechannel =', SideChan,             &
     &                ' m3/s:  gateflow=', gatedFlow,                   &
     &                ' m3/s,  totalflow=', totalFlow, ' m3/s'
               ENDIF
!
               IF( totalFlow .GE. PreProjFlow ) THEN
!                 Opened enough, now set gates and jump out 
                  Gates = NumGates
                  GOTO 367      
               ENDIF
!
  352          CONTINUE
!
!              If reached here then could not get above Pre-Project 
!              even with all gates open.  Probably not possible, 
!              indicating that simulation is too far from reality
!              to be meaningful, and may as well stop.    
!              This should be around 184.57m IGLD 85 (according to Rob 
!              Caldwell 8Oct08)
               STOP 'FAIL OVERTOP'
!
  367       WRITE(*,'(A,F6.2,A,F11.4,A)')             &
     &         '         adjusted to ',Gates,         &
     &         ' gates and sidechannel=',SideChan, ' m3/s'
         ENDIF
!
      ENDIF
!     
      END



      SUBROUTINE SolveGatesSide( Gates, SideChannel, Mon, BOP, NTS,     &
     &   PeriodLength, LS2SWP, EOP, MeanLevel, SWPierLevel, GateFlow,   &
     &   MeanTotalFlow)
!      
!     This module solves the storage equation for Lake Superior 
!     for a specific sidechannel flow and gate setting.  Mean
!     outflow and lake level computed, along with mean monthly 
!     level.
!
! modules
      USE par_kind
!
      IMPLICIT NONE 
!
!     Declarations regarding subroutine arguments:
!      
      REAL             Gates
!                      (input) The number of gates open.
!
      INTEGER          SideChannel, Mon, LS2SWP
!                      (input) The constant non-gated outflow, the 
!                      month number, and which Superior:SWP eqn to use.
!
      real(wp) :: BOP,  NTS,  PeriodLength
!                      (input) The BOP Lake Superior level, sum
!                      of NBS and Long Lac/Ogoki, and number of seconds
!                      in the time slice.
!
      real(wp) :: EOP, MeanLevel,MeanTotalFlow,GateFlow,SWPierLevel
!                      (output) EOP and average Lake Superior levels,
!                      corresponding SWP level, gated flow, and total 
!                      St. Marys flow.
!
!     Declarations regarding local variables or functions:
!
      INTEGER          I
!                      Scratch variable
!
      INTEGER          MaxIterations, flowtest
!
      real(wp) :: GETAREA
!                      Function that returns surface area for a lake
!
      DATA MaxIterations / 100 /
      flowtest = 0
      MeanLevel = BOP


!     initiliaze to some reasonable value
      MeanTotalFlow = 2000._wp
!
      DO 322 I=1, MaxIterations            
         EOP = BOP + (NTS - MeanTotalFlow) *                       &
     &      PeriodLength / GETAREA(1,MeanLevel)
         MeanLevel = (EOP + BOP) / 2.0_wp
         CALL GetSWPier(MeanLevel, Mon, Gates, SideChannel, -99,   &
     &      LS2SWP, SWPierLevel, GateFlow, 0)
         MeanTotalFlow = SideChannel + GateFlow
         IF( flowtest .EQ. INT(GateFlow * 100000._wp) ) GOTO 323
!
  322    flowtest = INT(GateFlow * 100000._wp)
!
!     Should never get here - there's no rounding, so should always
!     be able to converge to within 8-10 sigfigs 
      STOP ' Wilma!'
  323 END




!
!
      SUBROUTINE GetSWPier(SupLev, Month, Gates, NonGatedFlow, SWPRnd,  &
     &   LS2SWP, SWPierLev, GateFlow, verbose)
!      
!    This function takes a Lake Superior (ie, Pt Iroquois level) and
!    a month number, determines the corresponding level at S.W. Pier,
!    and applies it to the SW Pier gate equations.  The gate equations
!    are calibrated periodically, and fairly reliable.  The original
!    relationship between Pt. Iroquois and S.W. Pier is quite old, and
!    possibly based on very limited data.  Rob Caldwell at EC has come
!    up with an improved alternative.  It incorporates total flow, 
!    requiring that the non-gated flow be specified.
!
!    As far as Matt knows, the Pt Iroquois equations are derivatives of 
!    the old SW Pier relationship and gate equations, inheriting all 
!    their flaws, and then muddling things up a bit further.
!    
!
! modules
      USE par_kind
      USE lakesup_mod
      USE outflow_mod

      IMPLICIT NONE 
!
!     Declarations regarding subroutine arguments:
!      
      real(wp) :: SupLev
!                      (input) The Lake Superior level.
!
      INTEGER          NonGatedFlow, Month, SWPRnd
!                      (input) The month of the lake level, and the 
!                      rounding position for SW Pier levels.
!
      INTEGER          LS2SWP
!                      (input) Indicates which LS:SWP relationship to use:     
!                         1 = old Pt Iroquois / SW Pier equation
!                         2 = Caldwell's new equation of 2000
!
      REAL             Gates
!                      (input) The number of gates open in Month.
!
      real(wp) :: SWPierLev, GateFlow
!                      (output) Level at S.W. Pier, and flow through
!                      compensating works. 
!
!
!     Declarations regarding local variables or functions:
!
      INTEGER          I, flowtest, Flow2ItsAgo
!                      Scratch variables
!     
      real(wp) :: ROUND
!                      Returns value rounded to a certain accuracy,
!                      according to the old Lake Survey method.
!
      real(wp) :: SWPierGateFlow
!                      Returnd flow through gates, given SWP level
      INTEGER verbose
!
!
!
 6899 FORMAT('         Entering GetSWPier() with ', A3,                 &
     &    ' Superior level of ',F14.9,', and ', F8.1,' gates . . .')
      IF ( verbose .GT. 0) THEN
         WRITE(*,6899) MonthAbbrev(Month), SupLev, Gates
      ENDIF
!
!     XX TODO: make a nice error message here
      IF( SupLev .LT. 100) STOP ' Bogus Sup level!'
!
!
      IF( LS2SWP .EQ. 1) THEN 
!        Using original (old) LS:SWP relationship
         SWPierLev = Round(                                             &
     &      (SupLev + OldSWPA(Month))/OldSWPB(Month) + OldSWPC, SWPRnd)
         IF( Gates .LT. 0.45 .OR. Gates .GT. 16.1) THEN
!           Bogus number.  No gates open, or don't care.
            GateFlow = 0.0_wp
         ELSE
            GateFlow = SWPierGateFlow( SWPierLev, Gates )
         ENDIF
         IF ( verbose .GT. 0) THEN
            WRITE(*,'(A, F10.2)') '    . . . gate '//  &
     &         ' flow of ',                            &
     &         GateFlow
         ENDIF
!        
         
      ELSE IF( LS2SWP .EQ. 2) THEN 
!        Using EC 2000 (new) LS:SWP relationship
!        Solve simultaneous eqn's iteratively because new SWP eqn is
!        f(gateflow) and gateflow is f(SWP)
         IF ( verbose .GT. 0) THEN
            WRITE(*,'(A, I5)') '    . . . using new EC '//         &
     &         'relationship for LS:SWP and non-gated flow of ',   &
     &         NonGatedFlow
         ENDIF
!
         GateFlow = 100
         Flow2ItsAgo = 101
         IF( GateFlow + NonGatedFlow .LT. 0) STOP 'Kazoo'
         flowtest = 102
         DO 233, I = 1, 9999
            SWPierLev = Round( EC2000SWPA * SupLev -               &
     &         EC2000SWPB * LOG(GateFlow + NonGatedFlow) -         &
     &         EC2000SWPC(Month), SWPRnd)
            GateFlow = SWPierGateFlow( SWPierLev, Gates )
            IF( flowtest .EQ. INT(GateFlow * 100000._wp) ) GOTO 5988
!           Check to see if oscillating between two values 
            IF(SWPRnd .NE. -99 )THEN
               IF(INT(GateFlow*100000._wp) .EQ. Flow2ItsAgo) GOTO 5988
               Flow2ItsAgo = flowtest
            ENDIF
!            
  233       flowtest = INT(GateFlow * 100000._wp)
!
 5988    CONTINUE
      ELSE IF( LS2SWP .EQ. 3) THEN 
!        Using EC 2008 (new) LS:SWP relationship
!        Solve simultaneous eqn's iteratively because new SWP eqn is
!        f(gateflow) and gateflow is f(SWP)
         IF ( verbose .GT. 0) THEN
            WRITE(*,'(A, I5)') '    . . . using new EC '//         &
     &         'relationship for LS:SWP and non-gated flow of ',   &
     &         NonGatedFlow
         ENDIF
!
         GateFlow = 100
         Flow2ItsAgo = 101
         IF( GateFlow + NonGatedFlow .LT. 0) STOP 'Kazoo'
         flowtest = 102
         DO 234, I = 1, 9999
            SWPierLev = Round( EC2008SWPA * SupLev -               &
     &         EC2008SWPB * LOG(GateFlow + NonGatedFlow) -         &
     &         EC2008SWPC(Month), SWPRnd)
            GateFlow = SWPierGateFlow( SWPierLev, Gates )
            IF( flowtest .EQ. INT(GateFlow * 100000._wp) ) GOTO 5989
!           Check to see if oscillating between two values 
            IF(SWPRnd .NE. -99 )THEN
               IF(INT(GateFlow*100000._wp) .EQ. Flow2ItsAgo) GOTO 5989
               Flow2ItsAgo = flowtest
            ENDIF
!            
  234       flowtest = INT(GateFlow * 100000._wp)
!
 5989    CONTINUE
 
      ELSE
         STOP 'Alice!'
      ENDIF
!

      IF ( verbose .GT. 0) THEN
         WRITE(*,'(A,I3,A,2F16.10)')                          &
     &      '            . . . with level rounding at ',      &
     &      SWPRnd, ' SWP level and gated flow compute to ',  &
     &      SWPierLev, GateFlow
      ENDIF
!
!  From Rob in summer 2000:
!  
! The new relationship is a linear equation with monthly mean Lake Superior
! (LS) levels and the natural logarithm of Lake Superior monthly mean 
! outflow (ln(Q)) as the only independent variants.  There are constant 
! monthly fudge factors to the basic y-intercept (-8.125 m).  I'd enter 
! those using the SWPA array, though you might want to rename it SWPC 
! for clarity.  The basic form of equation is:
! 
! SWPierLev = 1.046385*LS - 0.10015*ln(Q) + SWPC, with everything in 
! metres and 10 cms, and SWPC as:
! 
! MONTH     SWPC (m)
! Jan       -8.152
! Feb       -8.161
! Mar       -8.163
! Apr       -8.158
! May       -8.129
! Jun       -8.129
! Jul       -8.125
! Aug       -8.124
! Sep       -8.125
! Oct       -8.126
! Nov       -8.118
! Dec       -8.127
! 
! The above new and improved relation was developed on data from 1970 to 
! 1998, since there was a funky discontinuity in the set prior to that 
! (your guess is as good as ours as to what caused it) for the open 
! water months from May to November.  Soooo, the above correction 
! factors mainly account for ice(, weeds, etc), and the equation 
! shouldn't be stretched beyond it's derivation limitations (183.08<LS<183.91 m & 125<Q<374 10 cms).

!   ---------------------------------------------------------------------------------   

!	From Rob's 2008 report: DEVELOPMENT OF A REVISED S.W. PIER WATER LEVEL RELATIONSHIP
	
!	The study described above resulted in the development of a new Lake Superior: S.W. Pier water level relationship.  The authors developed an updated relationship [EC2008 henceforth] using May-November data from 1979-2007:  
	
!	SWP = aSUP+bln(Q)+c  		(3)
	
!	Where: 	SWP 	= monthly mean water level at S.W. Pier gauge (m IGLD 1985),
!	a	= 1.033806,	
!	SUP 	= monthly mean lake-wide average water level of Lake Superior (m IGLD 1985), 
!	b	= -0.08622,
!	ln(Q)	= natural logarithm of monthly mean Lake Superior outflow (10 m3/s), and
!		c	= base intercept (-5.892 m) and a monthly correction factor (see Table 3 below).
			
!	As this relationship was derived using open-water conditions, the following constants, "c", are applied to account for monthly variabilities due to ice, etc.:
	
!	Table 3: Monthly Constants for EC2008 Relationship
!	Mth.	Jan		Feb		Mar		Apr		May		Jun		Jul		Aug		Sep		Oct		Nov		Dec
!	c (m) -5.916  -5.923  -5.925  -5.923  -5.897  -5.897  -5.892  -5.891 -5.894   -5.890  -5.886  -5.884
		
      END
!



      SUBROUTINE CheckSideChannelLimits (ProposedSideChannelFlow,    &
     & RegulatedMonth, verbose)
! This subroutine grew out of conversations regarding the physical  
! constraints that became obvious during Fencepost Plan creation.  Actual 
! values established primarily by Rob Caldwell & Missy Kropfreiter in 2008.
!
! Notes below compiled by Matt McPherson, Travis Dahl, Missy Kropfreiter, 
! Yin Fan, & Rob Caldwell on 8 Oct 2008.
!
! Sidechannel capacity:  The total sidechannel flow used by the routing model
! included all outflow not going through an open gate.  We decided to stick 
! with 2320 m3/s for a summer capacity, pending other information.  This 
! breaks down to:
! 1145 m3/s for Brookfield Renewable Power (typical capacity)
! 1145 m3/s for Edison Sault/US Gov't plant (assuming need to maintain Treaty)   
! 11 m3/s for leakage through gates (0.7m3/s*16)
! 19 m3/s for municipal, industrial, lockages
! 
! Two considerations were discussed, but chose not to change value:
! 1) Leakage isn't known with any confidence, but suspected to be somewhat 
!     higher than the amount given due to locks, stop logs, valves, sandstone
!     formations. 
! 2) The total US capacity is actually 1250, but limited by Treaty to an equal
!     share with Canada, so assume 1150 m3/s.  Modelers should check simulation
!     results and count the number of times at max sidechannel.  If many times,
!     then should note that we're probably not capturing the full hydropower
!     opportunity.
! 
! Physical minimums reflect lowest ability to stay operating - not leakage in
! case of shutdown.
! 
! Minimum winter sidechannel (Dec-Apr):
! 565 BRP
! 305 ES
! 260 US Gov't plant
! 10 m3/s for leakage through gates (0.7m3/s*16)
! 10 m3/s for municipal, industrial
! =1150 m3/s minimum flow to keep hydropower open.  Track flows attempted below
! 1150 m3/s to determine how often the Treaty assumption comes into play.
! Winter = Dec-Apr assumed
! 
! Minimum summer sidechannel (May-Nov)
! 40 m3/s USGP for plant power
! 90 m3/s ES to justify operation
! 130 m3/s BRP by Treaty
! 10 m3/s for leakage through gates (0.7m3/s*16)
! 10 m3/s for municipal, industrial
! =280 m3/s summer min
! 
! Minimum reasonable flow - 1200 m3/s is historical low flow - flag any lower
! flows.
! 
! Mean Superior to SW Pier relationship 
! 
! In 1989, 1969, and 1936, 10 cm difference between Superior lake-wide average
! and SW Pier - should cause a warning that the equation is out of range. 
! 5 cm in 1929, but isostatic rebound affects.
! 
! EC 2008 SWP Relationship range of conditions was Q=1250-3740 m3/s ,
! SUP=182.87-183.91 m.

! modules
      USE par_kind
!
      IMPLICIT NONE

!     Variables
      real(wp) :: ProposedSideChannelFlow
      INTEGER RegulatedMonth
      INTEGER verbose ! input
      real(wp) :: MaxCap, MinSummerCap, MinWinterCap     
!
!
!     Dump entering information
!

      if (verbose .gt. 0)                                               &
     &WRITE(*,'(A,F8.1,A,I2)')                                          &
     &  '  Entering CheckSideChannelLimits, ProposedSideChannelFlow= ', &
     &  ProposedSideChannelFlow, ' RegulatedMonth=', RegulatedMonth

!     Flow limits based on known physical infrastructure (see note at 
!     beginning of subroutine)
!     XX These should probably be input parameters
!     Maximum Sidechannel Capacity
!     2320 chosen to avoid backward compatibility problems with Plan 77A
!     Brookfield Power Rating       = 1140 cms
!     US/Edison Soo Plants          = 1140 cms (Per treaty requirements)
!     Fisheries Remedial Gate       =   15 cms (As long as < 16 gates open)
!     Comp Works Leakage            =    8 cms
!     Municipal/Industrial/Lockages =   19 cms
!                             TOTAL ~ 2320 cms
      MaxCap = 2320 
!     Minimum Winter Sidechannel Capacity
      MinWinterCap = 1150
!     Minimum Summer Sidechannel Capacity
      MinSummerCap = 280
      
      if (verbose .gt. 0) then
      IF (ProposedSideChannelFlow .GT. MaxCap) THEN
         WRITE(*,'(A,F8.1,A,F6.0)')                                     &
     &    '  Proposed SideChannel flow of ',  ProposedSideChannelFlow,  &
     &    ' changed to physical capacity of ', MaxCap
         ProposedSideChannelFlow = MaxCap
!    XX Add a counter here
      ELSE
        IF (RegulatedMonth .GT. 4 .AND. RegulatedMonth .LT. 12) THEN
           IF (ProposedSideChannelFlow .LT. MinSummerCap) THEN
              WRITE(*,'(A,F8.1,A,F6.0)')                                &
     &    '  Proposed SideChannel flow of ',  ProposedSideChannelFlow,  &
     &    ', changed to minimum summer flow of ', MinSummerCap
              ProposedSideChannelFlow = MinSummerCap
!    XX Add a counter here
           END IF
        ELSE
           IF (ProposedSideChannelFlow .LT. MinWinterCap) THEN
              WRITE(*,'(A,F8.1,A,F6.0)')                                &
     &    '  Proposed SideChannel flow of ',  ProposedSideChannelFlow,  &
     &    ' changed to minimum winter flow of ', MinWinterCap
              ProposedSideChannelFlow = MinWinterCap
!    XX Add a counter here
           END IF          
        END IF
      END IF
      endif
      END

