!
!
!-----------------------------------------------------------------------
!
!
      LOGICAL FUNCTION ReadDaily(Line, Year, Month, Qmonth, Floats)
!
!     This routine reads a line from file FUNIT according to standard
!     format for daily data (one line each qmonth, 7-8 values on line),
!     i.e.:  yr, mon, qmonth, values ==> I5,I3,I2,8(F8)
!
! modules
      USE par_kind
!
      IMPLICIT NONE
!
!     Declarations regarding subroutine arguments:
!      
      CHARACTER Line*(*)
!               (Input) The data line passed to this subroutine.
!
      INTEGER   Year, Month, Qmonth
!               (Output) The year, month, and quarter-month read from the 
!               data line if parsed OK, or the IOSTAT value if READ() 
!               fails (the function returns in this case .FALSE.).
!
      real(wp) :: Floats(8)
!               (Output) The 7 or 8 64-bit floating point values read from
!               the data line.  Compilers may squawk about re-dimensioning
!               here, since usually uses a column from the much bigger
!               array of scratch 64-bit floats (FWRK), but the usage seems
!               consistent with the ANSI F77 standard, so don't let the
!               warnings bother you too much.
!
!
!     Declarations regarding local variables or functions:
!
      INTEGER   I 
! 
!
      ReadDaily = .FALSE.
!
      READ(Line,'(I5, I3, I2, 8D8.0)',ERR=233,IOSTAT=I) Year, Month,    &
     &    Qmonth, Floats
!
!     See if got bogus dates
      IF( Year .GE. 1 .AND. (Month .GE. 1 .AND. Month .LE. 12) .AND.    &
     &   (Qmonth .GE. 1 .AND. Qmonth .LE. 4) ) THEN
         ReadDaily = .TRUE.
         RETURN
      ENDIF
   
  233 Year = I
      END
!      
!
!
!-----------------------------------------------------------------------
!
      INTEGER FUNCTION LASTKC (STRING, STRLEN)
!
      IMPLICIT NONE
!
      INTEGER STRLEN, ASEQNO, I
      CHARACTER STRING*(*)
!
!     Start at end of string, and work backwards, looking for a regular
!     "keyboard" character (ASCII char between decimal 33 and 126, not
!     including the space character).
!
!     THIS ROUTINE RETURNS A VALUE OF 1 IF NO "KEYBOARD" CHARACTERS FOUND!
!     This behaviour is so it can used fearlessly as a string limit, such
!     as in SAMPLE(1:LASTKC(SAMPLE, 127)).  If returned 0 or some other
!     spurious value, then could have unpredictable results cropping up 
!     all over code.  So the moral is to always make sure that there is
!     at least one "keyboard" character in the string when you call this.
!
!     No doubt this could be a lot more efficient.  Maybe remove call
!     to ICHAR and just assume that the value in each byte is actually
!     the ASCII code.  But I don't think the CGLRRM will spend too much
!     of its time processing strings, so no big savings to had anyway.
!
      DO 10 I=STRLEN,1,-1
!        Get ASCII sequence number using ICHAR:
         ASEQNO=ICHAR(STRING(I:I))
   10    IF(ASEQNO .GE. 33 .AND. ASEQNO .LE. 126)GOTO 20
!
!     In case goes through loop, make sure don't return 0
      I=1
   20 LastKC = I
      END
!      
!-------------------------------------------------------------------------
!
!
      SUBROUTINE GETQTRMONBOUNDS (YEAR, MONTH, QTR)
!
!     GetQtrMonthBounds determines the last day of each quarter-month based on    
!     the definitions received from USACE - Buffalo.                              
!
      IMPLICIT NONE
!
!
!     Declarations regarding subroutine arguments:
!      
      INTEGER      YEAR
!                  (Input) The year of interest.
!
      INTEGER      MONTH
!                  (Input) The month of interest.
!
      INTEGER      QTR(4)
!                  (Output) Array containing the ending day of each quarter-month
!                  in MONTH and YEAR. 
!
!                  
!     Don't need number of days in year, so just use QTR(1) as scratch for now:
      CALL NUMDAYS(YEAR, MONTH, QTR(1), QTR(4))
!
      IF (QTR(4) .LE. 29 ) THEN
         QTR(1) = 7
         QTR(2) = 14
         QTR(3) = 21
      ELSE
         QTR(1) = 8
         QTR(2) = 15
         QTR(3) = 23
      ENDIF
!
      END
!
!-------------------------------------------------------------------------
!
!
      INTEGER FUNCTION GetDaysInQMon(Year, Month, Qmonth)
!
!     Determines length of the specified quarter-month. 
!
      IMPLICIT NONE
!
!
!     Declarations regarding subroutine arguments:
!      
      INTEGER      Year, Month, Qmonth
!                  (Input) Identify the quarter-month of interest.
!
!
!     Declarations regarding local variables:
!
      INTEGER      QTRBNDS(4)
!                  Just scratch, holding the ending day of the month 
!                  for each quarter-month
! 
!
      CALL GetQtrMonBounds (Year, Month, QTRBNDS)
!
      IF ( Qmonth .EQ. 1 ) THEN
         GetDaysInQMon = QtrBnds(Qmonth)
      ELSE
         GetDaysInQMon = QtrBnds(Qmonth) - QtrBnds(Qmonth - 1)
      ENDIF
!
      END
!
!-------------------------------------------------------------------------
!
!
      INTEGER FUNCTION GETQMONNUM (YEAR, MONTH, DAY)
!
!     Determines which quarter-month the specified date is in. 
!
      IMPLICIT    NONE
!
!
!     Declarations regarding subroutine arguments:
!      
      INTEGER      YEAR
!                  (Input) The year of interest.
!
      INTEGER      MONTH
!                  (Input) The month of interest.
!
      INTEGER      DAY
!                  (Input) The day of interest.
!
!
!     Declarations regarding local variables:
!
      INTEGER      QTRBNDS(4)
!                  Just scratch, holding the ending day of the month 
!                  for each quarter-month
! 
!
      CALL GETQTRMONBOUNDS (YEAR, MONTH, QTRBNDS)
!
      DO 33, GETQMONNUM = 1, 4
!
   33    IF( DAY .LE. QTRBNDS(GETQMONNUM) ) RETURN
!
!
      STOP 'Invalid DAY in GETQMONNUM'
!
      END
!
!-------------------------------------------------------------------------
!
!
!
      LOGICAL FUNCTION LEAPYEAR (YEAR)
!
!     Returns .TRUE. if the specified year is a leap year
!
      IMPLICIT   NONE
!
!     Declarations regarding subroutine arguments:
!      
      INTEGER      YEAR
!                  (Input) The year of interest.
!
      LEAPYEAR = .FALSE.
!
      IF(  MOD(YEAR, 4) .EQ. 0 )THEN
!        Divisible by four, so might be a leap year
!
         IF( MOD(YEAR, 100) .NE. 0 .OR. MOD(YEAR, 400) .EQ. 0 ) THEN
!           It's either not a century, or divisible by 400, so must be a leap year
            LEAPYEAR = .TRUE.
         ENDIF
      ENDIF
!
      END
!
!
!
!-------------------------------------------------------------------------
!
!
      SUBROUTINE NUMDAYS(YEAR,MONTH,DAYSINYEAR,DAYSINMONTH)
!
!     Computes the number of days in the specified month and in the year.
!
      IMPLICIT   NONE
!
!     Declarations regarding subroutine arguments:
!      
      INTEGER      YEAR
!                  (Input) The year of interest.
!
      INTEGER      MONTH
!                  (Input) The month of interest.
!
      INTEGER      DAYSINYEAR
!                  (Output) The number of days in YEAR.
!
      INTEGER      DAYSINMONTH
!                  (Output) The number of days in MONTH.
!
!     Declarations regarding local variables or functions:
!
      LOGICAL      LEAPYEAR
!                  Returns .TRUE. if a leap year
!
      INTEGER      DAYSEACHMONTH(12) 
!                  Contains the number of days in each month
!
!
      DATA DAYSEACHMONTH /31,28,31,30,31,30,31,31,30,31,30,31/
!
!     The following should never happen if called properly.
!     If it happens, logic is bad in calling routine.
      IF(MONTH .LT. 1 .OR. MONTH .GT. 12) STOP 'Ren - get help!'
!
!
      DAYSINYEAR = 365
      DAYSINMONTH = DAYSEACHMONTH(MONTH)
!
      IF( LEAPYEAR(YEAR) ) THEN
!         
         DAYSINYEAR = DAYSINYEAR + 1
!
         IF( MONTH .EQ. 2 ) THEN
            DAYSINMONTH = DAYSINMONTH + 1
         ENDIF
      ENDIF
!
      END
!
!
!-------------------------------------------------------------------------
!
!-----------------------------------------------------------------------
!
!
      INTEGER FUNCTION DateSequence (DAY, MONTH, YEAR)
!
!     This routine computes the number of the date, with days numbered
!     sequentially from 1 (corresponding to the base date).  the base
!     date must be 1 March on any LEAP YEAR or century boundary. Dates
!     must be between the base date and the base date plus 2,147,483,646.
!     This routine is set up to use only 4-byte integer variables and 
!     to avoid intermediate use of reals in the evaluation of expressions.
!     It is a modification of the previous version which used 2-byte integer
!     variables. For reference on the 2-byte version of this subroutine
!     see Dr. Dobb's Journal 80:66-70.  Adapted by Tim Hunter.
!
!     Note that this routine correctly handles the year 2000, as well as all
!     other leap year issues.
!
      IMPLICIT NONE
!
      INTEGER   BASEYEAR
      PARAMETER (BASEYEAR = 0)
!
      INTEGER   DAY, MONTH, YEAR
!
      INTEGER   NUMYEARS, TOTC, LEAPC, SKIPC, NONLEAPC
!
      IF (YEAR .LE. BASEYEAR) STOP 'YEAR .LE. BASEYEAR'
!
      NUMYEARS = YEAR - BASEYEAR - 1 + (MONTH + 9) / 12
      TOTC = (BASEYEAR + NUMYEARS)/100 - (BASEYEAR/100)
      SKIPC = 3 - MOD(BASEYEAR/100, 4)
      LEAPC = (TOTC + 3 - SKIPC) / 4
      NONLEAPC = TOTC - LEAPC
!
      DateSequence = (365 * NUMYEARS + (NUMYEARS/4) - NONLEAPC          &
     &                + (153 * MOD(MONTH + 9, 12) + 2) / 5 + DAY) 
      END
!
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
      INTEGER FUNCTION ENDSTR (STRING, STRLEN)
!
      IMPLICIT NONE
!
      INTEGER STRLEN, ASEQNO, I
      CHARACTER STRING*(*)
!      DIMENSION STRING (1,*)
!
!     Search from beginning of string looking for a regular "keyboard"  
!     character (ASCII char between decimal 33 and 126, not including 
!     the space character).  Return position of last "keyboard"
!     character before whitespace or control characters.
!
!     If the string contains no "keyboard" characters, 0 is returned.
!     DON'T USE THE RESULT OF THIS FUNCTION IN A STRING LIMIT UNLESS
!     YOU POSITIVELY KNOW THAT THE STRING CONTAINS A "KEYBOARD" CHARACTER!
!     For instance, SAMPLE(ENDSTR(SAMPLE, 127):) will cause unexpected
!     results if SAMPLE is all blanks or something.  So safe use of this
!     for parsing lines read in from data files would be two-step, as in:
!          I= ENDSTR(SAMPLE, 127)
!          IF(I .NE. 0)THEN
!              WRITE(90,2015)SAMPLE(1:I)
!          ENDIF
!
!     No doubt this could be a lot more efficient.  Maybe remove call
!     to ICHAR and just assume that the value in each byte is actually
!     the ASCII code.  But I don't think the CGLRRM will spend too much
!     of its time processing strings, so no big savings to had anyway.
!
      DO 10 I=1,STRLEN,1
!        Get ASCII sequence number using ICHAR:
         ASEQNO=ICHAR(STRING(I:I))
!        write(*,'(A,I2,A,A1)')' ENDSTR=',ENDSTR,'   ',STRING(1,ENDSTR)
   10    IF(ASEQNO .LT. 33 .OR. ASEQNO .GT. 126)GOTO 20
!
   20 IF(I.GT. STRLEN .OR. ASEQNO .LT. 33 .OR. ASEQNO .GT. 126)THEN
         I = I - 1
      ENDIF

      ENDSTR = I
      END
!
!-----------------------------------------------------------------------
!      
!      
      CHARACTER*16 FUNCTION MakeIntervalName( IntervalNumber,   &
     &   Capitalize, Length)
!
!     Stupid little function that returns a string like "daily" or
!     "Daily" depending on the arguments.  The string is terminated by
!     a space character, for use with function ENDSTR().  The strings
!     returned by this function differ from the strings in array
!     IntervalName (in COMMON block RCRUNC) only by spelling out
!     "quarter-monthly" and optionally capitalizing.
!
      IMPLICIT NONE 
!
!     Declarations regarding subroutine arguments:
!      
      INTEGER      IntervalNumber 
!                  (Input) A number from 1-4 designating the routing 
!                  interval of interest.  1=daily, 2=weekly, 
!                  3=qmonthly, 4=monthly.
!      
      LOGICAL      Capitalize 
!                  (Input) Tells whether to capitalize the first letter
!                  (and ninth letter for Quarter-Monthly). 
!      
      INTEGER      Length 
!                  (Output) The number of characters to use in the
!                  string being returned.
!      
!     Declarations regarding local variables:
!
      CHARACTER*26 IntervalNames
!                  Contains the text strings 'DAILY' 'MONTHLY' etc.  
!                  Quarter-monthly string runs from characters 12-26; 
!                  monthly string runs from characters 20-26; weekly
!                  runs from 6-11; daily string runs from 1-5.
!
      CHARACTER*16 TempString

      DATA IntervalNames /'dailyweeklyquarter-monthly'/
!
      IF ( IntervalNumber .EQ. 1 ) THEN
         TempString = IntervalNames(1:5)//' '
         Length = 5
!
      ELSE IF ( IntervalNumber .EQ. 2 ) THEN
         TempString = IntervalNames(6:11)//' '
         Length = 6
!
      ELSE IF ( IntervalNumber .EQ. 3 ) THEN
         TempString = IntervalNames(12:26)//' '
         IF(Capitalize) TempString(9:9) = 'M'
         Length = 15
!
      ELSE IF ( IntervalNumber .EQ. 4 ) THEN
         TempString = IntervalNames(20:26)//' '
         Length = 7
!
      ELSE
         WRITE(*,'(A, I8)')                                 &
     &      ' Bad IntervalNumber in MakeIntervalName():',   &
     &      IntervalNumber
         STOP 'The Brain'
      ENDIF
!
      IF(Capitalize) TempString(1:1) =                      &
     &   CHAR(ICHAR( TempString(1:1) ) - 32)
!
      MakeIntervalName = TempString

      END
!
!
!-----------------------------------------------------------------------
!
      CHARACTER*1 FUNCTION EOPType( RoutLoopIndex ) 
!
!     This routine returns a character indicating whether the current 
!     period for a given routing loop (i.e., 1=Superior, 2=middle lakes, 
!     3=Ontario) ends a quarter-month or month.  If not, returns a 'D'
!     for daily routing.  Weekly routing always returns a 'W'.  The date
!     information is read from the PeriodEnd arrays in the COMMON block.
! 
! modules
      USE par_kind
      USE lakesup_mod

      IMPLICIT NONE
!
!     Declarations regarding subroutine arguments:
!      
      INTEGER          RoutLoopIndex
!                      (Input)  Index into RoutLoopIndex
!
!
!     Declarations regarding local variables or functions:
!
      INTEGER          I
!                      Index to the routing interval
!
      INTEGER          QTRBNDS(4), D
!                      Holds the ending day of the month for 
!                      each quarter-month, and a scratch counter
! 
      I = RoutingInterval(RoutLoopIndex)
!
      IF( I .EQ. 4) THEN
!        If monthly, then all periods get labeled 'EOM'
         EOPType = 'M'
      ELSEIF( I .EQ. 3) THEN
!        If qmonthly, then check if it the last qmonth (i.e., EOM)
         EOPType = 'Q'
         IF( PeriodEnd(1,RoutLoopIndex) .GT. 27) EOPType = 'M'
      ELSEIF( I .EQ. 2) THEN
!        If weekly, then all periods get labeled 'EOW' regardless of when end
         EOPType = 'W'
      ELSE
!        If daily check coincidence with month/qmonth
         EOPType = 'D'
         CALL GETQTRMONBOUNDS (PeriodEnd(4,RoutLoopIndex),              &
     &      PeriodEnd(3,RoutLoopIndex), QTRBNDS)
         DO 33, D = 1,3
   33       IF( PeriodEnd(1,RoutLoopIndex) .EQ. QTRBNDS(D)) EOPType= 'Q'
         IF( PeriodEnd(1,RoutLoopIndex) .EQ. QTRBNDS(4) ) EOPType = 'M'
      ENDIF
!
      END
!
!
!-----------------------------------------------------------------------
!
      FUNCTION GETAREA( LAKENUM, Z )
!
! modules
      USE par_kind
      USE surface_mod

      IMPLICIT NONE
!
!     Declarations regarding subroutine arguments:
!      
      real(wp) :: GETAREA
      real(wp) :: Z
!                  The water surface elevation of the lake.
!
      INTEGER      LAKENUM
!                  The lake id number to be used as the first element of
!                  array AVSZ.
!
!     Declarations regarding local variables or functions:
!
      INTEGER      I, J
      real(wp) :: TMPX1, TMPX2
      real(wp) :: POLATE
!                  scratch variable
!
      IF ( NUMAVSZ(LAKENUM) .LE. 1 ) THEN
!        constant area:
         GETAREA=AVSZ(LAKENUM,1,2) 
      ELSE
!
!        inter/extra polate from curve.  find closest X values to use:
         J = 2
         TMPX1 = ABS((Z - AVSZ(LAKENUM,1,1))+(Z - AVSZ(LAKENUM,2,1))) 
         DO 311, I = 3, NUMAVSZ(LAKENUM)
            TMPX2=ABS((Z-AVSZ(LAKENUM,I-1,1))+(Z-AVSZ(LAKENUM,I,1)))
            IF( TMPX2 .LT. TMPX1 ) THEN
               J = I
               TMPX1 = TMPX2
            ENDIF
  311       CONTINUE
!
!        Figure out which message level to use
         GETAREA=POLATE(AVSZ(LAKENUM,J-1,1),AVSZ(LAKENUM,J-1,2),        &
     &      AVSZ(LAKENUM,J,1), AVSZ(LAKENUM,J,2), Z, I)
      ENDIF
      END        
!

!
!-----------------------------------------------------------------------
!
      FUNCTION POLATE( X1, Y1, X2, Y2, X, Verbosity )
!
!     This funtion returns the value corresponding to X that is inter- 
!     or extrapolated from the line segment defined by (X1,Y1) and 
!     (X2, Y2).  It tries to catch undefined slopes and avoid division
!     by zero, but relies on the calling procedure to provide the 
!     messaging information that describes how the situation came about.
!
!
! modules
      USE par_kind
!
      IMPLICIT NONE
!
!     Declarations regarding subroutine arguments:
!      
      real(wp) :: POLATE
      real(wp) :: X1, Y1, X2, Y2
!                  Floating point numbers defining the line to 
!                  interpolate within or extrapolate from.
!
      real(wp) :: X
!                  The floating point ordinate that we are
!                  interpolating an abscissa for.
!
      INTEGER      Verbosity
!
!     Declarations regarding local variables or functions:
!
      INTEGER      I, J
      real(wp) :: ExtrapDist
      LOGICAL      LOGUSE(8)
!                  scratch variables
!
!     Declarations regarding COMMON block variables:
!
      INTEGER      Extrapolations
      real(wp) :: LargestExtrapolation
!                  Variables used to describe inadequate area-elev data
      COMMON /EXTRAP/   LargestExtrapolation, Extrapolations 
!
!
!
      I = 0
!     First make sure we don't divide by zero.  Should get 15 
!     significant digits in a DOUBLE, so test at that level:
!
      IF( (X2 - X1)*1.D14 .LT. 1._wp) THEN
         I = 1051
      ELSE
!        Should be able to compute a result, so do it:
         POLATE = Y1 + (X-X1) * (Y2-Y1)/(X2-X1)
!
!        If extrapolating, squawk about it, but do it anyway.
         IF((X .LT. X1 .AND. X .LT. X2) .OR.       &
     &      (X .GT. X2 .AND. X .GT. X1)) THEN
            Extrapolations = Extrapolations + 1
            IF(X .LT. X1 .AND. X .LT. X2) THEN
               IF ( X2 .LT. X1 ) THEN 
                  ExtrapDist = X2 - X
               ELSE
                  ExtrapDist = X1 - X
               ENDIF
            ELSE 
               IF ( X2 .GT. X1 ) THEN 
                  ExtrapDist = X - X2
               ELSE
                  ExtrapDist = X - X1
               ENDIF
            ENDIF
            IF ( ExtrapDist > LargestExtrapolation ) THEN
               LargestExtrapolation = ExtrapDist
            ENDIF
            I = 1052
         ENDIF
      ENDIF 


!
    5 FORMAT(A,D27.17,A,D27.17,A1)
      END
!

!----&------------------------------------------------------------------
!----&------------------------------------------------------------------
!
!
      FUNCTION USS (SUPFLOW, MHUMEAN)
!
! modules
      USE par_kind
!
      IMPLICIT NONE
!
!     Compute the level at the US Slip Gauge by method of bisection of GLERL
!
!     Lifted out of Env Canada Plan 77A model, with considerable mutation
!  
!
      real(wp) :: USS
      INTEGER SUPFLOW
!     Must be to 10m3s if MHUMean is to cm, or won't converge.
!
      real(wp) :: MHUMEAN, UPLIMIT, LOLIMIT
      INTEGER VALUE, Tmp, I
!     real(wp) :: ROUND
!
!     initial limits are Uplimit = Superior max. Lolimit= current MHU
!     and USS = a typical US slip level
!
!
      IF (MHUMEAN.LE.(172.30_wp)) THEN
         USS = DBLE(-9999)
         RETURN
      ENDIF 
!
!     SUPFLOW = NINT(Round( DBLE(SupFlow), 1 ))
!     SupFlow already integer
      UPLIMIT = 183.8_wp
      LOLIMIT = MHUMEAN
      USS = 176.5_wp
!
!      Write(*, '(I8,4F9.3)') SUPFLOW, MHUMEAN, USS, UpLimit, LoLimit
!
!     Repeat to achieve tolerance of 1 cm. SUPFLOW in 10m3s
!
      DO 11, I = 1, 99
!
!         Tmp = NINT(Round(USS, -2) * 100)
         Tmp = NINT(USS * 100)
         USS = (UpLimit + LoLimit) / 2 
!
!
!         VALUE = NINT( Round(347.3_wp * 
!     &      ((USS-MHUMEAN-0.06_wp)**0.4_wp)*((USS - 172.30_wp)**1.5_wp), 1))
         VALUE = NINT( 347.3_wp *                                       &
     &      ((USS-MHUMEAN-0.06_wp)**0.4_wp)*((USS - 172.30_wp)**1.5_wp))
!
!        Write(*, '(2I8,3F9.3)') Tmp, Value, USS, 34.73_wp * 
!    &      ((USS-MHUMEAN-0.06_wp)**0.4_wp)*((USS - 172.30_wp)**1.5_wp)
!
!         VALUE = NINT(Round( 34.73_wp * 
!     &      ((USS-MHUMEAN-0.06_wp)**0.4_wp)*((USS - 172.30_wp)**1.5_wp)
!     &      , 0)) - SUPFLOW
!
         IF (VALUE .GE. SUPFLOW) UPLIMIT=USS
         IF (VALUE .LT. SUPFLOW) LOLIMIT=USS
!
!   11    IF ( NINT(Round(USS,-2) * 100) .EQ. Tmp
!     &      .AND. VALUE .EQ. SUPFLOW) GOTO 12
   11    IF(NINT(USS * 100) .EQ. Tmp .AND. VALUE .EQ. SUPFLOW) GOTO 12
!
!  10 MDPNT = MIDPOINT
!     MIDPOINT=(UPLIMIT+LOLIMIT)*0.5
!       VALUE =  DBLE(34.73) * ((MIDPOINT-MHUMEAN-0.06)**0.4)*
!     &          ((MIDPOINT-DBLE(172.30))**1.5)-SUPFLOW
!       IF (VALUE.GE.DBLE(0.0)) UPLIMIT=MIDPOINT
!       IF (VALUE.LT.DBLE(0.0)) LOLIMIT=MIDPOINT
!       IF ((ROUND(MIDPOINT,DBLE(100)).NE.ROUND(MDPNT,DBLE(100))).AND.
!     &       (VALUE.NE.DBLE(0.))) GOTO 10
!      ENDIF 
!
!       USS = MIDPOINT
!
         STOP ' Gilligan'
!
   12 END
!




      FUNCTION SWPierGateFlow( SWPLev, Gates )
!
!     Computes gate flow from SW Pier gate equations
!
! modules
      USE par_kind
      USE outflow_mod

      IMPLICIT NONE
!
!     Declarations regarding subroutine arguments:
!      
      real(wp) :: SWPierGateFlow
      real(wp) :: SWPLev
!                      (input) The SW Pier level.
!
      REAL             Gates
!                      (input) The number of gates open
!
!
!     Declarations regarding local variables or functions:
!
      INTEGER          I
!                      Scratch variable
!     
!
      I = NINT(Gates - 0.1) 
      IF(I .EQ. 0) THEN
!        Gates must be 0.5. Use half of diff between 1 and 2 gates
         SWPierGateFlow=                                                &
     &            (SWPGateEqA(2) * (SWPLev - SWPGateEqB(2))**1.5_wp -   &
     &            SWPGateEqA(1) * (SWPLev - SWPGateEqB(1))**1.5_wp) /2
      ELSE
         SWPierGateFlow=SWPGateEqA(I) * (SWPLev-SWPGateEqB(I))**1.5_wp 
      ENDIF
      END



!
!
      FUNCTION NonCompWorksFlow( SupLevel, MHLevel )
!
!     Returns the "Side Channel" flow (everthing that doesn't go
!     through the Compensating Works.
!
! modules
      USE par_kind
!
      IMPLICIT NONE 
!
!     Declarations regarding subroutine arguments:
!      
      real(wp) :: NonCompWorksFlow
      real(wp) :: SupLevel, MHLevel
!                   (Input)  The levels of lakes Superior and Huron.
!
!
!
      NonCompWorksFlow = 2320._wp
!
      END




      FUNCTION CompWorksFreeboard(             &
     &   Mon, SupLvl, CompWorksTopElev)
!
!     returns the amount of "freeboard" before the structure is threatened
!      
! modules
      USE par_kind
!
      IMPLICIT NONE 
!
!     Declarations regarding subroutine arguments:
!      
      real(wp) :: CompWorksFreeboard
      INTEGER          Mon
!                      (input) The month number
!
      real(wp) :: SupLvl, CompWorksTopElev
!                      (input) The Lake Superior level and the elevation
!                      at which the Compensating Works becomes threatened
!
!
!     Declarations regarding local variables or functions:
!
!      INTEGER          I
!                      Scratch variable
!
      INTEGER          SWPMethod, SWPRoundLvl
!
      real(wp) :: SWPierLevel
!                      Computed SW Pier level
!
      INTEGER          NonGatedFlow
      real(wp) :: GateFlow
      REAL             Gates
!                      Presently only required as dummy arguments
!                      to pass along to GetSWPier(), but may be
!                      used in future 
!
!     Presently hardcoded to use old relationship between SW Pier and 
!     Lake Superior instead of the one developed in 2000 by EC, since 
!     old one is simpler for this initial implementation.  Newer method 
!     is also function of St. Marys River flow, which requires 
!     decisions about the outflow, so leave that as a refinement.
      SWPMethod = 1
      Gates = -9999.0
      NonGatedFlow = -9999
!
!     Don't do any rounding of intermediate computations
      SWPRoundLvl = -99
!
      CALL GetSWPier(SupLvl, Mon, Gates, NonGatedFlow, SWPRoundLvl,     &
     &   SWPMethod, SWPierLevel, GateFlow, 0)
!      
      CompWorksFreeboard = CompWorksTopElev - SWPierLevel
!
      END

